package com.agilex.bcm.tests.messages;

import com.agilex.bcm.framework.base.BcmTest;
import com.agilex.bcm.framework.base.FacilityMessageModel;
import com.agilex.bcm.framework.pages.staff.BcmAddMessage;
import com.agilex.common.helpers.DateRange;
import com.agilex.common.helpers.TimeOfDay;
import com.agilex.common.helpers.ValueUnits;
import org.junit.Test;
import com.agilex.bcm.framework.base.StaffUserModel;

import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

import org.junit.Assert;

public class FacilityMessages extends BcmTest {
    @Test
    public void verifyDefaultFacilityMessageFormValues() {
        // Set up test variables
        StaffUserModel testUser = new StaffUserModel(deployment.staffUser4);
        String dateToday = generateDateString();


        // Open the Annie staff app, log in as a clinical staff/admin, and switch to the Broadcast Admin view
        bcm.openAnnieStaffApp();
        bcm.staffLogin().logInAs(testUser.accessCode, testUser.verifyCode, testUser.station);
        bcm.staffPatientSearch().switchToBroadcastAdminView();

        // Add a new message
        bcm.bcmVaFacilityMessages().clickAddMessageButton();
        bcm.bcmAddMessage().waitForAddTitleHeader();

        // Verify pre-populated fields
        Assert.assertTrue(bcm.bcmAddMessage().isMessageCreatedByValueCorrect(testUser.getFullName()));
        Assert.assertTrue(bcm.bcmAddMessage().isMessageCreatedOnValueCorrect(dateToday));
        Assert.assertTrue(bcm.bcmAddMessage().isMessageTitleValueCorrect(EMPTY_STRING));
        Assert.assertTrue(bcm.bcmAddMessage().isMessageMessageValueCorrect(EMPTY_STRING));
        Assert.assertTrue(bcm.bcmAddMessage().isMessageScheduleValueCorrect(BcmAddMessage.OPTION_NONE));
        Assert.assertTrue(bcm.bcmAddMessage().isMessageSendMessageTimeHoursValueCorrect(BcmAddMessage.OPTION_NONE));
        Assert.assertTrue(bcm.bcmAddMessage().isMessageSendMessageTimeMinutesValueCorrect(BcmAddMessage.OPTION_NONE));
        Assert.assertTrue(bcm.bcmAddMessage().isMessageSendMessageTimeAmPmValueCorrect(BcmAddMessage.OPTION_UNDEFINED));
        Assert.assertTrue(bcm.bcmAddMessage().isMessageStartDateValueCorrect(EMPTY_STRING));
        Assert.assertTrue(bcm.bcmAddMessage().isMessageEndDateValueCorrect(EMPTY_STRING));

        // Verify that the schedule select options are correct
        Assert.assertTrue(bcm.bcmAddMessage().areMessageScheduleOptionsCorrect());

        // Verify schedule field combinations
        bcm.bcmAddMessage().setMessageScheduleByValue("Once");
        Assert.assertFalse(bcm.bcmAddMessage().areScheduleSpecifiedDaysCheckboxesVisible());
        Assert.assertFalse(bcm.bcmAddMessage().areScheduleTimeBetweenCyclesFieldsVisible());

        bcm.bcmAddMessage().setMessageScheduleByValue("Specified Days");
        Assert.assertTrue(bcm.bcmAddMessage().areScheduleSpecifiedDaysCheckboxesVisible());
        Assert.assertFalse(bcm.bcmAddMessage().areScheduleTimeBetweenCyclesFieldsVisible());

        bcm.bcmAddMessage().setMessageScheduleByValue("Custom");
        Assert.assertFalse(bcm.bcmAddMessage().areScheduleSpecifiedDaysCheckboxesVisible());
        Assert.assertTrue(bcm.bcmAddMessage().areScheduleTimeBetweenCyclesFieldsVisible());
        Assert.assertTrue(bcm.bcmAddMessage().areScheduleTimeBetweenCyclesUnitsOptionsCorrect());

        // Verify that the schedule send time select options are correct
        Assert.assertTrue(bcm.bcmAddMessage().areMessageSendMessageTimeHoursOptionsCorrect());
        Assert.assertTrue(bcm.bcmAddMessage().areMessageSendMessageTimeMinutesOptionsCorrect());
        Assert.assertTrue(bcm.bcmAddMessage().areMessageSendMessageTimeAmPmOptionsCorrect());

        // Cancel message creation and abort the cancellation
        bcm.bcmAddMessage().cancelMessageCreation();
        bcm.bcmFacilityMessageCancellation().abortCancellation();
        bcm.bcmAddMessage().waitForAddTitleHeader();

        // Cancel message creation and confirm the cancellation
        bcm.bcmAddMessage().cancelMessageCreation();
        bcm.bcmFacilityMessageCancellation().confirmCancellation();
        bcm.bcmVaFacilityMessages().waitForTitleHeader();
    }

    @Test
    public void addAndEditFacilityMessageCheckValidation() {
        // Test users
        StaffUserModel testUser = new StaffUserModel(deployment.staffUser4);

        // Test flow flags
        boolean isMessageAdded = false, isMessageEdited = false;

        // Start and end dates
        String dateToday = generateDateString(), invalidStartDateFormat = "strawberry", invalidEndDateFormat = "banana";
        DateRange yearSpan = new DateRange(dateToday, generateDateString(Calendar.DATE, 364)),
                daySpan = new DateRange(dateToday, generateDateString(Calendar.DATE, 1)),
                daySpanInverted = new DateRange(dateToday, generateDateString(Calendar.DATE, -1)),
                daySpanBeforeToday = new DateRange(generateDateString(Calendar.DATE, -1), dateToday);

        // Start and end dates for the edit test
        String dateEditTwoWeeks = generateDateString(Calendar.DATE, 11);
        DateRange yearSpanEdit = new DateRange(dateEditTwoWeeks, generateDateString(Calendar.DATE, 21));

        // Days of the week sets
        Map<String, Boolean> emptyScheduleDays = new HashMap<String, Boolean>(0);
        emptyScheduleDays.put("Monday", false);
        emptyScheduleDays.put("Tuesday", false);
        emptyScheduleDays.put("Wednesday", false);
        emptyScheduleDays.put("Thursday", false);
        emptyScheduleDays.put("Friday", false);
        emptyScheduleDays.put("Saturday", false);
        emptyScheduleDays.put("Sunday", false);
        Map<String, Boolean> fullScheduleDays = new HashMap<String, Boolean>(0);
        fullScheduleDays.put("Monday", true);
        fullScheduleDays.put("Tuesday", true);
        fullScheduleDays.put("Wednesday", true);
        fullScheduleDays.put("Thursday", true);
        fullScheduleDays.put("Friday", true);
        fullScheduleDays.put("Saturday", true);
        fullScheduleDays.put("Sunday", true);

        // Times between cycles
        ValueUnits validTimeBetweenCycles = new ValueUnits(10, "Minute");
        ValueUnits validEditedTimeBetweenCycles = new ValueUnits(3, "Hour");
        ValueUnits emptyTimeBetweenCycles = new ValueUnits(EMPTY_STRING, EMPTY_STRING);
        ValueUnits[] invalidTimesBetweenCycles = {
                new ValueUnits(EMPTY_STRING, "Minute"),
                new ValueUnits(10, EMPTY_STRING)
        };

        // Test times
        TimeOfDay validTime = new TimeOfDay(8, 30, TimeOfDay.AM);
        TimeOfDay validEditTime = new TimeOfDay(2, 15, TimeOfDay.PM);

        // Facility message models
        FacilityMessageModel validMessage = new FacilityMessageModel(testUser.getFullName(), dateToday, "Test Title",
                "Test Message", "Once", validTime, yearSpan.getStartDate(), yearSpan.getEndDate());
        FacilityMessageModel validEditMessage = new FacilityMessageModel(testUser.getFullName(), dateToday,
                "Test Title EDITED", "Test Message EDITED", "Custom", validEditedTimeBetweenCycles, validEditTime,
                yearSpanEdit.getStartDate(), yearSpanEdit.getEndDate());
        FacilityMessageModel[] invalidMessages = {
                // Title & message are whitespace
                // Specified days schedule with no days selected
                // Invalid start and end date formats
                new FacilityMessageModel(testUser.getFullName(), dateToday, " ", " ", "Specified Days",
                        emptyScheduleDays, validTime, invalidStartDateFormat, invalidEndDateFormat),

                // Custom schedule with no time between cycles set
                // End date set to before the start date
                new FacilityMessageModel(testUser.getFullName(), dateToday, "Test Title", "Test Message", "Custom",
                        emptyTimeBetweenCycles, validTime, daySpanInverted.getStartDate(), daySpanInverted.getEndDate()),

                // Specified days schedule with all days select when start to end date duration is only one day
                new FacilityMessageModel(testUser.getFullName(), dateToday, "Test Title", "Test Message",
                        "Specified Days", fullScheduleDays, validTime, daySpan.getStartDate(), daySpan.getEndDate()),

                // Start date set to before today
                new FacilityMessageModel(testUser.getFullName(), dateToday, "Test Title", "Test Message", "Once",
                        validTime, daySpanBeforeToday.getStartDate(), daySpanBeforeToday.getEndDate())
        };


        // Open the Annie staff app, log in as a clinical staff/admin, and switch to the Broadcast Admin view
        bcm.openAnnieStaffApp();
        bcm.staffLogin().logInAs(testUser.accessCode, testUser.verifyCode, testUser.station);
        bcm.staffPatientSearch().switchToBroadcastAdminView();
        bcm.bcmVaFacilityMessages().waitForTitleHeader();

        while(!(isMessageAdded && isMessageEdited)) {
            if(!isMessageAdded) {
                // Navigate to the page to add a new message
                bcm.bcmVaFacilityMessages().clickAddMessageButton();
                bcm.bcmAddMessage().waitForAddTitleHeader();

                // Set the test flow flag
                isMessageAdded = true;
            } else if(!isMessageEdited) {
                // Navigate to the page to edit an existing message
                bcm.bcmVaFacilityMessages().editMessage(bcm.bcmVaFacilityMessages().getMessageEntryIndexByTitle(validMessage.title));
                bcm.bcmAddMessage().waitForEditTitleHeader();

                // Verify that the fields are pre-populated correctly
                Assert.assertTrue(bcm.bcmAddMessage().isMessageTitleValueCorrect(validMessage.title));
                Assert.assertTrue(bcm.bcmAddMessage().isMessageMessageValueCorrect(validMessage.message));
                Assert.assertTrue(bcm.bcmAddMessage().isMessageScheduleValueCorrect(validMessage.schedule));
                if(validMessage.schedule.equals("Specified Days")) {
                    Assert.assertTrue(bcm.bcmAddMessage().areScheduleSpecifiedDaysCheckboxesCorrect(validMessage.scheduleSpecifiedDays));
                } else if(validMessage.schedule.equals("Custom")) {
                    Assert.assertTrue(bcm.bcmAddMessage().areScheduleTimeBetweenCyclesCorrect(validMessage.scheduleTimeBetweenCycles));
                }
                Assert.assertTrue(bcm.bcmAddMessage().isMessageSendMessageTimeCorrect(validMessage.sendMessageTime));
                Assert.assertTrue(bcm.bcmAddMessage().isMessageStartDateValueCorrect(validMessage.startDate));
                Assert.assertTrue(bcm.bcmAddMessage().isMessageEndDateValueCorrect(validMessage.endDate));

                // Set the test flow flag
                isMessageEdited = true;
            }

            // Fill in the message form with the first set of invalid values and check validation error messages
            // Title & message are whitespace
            // Specified days schedule with no days selected
            // Invalid start and end date formats
            bcm.bcmAddMessage().fillOutMessageFields(invalidMessages[0]);
            bcm.bcmAddMessage().saveMessageExpectValidationError();

            Assert.assertTrue(bcm.bcmAddMessage().isMessageHeaderValidationShown());
            Assert.assertTrue(bcm.bcmAddMessage().isMessageTitleEmptyValidationShown());
            Assert.assertTrue(bcm.bcmAddMessage().isMessageMessageEmptyValidationShown());
            Assert.assertTrue(bcm.bcmAddMessage().isMessageScheduleDaysEmptyValidationShown());
            Assert.assertTrue(bcm.bcmAddMessage().isMessageStartDateInvalidFormatValidationShown());
            Assert.assertTrue(bcm.bcmAddMessage().isMessageEndDateInvalidFormatValidationShown());

            // Fill in the message form with the second set of invalid values and check validation error messages
            // Custom schedule with no time between cycles set
            // End date set to before the start date
            bcm.bcmAddMessage().fillOutMessageFields(invalidMessages[1]);
            bcm.bcmAddMessage().saveMessageExpectValidationError();

            Assert.assertTrue(bcm.bcmAddMessage().isMessageHeaderValidationShown());
            Assert.assertTrue(bcm.bcmAddMessage().isMessageScheduleTimeBetweenCyclesValueEmptyValidationShown());
            Assert.assertTrue(bcm.bcmAddMessage().isMessageScheduleTimeBetweenCyclesUnitsEmptyValidationShown());
            Assert.assertTrue(bcm.bcmAddMessage().isMessageStartDateAfterEndDateValidationShown());
            Assert.assertTrue(bcm.bcmAddMessage().isMessageEndDateBeforeStartDateValidationShown());

            // Fill in the message form with the third set of invalid values and check validation error messages
            // Specified days schedule with all days select when start to end date duration is only one day
            bcm.bcmAddMessage().fillOutMessageFields(invalidMessages[2]);
            bcm.bcmAddMessage().saveMessageExpectValidationError();

            Assert.assertTrue(bcm.bcmAddMessage().isMessageHeaderValidationShown());
            Assert.assertTrue(bcm.bcmAddMessage().isMessageScheduleDaysOutOfRangeValidationShown());

            // Fill in the message form with the fourth set of invalid values and check validation error messages
            // Start date set to date before today
            bcm.bcmAddMessage().fillOutMessageFields(invalidMessages[3]);
            bcm.bcmAddMessage().saveMessageExpectValidationError();

            Assert.assertTrue(bcm.bcmAddMessage().isMessageHeaderValidationShown());
            Assert.assertTrue(bcm.bcmAddMessage().isMessageStartDateBeforeTodayValidationShown());

            // Run through the list of invalid Time Between Cycles values
            bcm.bcmAddMessage().setMessageScheduleByValue("Custom");
            for(ValueUnits invalidTimeBetweenCyclesEntry : invalidTimesBetweenCycles) {
                bcm.bcmAddMessage().setScheduleTimeBetweenCyclesValues(invalidTimeBetweenCyclesEntry);
                bcm.bcmAddMessage().saveMessageExpectValidationError();
                bcm.bcmAddMessage().isMessageScheduleTimeBetweenCyclesValueEmptyValidationShown();
            }

            // Fill in the message form with the correct message values and click the Save button
            bcm.bcmAddMessage().fillOutMessageFields(validMessage);
            bcm.bcmAddMessage().saveMessageExpectSuccess();
            bcm.bcmFacilityMessageConfirmation().dismissModal();
            bcm.bcmVaFacilityMessages().waitForTitleHeader();
        }
    }

    @Test
    public void verifyEnableAndDisable() {
        // Test variables
        StaffUserModel testUser = new StaffUserModel(deployment.staffUser4);
        FacilityMessageModel testMessage = new FacilityMessageModel(testUser.getFullName(), generateDateString(),
                "Enable Disable Test", "Enable disable test message", "Once", new TimeOfDay(8, 30, TimeOfDay.PM),
                generateDateString(), generateDateString(Calendar.YEAR, 1));
        int testMessageIndex;


        // Open the Annie staff app, log in as a clinical staff/admin, and switch to the Broadcast Admin view
        bcm.openAnnieStaffApp();
        bcm.staffLogin().logInAs(testUser.accessCode, testUser.verifyCode, testUser.station);
        bcm.staffPatientSearch().switchToBroadcastAdminView();

        // Create a new facility message to work with
        bcm.bcmVaFacilityMessages().clickAddMessageButton();
        bcm.bcmAddMessage().waitForAddTitleHeader();
        bcm.bcmAddMessage().fillOutMessageFields(testMessage);
        bcm.bcmAddMessage().saveMessageExpectSuccess();
        bcm.bcmFacilityMessageConfirmation().waitForModalTitle();
        bcm.bcmFacilityMessageConfirmation().dismissModal();
        bcm.bcmVaFacilityMessages().waitForTitleHeader();

        // Get the message index
        testMessageIndex = bcm.bcmVaFacilityMessages().getMessageEntryIndexByTitle(testMessage.title);

        // Disable a facility message and cancel the disabling
        bcm.bcmVaFacilityMessages().disableMessage(testMessageIndex);
        bcm.bcmFacilityMessageDisable().waitForModalHeader();
        bcm.bcmFacilityMessageDisable().abortDisable();
        bcm.bcmVaFacilityMessages().waitForTitleHeader();

        // Make sure the message is still enabled
        Assert.assertTrue(bcm.bcmVaFacilityMessages().isMessageEntryEnabled(testMessageIndex));

        // Disable a facility message and confirm the disabling
        bcm.bcmVaFacilityMessages().disableMessage(testMessageIndex);
        bcm.bcmFacilityMessageDisable().waitForModalHeader();
        bcm.bcmFacilityMessageDisable().confirmDisable();
        bcm.bcmVaFacilityMessages().waitForLoadingToClear();
        bcm.bcmVaFacilityMessages().waitForTitleHeader();

        // Make sure the message is disabled
        Assert.assertTrue(bcm.bcmVaFacilityMessages().isMessageEntryDisabled(testMessageIndex));

        // Enable a facility message and cancel the enabling
        bcm.bcmVaFacilityMessages().enableMessage(testMessageIndex);
        bcm.bcmFacilityMessageEnable().waitForModalHeader();
        bcm.bcmFacilityMessageEnable().abortEnable();
        bcm.bcmVaFacilityMessages().waitForTitleHeader();

        // Make sure the message is still disabled
        Assert.assertTrue(bcm.bcmVaFacilityMessages().isMessageEntryDisabled(testMessageIndex));

        // Enable a facility message and confirm the enabling
        bcm.bcmVaFacilityMessages().enableMessage(testMessageIndex);
        bcm.bcmFacilityMessageEnable().waitForModalHeader();
        bcm.bcmFacilityMessageEnable().confirmEnable();
        bcm.bcmAddMessage().waitForEditTitleHeader();
        bcm.bcmAddMessage().saveMessageExpectSuccess();
        bcm.bcmFacilityMessageConfirmation().waitForModalTitle();
        bcm.bcmFacilityMessageConfirmation().dismissModal();

        // Make sure the facility message is enabled
        bcm.bcmVaFacilityMessages().waitForTitleHeader();
        Assert.assertTrue(bcm.bcmVaFacilityMessages().isMessageEntryEnabled(testMessageIndex));
    }

    @Test
    public void verifyFacilityMessageFiltering() {
         // Set up test variables
         StaffUserModel testUser = new StaffUserModel(deployment.staffUser4);
         DateRange prePopulatedDates, invertedDates, futureDates;
         String[] progressValues = { "Pending Delivery", "Delivery In Progress", "Delivered" };


         // Open the Annie staff app, log in as a clinical staff/admin, and switch to the Broadcast Admin view
         bcm.openAnnieStaffApp();
         bcm.staffLogin().logInAs(testUser.accessCode, testUser.verifyCode, testUser.station);
         bcm.staffPatientSearch().switchToBroadcastAdminView();
         bcm.bcmVaFacilityMessages().waitForTitleHeader();

         // Verify that the options on the Progress and View select elements are correct
         Assert.assertTrue(bcm.bcmVaFacilityMessages().areProgressOptionsCorrect());
         Assert.assertTrue(bcm.bcmVaFacilityMessages().areViewOptionsCorrect());

         // Get the pre-populated Created From and Created To dates and generate other dates based on them
         prePopulatedDates = new DateRange(bcm.bcmVaFacilityMessages().getCreatedFromValue(),
                 bcm.bcmVaFacilityMessages().getCreatedToValue());
         invertedDates = new DateRange(prePopulatedDates.getEndDate(), prePopulatedDates.getStartDate());
         futureDates = new DateRange(generateDateString(prePopulatedDates.getStartDate(), Calendar.YEAR, 2),
                 generateDateString(prePopulatedDates.getEndDate(), Calendar.YEAR, 2));

         // Verify that there are no more than the maximum number of messages on the page
         Assert.assertTrue(bcm.bcmVaFacilityMessages().isMessageEntryCountWithinBounds());

         // Verify that the initial Created From and Created To values are consistent with the messages shown
         Assert.assertTrue(bcm.bcmVaFacilityMessages().doMessageCreatedOnValuesMatchFiltering());

         // Verify a date range for which there are no messages present
         bcm.bcmVaFacilityMessages().setCreatedFromValue(futureDates.getStartDate());
         bcm.bcmVaFacilityMessages().setCreatedToValue(futureDates.getEndDate());
         Assert.assertTrue(bcm.bcmVaFacilityMessages().isDataNotFoundMessageShown());

         // Verify an invalid date range, even when messages are present between them
         bcm.bcmVaFacilityMessages().setCreatedFromValue(invertedDates.getStartDate());
         bcm.bcmVaFacilityMessages().setCreatedToValue(invertedDates.getEndDate());
         Assert.assertTrue(bcm.bcmVaFacilityMessages().isDataNotFoundMessageShown());

         // Reset the date filters to their original pre-populated values
         bcm.bcmVaFacilityMessages().clickResetButton();

         // Verify Progress value filtering
         for(String value : progressValues) {
             bcm.bcmVaFacilityMessages().setProgressByValue(value);
             Assert.assertTrue(bcm.bcmVaFacilityMessages().doMessageProgressValuesMatchFiltering());
         }

         // Verify "Created By Me" View value filtering
         bcm.bcmVaFacilityMessages().clickResetButton();
         bcm.bcmVaFacilityMessages().setViewByValue("Created By Me");
         Assert.assertTrue(bcm.bcmVaFacilityMessages().doMessageCreatedByValuesMatchUser(testUser.getFullName()));
    }
}