package com.agilex.common.selenium;

import java.io.File;
import java.lang.reflect.Array;
import java.util.*;
import java.util.concurrent.TimeUnit;

import com.google.common.collect.ArrayListMultimap;
import org.openqa.selenium.*;
import org.openqa.selenium.NoSuchElementException;
import org.openqa.selenium.remote.BrowserType;
import org.openqa.selenium.remote.LocalFileDetector;
import org.openqa.selenium.support.ui.ExpectedCondition;
import org.openqa.selenium.support.ui.Select;
import org.openqa.selenium.support.ui.WebDriverWait;
import org.openqa.selenium.interactions.Actions;

/**
 * <font color="red"><b>NOTE:</b> Internet Explorer will NOT work locally unless you allow
 * active content to run locally.</font><br/>
 * 1. From Internet Explorer, select the Tools menu, then the Options...<br/>
 * 2. In the Internet Options dialog, select the Advanced tab...<br/>
 * 3. Scroll down until you see the Security options.  Enable the checkbox
 * "Allow active content to run in files on My Computer".<br/>
 * 4. Close the dialog, and quit Internet Explorer.  The changes will take
 * effect the next time you start IE.<br/>
 * This information was obtained here: http://www.genopro.com/help/report-generator/allow-blocked-content/
 * <br/><br/>
 * 
 * <hr>
 * Locators are used to find the Elements on the page.
 * It is possible for the locator to have an optional prefix to the string that would start
 * with one of the following:<br/><br/>
 * <table>
 * <tr><td valign="top">&quot;<b>name=</b>&quot;</td><td>Find the input element with matching &quot;name&quot; attribute.</td></tr>
 *
 * <tr><td valign="top">&quot;<b>xpath=</b>&quot;</td><td><font color="red">NOTE: Different behavior on different browsers</font><br/>
 * At a high level, WebDriver uses a
 * browser's native XPath capabilities wherever possible. On those browsers
 * that don't have native XPath support, WebDriver provides its own
 * implementation. This can lead to some unexpected behaviour unless you are
 * aware of the differences in the various xpath engines.
 * <table border="1"><thead>
 * <tr><th>Driver</th><th>Tag and Attribute Name</th><th>Attribute Values</th><th>Native XPath Support</th></tr>
 * </thead>
 * <tbody>
 * <tr><td>HtmlUnit Driver</td><td>Lower-cased</td><td>As they appear in the HTML</td><td>Yes</td></tr>
 * <tr><td>Internet Explorer Driver</td><td>Lower-cased</td><td>As they appear in the HTML</td><td>No</td></tr>
 * <tr><td>Firefox Driver</td><td>Case insensitive</td><td>As they appear in the HTML</td><td>Yes</td></tr>
 * </tbody></table>
 * <p>This is a little abstract, so for the following piece of HTML:</p>
 * <code>&lt;input type=&quot;text&quot; name=&quot;example&quot; /&gt;</code><br/>
 * <code>&lt;INPUT type=&quot;text&quot; name=&quot;other&quot; /&gt;</code><br/><br/>
 * And the following Java Code:<br/>
 * <code>List&lt;WebElement&gt; inputs = driver.findElements(By.xpath(&quot;//input&quot;));</code><br/>
 * The following number of matches will be found
 * <table border="1"><thead>
 * <tr><th>XPath expression</th><th>HtmlUnit Driver</th><th>Firefox Driver</th><th>Internet Explorer Driver</th></tr>
 * <tbody>
 * <tr><td>//input</td><td>1 (&quot;example&quot;)</td><td>2</td><td>2</td></tr>
 * <tr><td>//INPUT</td><td>0</td><td>2</td><td>0</td></tr>
 * </tbody></table>
 * Sometimes HTML elements do not need attributes to be explicitly declared
 * because they will default to known values. For example, the &quot;input&quot;
 * tag does not require the &quot;type&quot; attribute because it defaults to
 * &quot;text&quot;. The rule of thumb when using xpath in WebDriver is that
 * you should not expect to be able to match against these implicit attributes.
 * </td></tr>
 *
 * <tr><td valign="top">&quot;<b>/</b>&quot;</td><td><font color="red">NOTE: Different behavior on different browsers</font><br/>
 * A string starting with a slash is assumed to be an xpath statement
 * </td></tr>
 *
 * <tr><td valign="top">&quot;<b>domtagname=</b>&quot;</td><td>The DOM Tag Name of the element.</td></tr>
 *
 * <tr><td valign="top">&quot;<b>domclass=</b>&quot;</td><td>Class in this case refers to the attribute
 * on the DOM element. <font color="red">NOTE: Often in practical use there are many DOM elements
 * with the same class name, thus using this would find multiple elements
 * and only return the first element (which may not be what you want).</font></td></tr>
 *
 * <tr><td valign="top">&quot;<b>css=</b>&quot;</td><td>Like the name implies it is a locator strategy by
 * css. <font color="red">NOTE: Native browser support is used by default, so please refer to w3c
 * css selectors <http://www.w3.org/TR/CSS/#selectors> for a list of
 * generally available css selectors. If a browser does not have native
 * support for css queries, then Sizzle is used. IE 6,7 and FF3.0 currently
 * use Sizzle as the css query engine.Beware that not all browsers were created equal, some css that might
 * work in one version may not work in another.</font></td></tr>
 * 
 * <tr><td valign="top">&quot;<b>link=</b>&quot;</td><td>Find the link element with matching visible text.</td></tr>
 *
 * <tr><td valign="top">&quot;<b>partiallink=</b>&quot;</td><td>Find the link element with partial matching visible text.</td></tr>
 *
 * <tr><td valign="top">&quot;<b>id=</b>&quot;</td><td><i>(default)</i> This is the most efficient and preferred
 * way to locate an element.  If the string does not contain any of these
 * prefixes, it is assumed to be the equivalent of id=.</td></tr>
 *
 * <tr><td valign="top">&quot;<b>identifier=</b>&quot;</td><td>A synonym for &quot;id=&quot;</td></tr>
 * </table>
 * <br/>
 * <font color="red">NOTE: domclass, domtagname, and partiallink do not exist in Selenium RC -
 * they are specific to WebDriver. So you risk losing backwards compatibility by
 * using them.</font><br/><br/>
 *
 * Examples using each of the locators above:
 * <table border="1"><thead>
 * <tr><th>locator</th><th>Html source it would find</th></tr>
 * <tbody>
 * <tr><td>name=One</td><td>&lt;input type=&quot;checkbox&quot; id=&quot;chbx1&quot; name=&quot;One&quot; /&gt;</td></tr>
 * <tr><td>xpath=//*[@id=\&quot;chbx4\&quot;]</td><td>&lt;input type=&quot;checkbox&quot; id=&quot;chbx4&quot; name=&quot;Four&quot; /&gt;</td></tr>
 * <tr><td>//*[@id=\&quot;chbx3\&quot;]</td><td>&lt;input type=&quot;checkbox&quot; id=&quot;chbx3&quot; name=&quot;Three&quot; /&gt;</td></tr>
 * <tr><td>domtagname=a</td><td>&lt;a href=&quot;javascript:toggle_visibility('toggleText');&quot;>Toggle Me&lt;/a&gt;</td></tr>
 * <tr><td>domclass=gsfi</td><td>&lt;input type=&quot;text&quot; class=&quot;gsfi&quot; /&gt;</td></tr>
 * <tr><td>css=input#chbx6.chk</td><td>&lt;input type=&quot;checkbox&quot; id=&quot;chbx6&quot; name=&quot;Six&quot; class=&quot;chk&quot; /&gt;</td></tr>
 * <tr><td>link=Toggle Me</td><td>&lt;a href=&quot;javascript:toggle_visibility('toggleText');&quot;>Toggle Me&lt;/a&gt;</td></tr>
 * <tr><td>partiallink=Toggle</td><td>&lt;a href=&quot;javascript:toggle_visibility('toggleText');&quot;>Toggle Me&lt;/a&gt;</td></tr>
 * <tr><td>identifier=male<br/>id=male<br/>male</td><td>&lt;input type=&quot;radio&quot; name=&quot;sex&quot; id=&quot;male&quot; /&gt;</td></tr>
 * </tbody></table>
 */
public class DriverUtil
{
	private WebDriver driver = null;
    private final static int explicitWaitSeconds = 10;


//----------------------Constructors--------------------------------------------

	public DriverUtil(WebDriver myDriver)
	{
		driver = myDriver;
		driver.manage().timeouts().implicitlyWait(10, TimeUnit.SECONDS);
	}
	
//----------------------Utility Methods-----------------------------------------

	/**
	 * See the description of {@link DriverUtil} for how we are using this parseLocator method.
	 */
	protected By parseLocator(String locator)
	{
		if (locator.toLowerCase().startsWith("name="))
			return By.name(locator.substring(5));
		else if (locator.toLowerCase().startsWith("xpath="))
			return By.xpath(locator.substring(6));
		else if (locator.toLowerCase().startsWith("/"))
			return By.xpath(locator);
		else if (locator.toLowerCase().startsWith("domtagname="))
			return By.tagName(locator.substring(11));
		else if (locator.toLowerCase().startsWith("domclass="))
			return By.className(locator.substring(9));
		else if (locator.toLowerCase().startsWith("css="))
			return By.cssSelector(locator.substring(4));
		else if (locator.toLowerCase().startsWith("link="))
			return By.linkText(locator.substring(5));
		else if (locator.toLowerCase().startsWith("partiallink="))
			return By.partialLinkText(locator.substring(12));
		else if (locator.toLowerCase().startsWith("id="))
			return By.id(locator.substring(3));
		else if (locator.toLowerCase().startsWith("identifier="))
			return By.id(locator.substring(11));

		return By.id(locator);
	}
	
	protected By parseLocator(StringBuilder locator) {
		return(parseLocator(locator.toString()));
	}
	
	/**
	 * Returns the attribute's current value or null if the value is not set.<br/>
	 * Get the value of a the given attribute of the element. Will return the
	 * current value, even if this has been modified after the page has been
	 * loaded. More exactly, this method will return the value of the given
	 * attribute, unless that attribute is not present, in which case the value
	 * of the property with the same name is returned. If neither value is set,
	 * null is returned. The &quot;style&quot; attribute is converted as best can be to a
	 * text representation with a trailing semi-colon. The following are deemed
	 * to be &quot;boolean&quot; attributes, and will return either &quot;true&quot; or &quot;false&quot;:
	 * async, autofocus, autoplay, checked, compact, complete, controls, declare,
	 * defaultchecked, defaultselected, defer, disabled, draggable, ended,
	 * formnovalidate, hidden, indeterminate, iscontenteditable, ismap,
	 * itemscope, loop, multiple, muted, nohref, noresize, noshade, novalidate,
	 * nowrap, open, paused, pubdate, readonly, required, reversed, scoped,
	 * seamless, seeking, selected, spellcheck, truespeed, willvalidate<br/>
	 * Finally, the following commonly mis-capitalized attribute/property names
	 * are evaluated as expected:<br/>&quot;class&quot;<br/>&quot;readonly&quot;
	 *
	 * @param locator For a complete description of locators and how to use them,
	 * see the Javadoc for the class {@link SeleniumWebDriver}.
	 * @param attribute The name of the attribute.
	 */
	public String getAttribute(String locator, String attribute) {
		try {
			WebElement element = driver.findElement(parseLocator(locator));
			String retValue = element.getAttribute(attribute);

			if (retValue == null)
				throw new WebTestingException("Unable to find the attribute \"" + attribute + "\" for " + locator + ", it was null.");
			return retValue;
		}
		catch (org.openqa.selenium.NoSuchElementException t)  {
			throw new WebTestingException("Unable to find the attribute \"" + attribute + "\" for " + locator + ": " + t.getMessage(), t);
		}
		catch (Throwable t)  {
			throw new WebTestingException("Problem with attribute \"" + attribute + "\" for " + locator + ": " + t.getMessage(), t);
		}
	}
	
	public String getAttribute(StringBuilder locator, String attribute) {
		return(getAttribute(locator.toString(), attribute));
	}
	
//------------------------------Methods-----------------------------------------

	/**
	 * Used to quit browsing the application.
	 */
	public void quit() {
		driver.quit();
	}
	
	/**
	 * Clicks a link, button, check box, radio button, etc.<br/>
	 * If this causes a new page to load, this method will
	 * block until the page has loaded. If click() causes a new page to be
	 * loaded via an event or is done by sending a native event (which is a
	 * common case on Firefox, IE on Windows) then the method will <b><u>not</u></b> wait
	 * for it to be loaded and the caller should verify that a new page has been
	 * loaded (for example, by using {@link #waitForPageToLoad()}).<br/>
	 * If this element is not clickable, then this operation is a no-op since
	 * it's pretty common for someone to accidentally miss the target when clicking in Real Life.
	 *
	 * @param locator For a complete description of locators and how to use them,
	 * see the Javadoc for the class {@link DriverUtil}.
	 */
	public void click(String locator) {
        int attempts = 0;

        while(attempts < explicitWaitSeconds) {
            try {
                WebElement element = driver.findElement(parseLocator(locator));
                element.click();
                break;
            } catch(StaleElementReferenceException sere) {
                attempts++;
                waitSeconds(1);
            } catch(NoSuchElementException nsee) {
                attempts++;
                waitSeconds(1);
            } catch(Throwable t) {
                throw new WebTestingException(t.getMessage(), t);
            }
        }
	}
	
	public void click(StringBuilder locator) {
		click(locator.toString());
	}

    public void clearElementValue(String locator) {
        try {
            WebElement element = driver.findElement(parseLocator(locator));
            element.clear();
        } catch(Throwable t) {
            throw new WebTestingException("clearElementValue: " + t.getMessage(), t);
        }
    }
	
	/**
	 * enters text into the input field specified<br/>
	 * The element is first cleared.  Passing in a null or empty string will
	 * clear the element and the WebElement's sendKeys will not be called.
	 *
	 * @param locator For a complete description of locators and how to use them,
	 * see the Javadoc for the class {@link DriverUtil}.
	 * @param text the text to enter
	 */
	public void setElementValue(String locator, String text) {
		try {
			WebElement element = driver.findElement(parseLocator(locator));
			//NOTE: The behavior of the selenium driver was to always clear
			//the input field before sending the text to it.  Calling this
			//method with an empty string would clear the value as well.
			String type = element.getAttribute("type");
			if ("text".equalsIgnoreCase(type))
				element.clear();
			if (text != null && !text.isEmpty())
				element.sendKeys(text);
		} catch (Throwable t)  {
			throw new WebTestingException("setElementValue: " + t.getMessage(), t);
		}
	}
	
	public void setElementValue(StringBuilder locator, String text) {
		setElementValue(locator.toString(), text);
	}
	
	/**
	 * Returns the value of an input field (or anything with a value parameter).
	 * Internally, this just retrieves the contents of the &quot;value&quot;
	 * attribute of the element passed in the locator using {@link #getAttribute(String)}.
	 *
	 * @param locator For a complete description of locators and how to use them,
	 * see the Javadoc for the class {@link SeleniumWebDriver}.
	 */
	public String getElementValue(String locator) {
		return this.getAttribute(locator, "value");
	}
	
	public String getElementValue(StringBuilder locator) {
		return(getElementValue(locator.toString()));
	}
	
	/**
	 * Get the visible (i.e. not hidden by CSS) innerText of this element,
	 * including sub-elements, without any leading or trailing whitespace.<br/>
	 * NOTE: {@link BrowserType#HtmlUnit} has issues with concatenating
	 * values returned from this method.  For example, if you have the following:
	 * &quot;&lt;b&gt;Test &lt;/b&gt;this&quot; it will return &quot;Testthis&quot;
	 * rather than &quot;Test this&quot;  The same is true of span tags
	 * and text that spans columns.  See {@link #isTextOnPage(String)} for
	 * more details.
	 */
	public String getText(String locator) {
		try {
			return driver.findElement(parseLocator(locator)).getText();
		} catch (Throwable t)  {
			throw new WebTestingException("Unable to get text for " + locator, t);
		}
	}
	
	public String getText(StringBuilder locator) {
		return(getText(locator.toString()));
	}
	
	/**
	 * Load a new web page in the current browser window.<br/>
	 * This is done using an
	 * HTTP GET operation, and the method will block until the load is complete.
	 * This will follow redirects issued either by the server or as a
	 * meta-redirect from within the returned HTML. Should a meta-redirect
	 * &quot;rest&quot; for any duration of time, it is best to wait until this timeout
	 * is over, since should the underlying page change whilst your test is
	 * executing the results of future calls against this interface will be
	 * against the freshly loaded page.
	 *
	 * @param url The url to browse to.
	 */
	public void open(String url) {
		if (url == null || url.isEmpty())
			throw new WebTestingException("Unable to navigate to a null or empty url");

		try {
			driver.get(url);
		} catch (Throwable t)  {
			throw new WebTestingException("Unable to navigate to: " + url, t);
		}
	}
	
	/**
	 * Returns <code>true</code> if an input field is enabled (not disabled).
	 * This will return <code>true</code> for everything except for a disabled
	 * input field.
	 *
	 * @param locator For a complete description of locators and how to use them,
	 * see the Javadoc for the class {@link SeleniumWebDriver}.
	 */
	public boolean isElementEditable(String locator) {
		try {
			WebElement element = driver.findElement(parseLocator(locator));
			return element.isEnabled();
		} catch (Throwable t)  {
			throw new WebTestingException("Unable to check if element is enabled for "+ locator, t);
		}
	}
	
	public boolean isElementEditable(StringBuilder locator) {
		return(isElementEditable(locator.toString()));
	}
	
	/**
	 * Returns the current HTML page's title, with leading and trailing
	 * whitespace stripped, or null if one is not already set
	 */
	public String getPageTitle() {
		try {
			return driver.getTitle();
		} catch (Throwable t)  {
			throw new WebTestingException("Unable to obtain the current page's title", t);
		}
	}
		
	/**
	 * Returns all selected options belonging to this select tag
	 */

	/*
	// Original implementation
	public String[] getSelectOptions(String locator) {
		try {
			WebElement element = driver.findElement(parseLocator(locator));
			Select select = new Select(element);
			java.util.List<WebElement> selected = select.getAllSelectedOptions();
			String[] retValue = new String[selected.size()];
			int i=0;
			for (WebElement selectedElement : selected) {
				String text = selectedElement.getText();
				retValue[i++] = text;
			}
			return retValue;
		} catch (Throwable t)  {
			throw new WebTestingException("Unable to get selected options for " + locator);
		}
	}
	
	public String[] getSelectOptions(StringBuilder locator) {
		return(getSelectOptions(locator.toString()));
	}
    */

    public List<String> getSelectOptions(String locator) {
        Select selector = new Select(driver.findElement(parseLocator(locator)));
        List<WebElement> selectorOptions = selector.getOptions();
        ArrayList<String> optionList = new ArrayList<String>(0);

        for(WebElement optionElement : selectorOptions) {
            optionList.add(optionElement.getAttribute("value"));
        }

        return optionList;
    }

	/**
	 * Populates a file upload box with the text passed into uploadFilePath.
	 * LocalFileDetector is used to get the absolutePath to the filename.
	 */
	public void uploadFile(String locator, String upLoadFilePath) {
		
		LocalFileDetector detector = new LocalFileDetector();
		File f = detector.getLocalFile(upLoadFilePath);

		this.setElementValue(locator, f.getAbsolutePath());
	}
	
	public void uploadFile(StringBuilder locator, String uploadFilePath) {
		uploadFile(locator.toString(), uploadFilePath);
	}
	
	/**
	 * Wait for text in an element to be made visible most likely following the
	 * execution of some JavaScript.
	 *
	 * @param locator For a complete description of locators and how to use them,
	 * see the Javadoc for the class {@link SeleniumWebDriver}.
	 * @param text the text to look for
	 * @param waitTimeOutInSeconds the number of seconds to wait before giving up.
	 */
	public void waitForVisibleText(String locator, final String text, int waitTimeOutInSeconds) {
        int attempts = 0;
        boolean success = false;

        while(!success) {
            final WebElement element = driver.findElement(parseLocator(locator));
            try {
                new WebDriverWait(driver, waitTimeOutInSeconds).until(new ExpectedCondition<Boolean>() {
                    @Override
                    public Boolean apply(WebDriver d) {
                        return element.isDisplayed() && element.getText().equals(text);
                    }
                });
                success = true;
            } catch(NoSuchElementException nsee) {
                attempts++;
                if(attempts < explicitWaitSeconds) {
                    waitSeconds(1);
                } else {
                    throw new WebTestingException(
                            nsee.getMessage() + ", seconds=\"" + waitTimeOutInSeconds + "\", locator=\"" + locator +
                            "\", text=\"" + text + "\"", nsee);
                }
            } catch(Throwable t) {
                throw new WebTestingException(
                        t.getMessage() + ", seconds=\"" + waitTimeOutInSeconds + "\", locator=\"" + locator +
                        "\", text=\"" + text + "\"", t);
            }
        }
	}

    public void waitForVisibleElement(String locator, int waitTimeOutInSeconds) {
        int attempts = 0;
        boolean success = false;

        while(!success) {
            final WebElement element = driver.findElement(parseLocator(locator));
            try {
                new WebDriverWait(driver, waitTimeOutInSeconds).until(new ExpectedCondition<Boolean>() {
                    @Override
                    public Boolean apply(WebDriver d) {
                        return element.isDisplayed();
                    }
                });
                success = true;
            } catch(NoSuchElementException nsee) {
                attempts++;
                if(attempts < explicitWaitSeconds) {
                    waitSeconds(1);
                } else {
                    throw new WebTestingException(
                            nsee.getMessage() + ", seconds=\"" + waitTimeOutInSeconds + "\", locator=\"" + locator);
                }
            } catch(Throwable t) {
                throw new WebTestingException(
                        t.getMessage() + ", seconds=\"" + waitTimeOutInSeconds + "\", locator=\"" + locator);
            }
        }
    }
	
	public void waitForVisibleText(StringBuilder locator, final String text, int waitTimeOutInSeconds) {
		waitForVisibleText(locator.toString(), text, waitTimeOutInSeconds);
	}
	
	public boolean isElementVisible(String locator) {
        WebElement element;

		try {
            element = driver.findElement(parseLocator(locator));
        } catch(NoSuchElementException nsee) {
            return false;
        }

        return element.isDisplayed();
	}
	
	public boolean isElementVisible(StringBuilder locator) {
		return(isElementVisible(locator.toString()));
	}
	
	public void setElementAttribute(String locator, String attribute, String value) {
		JavascriptExecutor js = (JavascriptExecutor) driver;
		js.executeScript("document.getElementByID('" + parseLocator(locator) + "').setAttribute('" + attribute + "', '" + value + "')");
	}
	
	public void setElementAttribute(StringBuilder locator, String attribute, String value) {
		setElementAttribute(locator.toString(), attribute, value);
	}
	
	public WebElement getWebElement(String locator) {
		return(driver.findElement(parseLocator(locator)));
	}
	
	public WebElement getWebElement(StringBuilder locator) {
		return(getWebElement(locator.toString()));
	}
	
	public void selectOptionByValue(String selectLocator, String optionToSelect) {
		Select selector = new Select(driver.findElement(parseLocator(selectLocator)));
		selector.selectByValue(optionToSelect);
	}
	
	public void selectOptionByValue(StringBuilder selectLocator, String optionToSelect) {
		selectOptionByValue(selectLocator.toString(), optionToSelect);
	}
	
	public void selectOptionByIndex(String selectLocator, int optionToSelect) {
		Select selector = new Select(driver.findElement(parseLocator(selectLocator)));
		List<WebElement> selectorOptions = selector.getOptions();
        selectorOptions.get(optionToSelect).click();
	}
	
	public void selectOptionByIndex(StringBuilder selectLocator, int optionToSelect) {
		selectOptionByIndex(selectLocator.toString(), optionToSelect);
	}
	
	public boolean isElementSelected(String locator) {
		return(driver.findElement(parseLocator(locator)).isSelected());
	}
	
	public boolean isElementSelected(StringBuilder locator) {
		return(isElementSelected(locator.toString()));
	}

    public void hitEnterKey() {
        Actions action = new Actions(driver);
        action.sendKeys(Keys.ENTER).perform();
    }

    public void hitTabKey() {
        Actions action = new Actions(driver);
        action.sendKeys(Keys.TAB).perform();
    }
	
	public void hitShiftTabKeys() {
		Actions action = new Actions(driver);
		action.keyDown(Keys.SHIFT).perform();
		action.sendKeys(Keys.TAB).perform();
		action.keyUp(Keys.SHIFT).perform();
	}

    public void waitSeconds(int seconds) {
        try {
            TimeUnit.SECONDS.sleep(seconds);
        } catch(InterruptedException ex) {
            Thread.currentThread().interrupt();
        }
    }
}
