package com.agilex.common.selenium;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.concurrent.TimeUnit;

import org.openqa.selenium.WebDriver;

/**
 * The WebSite class serves as a basis for navigating the entire site.
 * The project you are working on should override this and provide methods
 * for accessing the individual page objects for your application.<br/>
 * <b>Example:</b><br/>
 * <code>
 * public PcccLogin login() { return (PcccLogin) this.getCurrentPage(PcccLogin.class); }
 * </code>
 */
public abstract class WebSite {
	private WebPage currentPage;
	protected DriverUtil driverutil;
    private final static int pageNavigationTimeout = 30;
	
//-------------------------Constructor------------------------------------------
	
	/**
	 * The WebDriver passed in will be hidden from all classes.  However, it will
	 * be available only through the driverutil object that the WebPage objects will
	 * have access to.
	 */
	public WebSite(WebDriver driver) {
		this.driverutil = new DriverUtil(driver);
	}
	
//-------------------------public methods---------------------------------------
	
	/**
	 * Your test class should call this in the @After tag to properly quit the WebDriver and close the browser.
	 * This is a convenience method to driverutil's quit method.
	 */
	public void quit() {
		driverutil.quit();
	}

	/**
	 * Return the current <code>WebPage</code> object.  This should be the same as the
	 * current page under test.  However, this method does not guarantee this to be true.
	 *
	 * This method does a check for <code>null</code> and a check to determine if the
	 * current page is an instance of the pageClass parameter.  If either check fails then
	 * a WebPageException is thrown.
	 *
	 * A concrete <code>Website</code> instance might implement a method to return type-specific
	 * instances of <code>WebPage</code> classes.  For example:
	 *
	 * <pre>
	 *  public EditPersonPage editPerson() {
	 *      return (EditPersonPage) getCurrentPage(EditPersonPage.class);
	 *  }
	 * </pre>
	 *
	 * @param pageClass - concrete class to check for before returning the current page
	 * @throws WebTestingException - thrown if current page is <code>null</code> or
	 * the current page is not an instance of the pageClass parameter
	 */
	public WebPage getCurrentPage(Class<? extends WebPage> pageClass) {
		if (currentPage == null)
			throw new WebTestingException("The current page is not an instance of the " + pageClass.getName()
										+ " class.  The current page is not set (null).");

		if (!pageClass.isInstance(currentPage))
			throw new WebTestingException("The current page is not an instance of the " + pageClass.getName()
											+ " class.  The current page is an instance of the "
											+ currentPage.getClass().getName() + " class.");

		return currentPage;
	}
	
	/**
	 * Returns true if the current page is an instance of the pageClass parameter.
	 */
	public boolean isCurrentPage(Class<? extends WebPage> pageClass) {
		return pageClass.isInstance(currentPage);
	}
	
	/**
	 * Using reflection this method attempts to create an instance of the class defined in
	 * the pageClass parameter and set the newly created instance as the current page.
	 *
	 * This method can be used in <code>Website</code> and <code>WebPage</code> instances to
	 * set the current page.  From a <code>WebPage</code> instance for example:
	 * <pre>
	 * public void clickNextButton() throws PageCreationExecption {
	 *	  click(NEXT_BUTTON);
     *
	 *	  webSite.setCurrentPage(EditPersonPage.class);
	 * }
	 * </pre>
	 *
	 * @param pageClass - concrete class to create and set as the current page
	 * @throws WebTestingException - thrown if there is an exception creating an instance
	 * of pageClass
	 * @throws WebTestingException - throw if the <code>WebPage</code> instance returns
	 * False from the <code>isPageTitleEqual(String pageTitle)</code> method call.  The
	 * pageTitle parameter is set to the Driver.getPageTitle() value.
	 */
	public WebPage setCurrentPage(Class<? extends WebPage> pageClass) {
		WebPage page = createPageInstance(pageClass);
		checkSamePageAsDriverPage(page);

		currentPage = page;

		return currentPage;
	}
	
//-------------------------private helper methods-------------------------------
	
	private void checkSamePageAsDriverPage(WebPage page) {
        int secondsElapsed = 0;

        while(!page.verifyTitle() && secondsElapsed < pageNavigationTimeout) {
            try {
                TimeUnit.SECONDS.sleep(1);
            } catch(InterruptedException ex) {
                Thread.currentThread().interrupt();
            }

            secondsElapsed++;
        }

		if(secondsElapsed >= pageNavigationTimeout) {
            throw new WebTestingException("Page titles mismatch.  The expected page title is " + page.getPageTitle() +
                    " and the actual page title is " + driverutil.getPageTitle() + ".");
        }
	}
	
	private WebPage createPageInstance(Class<? extends WebPage> pageClass) {
		WebPage page;

		try {
			page = createWebPageClass(pageClass);
		} catch (Exception e) {
			throw new WebTestingException("Unable to create an instance of class " + pageClass.getName() + ".", e);
		}

		return page;
	}
	
	private WebPage createWebPageClass(Class<? extends WebPage> pageClass) throws NoSuchMethodException, InstantiationException,
	  IllegalAccessException, InvocationTargetException {
		Constructor<? extends WebPage> ctor = pageClass.getDeclaredConstructor(new Class[] {DriverUtil.class, WebSite.class});
		Object obj = ctor.newInstance(driverutil, this);
		
		return pageClass.cast(obj);
	}
}
