package gov.va.mobilehealth.courier.core.protocol.dictionary;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import gov.va.mobilehealth.cms.jackson.Jackson;
import gov.va.mobilehealth.courier.core.protocol.*;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeDaysOfWeek;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeOnce;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.validation.Validation;
import javax.validation.Validator;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static gov.va.mobilehealth.courier.testing.FixtureHelpers.fixture;
import static org.fest.assertions.api.Assertions.assertThat;

public class DictionaryTest {
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
    private static final Logger LOGGER = LoggerFactory.getLogger(DictionaryTest.class);
    private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

    @Test
    public void serializesToJSON() throws IOException {
        final Dictionary expected = getDictionary();

        final Dictionary given = MAPPER.readValue(fixture("fixtures/dictionary.json"), Dictionary.class);

        LOGGER.info(MAPPER.writeValueAsString(expected));
        LOGGER.info(MAPPER.writeValueAsString(given));

        assertThat(MAPPER.writeValueAsString(expected)).isEqualTo(
            MAPPER.writeValueAsString(given));
    }

    @Test
    public void deserializesFromJSON() throws IOException {
        final Dictionary expected = getDictionary();

        final Dictionary given = MAPPER.readValue(fixture("fixtures/dictionary.json"), Dictionary.class);

        LOGGER.info(MAPPER.writeValueAsString(expected));
        LOGGER.info(MAPPER.writeValueAsString(given));

        assertThat(given).isEqualTo(expected);
    }

    @Test
    public void servicePatternTest() throws IOException {
        final List<ServiceDictionary> given = MAPPER.readValue(fixture("fixtures/services.json"),
            new TypeReference<List<ServiceDictionary>>() {
            }
        );

        CMSService service = new CMSService();

        String test = "ox 95";
        service.setPatternFromKeywords(given.get(1).getKeywords());
        assertThat(test.matches(service.getPattern())).isTrue();

        test = "ox95";
        service.setPatternFromKeywords(given.get(1).getKeywords());
        assertThat(test.matches(service.getPattern())).isTrue();

        test = "my ox95";
        service.setPatternFromKeywords(given.get(1).getKeywords());
        assertThat(test.matches(service.getPattern())).isTrue();

        test = "my oxis 95";
        service.setPatternFromKeywords(given.get(1).getKeywords());
        assertThat(test.matches(service.getPattern())).isFalse();

        test = "oxis 95";
        service.setPatternFromKeywords(given.get(1).getKeywords());
        assertThat(test.matches(service.getPattern())).isFalse();

        test = "bg 24.5";
        service.setPatternFromKeywords(given.get(2).getKeywords());
        assertThat(test.matches(service.getPattern())).isTrue();

        test = "bg24.5";
        service.setPatternFromKeywords(given.get(2).getKeywords());
        assertThat(test.matches(service.getPattern())).isTrue();

        test = "bgb 24";
        service.setPatternFromKeywords(given.get(3).getKeywords());
        assertThat(test.matches(service.getPattern())).isTrue();

        test = "bga 5";
        service.setPatternFromKeywords(given.get(4).getKeywords());
        assertThat(test.matches(service.getPattern())).isTrue();

        test = "we 125";
        service.setPatternFromKeywords(given.get(5).getKeywords());
        assertThat(test.matches(service.getPattern())).isTrue();

        test = "calories 1200";
        service.setPatternFromKeywords(given.get(6).getKeywords());
        assertThat(test.matches(service.getPattern())).isTrue();

        test = "exercise 40";
        service.setPatternFromKeywords(given.get(7).getKeywords());
        assertThat(test.matches(service.getPattern())).isTrue();

        test = "temp 101.1";
        service.setPatternFromKeywords(given.get(8).getKeywords());
        assertThat(test.matches(service.getPattern())).isTrue();

        test = "temp 98";
        service.setPatternFromKeywords(given.get(8).getKeywords());
        assertThat(test.matches(service.getPattern())).isTrue();

        test = "pulse 50";
        service.setPatternFromKeywords(given.get(9).getKeywords());
        assertThat(test.matches(service.getPattern())).isTrue();

    }

    private Dictionary getDictionary() {
        final Dictionary dictionary = new Dictionary();

        // services list
        List<String> l = new ArrayList<String>();
        l.add("Blood");
        l.add("BP");

        ServiceDictionary bp = new ServiceDictionary();

        Reading sys = new Reading("Systolic", new ReadingThreshold(70, 262), new ReadingThreshold(80, 140));
        sys.setUnit("mmHg");
        Reading dia = new Reading("Diastolic", new ReadingThreshold(50, 120), new ReadingThreshold(60, 90));
        dia.setUnit("mmHg");

        List<Reading> readings = new ArrayList<Reading>();
        readings.add(sys);
        readings.add(dia);
        bp.setReadings(readings);

        bp.setKeywords(l);
        bp.setName("Blood Pressure");

        List<ServiceDictionary> services = new ArrayList<ServiceDictionary>();


        TimeZone timeZone = new TimeZone();
        List<TimeZone> timeZones = new ArrayList<TimeZone>();

        timeZone.setValue("US/Alaska");
        timeZone.setDescription("Alaska (GMT-09:00)");
        timeZones.add(timeZone);


        services.add(bp);
        dictionary.setServices(services);
        dictionary.setTimeZones(timeZones);

        // schedule
        Schedule schedule = new Schedule();
        List<ScheduleType> scheduleTypeList = new ArrayList<ScheduleType>();

        TypeOnce scheduleType1 = new TypeOnce();
        TypeEverySoMany scheduleType2 = new TypeEverySoMany();
        TypeDaysOfWeek scheduleType3 = new TypeDaysOfWeek();

        scheduleType1.setName("Once");
        scheduleType2.setName("Custom");
        scheduleType3.setName("Specified Days");
        scheduleType2.setInterval(new TimeInterval(TimeIntervalUnit.MINUTE, 0));

        scheduleTypeList.add(scheduleType1);
        scheduleTypeList.add(scheduleType2);
        scheduleTypeList.add(scheduleType3);

        schedule.setScheduleType(scheduleTypeList);

        dictionary.setSchedule(schedule);


        return dictionary;
    }
}
