package gov.va.mobilehealth.courier.bcm.appointments.scheduler;

import gov.va.mobilehealth.courier.bcm.appointments.*;
import gov.va.mobilehealth.courier.helper.QuartzScheduleFactoryTestHelper;
import org.joda.time.DateTime;
import org.joda.time.Seconds;
import org.junit.*;
import org.junit.rules.ExpectedException;
import org.quartz.JobDetail;
import org.quartz.JobKey;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.impl.matchers.GroupMatcher;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import static org.junit.Assert.assertEquals;

public class SendAppointmentReminderJobCreatorTest {
  private SendAppointmentReminderJobCreator reminderJobCreator;
  private Scheduler scheduler;

  @Rule
  public ExpectedException thrown = ExpectedException.none();

  @Before
  public void setup() throws SchedulerException{
    scheduler = QuartzScheduleFactoryTestHelper.initTestScheduler();

    reminderJobCreator = new SendAppointmentReminderJobCreator();
}

  @After
  public void finish() throws IOException, SchedulerException {
    scheduler.shutdown();
  }

  @BeforeClass
  public static void classSetup() throws IOException {
  }

  @AfterClass
  public static void tearDown() {

  }

  @Test
  public void createReminderJob() throws Exception {
    /**
     * Setup, create entities
     */
    AppointmentReminder reminder = createAppointmentReminder();

    /**
     * scheduler pause
     * do action that affects scheduler
     */
    // save the BCM message, the send message and expire job should be scheduled
    scheduler.pauseAll();
    createReminderJob(reminder);

    /**
     * check jobs
     */
    // check init and expire job scheduled
    //TODO: For now, work with entity id, may need to re-evaluate since we blow away the id's on each run...  may need to be patient id + original hdr appt id
    JobKey sendReminderJobKey = JobKey.jobKey(reminder.getFutureAppointment().getId() + "_0", SendAppointmentReminderJobCreator.getReminderJobGroupName(reminder));
    boolean JobScheduled = scheduler.checkExists(sendReminderJobKey);
    Assert.assertTrue("JobScheduled not found", JobScheduled);

    //TODO: Check meta data on the job...
    JobDetail jobDetail = scheduler.getJobDetail(sendReminderJobKey);
    Assert.assertNotNull(jobDetail.getJobDataMap().getString(SendAppointmentReminderJob.PatientEdipiIdKey));
    Assert.assertNotNull(jobDetail.getJobDataMap().getString(SendAppointmentReminderJob.AppointmentReminderMessageKey));

    scheduler.resumeAll();

    /**
     * run cleanup job, sleep
     */
    scheduler.deleteJob(sendReminderJobKey);
  }


  /**
   * With an appointment, and config to send out 2 reminders, ensure both jobs exist after the process is complete
   * @throws Exception
   */
  @Test
  public void createMultipleRemindersForSamePatientResultsInCorrectNumberOfJobs() throws Exception {
    /**
     * Setup, create entities
     */
    AssessmentProfileFutureAppointment appointment = createFutureAppointment();
    appointment.setStartDate(DateTime.now().plusDays(1));

    AppointmentReminderGenerator generator = new AppointmentReminderGenerator();
    List<AppointmentReminder> appointmentReminders = generator.createAppointmentReminders(appointment);

    assertEquals("Correct number of reminders created", 2, appointmentReminders.size());
    /**
     * scheduler pause
     * do action that affects scheduler
     */
    // save the BCM message, the send message and expire job should be scheduled
    scheduler.pauseAll();
    createReminderJob(appointmentReminders, appointment.getpatientEdipiIdentifier());

    /**
     * check jobs
     */
    // check init and expire job scheduled
    GroupMatcher<JobKey> matcher = GroupMatcher.groupEquals(SendAppointmentReminderJobCreator.getReminderJobGroupName(appointmentReminders.get(0)));
    Set<JobKey> jobKeys = scheduler.getJobKeys(matcher);

    assertEquals(2, jobKeys.size());
    scheduler.resumeAll();

    /**
     * run cleanup job, sleep
     */
    for (JobKey jobKey : jobKeys) {
      scheduler.deleteJob(jobKey);
    }
  }

  /**
   * Since the job key is based on the appointment id, need to ensure the initial job is properly removed if the future
   * appointment id changes (since we delete and insert appointments, the id's are always different)
   * @throws Exception
   */
  @Test
  public void createReminderJobCleansJobsWithDbResave() throws Exception {
    /**
     * Setup, create entities
     */
    AppointmentReminder reminder = createAppointmentReminder();

    /**
     * scheduler pause
     * do action that affects scheduler
     */
    // save the BCM message, the send message and expire job should be scheduled
    scheduler.pauseAll();
    createReminderJob(reminder);

    GroupMatcher<JobKey> matcher = GroupMatcher.groupEquals(SendAppointmentReminderJobCreator.getReminderJobGroupName(reminder));
    Set<JobKey> jobKeys = scheduler.getJobKeys(matcher);
    assertEquals(1, jobKeys.size());

    //change the id
    reminder.getFutureAppointment().setId("11111111");
    //and run a second time
    createReminderJob(reminder);

    GroupMatcher<JobKey> matcher2 = GroupMatcher.groupEquals(SendAppointmentReminderJobCreator.getReminderJobGroupName(reminder));
    Set<JobKey> jobKeys2 = scheduler.getJobKeys(matcher2);

    assertEquals(1, jobKeys2.size());
    scheduler.resumeAll();

    /**
     * run cleanup job, sleep
     */
    for (JobKey jobKey : jobKeys) {
      scheduler.deleteJob(jobKey);
    }
  }

  /**
   * Calling the creator with an empty list of reminders clears out any pre-existing scheduled messages
   */
  @Test
  public void emptyListOfRemindersClearsExistingJobs() throws Exception {
    /**
     * Setup, create entities
     */
    AppointmentReminder reminder = createAppointmentReminder();

    /**
     * scheduler pause
     * do action that affects scheduler
     */
    // save the BCM message, the send message and expire job should be scheduled
    scheduler.pauseAll();
    createReminderJob(reminder);

    GroupMatcher<JobKey> matcher = GroupMatcher.groupEquals(SendAppointmentReminderJobCreator.getReminderJobGroupName(reminder));
    Set<JobKey> jobKeys = scheduler.getJobKeys(matcher);
    //Make sure initial job is there
    assertEquals(1, jobKeys.size());


    List<AppointmentReminder> emptyReminders = new ArrayList<AppointmentReminder>();
    //and run a second time, this time with empty list
    createReminderJob(emptyReminders, reminder.getFutureAppointment().getpatientEdipiIdentifier());

    GroupMatcher<JobKey> matcher2 = GroupMatcher.groupEquals(SendAppointmentReminderJobCreator.getReminderJobGroupName(reminder));
    Set<JobKey> jobKeys2 = scheduler.getJobKeys(matcher2);

    assertEquals(0, jobKeys2.size());
    scheduler.resumeAll();
  }

  /**
   * Clean jobs successfully clears scheduled jobs.
   */
  @Test
  public void cleanJobsForPatientClearsExistingJobs() throws Exception {
    /**
     * Setup, create entities
     */
    AppointmentReminder reminder = createAppointmentReminder();

    /**
     * scheduler pause
     * do action that affects scheduler
     */
    // save the BCM message, the send message and expire job should be scheduled
    scheduler.pauseAll();
    createReminderJob(reminder);

    GroupMatcher<JobKey> matcher = GroupMatcher.groupEquals(SendAppointmentReminderJobCreator.getReminderJobGroupName(reminder));
    Set<JobKey> jobKeys = scheduler.getJobKeys(matcher);
    //Make sure initial job is there
    assertEquals(1, jobKeys.size());


    //Explicitly clean jobs for the patient
    EdipiIdentifier edipiIdentifier = reminder.getFutureAppointment().getpatientEdipiIdentifier();
    reminderJobCreator.cleanJobsForPatient(edipiIdentifier);


    GroupMatcher<JobKey> matcher2 = GroupMatcher.groupEquals(SendAppointmentReminderJobCreator.getReminderJobGroupName(edipiIdentifier));
    Set<JobKey> jobKeys2 = scheduler.getJobKeys(matcher2);

    assertEquals(0, jobKeys2.size());
    scheduler.resumeAll();
  }

  private void createReminderJob(AppointmentReminder reminder) {
    List<AppointmentReminder> reminders = new ArrayList<AppointmentReminder>();
    reminders.add(reminder);
    reminderJobCreator.createReminderJobs(reminders, reminder.getFutureAppointment().getpatientEdipiIdentifier());
  }

  private void createReminderJob(List<AppointmentReminder> reminders, EdipiIdentifier edipiIdentifier) {
    reminderJobCreator.createReminderJobs(reminders, edipiIdentifier);
  }

  private AppointmentReminder createAppointmentReminder(){
    AppointmentReminder reminder = new AppointmentReminder();
    reminder.setFutureAppointment(createFutureAppointment());
    //10 seconds in the future?
    reminder.setDeliveryDateTime(DateTime.now().plus(Seconds.seconds(3)));
    return reminder;
  }
  private AssessmentProfileFutureAppointment createFutureAppointment() {
    AssessmentProfileFutureAppointment appointment = new AssessmentProfileFutureAppointment();
    appointment.setId("asd9f87asdf87asdf98a0sdfg8as0g");
    appointment.setFacility("Test Facility");
    appointment.setStartDate(DateTime.now());  //evaluate what time we want here
    appointment.setPatientEdipiIdentifier(new EdipiIdentifier("100840"));
    appointment.setPatientIcnIdentifier(new IcnIdentifier("100840"));
    return appointment;
  }
}
