package gov.va.mobilehealth.courier.db;

import com.google.common.base.Optional;
import com.mongodb.*;

import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.UserIdentifier;
import gov.va.mobilehealth.courier.core.PatientPreference;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;

import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.IOException;
import java.util.List;

import static org.fest.assertions.api.Assertions.assertThat;
import static org.junit.Assert.assertEquals;

public class PatientDaoTest {

  static String patientId = "112233";
  private static MongoClient mongoClient;

  @BeforeClass
  public static void setup() throws IOException {
    MongoDBConnection.init("mongodb://localhost", "courier");
    mongoClient = new MongoClient();
  }

  @Before
  public void setUp() throws Exception {
    mongoClient.getDB("courier").getCollection("patients").drop();
    BasicDBObject index1 = new BasicDBObject("preference.phoneNumber", 1);
    BasicDBObject option1 = new BasicDBObject("unique",true);
    DBCollection patientsCollection = mongoClient.getDB("courier").getCollection("patients");
    patientsCollection.drop();
    patientsCollection.ensureIndex(index1, option1);
  }

  @AfterClass
  public static void tearDown() {
    MongoDBConnection.close();
  }

  @Test
  public void canFindAll() throws IOException {
    PatientDAO dao = new PatientDAO();
//    	dao.prePopulate();
    dao.findAll();
  }

  @Test
  public void canFindbyPhone() throws IOException {
    PatientDAO dao = new PatientDAO();

    Patient expected = getPatient();
    dao.update(expected);

    Optional<Patient> actual = dao.findByMDN(expected.getPreference().getPhoneNumber());

    assertThat(actual.isPresent()).isTrue();
    assertThat(actual.get()).isEqualTo(expected);

    dao.removeFromDB(expected.getId());
  }

  @Test
  public void canCreatePatient() throws IOException {
    PatientDAO dao = new PatientDAO();

    Patient p1 = getPatient();
    dao.update(p1);

    p1.getPreference().setEmail("newemail@abc.com");
    p1.setSsn("333-222-1111");

    dao.update(p1);

    Patient p2 = dao.findById(p1.getId()).get();
    assertThat(p2.getPreference().getEmail()).isEqualTo("newemail@abc.com");
    assertThat(p2.getLast4SSN()).isEqualTo("3333");
    assertThat(p2.getLastNameInitial()).isEqualTo("S");

    dao.removeFromDB(p1.getId());

  }

  @Test
  public void findProfileByName() throws IOException {
    PatientDAO dao = new PatientDAO();
    Patient p1 = getPatient();
    dao.update(p1);
    Patient p2 = getPatient();
    p2.getPreference().setPhoneNumber("1-123-456-7891");
    p2.setFirstName("Tom");
    dao.update(p2);

    List<Patient> pList = dao.findByName("Smith", "John", null);
    assertThat(pList.size()).isEqualTo(1);

    pList = dao.findByName("Smith", null, null);
    assertThat(pList.size()).isEqualTo(2);

    dao.removeFromDB(p1.getId());
    dao.removeFromDB(p2.getId());
  }

  @Test
  public void findProfileBySsn() throws IOException {
    PatientDAO dao = new PatientDAO();
    Patient p1 = getPatient();
    dao.update(p1);
    Patient p2 = getPatient();
    p2.getPreference().setPhoneNumber("1-123-456-7891");
    p2.setSsn("222-33-2222");
    dao.update(p2);

    List<Patient> pList = dao.findBySsn("111-22-3333");
    assertThat(pList.size()).isEqualTo(1);

    dao.removeFromDB(p1.getId());
    dao.removeFromDB(p2.getId());
  }

  @Test
  public void findProfileByMobile() throws IOException {
    PatientDAO dao = new PatientDAO();
    Patient p1 = getPatient();
    dao.update(p1);
    Patient p2 = getPatient();
    p2.getPreference().setPhoneNumber("1-123-456-9999");
    dao.update(p2);
    Patient p3 = getPatient();
    p3.getPreference().setPhoneNumber("1-123-456-7891");
    dao.update(p3);

    List<Patient> pList = dao.findByMobile("1-123-456-7890");
    assertThat(pList.size()).isEqualTo(1);

    dao.removeFromDB(p1.getId());
    dao.removeFromDB(p2.getId());
    dao.removeFromDB(p3.getId());
  }

  @Test
  public void findPatientByPatientIdentifier() throws IOException {
	  PatientDAO dao = new PatientDAO();
    Patient p1 = getPatient();
    dao.update(p1);

	  Optional<Patient> patient= dao.findByPatientIdentifier(p1.getPatientIdentifier());

	  assertThat(patient.get().getPatientIdentifier().getAssigningAuthority()).isEqualTo("EDIPI");
	  assertThat(patient.get().getPatientIdentifier().getUniqueId()).isEqualTo("PATID01");

	  dao.removeFromDB(p1.getId());
  }

  @Test
  public void findByLastNameInitialAnd4Ssn() throws IOException {
	  String last4SSN="3333";
	  String lastNameInitial="S";

	  PatientDAO dao = new PatientDAO();
	  Patient p1 = getPatient();
	  dao.update(p1);

	  List<Patient> pList =  dao.findByLastnameInit4Ssn(last4SSN,lastNameInitial);
	  List<DBObject> indexInfoList = mongoClient.getDB("courier").getCollection("patients").getIndexInfo();

	  for (DBObject indexInfo : indexInfoList) {
		  System.out.println(indexInfo);
	  }

	  assertThat(pList.size()).isEqualTo(1);
	  assertEquals(last4SSN, pList.get(0).getLast4SSN());
	  assertEquals(lastNameInitial, pList.get(0).getLastNameInitial());
	  dao.removeFromDB(p1.getId());

  }

  @Test (expected = MongoException.class)
  public void noDuplicatePhoneNumber() throws IOException {
    PatientDAO dao = new PatientDAO();
    Patient p1 = getPatient();
    dao.update(p1);
    Patient p2 = getPatient();
    dao.update(p2);


  }

  @Test
  public void canQueryByAppointmentReminderPreference() throws IOException {
    PatientDAO dao = new PatientDAO();

    //Create 3 patients, 2 having appointment pref set true
    Patient p1 = getPatient();
    p1.getPreference().setReceiveApptReminder(true);
    p1.getPreference().setPhoneNumber("1-123-456-7892");
    dao.update(p1);


    Patient p2 = getPatient();
    p2.getPreference().setReceiveApptReminder(true);
    p2.getPreference().setPhoneNumber("1-123-456-7893");
    dao.update(p2);

    Patient p3 = getPatient();
    p3.getPreference().setReceiveApptReminder(false);
    p2.getPreference().setPhoneNumber("1-123-456-7894");
    dao.update(p3);

    List<Patient> foundPatients = dao.findAppointmentReminderPatients();
    assertEquals(2, foundPatients.size());
  }

  private Patient getPatient(){
    return PatientDaoTestHelper.getPatient();
  }
}
