package gov.va.mobilehealth.courier.db;

import com.mongodb.BasicDBObject;
import com.mongodb.MongoClient;
import gov.va.mobilehealth.courier.core.*;
import gov.va.mobilehealth.courier.core.protocol.PatientResponseStatus;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import org.bson.types.ObjectId;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import static org.fest.assertions.api.Assertions.assertThat;

public class PatientMessageDaoTest {

  private static final String messageId1 = "73b9f1519d16ccbfab2fcff1";
  private static final String messageId2 = "89b9f1519d16ccbfab2fcff2";
  private static final String patientId = "52b9f1519d16ccbfab2fcff4";
  private static MongoClient mongoClient;

  @BeforeClass
  public static void setup() throws IOException {
    MongoDBConnection.init("mongodb://localhost", "courier");
    mongoClient = new MongoClient();
    mongoClient.getDB("courier").getCollection("patientMessages").drop();
  }

  @AfterClass
  public static void tearDown() {
    MongoDBConnection.close();
  }

  @Test
  public void canFindAll() throws IOException {
    PatientMessageDAO dao = new PatientMessageDAO();
    PatientIncomingMessage p1 = getPatientResponse(messageId1, patientId);

    insertPatientIncomingMessage(p1);

    PatientOutgoingMessage p2 = getPatientOutgoingMessage(messageId2, patientId);

    insertPatientOutgoingMessage(p2);

    Collection<PatientMessage> patientMessages = dao.findAll();
    assertThat(patientMessages.size()).isEqualTo(2);

    mongoClient.getDB("courier").getCollection("patientMessages").drop();
  }

  @Test
  public void canUpdateStatus() throws IOException {
    PatientMessageDAO dao = new PatientMessageDAO();
    String serviceName = "Blood Pressure";

    PatientIncomingMessage p1 = getPatientResponse(null, patientId);
    dao.insertIncomingMessage(p1);

    Patient patient = new Patient();
    patient.setPreference(new PatientPreference());
    CMSMessage msg = new CMSMessage("", patient, "", MessageType.OUT);
    msg.setIncomingMessageId(p1.getId());
    msg.setAlertLevel(PatientResponseStatus.CRITICAL.toString());
    msg.setServiceName(serviceName);
    dao.updateStatus(msg);

    PatientMessageHistoryDAO dao1 = new PatientMessageHistoryDAO();
    List<PatientMessageHistory> phMsgList = dao1.findByPatientId(patientId, false);

    assertThat(phMsgList.get(0).getAlertLevel()).isEqualTo(PatientResponseStatus.CRITICAL.toString());
    assertThat(phMsgList.get(0).getServiceName()).isEqualTo(serviceName);

    mongoClient.getDB("courier").getCollection("patientMessages")
      .remove(new BasicDBObject().append("_id", new ObjectId(p1.getId())));
  }

  private PatientIncomingMessage getPatientResponse(String messageId, String patientId) {
    PatientIncomingMessage p1 = (new PatientIncomingMessage("7030001111",
      "7034442222"));
    p1.setId(messageId);
    p1.setClientSubmitTime("2014-02-28 13:00:10");
    p1.setNetworkSubmitTime("2014-02-28 12:10:10");
    p1.setText("Your blood pressure is high");
    p1.setClientMessageId("testing");
    p1.setCostCentre("Centre");
    p1.setMssageId("8000");
    p1.setDestination("201-222-3333");
    p1.setSource("222-333-4444");
    p1.setSegmentCount(1);
    p1.setReferenceNumber(10);
    p1.setSegmentNumber(15);
    p1.setPatientId(patientId);
    p1.setType(MessageType.IN);

    return p1;
  }

  private PatientOutgoingMessage getPatientOutgoingMessage(String messageId, String patientId) {
    PatientOutgoingMessage patientMsg = new PatientOutgoingMessage(
      "7031112222", "2021112222");

    patientMsg.setText("Please make sure you eat more healthy food!");
    patientMsg.setNetworkSubmitTime("2014-02-28 13:10:00");
    patientMsg.setClientSubmitTime("2014-02-28 10:00:00");
    patientMsg.setType(MessageType.OUT);
    patientMsg.setPatientId(patientId);
    patientMsg.setId(messageId);
    patientMsg.setType(MessageType.OUT);

    Message msg = new Message();
    msg.setMessageId("69879417");
    msg.setResponseCode("001");
    msg.setResponseString("OK");
    msg.setSuccessCode("0");
    msg.setId(null);
    List<Message> msgList = new ArrayList<Message>();
    msgList.add(msg);

    patientMsg.setMessages(msgList);
    return patientMsg;
  }

  private void insertPatientIncomingMessage(PatientIncomingMessage patientResponse) throws IOException {
    PatientMessageDAO dao = new PatientMessageDAO();

    dao.insertIncomingMessage(patientResponse);
  }

  private void insertPatientOutgoingMessage(PatientOutgoingMessage patientResponse) throws IOException {
    PatientMessageDAO dao = new PatientMessageDAO();

    dao.insertOutgoingMessage(patientResponse);
  }
}
