package gov.va.mobilehealth.courier.db;

import gov.va.mobilehealth.courier.core.protocol.*;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import static org.fest.assertions.api.Assertions.assertThat;

public class PatientProtocolDaoTest {
  private static PatientProtocolDAO dao;
  private String patientId = "528a739eda06d04179851fa9";

  @BeforeClass
  public static void setup() throws IOException {
    MongoDBConnection.init("mongodb://localhost", "courier");

    dao = new PatientProtocolDAO();
  }

  @AfterClass
  public static void tearDown() {
    MongoDBConnection.close();
  }

  @Test
  public void canFindActivebyPatientId() throws Exception {
    PatientProtocol p = ProtocolTemplateBuilder.buildPatientProtocol(patientId, null);
    Comment cm1 = new Comment();
    cm1.setContent("Comment1");
    cm1.setUser("UserOne");
    cm1.setTime("02-10-2014");
    dao.update(p);
    dao.delete(p.getId(), cm1);
    PatientProtocol p2 = ProtocolTemplateBuilder.buildPatientProtocol(patientId, null);
    dao.update(p2);

    Collection<PatientProtocol> patientProtocolList = dao.findByPatientIdAndStatus(p.getPatientId(),
      PatientProtocolStatus.ACTIVE);

    assertThat(patientProtocolList.size()).isEqualTo(1);

    dao.removeFromDB(p.getId());
    dao.removeFromDB(p2.getId());
  }

  @Test
  public void canFindActiveProtocolsByPatientID() throws Exception {
    PatientProtocol p = ProtocolTemplateBuilder.buildPatientProtocol(patientId, null);
    Comment cm1 = new Comment();
    cm1.setContent("Comment1");
    cm1.setUser("UserOne");
    cm1.setTime("02-10-2014");
    dao.update(p);
    dao.delete(p.getId(), cm1);
    PatientProtocol p2 = ProtocolTemplateBuilder.buildPatientProtocol(patientId, null);
    p2.setStatus(PatientProtocolStatus.RESUME);
    dao.update(p2);

    Collection<PatientProtocol> patientProtocolList = dao.findByPatientIdAndStatus(p.getPatientId(),
      PatientProtocol.ACTIVE_PATIENT_PROTOCOL_STATUSES);

    assertThat(patientProtocolList.size()).isEqualTo(1);

    dao.removeFromDB(p.getId());
    dao.removeFromDB(p2.getId());
  }


  @Test
  public void testUpdateStatus() throws Exception {
    PatientProtocol p = ProtocolTemplateBuilder.buildPatientProtocol(patientId, null);
    dao.update(p);
    dao.updateStatus(p.getId(), PatientProtocolStatus.EXPIRED);

    Collection<PatientProtocol> patientProtocolList = dao.findByPatientId(p.getPatientId());

    assertThat(patientProtocolList.size()).isEqualTo(1);
    for (PatientProtocol pc : patientProtocolList) {
      assertThat(PatientProtocolStatus.EXPIRED).isEqualTo(pc.getStatus());
    }
    dao.removeFromDB(p.getId());
  }

  @Test
  public void canFindAll() throws Exception {
    PatientProtocol p = ProtocolTemplateBuilder.buildPatientProtocol(patientId, null);
    Comment cm1 = new Comment();
    cm1.setContent("Comment1");
    cm1.setUser("UserOne");
    cm1.setTime("02-10-2014");
    dao.update(p);
    dao.delete(p.getId(), cm1);
    PatientProtocol p2 = ProtocolTemplateBuilder.buildPatientProtocol(patientId, null);
    dao.update(p2);
    Comment cm2 = new Comment();
    cm2.setContent("Comment1");
    cm2.setUser("UserOne");
    cm2.setTime("02-10-2014");
    dao.delete(p2.getId(), cm2);

    Collection<PatientProtocol> patientProtocolList = dao.findByPatientId(p.getPatientId());

    assertThat(patientProtocolList.size()).isEqualTo(2);
    for (PatientProtocol pc : patientProtocolList) {
      assertThat(PatientProtocolStatus.REMOVED).isEqualTo(pc.getStatus());
    }


    dao.removeFromDB(p.getId());
    dao.removeFromDB(p2.getId());
  }


  @Test
  public void canFindByPatientIdAndProtocolName() throws Exception {
    PatientProtocol p = ProtocolTemplateBuilder.buildPatientProtocol(patientId, null);
    dao.save(p);

    List<PatientProtocol> patientProtocols = dao.findByPatientIdAndProtocolName(p.getPatientId(), p.getProtocol().getName());
    assertThat(patientProtocols.size()).isEqualTo(1);

    dao.removeFromDB(p.getId());
  }

  @Test
  public void canDelete() throws Exception {
    PatientProtocol p = new PatientProtocol();
    p.setStatus(PatientProtocolStatus.ACTIVE);
    Vital t = new Vital();

    List<Template> templates = new ArrayList<Template>();
    templates.add(t);
    Protocol protocol = new Protocol();
    protocol.setTemplates(templates);
    protocol.setName("Blood Pressure");
    p.setProtocol(protocol);

    PatientProtocol result;
    dao.insert(p);

    String id = p.getId();

    Comment c = new Comment();
    c.setContent("disable");
    dao.delete(id, c);

    result = dao.findById(id).get();
    assertThat(result.getProtocol().isStatus()).isEqualTo(false);
    assertThat(result.getComments().get(0).getContent()).isEqualTo("disable");

    dao.removeFromDB(id);
  }

  @Test
  public void canExpireTemplate() throws Exception {
    PatientProtocol p = new PatientProtocol();
    p.setStatus(PatientProtocolStatus.ACTIVE);
    List<Template> templates = new ArrayList<Template>();

    Vital t = new Vital();
    t.setStatus(PatientProtocolStatus.ACTIVE);
    templates.add(t);

    Vital t2 = new Vital();
    t2.setStatus(PatientProtocolStatus.ACTIVE);
    templates.add(t2);

    Protocol protocol = new Protocol();
    protocol.setTemplates(templates);
    protocol.setName("Blood Pressure");
    p.setProtocol(protocol);

    PatientProtocol result;
    dao.insert(p);

    String id = p.getId();

    Comment c = new Comment();
    c.setContent("disable");
    dao.expireProtocol(id, 0);

    result = dao.findById(id).get();
    assertThat(result.getStatus()).isEqualTo(PatientProtocolStatus.ACTIVE);
    assertThat(result.getProtocol().getTemplates().get(0).getStatus()).isEqualTo(PatientProtocolStatus.EXPIRED);
    assertThat(result.getProtocol().getTemplates().get(1).getStatus()).isEqualTo(PatientProtocolStatus.ACTIVE);

    dao.expireProtocol(id, 1);

    result = dao.findById(id).get();
    assertThat(result.getStatus()).isEqualTo(PatientProtocolStatus.EXPIRED);
    assertThat(result.getProtocol().getTemplates().get(0).getStatus()).isEqualTo(PatientProtocolStatus.EXPIRED);
    assertThat(result.getProtocol().getTemplates().get(1).getStatus()).isEqualTo(PatientProtocolStatus.EXPIRED);

    dao.removeFromDB(id);
  }

}
