package gov.va.mobilehealth.courier.db;

import com.mongodb.BasicDBObject;
import com.mongodb.DBObject;
import com.mongodb.MongoClient;
import com.mongodb.util.JSON;
import gov.va.mobilehealth.cms.jackson.Jackson;
import gov.va.mobilehealth.courier.core.PatientReading;
import gov.va.mobilehealth.courier.core.PatientResponseServiceData;
import gov.va.mobilehealth.courier.core.PatientServiceReading;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import static org.fest.assertions.api.Assertions.assertThat;

public class PatientResponseServiceDataDaoTest {

  private static final String patientId1 = "52b9f1519d16ccbfab2fcff4";
  private static final String patientId2 = "52b9f1519d16ccbfab2fcff1";
  private static final String serviceId1 = "52e829f9bf890cf9ee685d85";
  private static final String serviceId2 = "52e829f9bf890cf9ee685d86";
  private static MongoClient mongoClient;

  @BeforeClass
  public static void setup() throws IOException {
    MongoDBConnection.init("mongodb://localhost", "courier");
    mongoClient = new MongoClient();
    mongoClient.getDB("courier").getCollection("patientResponseServiceData").drop();
  }

  @AfterClass
  public static void tearDown() {
    MongoDBConnection.close();
    mongoClient.close();
  }

  @Test
  public void notFound() throws IOException {
    PatientResponseServiceDataDAO dao = new PatientResponseServiceDataDAO();
    PatientResponseServiceData p1 = getPatientResponseServiceData(patientId1);
    insertPatientResponseServiceData(p1);

    Collection<PatientResponseServiceData> patientPatientResponseServicedata = dao.findByPatientId(patientId2);
    assertThat(patientPatientResponseServicedata.size()).isEqualTo(0);

    patientPatientResponseServicedata = dao.findByPatientIdAndServiceId(patientId2, serviceId1);
    assertThat(patientPatientResponseServicedata.size()).isEqualTo(0);
  }

  @Test
  public void canFindByPatientId() throws IOException {
    PatientResponseServiceDataDAO dao = new PatientResponseServiceDataDAO();

    PatientResponseServiceData p1 = getPatientResponseServiceData(patientId1);
    DBObject dbObject = (DBObject) JSON.parse(Jackson.newObjectMapper()
            .writeValueAsString(p1));
    mongoClient.getDB("courier").getCollection("patientResponseServiceData")
            .insert(dbObject);


    assertThat(p1).isEqualTo(getPatientResponseServiceData(patientId1));

    mongoClient.getDB("courier").getCollection("patientResponseServiceData")
            .remove(new BasicDBObject().append("patientId", patientId1));
  }

  @Test
  public void canSave() throws IOException {
    PatientResponseServiceDataDAO dao = new PatientResponseServiceDataDAO();
    PatientResponseServiceData p1 = getPatientResponseServiceData(patientId1);
    insertPatientResponseServiceData(p1);

    Collection<PatientResponseServiceData> patientPatientResponseServicedata = dao.findByPatientId(patientId1);
    assertThat(patientPatientResponseServicedata.size()).isEqualTo(1);
    mongoClient.getDB("courier").getCollection("patientResponseServiceData")
            .remove(new BasicDBObject().append("patientId", patientId1));
  }

  @Test
  public void findServiceNameByPatientId() throws IOException {
    PatientResponseServiceDataDAO dao = new PatientResponseServiceDataDAO();
    PatientResponseServiceData p1 = getPatientResponseServiceData(patientId1);
    insertPatientResponseServiceData(p1);
    insertPatientResponseServiceData(p1);

    Collection<PatientServiceReading> patientPatientResponseServicedata = dao.findServiceNameByPatientId(patientId1);
    assertThat(patientPatientResponseServicedata.size()).isEqualTo(1);
    mongoClient.getDB("courier").getCollection("patientResponseServiceData")
      .remove(new BasicDBObject().append("patientId", patientId1));
  }

  private PatientResponseServiceData getPatientResponseServiceData(String patientId) {
    PatientResponseServiceData patientResponseServiceData = new PatientResponseServiceData();

    patientResponseServiceData.setPatientId(patientId);
    patientResponseServiceData.setPatientResponseReceivedTime("2014-02-28 13:10:00");

    PatientServiceReading patientServiceReading = getPatientServiceReading();
    patientResponseServiceData.setPatientServiceReading(patientServiceReading);

    return patientResponseServiceData;
  }

  public PatientServiceReading getPatientServiceReading() {
    PatientServiceReading patientServiceReading = new PatientServiceReading();
    patientServiceReading.setServiceName("Blood Pressure");

    patientServiceReading.setServiceId(serviceId1);

    List<PatientReading> patientReadings = getPatientReadings();
    patientServiceReading.setReadings(patientReadings);
    return patientServiceReading;
  }

  public List<PatientReading> getPatientReadings() {
    List<PatientReading> patientReadings = new ArrayList<PatientReading>();

    Float tmpFloat = new Float(80);
    PatientReading patientReading = new PatientReading();
    patientReading.setName("Diastolic");
    patientReading.setUnit("mmHg");
    patientReading.setValue(tmpFloat);

    patientReadings.add(patientReading);

    patientReading = new PatientReading();
    patientReading.setName("Systolic");
    tmpFloat = new Float(130.0);
    patientReading.setValue(tmpFloat);
    patientReading.setUnit("mmHg");

    patientReadings.add(patientReading);
    return patientReadings;

  }

  private void insertPatientResponseServiceData(PatientResponseServiceData patientResponseServiceData) {
    PatientResponseServiceDataDAO dao = new PatientResponseServiceDataDAO();

    try {

      dao.save(patientResponseServiceData);
    } catch (Exception exp) {

    }
  }

  @Test
  public void canFindByPatientIdAndServiceId() throws IOException {
    PatientResponseServiceDataDAO dao = new PatientResponseServiceDataDAO();

    PatientResponseServiceData p1 = getPatientResponseServiceData(patientId1);
    insertPatientResponseServiceData(p1);
    PatientResponseServiceData p2 = getPatientResponseServiceData(patientId2);
    insertPatientResponseServiceData(p2);

    List<PatientResponseServiceData> patientResponseServiceDataList = new ArrayList<PatientResponseServiceData>();
    patientResponseServiceDataList = dao.findByPatientIdAndServiceId(patientId1, serviceId1);

    assertThat(patientResponseServiceDataList.size()).isEqualTo(1);
    mongoClient.getDB("courier").getCollection("patientResponseServiceData")
            .remove(new BasicDBObject().append("patientId", patientId1));
    mongoClient.getDB("courier").getCollection("patientResponseServiceData")
            .remove(new BasicDBObject().append("patientId", patientId2));
  }

}
