package gov.va.mobilehealth.courier.helper;

import com.mongodb.MongoClient;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.PatientIncomingMessage;
import gov.va.mobilehealth.courier.core.PatientPreference;
import gov.va.mobilehealth.courier.core.UserIdentifier;
import gov.va.mobilehealth.courier.core.protocol.PatientProtocol;
import gov.va.mobilehealth.courier.core.protocol.PatientProtocolStatus;
import gov.va.mobilehealth.courier.core.protocol.ProtocolTemplateBuilder;
import gov.va.mobilehealth.courier.db.DictionaryDAO;
import gov.va.mobilehealth.courier.db.PatientDAO;
import gov.va.mobilehealth.courier.db.PatientProtocolDAO;
import gov.va.mobilehealth.courier.protocol.MessageProcessorService;
import gov.va.mobilehealth.courier.resources.helper.IncomingMessageProcessor;
import gov.va.mobilehealth.courier.resources.helper.ResponseMessageGenerator;
import gov.va.mobilehealth.courier.scheduler.PatientProtocolScheduler;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.utils.DateParser;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import org.apache.camel.ProducerTemplate;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.quartz.SchedulerException;
import org.quartz.impl.StdSchedulerFactory;

import java.io.IOException;
import java.util.List;

public class IncomingMessageProcessorTest {

  private static DictionaryDAO dao;
  private static MongoClient mongoClient;

  @BeforeClass
  public static void setup() throws IOException {
    MongoDBConnection.init("mongodb://localhost", "courier");
    mongoClient = new MongoClient();
    mongoClient.getDB("courier").getCollection("dictionary").drop();

    dao = new DictionaryDAO();
  }

  @AfterClass
  public static void tearDown() {
    MongoDBConnection.close();
  }


  //TODO: Now that I have good tests around message creation, can i extract all that logic to a class and reduce dependencies further?
  //TODO: How can I assert the schedule changes
  //TODO: How can I assert the patient preference changes

  @Test
  public void startSendsCorrectMessage() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(false);
    updatePatient(p1);

    MockMessageProducer messageProducer = new MockMessageProducer();
    IncomingMessageProcessor imp = getIncomingMessageProcessor(messageProducer);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    imp.processStart(patientResponse);

    assertLastMessage(messageProducer, patient, patientResponse, patientResponse.getDestination(), ResponseMessageGenerator.START_RESPONSE);
  }

  @Test
  public void helpSendsCorrectMessage() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(false);
    updatePatient(p1);

    MockMessageProducer messageProducer = new MockMessageProducer();
    IncomingMessageProcessor imp = getIncomingMessageProcessor(messageProducer);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    imp.processHelp(patientResponse, patient);

    assertLastMessage(messageProducer, patient, patientResponse, patientResponse.getDestination(), new ResponseMessageGenerator().getHelpResponse());
  }

  @Test
  public void stopWithNoStartSendsStartError() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(false);
    updatePatient(p1);

    MockMessageProducer messageProducer = new MockMessageProducer();
    IncomingMessageProcessor imp = getIncomingMessageProcessor(messageProducer);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    String responseType = "expectedSource";

    imp.stopProtocol(patientResponse, patient, responseType);

    assertLastMessage(messageProducer, patient, patientResponse, responseType, ResponseMessageGenerator.START_ERROR);
  }

  @Test
  public void stopSendsStopResponse() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(true);
    updatePatient(p1);

    MockMessageProducer messageProducer = new MockMessageProducer();
    IncomingMessageProcessor imp = getIncomingMessageProcessor(messageProducer);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    String responseType = "expectedSource";

    imp.stopProtocol(patientResponse, patient, responseType);

    CMSMessage lastMessage = (CMSMessage) messageProducer.getLastSentMessage();
    //assert lastMessage values
    assertCommonMessageTraits2(patient, patientResponse, responseType, lastMessage);
    Assert.assertEquals("Text", ResponseMessageGenerator.STOP_RESPONSE, lastMessage.getText());
  }

  @Test
  public void pauseWithNoStartSendsStartError() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(false);
    updatePatient(p1);

    MockMessageProducer messageProducer = new MockMessageProducer();
    IncomingMessageProcessor imp = getIncomingMessageProcessor(messageProducer);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    String responseType = "expectedSource";

    imp.pauseProtocol(patientResponse, patient, responseType);

    assertLastMessage(messageProducer, patient, patientResponse, responseType, ResponseMessageGenerator.START_ERROR);
  }

  @Test
  public void pauseWithStartSendsValidPauseResponse() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(true);
    updatePatient(p1);

    MockMessageProducer messageProducer = new MockMessageProducer();
    IncomingMessageProcessor imp = getIncomingMessageProcessor(messageProducer);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    String responseType = "expectedSource";

    imp.pauseProtocol(patientResponse, patient, responseType);

    assertLastMessage(messageProducer, patient, patientResponse, responseType, ResponseMessageGenerator.PAUSE_RESPONSE);
  }

  @Test
  public void resumeWithNoStartSendsStartError() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(false);
    updatePatient(p1);

    MockMessageProducer messageProducer = new MockMessageProducer();
    IncomingMessageProcessor imp = getIncomingMessageProcessor(messageProducer);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    String responseType = "expectedSource";

    imp.resumeProtocol(patientResponse, patient, responseType);

    assertLastMessage(messageProducer, patient, patientResponse, responseType, ResponseMessageGenerator.START_ERROR);
  }

  @Test
  public void resumeWithNoPauseSendsPauseError() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(true);
    p1.getPreference().setMessagePaused(false);
    updatePatient(p1);

    MockMessageProducer messageProducer = new MockMessageProducer();
    IncomingMessageProcessor imp = getIncomingMessageProcessor(messageProducer);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    String responseType = "expectedSource";

    imp.resumeProtocol(patientResponse, patient, responseType);

    assertLastMessage(messageProducer, patient, patientResponse, responseType, ResponseMessageGenerator.PAUSE_ERROR);
  }

  @Test
  public void resumeWithNoProtocolsSendsValidResumeResponse() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(true);
    p1.getPreference().setMessagePaused(true);
    updatePatient(p1);

    MockMessageProducer messageProducer = new MockMessageProducer();
    IncomingMessageProcessor imp = getIncomingMessageProcessor(messageProducer);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    String responseType = "expectedSource";

    imp.resumeProtocol(patientResponse, patient, responseType);

    assertLastMessage(messageProducer, patient, patientResponse, responseType, ResponseMessageGenerator.RESUME_RESPONSE);
  }

  @Test
  public void resumeWithProtocolsSendsValidResumeResponse() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(true);
    p1.getPreference().setMessagePaused(true);
    p1.getPreference().setTimeZone("EST");
    updatePatient(p1);

    PatientProtocol p = ProtocolTemplateBuilder.buildPatientProtocol(p1.getId(), null);
    p.setStatus(PatientProtocolStatus.PAUSED);
    PatientProtocolDAO protocolDao = new PatientProtocolDAO();
    protocolDao.save(p);


    MockMessageProducer messageProducer = new MockMessageProducer();
    IncomingMessageProcessor imp = getIncomingMessageProcessor(messageProducer);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    String responseType = "expectedSource";

    imp.resumeProtocol(patientResponse, patient, responseType);

    assertLastMessage(messageProducer, patient, patientResponse, responseType, ResponseMessageGenerator.RESUME_RESPONSE);
  }

    @Test
    public void pauseAndResumeOnlyResumesActiveProtocols() throws Exception {
        Patient patient = getPatient();
        patient.getPreference().setMessageStarted(true);
        patient.getPreference().setMessagePaused(true);
        patient.getPreference().setTimeZone("EST");
        updatePatient(patient);

        // Build the first protocol, which we will set as active
        PatientProtocol activeProtocol = ProtocolTemplateBuilder.buildPatientProtocol(patient.getId(), null);
        PatientProtocolDAO protocolDao = new PatientProtocolDAO();
        String activeProtocolId = protocolDao.save(activeProtocol);

        // Build another protocol but put it in the inactive protocol list
        PatientProtocol inactiveProtocol = ProtocolTemplateBuilder.buildPatientProtocol(patient.getId(), null);
        inactiveProtocol.setStatus(PatientProtocolStatus.REMOVED);
        String inactiveProtocolId = protocolDao.save(inactiveProtocol);

        MockMessageProducer messageProducer = new MockMessageProducer();
        IncomingMessageProcessor imp = getIncomingMessageProcessor(messageProducer);

        PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

        String responseType = "expectedSource";

        // This should only pause ACTIVE protocols
        imp.pauseProtocol(patientResponse, patient, responseType);

        assertLastMessage(messageProducer, patient, patientResponse, responseType, ResponseMessageGenerator.PAUSE_RESPONSE);

        // Test that the active protocol is paused
        Assert.assertEquals("PausedProtocol", PatientProtocolStatus.PAUSED, protocolDao.findById(activeProtocolId).get().getStatus());

        // Test that the REMOVED protocol is still inactive
        Assert.assertEquals("InactiveProtocol", PatientProtocolStatus.REMOVED, protocolDao.findById(inactiveProtocolId).get().getStatus());

        // Resume the protocol list - should only resume active protocols...
        imp.resumeProtocol(patientResponse, patient, responseType);

        assertLastMessage(messageProducer, patient, patientResponse, responseType, ResponseMessageGenerator.RESUME_RESPONSE);

        // Test that the active protocol is resumed
        Assert.assertEquals("PausedProtocol", PatientProtocolStatus.RESUME, protocolDao.findById(activeProtocolId).get().getStatus());

        // Test that the REMOVED protocol is still inactive
        Assert.assertEquals("InactiveProtocol", PatientProtocolStatus.REMOVED, protocolDao.findById(inactiveProtocolId).get().getStatus());

    }

  //sends resume after stop message was sent
  @Test
  public void resumeWithStop() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStopped(true);
    p1.getPreference().setMessageStarted(false);
    p1.getPreference().setMessagePaused(false);
    p1.getPreference().setTimeZone("EST");
    updatePatient(p1);

    PatientProtocol p = ProtocolTemplateBuilder.buildPatientProtocol(p1.getId(), null);
    p.setStatus(PatientProtocolStatus.REMOVED);
    PatientProtocolDAO protocolDao = new PatientProtocolDAO();
    protocolDao.save(p);


    MockMessageProducer messageProducer = new MockMessageProducer();
    IncomingMessageProcessor imp = getIncomingMessageProcessor(messageProducer);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    String responseType = "expectedSource";

    imp.resumeProtocol(patientResponse, patient, responseType);

    assertLastMessage(messageProducer, patient, patientResponse, responseType, ResponseMessageGenerator.START_ERROR);
  }

  //This tests the functionality through the IncomingMessageProcessor.  More extensive testing of exact reading responses
  //should be handled against MessageProcessorService
  @Test
  public void readingNoMatchSendsCorrectMessage() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(false);
    updatePatient(p1);

    MockMessageProducer messageProducer = new MockMessageProducer();
    IncomingMessageProcessor imp = getIncomingMessageProcessor(messageProducer);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    imp.processReading(patientResponse, patient.getId());

    assertLastMessage(messageProducer, patient, patientResponse, patientResponse.getDestination(), MessageProcessorService.NO_MATCH_MESSAGE);
  }

  @Test
  public void readingBeforeStartSendsStartError() throws Exception {
    Patient p1 = getPatient();
    p1.getPreference().setMessageStarted(false);
    updatePatient(p1);

    MockMessageProducer messageProducer = new MockMessageProducer();
    IncomingMessageProcessor imp = getIncomingMessageProcessor(messageProducer);

    Patient patient = p1;
    PatientIncomingMessage patientResponse = createPatientIncomingMessage(patient);

    imp.processReadingBeforeStart(patientResponse, patient.getId());

    assertLastMessage(messageProducer, patient, patientResponse, patientResponse.getDestination(), ResponseMessageGenerator.START_ERROR);
  }

  private void assertLastMessage(MockMessageProducer messageProducer, Patient patient, PatientIncomingMessage expectedSource, String responseType, String expectedMessageText) {
    CMSMessage lastMessage = (CMSMessage) messageProducer.getLastSentMessage();
    //assert lastMessage values
    assertCommonMessageTraits2(patient, expectedSource, responseType, lastMessage);
    Assert.assertEquals("Text", expectedMessageText, lastMessage.getText());
  }

  private void assertCommonMessageTraits(Patient patient, PatientIncomingMessage patientResponse, CMSMessage lastMessage) {
    assertCommonMessageTraits2(patient, patientResponse, patientResponse.getDestination(), lastMessage);
  }

  private void assertCommonMessageTraits2(Patient patient, PatientIncomingMessage patientResponse, String expectedSource, CMSMessage lastMessage) {
    Assert.assertNotNull(lastMessage);

    Assert.assertEquals("IncomingMessageId", patientResponse.getId(), lastMessage.getIncomingMessageId());
    Assert.assertEquals("Source", expectedSource, lastMessage.getSource());
    Assert.assertEquals("Destination", patientResponse.getSource(), lastMessage.getDestination());
    Assert.assertEquals("Type", "Out", lastMessage.getType());
    Assert.assertEquals("PatientId", patient.getId(), lastMessage.getPatientId());
    Assert.assertEquals("SendSMS", false, lastMessage.isForceSendViaSMS());
  }


  private IncomingMessageProcessor getIncomingMessageProcessor(final MockMessageProducer messageProducer) throws SchedulerException {
    return new IncomingMessageProcessor(null) {
      protected PatientProtocolScheduler getPatientProtocolScheduler() throws SchedulerException {
        StdSchedulerFactory schedulerFactory = new StdSchedulerFactory("quartz_test.properties");

        return new PatientProtocolScheduler(schedulerFactory.getScheduler());
      }

      protected ProducerTemplate getOutgoingMsgProducer() {
        return messageProducer;
      }
    };
  }

  private void updatePatient(Patient p1) throws IOException {
    PatientDAO dao = new PatientDAO();
    dao.update(p1);
  }

  private Patient getPatient() {
    Patient p1 = new Patient();
    p1.setFirstName("John");
    p1.setLastName("Smith");
    p1.setMiddleName("Rob");

    PatientPreference pref = new PatientPreference();
    pref.setPhoneNumber("1-123-456-7890");

    p1.setDob("1966-01-01");
    p1.setSsn("111-22-3333");
    p1.setPreference(pref);

    UserIdentifier pId = new UserIdentifier();
    pId.setAssigningAuthority("EDIPI");
    pId.setUniqueId("PATID01");
    p1.setPatientIdentifier(pId);
    return p1;
  }

  PatientIncomingMessage createPatientIncomingMessage(Patient patient) {
    PatientIncomingMessage message = new PatientIncomingMessage("", "");
//    message.setId("123");
    message.setSource("messageSource");
    message.setDestination("destination");

    //TODO: Sometimes patientId off of message is used, sometimes off of passed in patient
    message.setPatientId(patient.getId());

    return message;
  }


}
