package gov.va.mobilehealth.courier.scheduler;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.mongodb.DB;
import com.mongodb.DBObject;
import com.mongodb.util.JSON;
import gov.va.mobilehealth.cms.jackson.Jackson;
import gov.va.mobilehealth.courier.bcm.appointments.scheduler.BcmQuartzInitializerListener;
import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.PatientPreference;
import gov.va.mobilehealth.courier.core.protocol.*;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany;
import gov.va.mobilehealth.courier.protocol.MessageProcessorService;
import gov.va.mobilehealth.courier.resources.helper.IncomingMessageProcessor;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import org.bson.types.ObjectId;
import org.joda.time.DateTime;
import org.joda.time.LocalTime;
import org.joda.time.Period;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.quartz.JobKey;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.impl.StdSchedulerFactory;
import org.springframework.mock.web.MockServletContext;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

/**
 * Created by chiy on 1/16/14.
 */
public class PatientProtocolSchedulerTest {
  private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
  public static final String TRIGGER_FIRST_GROUP = "initProtocol";

  private static DB mongoDB;

  static String protocolTemplateId = "52Rename8e87b0da065b48422945fb";
  static String patientId = "52a8f798da063c40a95fc9b7";
  static String testPatientId = "52a8f798da063c40a95fc9b4";
  static String patientProtocolId = "528e880bda065b48422945fe";
  static String deleteProtocolTemplateId = "52963d85da06c04b9f1eefe6";
  static String timezone = "US/Eastern";

  private static PatientProtocolScheduler patientScheduler;
  private static Scheduler scheduler;
  private static StdSchedulerFactory factory;

  @Before
  public void setup() throws IOException, SchedulerException {
    MongoDBConnection.init("mongodb://localhost", "courier");
    mongoDB = MongoDBConnection.getDB();

    mongoDB.getCollection("patients").drop();
    mongoDB.getCollection("patientProtocol").drop();

    //schedule it
    factory = new StdSchedulerFactory("quartz_test.properties");
    scheduler = factory.getScheduler();
    scheduler.start();
    patientScheduler = new PatientProtocolScheduler(scheduler);
    patientScheduler.SetPatientProtocolInitJobClass(PatientProtocolInitJobMockSMS.class);
    patientScheduler.SetPatientProtocolLastJobClass(PatientProtocolLastJobMockSMS.class);
    patientScheduler.SetPatientProtocolReminderJobClass(PatientProtocolReminderJobMockSMS.class);
  }

  @Test
  public void expireProtocol() throws Exception {

    Patient p1 = getPatient();
    p1.setId(patientId);
    DBObject dbObject = (DBObject) JSON.parse(MAPPER.writeValueAsString(p1));
    ObjectId oId1 = new ObjectId(p1.getId());
    dbObject.put("_id", oId1);
    mongoDB.getCollection("patients").insert(dbObject);

    PatientProtocol p2 = getPatientProtocol();
    p2.setId(patientProtocolId);
    dbObject = (DBObject) JSON.parse(MongoDocumentBase.newCustomObjectMapper().writeValueAsString(p2));
    ObjectId oId = new ObjectId(p2.getId());
    dbObject.put("_id", oId);
    mongoDB.getCollection("patientProtocol").insert(dbObject);

    scheduler.pauseAll();
    patientScheduler.update(p2, p1.getPreference().getTimeZone());

    for (int i = 0; i < p2.getProtocol().getTemplates().size(); i++) {
      String jobId = PatientProtocolScheduler.getJobId(p2.getId(), i);
      JobKey jobkey = JobKey.jobKey(jobId, PatientProtocolScheduler.JOB_EXPIRE_GROUP);
      boolean initJobScheduled = scheduler.checkExists(jobkey);
      Assert.assertTrue(initJobScheduled);
    }
    scheduler.clear();
  }


  @Test
  public void reminderMsg() throws Exception {

    Patient p1 = getPatient();
    p1.setId(patientId);
    DBObject dbObject = (DBObject) JSON.parse(MAPPER.writeValueAsString(p1));
    ObjectId oId1 = new ObjectId(p1.getId());
    dbObject.put("_id", oId1);
    mongoDB.getCollection("patients").insert(dbObject);

    PatientProtocol p2 = getPatientProtocol();
    p2.setId(patientProtocolId);
    dbObject = (DBObject) JSON.parse(MongoDocumentBase.newCustomObjectMapper().writeValueAsString(p2));
    ObjectId oId = new ObjectId(p2.getId());
    dbObject.put("_id", oId);
    mongoDB.getCollection("patientProtocol").insert(dbObject);

    scheduler.pauseAll();
    patientScheduler.update(p2, p1.getPreference().getTimeZone());

    // check if the initial jobs are scheduled correctly
    for (int i = 0; i < p2.getProtocol().getTemplates().size(); i++) {
      String jobId = PatientProtocolScheduler.getJobId(p2.getId(), i);
      JobKey jobkey = JobKey.jobKey(jobId, PatientProtocolScheduler.JOB_FIRST_GROUP);
      boolean initJobScheduled = scheduler.checkExists(jobkey);
      Assert.assertTrue(initJobScheduled);
    }

    // check the reminder jobs are scheduled correctly
    for (int i = 0; i < p2.getProtocol().getTemplates().size(); i++) {
      String jobId = PatientProtocolScheduler.getJobId(p2.getId(), i);
      JobKey jobkey = JobKey.jobKey(jobId, PatientProtocolScheduler.JOB_FIRST_GROUP);
      scheduler.resumeAll();
      if (scheduler.checkExists(jobkey)) {
        scheduler.triggerJob(jobkey);
      }

      while (scheduler.checkExists(jobkey)) {
        Thread.sleep(1000);
      }

      JobKey reminderJobkey = JobKey.jobKey(jobId, PatientProtocolScheduler.JOB_REMINDER_GROUP);
      boolean reminderJobsScheduled = scheduler.checkExists(reminderJobkey);

      Assert.assertTrue(reminderJobsScheduled);
    }
    scheduler.clear();

  }

  /*
   * This test will trigger msg processing since msg is created, scheduled, and processed.  This will
   * generate connection refuse exceptions but for this test purpose can be ignored.
   */
  @Test
  public void removeRemindersOnMsgReceive() throws Exception {

    Patient patient = getPatient();
    patient.setId(patientId);

    DBObject dbObject = (DBObject) JSON.parse(MAPPER.writeValueAsString(patient));
    ObjectId oId1 = new ObjectId(patient.getId());
    dbObject.put("_id", oId1);
    mongoDB.getCollection("patients").insert(dbObject);

    PatientProtocol patientProtocol = getPatientProtocol();
    //  ProtocolTemplateBuilder.buildProtocolTemplateWithScheduleTypeOnce().setR;
    //   ProtocolTemplateBuilder.buildProtocolTemplateWithScheduleTypeOnce()
    //   p2.getProtocol().
    Template template = patientProtocol.getProtocol().getTemplates().get(0);
    template.getSchedule().setReminderInterval(new TimeInterval(TimeIntervalUnit.MINUTE, 10));
    patientProtocol.setId(patientProtocolId);
    dbObject = (DBObject) JSON.parse(MongoDocumentBase.newCustomObjectMapper().writeValueAsString(patientProtocol));
    ObjectId oId = new ObjectId(patientProtocol.getId());
    dbObject.put("_id", oId);
    mongoDB.getCollection("patientProtocol").insert(dbObject);

    scheduler.pauseAll();
    patientScheduler.update(patientProtocol, patient.getPreference().getTimeZone());

    for (int index = 0; index < patientProtocol.getProtocol().getTemplates().size(); index++) {
      String jobId = PatientProtocolScheduler.getJobId(patientProtocol.getId(), index);
      JobKey jobkey = JobKey.jobKey(jobId, PatientProtocolScheduler.JOB_FIRST_GROUP);
/*          boolean initJobScheduled = scheduler.checkExists(jobkey);
          Assert.assertTrue(initJobScheduled);*/

      patientScheduler.scheduler.resumeAll();
      if (patientScheduler.scheduler.checkExists(jobkey)) {
        patientScheduler.scheduler.triggerJob(jobkey);
      }

      int i = 0;
      while (scheduler.checkExists(jobkey) && i++ < 5) {
        Thread.sleep(1000);
      }

      JobKey reminderJobkey = JobKey.jobKey(jobId, PatientProtocolScheduler.JOB_REMINDER_GROUP);
      boolean reminderJobsScheduled = scheduler.checkExists(reminderJobkey);
      Assert.assertTrue(reminderJobsScheduled);


      CMSMessage cmsg = new CMSMessage(null, patient, MessageProcessorService.SUCCESS_MESSAGE, MessageType.OUT);
      cmsg.setText(MessageProcessorService.SUCCESS_MESSAGE);
      cmsg.setPatientProtocolId(patientProtocol.getId());
      cmsg.setTemplateIndex(index);

      MockServletContext ctx = getMockServletContext();
      IncomingMessageProcessor incomingMessageProcessor = new IncomingMessageProcessor(ctx);
      incomingMessageProcessor.deletePatientReminderJobs(cmsg);
      reminderJobsScheduled = scheduler.checkExists(reminderJobkey);
      Assert.assertFalse(reminderJobsScheduled);
    }

  }

  private MockServletContext getMockServletContext() {
    MockServletContext ctx = new MockServletContext();
    ctx.setAttribute(BcmQuartzInitializerListener.QUARTZ_FACTORY_KEY, factory);
    return ctx;
  }

  @Test
  public void removeRemindersOnDeleteProtocol() throws Exception {

    Patient patient = getPatient();
    patient.setId(patientId);
    DBObject dbObject = (DBObject) JSON.parse(MAPPER.writeValueAsString(patient));
    ObjectId oId1 = new ObjectId(patient.getId());
    dbObject.put("_id", oId1);
    mongoDB.getCollection("patients").insert(dbObject);

    PatientProtocol patientProtocol = getPatientProtocol();
    //p2.getProtocol().setReminderInterval(new TimeInterval(TimeIntervalUnit.MINUTE, 10));
    patientProtocol.setId(patientProtocolId);
    dbObject = (DBObject) JSON.parse(MongoDocumentBase.newCustomObjectMapper().writeValueAsString(patientProtocol));
    ObjectId oId = new ObjectId(patientProtocol.getId());
    dbObject.put("_id", oId);
    mongoDB.getCollection("patientProtocol").insert(dbObject);

    scheduler.pauseAll();
    patientScheduler.update(patientProtocol, patient.getPreference().getTimeZone());

    for (int index = 0; index < patientProtocol.getProtocol().getTemplates().size(); index++) {
      String jobId = PatientProtocolScheduler.getJobId(patientProtocol.getId(), index);
      JobKey jobkey = JobKey.jobKey(jobId, PatientProtocolScheduler.JOB_FIRST_GROUP);
          /*boolean initJobScheduled = scheduler.checkExists(jobkey);
          Assert.assertTrue(initJobScheduled);*/

      patientScheduler.scheduler.resumeAll();
      if (patientScheduler.scheduler.checkExists(jobkey)) {
        patientScheduler.scheduler.triggerJob(jobkey);
      }

      int i = 0;
      while (scheduler.checkExists(jobkey) && i++ < 5) {
        Thread.sleep(1000);
      }

      JobKey reminderJobkey = JobKey.jobKey(jobId, PatientProtocolScheduler.JOB_REMINDER_GROUP);
      boolean reminderJobsScheduled = scheduler.checkExists(reminderJobkey);
      Assert.assertTrue(reminderJobsScheduled);
    }

    for (int index = 0; index < patientProtocol.getProtocol().getTemplates().size(); index++) {
      String jobId = PatientProtocolScheduler.getJobId(patientProtocol.getId(), index);
      JobKey reminderJobkey = JobKey.jobKey(jobId, PatientProtocolScheduler.JOB_REMINDER_GROUP);
      patientScheduler.delete(patientProtocol);
      boolean reminderJobsScheduled = scheduler.checkExists(reminderJobkey);
      Assert.assertFalse(reminderJobsScheduled);
    }


  }


  @After
  public void finish() throws IOException, SchedulerException {
    scheduler.shutdown();
    mongoDB.getCollection("patients").drop();
    mongoDB.getCollection("patientProtocol").drop();
    mongoDB.getCollection("test").drop();
    MongoDBConnection.close();
  }

  private Patient getPatient() {
    Patient p1 = new Patient();
    p1.setFirstName("John");
    p1.setLastName("Smith");

    PatientPreference pref = new PatientPreference();
    pref.setPhoneNumber("123456789");
    pref.setTimeZone("US/Eastern");

    p1.setDob("1966-01-01");
    p1.setPreference(pref);
    return p1;
  }

  private PatientProtocol getPatientProtocol() {
    PatientProtocol p = new PatientProtocol();
    p.setPatientId(patientId);
    DateTime today = new DateTime();
    DateTimeFormatter fmt = DateTimeFormat.forPattern("yyyy-MM-dd");
    Period pd = Period.days(1);

    Template template = getBloodPressureProtocol();
    template.setStartDate(today.toString(fmt));
    template.setEndDate((today.plus(pd)).toString(fmt));

    Template template1 = getBloodPressureProtocol();
    template1.setStartDate(today.toString(fmt));
    template1.setEndDate((today.plus(pd)).toString(fmt));

    List<Template> templates = new ArrayList<Template>();
    templates.add(template);
    templates.add(template1);
    Protocol protocol = new Protocol();
    protocol.setTemplates(templates);
    protocol.setName("Blood Pressure");
    p.setProtocol(protocol);

    p.setAssignedDate(today.toString(fmt));
    p.setLastEditedDate(today.toString(fmt));
    p.setStatus(PatientProtocolStatus.ACTIVE);

    return p;
  }

  protected static StdSchedulerFactory getStdSchedulerFactory() throws SchedulerException {
    Properties props = new Properties();
    props.setProperty("org.quartz.threadPool.threadCount", "1");
    props.setProperty("org.quartz.threadPool.class", "org.quartz.simpl.SimpleThreadPool");
    props.setProperty("org.quartz.jobStore.class", "org.quartz.simpl.RAMJobStore");
    props.setProperty("org.quartz.jobStore.misfireThreshold", "org.quartz.simpl.RAMJobStore");
    StdSchedulerFactory f = new StdSchedulerFactory(props);
    return f;
  }

  private Template getBloodPressureProtocol() {

    final Template protocol = ProtocolTemplateBuilder.buildProtocolTemplateWithScheduleTypeOnce();

    LocalTime lt = new LocalTime();
    ScheduleTime st = new ScheduleTime();
    if (lt.getHourOfDay() > 12) {
      st.setHour(lt.getHourOfDay() - 12);
      st.setMeridian(Meridian.PM);
    } else {
      st.setHour(lt.getHourOfDay());
      st.setMeridian(Meridian.AM);
    }
    st.setMin(lt.getMinuteOfHour());
    protocol.getSchedule().setScheduleTime(st);

    protocol.getSchedule().setReminderInterval(new TimeInterval(TimeIntervalUnit.YEAR,
      1));
    protocol.getSchedule().setReminderNumber(2);

    return protocol;
  }

  private Template getProtocolforEveryX() {

    final Template protocol = ProtocolTemplateBuilder.buildProtocolTemplateWithScheduleTypeEverySoMany();

    LocalTime lt = new LocalTime();
    ScheduleTime st = new ScheduleTime();
    if (lt.getHourOfDay() > 12) {
      st.setHour(lt.getHourOfDay() - 12);
      st.setMeridian(Meridian.PM);
    } else {
      st.setHour(lt.getHourOfDay());
      st.setMeridian(Meridian.AM);
    }
    st.setMin(lt.getMinuteOfHour());

    protocol.getSchedule().setScheduleTime(st);
    TypeEverySoMany typeEverySoMany = new TypeEverySoMany();

    typeEverySoMany.setName("Every so many X");
    typeEverySoMany.setInterval(new TimeInterval(TimeIntervalUnit.MINUTE, 1));

    protocol.getSchedule().setScheduleType(typeEverySoMany);


    return protocol;
  }

  private PatientProtocol getPatientProtocolEveryX() {
    PatientProtocol p = new PatientProtocol();
    p.setPatientId(patientId);
    DateTime today = new DateTime();
    DateTimeFormatter fmt = DateTimeFormat.forPattern("yyyy-MM-dd");

    Template template = getProtocolforEveryX();
    template.setStartDate(today.toString(fmt));
    template.setEndDate((today).toString(fmt));    // for test end date should be same date

    List<Template> templates = new ArrayList<Template>();
    templates.add(template);
    Protocol protocol = new Protocol();
    protocol.setTemplates(templates);
    protocol.setName("Blood Pressure");
    p.setProtocol(protocol);

    p.setAssignedDate(today.toString(fmt));
    p.setLastEditedDate(today.toString(fmt));
    p.setStatus(PatientProtocolStatus.ACTIVE);

    return p;
  }


  @Test
  public void scheduleEveryX() throws Exception {

    Patient p1 = getPatient();
    p1.setId(patientId);
    DBObject dbObject = (DBObject) JSON.parse(MAPPER.writeValueAsString(p1));
    ObjectId oId1 = new ObjectId(p1.getId());
    dbObject.put("_id", oId1);
    mongoDB.getCollection("patients").insert(dbObject);

    PatientProtocol p2 = getPatientProtocolEveryX();
    p2.setId(patientProtocolId);
    dbObject = (DBObject) JSON.parse(MongoDocumentBase.newCustomObjectMapper().writeValueAsString(p2));
    ObjectId oId = new ObjectId(p2.getId());
    dbObject.put("_id", oId);
    mongoDB.getCollection("patientProtocol").insert(dbObject);

    scheduler.pauseAll();
    patientScheduler.update(p2, p1.getPreference().getTimeZone());

    String jobId = PatientProtocolScheduler.getJobId(p2.getId(), 0);
    JobKey jobkey = JobKey.jobKey(jobId, PatientProtocolScheduler.JOB_FIRST_GROUP);
    boolean initJobScheduled = scheduler.checkExists(jobkey);
    Assert.assertTrue(initJobScheduled);
    scheduler.resumeAll();
        /*if (scheduler.checkExists(jobkey)) {
            scheduler.triggerJob(jobkey);
        }    */

       /* while (scheduler.checkExists(jobkey)) {
            Thread.sleep(1000);
        }  */

        /*JobKey reminderJobkey = JobKey.jobKey(jobId, PatientProtocolScheduler.JOB_REMINDER_GROUP);
        boolean reminderJobsScheduled = scheduler.checkExists(reminderJobkey);
        Assert.assertTrue(reminderJobsScheduled);     */
    scheduler.clear();


  }

  private Template getProtocolforTypeDaysOfWeek() {

    final Template protocol = ProtocolTemplateBuilder.buildProtocolTemplateWithScheduleTypeDaysOfWeek();

    LocalTime lt = new LocalTime();
    ScheduleTime st = new ScheduleTime();
    if (lt.getHourOfDay() > 12) {
      st.setHour(lt.getHourOfDay() - 12);
      st.setMeridian(Meridian.PM);
    } else {
      st.setHour(lt.getHourOfDay());
      st.setMeridian(Meridian.AM);
    }
    st.setMin(lt.getMinuteOfHour());
    protocol.getSchedule().setScheduleTime(st);


    return protocol;
  }

  private PatientProtocol getPatientProtocolTypeDaysOfWeek() {
    PatientProtocol p = new PatientProtocol();
    p.setPatientId(patientId);
    DateTime today = new DateTime();
    DateTimeFormatter fmt = DateTimeFormat.forPattern("yyyy-MM-dd");
    Period pd = Period.days(20);

    Template template = getProtocolforTypeDaysOfWeek();
    template.setStartDate(today.toString(fmt));
    template.setEndDate((today.plus(pd)).toString(fmt));

    List<Template> templates = new ArrayList<Template>();
    templates.add(template);
    Protocol protocol = new Protocol();
    protocol.setTemplates(templates);
    protocol.setName("Blood Pressure");
    p.setProtocol(protocol);

    p.setAssignedDate(today.toString(fmt));
    p.setLastEditedDate(today.toString(fmt));
    p.setStatus(PatientProtocolStatus.ACTIVE);

    return p;
  }

  @Test
  public void scheduleDaysOfWeek() throws Exception {

    Patient p1 = getPatient();
    p1.setId(patientId);
    DBObject dbObject = (DBObject) JSON.parse(MAPPER.writeValueAsString(p1));
    ObjectId oId1 = new ObjectId(p1.getId());
    dbObject.put("_id", oId1);
    mongoDB.getCollection("patients").insert(dbObject);

    PatientProtocol p2 = getPatientProtocolTypeDaysOfWeek();
    p2.setId(patientProtocolId);
    dbObject = (DBObject) JSON.parse(MongoDocumentBase.newCustomObjectMapper().writeValueAsString(p2));
    ObjectId oId = new ObjectId(p2.getId());
    dbObject.put("_id", oId);
    mongoDB.getCollection("patientProtocol").insert(dbObject);

    scheduler.pauseAll();
    patientScheduler.update(p2, p1.getPreference().getTimeZone());

    String jobId = PatientProtocolScheduler.getJobId(p2.getId(), 0);
    JobKey jobkey = JobKey.jobKey(jobId, PatientProtocolScheduler.JOB_FIRST_GROUP);
    boolean initJobScheduled = scheduler.checkExists(jobkey);
    Assert.assertTrue(initJobScheduled);
    scheduler.resumeAll();
        /*if (scheduler.checkExists(jobkey)) {
            scheduler.triggerJob(jobkey);
        }

        while (scheduler.checkExists(jobkey)) {
            Thread.sleep(1000);
        }

        JobKey reminderJobkey = JobKey.jobKey(jobId, PatientProtocolScheduler.JOB_REMINDER_GROUP);
        boolean reminderJobsScheduled = scheduler.checkExists(reminderJobkey);
        Assert.assertTrue(reminderJobsScheduled);  */
    scheduler.clear();


  }

  @Test
  public void scheduleTypeOnce() throws Exception {

    Patient p1 = getPatient();
    p1.setId(patientId);
    DBObject dbObject = (DBObject) JSON.parse(MAPPER.writeValueAsString(p1));
    ObjectId oId1 = new ObjectId(p1.getId());
    dbObject.put("_id", oId1);
    mongoDB.getCollection("patients").insert(dbObject);

    PatientProtocol p2 = getPatientProtocolTypeOnce();
    p2.setId(patientProtocolId);
    dbObject = (DBObject) JSON.parse(MongoDocumentBase.newCustomObjectMapper().writeValueAsString(p2));
    ObjectId oId = new ObjectId(p2.getId());
    dbObject.put("_id", oId);
    mongoDB.getCollection("patientProtocol").insert(dbObject);

    scheduler.pauseAll();
    scheduler.start();
    patientScheduler.update(p2, p1.getPreference().getTimeZone());

    String jobId = PatientProtocolScheduler.getJobId(p2.getId(), 0);
    JobKey jobkey = JobKey.jobKey(jobId, PatientProtocolScheduler.JOB_FIRST_GROUP);
    boolean initJobScheduled = scheduler.checkExists(jobkey);
    Assert.assertTrue(initJobScheduled);

        /*if (scheduler.checkExists(jobkey)) {
            scheduler.triggerJob(jobkey);
        }

        while (scheduler.checkExists(jobkey)) {
            Thread.sleep(1000);
        }

        JobKey reminderJobkey = JobKey.jobKey(jobId, PatientProtocolScheduler.JOB_REMINDER_GROUP);
        boolean reminderJobsScheduled = scheduler.checkExists(reminderJobkey);
        Assert.assertTrue(reminderJobsScheduled);  */
    scheduler.clear();


  }

  private PatientProtocol getPatientProtocolTypeOnce() {
    PatientProtocol p = new PatientProtocol();
    p.setPatientId(patientId);
    DateTime today = new DateTime();
    DateTimeFormatter fmt = DateTimeFormat.forPattern("yyyy-MM-dd");
    Period pd = Period.days(1);

    Template template = getProtocolforTypeOnce();
    template.setStartDate(today.toString(fmt));
    template.setEndDate((today.plus(pd)).toString(fmt));

    List<Template> templates = new ArrayList<Template>();
    templates.add(template);
    Protocol protocol = new Protocol();
    protocol.setTemplates(templates);
    protocol.setName("Blood Pressure");
    p.setProtocol(protocol);

    p.setAssignedDate(today.toString(fmt));
    p.setLastEditedDate(today.toString(fmt));
    p.setStatus(PatientProtocolStatus.ACTIVE);

    return p;
  }

  private Template getProtocolforTypeOnce() {

    final Template protocol = ProtocolTemplateBuilder.buildProtocolTemplateWithScheduleTypeOnce();

    LocalTime lt = new LocalTime();
    ScheduleTime st = new ScheduleTime();
    if (lt.getHourOfDay() > 12) {
      st.setHour(lt.getHourOfDay() - 12);
      st.setMeridian(Meridian.PM);
    } else {
      st.setHour(lt.getHourOfDay());
      st.setMeridian(Meridian.AM);
    }
    st.setMin(lt.getMinuteOfHour());
    protocol.getSchedule().setScheduleTime(st);


    return protocol;
  }


}
