package gov.va.mobilehealth.courier.scheduler;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.mongodb.DB;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.util.JSON;

import gov.va.mobilehealth.cms.jackson.Jackson;
import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.PatientPreference;
import gov.va.mobilehealth.courier.core.protocol.ProtocolTemplateBuilder;
import gov.va.mobilehealth.courier.core.protocol.PatientProtocol;
import gov.va.mobilehealth.courier.core.protocol.Vital;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;

import org.bson.types.ObjectId;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.quartz.*;
import org.quartz.impl.StdSchedulerFactory;

import java.io.IOException;

import static org.fest.assertions.api.Assertions.assertThat;
import static org.quartz.JobBuilder.newJob;

public class SchedulerAcceptanceTest {

  private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
  static String protocolTemplateId = "528e87b0da065b48422945fb";
  static String patientId = "52a8f798da063c40a95fc9b7";
  static String testPatientId = "52a8f798da063c40a95fc9b4";
  static String patientProtocolId = "528e880bda065b48422945fe";
  static String deleteProtocolTemplateId = "52963d85da06c04b9f1eefe6";
  static String serviceMsg = "service message";
  private static DB mongoDB;
  private static Scheduler scheduler;

  @BeforeClass
  public static void setup() throws IOException, SchedulerException {
    MongoDBConnection.init("mongodb://localhost", "courier");
    mongoDB = MongoDBConnection.getDB();

    mongoDB.getCollection("patients").drop();
    mongoDB.getCollection("patientProtocol").drop();

    //schedule it
    scheduler = new StdSchedulerFactory("quartz_test.properties").getScheduler();
    scheduler.start();
  }

  @AfterClass
  public static void finish() throws IOException, SchedulerException {
    scheduler.shutdown();
    mongoDB.getCollection("patients").drop();
    mongoDB.getCollection("patientProtocol").drop();
    mongoDB.getCollection("test").drop();
    MongoDBConnection.close();
  }

  @Test
  public void canExecuteJob() throws Exception {
    Patient p1 = getPatient();
    p1.setId(patientId);
    DBObject dbObject = (DBObject) JSON.parse(MAPPER.writeValueAsString(p1));
    ObjectId oId1 = new ObjectId(p1.getId());
    dbObject.put("_id", oId1);
    mongoDB.getCollection("patients").insert(dbObject);

    PatientProtocol p2 = ProtocolTemplateBuilder.buildPatientProtocol(patientId, null);
    Vital template = (Vital)p2.getProtocol().getTemplates().get(0);
    template.setServiceMessage(serviceMsg);
    p2.setId(patientProtocolId);
    dbObject = (DBObject) JSON.parse(MongoDocumentBase.newCustomObjectMapper().writeValueAsString(p2));
    ObjectId oId = new ObjectId(p2.getId());
    dbObject.put("_id", oId);
    mongoDB.getCollection("patientProtocol").insert(dbObject);
    String jobId = PatientProtocolScheduler.getJobId(patientProtocolId, 0);

    JobDetail job = newJob(PatientProtocolJobTestInit.class)
            .withIdentity(jobId, "group1")
            .build();

    Trigger trigger = TriggerBuilder.newTrigger()
            .withIdentity(jobId, "group1")
            .startNow()
            .build();

    scheduler.scheduleJob(job, trigger);
    // shutdown the scheduler after 10 seconds
    try {
      Thread.sleep(100);
    } catch (InterruptedException ex) {
      Thread.currentThread().interrupt();
    }

    DBCursor cursor = mongoDB.getCollection("test").find();

    assertThat(cursor.count()).isNotZero();

    while (cursor.hasNext()) {
      assertThat(cursor.next().get("message")).isEqualTo(serviceMsg);
      assertThat(cursor.curr().get("mdn")).isEqualTo("123456789");
    }

    scheduler.deleteJob(JobKey.jobKey(jobId, "group1"));
    scheduler.unscheduleJob(TriggerKey.triggerKey(jobId, "group1"));
  }

  private Patient getPatient() {
    Patient p1 = new Patient();
    p1.setFirstName("John");
    p1.setLastName("Smith");

    PatientPreference pref = new PatientPreference();
    pref.setPhoneNumber("123456789");

    p1.setDob("1966-01-01");
    p1.setPreference(pref);
    return p1;
  }

}
