package gov.va.mobilehealth.courier.service;

import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserFactory;
import com.mongodb.MongoClient;

import gov.va.mobilehealth.courier.core.Facility;
import gov.va.mobilehealth.courier.core.protocol.BroadcastFacilityMessage;
import gov.va.mobilehealth.courier.core.protocol.Meridian;
import gov.va.mobilehealth.courier.core.protocol.Schedule;
import gov.va.mobilehealth.courier.core.protocol.ScheduleTime;
import gov.va.mobilehealth.courier.core.protocol.TimeInterval;
import gov.va.mobilehealth.courier.core.protocol.TimeIntervalUnit;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeOnce;
import gov.va.mobilehealth.courier.db.DictionaryDAO;
import gov.va.mobilehealth.courier.db.FacilityDAO;
import gov.va.mobilehealth.courier.resources.helper.BadRequestException;
import gov.va.mobilehealth.courier.scheduler.BroadcastMessageScheduler;
import gov.va.mobilehealth.courier.utils.DateParser;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;

import org.joda.time.DateTime;
import org.junit.*;
import org.junit.rules.ExpectedException;
import org.quartz.JobKey;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.impl.StdSchedulerFactory;

import java.io.IOException;
import java.util.List;

import javax.servlet.ServletContext;

import static org.fest.assertions.api.Assertions.assertThat;
import static org.junit.Assert.assertNotNull;

public class BroadCastFacilityMessageServiceTest {
  private static MongoClient mongoClient;
  private static final String defaultAssigningAuthority = "dfn-688";
  private final MhpUser defaultMhpUser = createMhpUser(defaultAssigningAuthority);

  private BroadCastFacilityMessageService service;
  private BroadCastFacilityMessageService serviceToTestScheduler;
  private Scheduler scheduler; 

  @Rule
  public ExpectedException thrown = ExpectedException.none();

  @Before
  public void setup() throws SchedulerException{
    service = new BroadCastFacilityMessageService() {
        protected BroadcastMessageScheduler getBroadcastMessageScheduler(ServletContext context) throws SchedulerException {
            return new BroadcastMessageScheduler(null) {
                public void update(BroadcastFacilityMessage facilityMessage, String timezone) {
                }
                
                public void delete(final String jobId) {
                }
            };
        }
    };
    service.setMhpUser(defaultMhpUser);
    
    StdSchedulerFactory schedulerFactory = new StdSchedulerFactory("quartz_test.properties");
    scheduler = schedulerFactory.getScheduler();

    serviceToTestScheduler = new BroadCastFacilityMessageService() {
        protected BroadcastMessageScheduler getBroadcastMessageScheduler(ServletContext context) throws SchedulerException {
            return new BroadcastMessageScheduler(scheduler);
        };
    };
    serviceToTestScheduler.setMhpUser(defaultMhpUser);
    scheduler.start();
}

  @After
  public void finish() throws IOException, SchedulerException {
    scheduler.shutdown();
  }

  @BeforeClass
  public static void classSetup() throws IOException {
    MongoDBConnection.init("mongodb://localhost", "courier");
    mongoClient = new MongoClient();
    mongoClient.getDB("courier").getCollection("facility").drop();

    FacilityDAO dao = new FacilityDAO();
    dao.saveOrUpdate(createFacility());
  }

  @AfterClass
  public static void tearDown() {
    mongoClient.getDB("courier").getCollection("facility").drop();
    MongoDBConnection.close();
  }

  @Test
  public void getById() throws Exception {
    BroadcastFacilityMessage message = createFacilityMessage();

    BroadcastFacilityMessage savedMessage = service.updateBroadcastFacilityMessageTemplate(null, message);

    BroadcastFacilityMessage foundMessage = service.getById(savedMessage.getId());
    assertNotNull(foundMessage);

    service.remove(foundMessage);

  }

  @Test
  public void getByIdWithWrongAssigningAuthorityThrows() throws Exception {
    BroadcastFacilityMessage message = createFacilityMessage();
    BroadcastFacilityMessage savedMessage = service.updateBroadcastFacilityMessageTemplate(null, message);

    //set up exception expectations
    thrown.expect(BadRequestException.class);
    thrown.expectMessage(BroadCastFacilityMessageService.MSG_INCORRECT_ASSIGNING_AUTHORITY);

    try{
        //now execute what should throw
        //should throw based upon message assiginging authority not matching user assigning authority
        BroadCastFacilityMessageService newService = new BroadCastFacilityMessageService();
        newService.setMhpUser(createMhpUser("dfn-diff"));
        newService.getById(savedMessage.getId());
    } finally {
        service.remove(message);
    }

  }

  @Test
  public void getByIdWithWrongIdThrows() throws Exception {
    BroadcastFacilityMessage message = createFacilityMessage();
    service.updateBroadcastFacilityMessageTemplate(null, message);

    //set up exception expectations
    thrown.expect(BadRequestException.class);
    thrown.expectMessage(BroadCastFacilityMessageService.MSG_NOT_FOUND);

    try {
        //now execute what should throw
        //should throw based upon message assiginging authority not matching user assigning authority
        service.getById("badId");
    } finally {
        service.remove(message);
    }

  }

  @Test
  public void saveWithWrongAssigningAuthorityThrows() throws Exception {
    BroadcastFacilityMessage message = createFacilityMessage();
    message.setAssigningAuthority("dfn-different");

    //set up exception expectations
    thrown.expect(BadRequestException.class);
    thrown.expectMessage(BroadCastFacilityMessageService.MSG_INCORRECT_ASSIGNING_AUTHORITY);

    //now execute what should throw
    service.updateBroadcastFacilityMessageTemplate(null, message);

  }

  @Test
  public void createAndGetFacilityMessage() throws Exception {
    BroadcastFacilityMessage message = createFacilityMessage();

    service.updateBroadcastFacilityMessageTemplate(null, message);

    List<BroadcastFacilityMessage> messagesReturned = service.getMessages();

    // Verify the new protocol is there
    BroadcastFacilityMessage foundMessage = null;
    for (BroadcastFacilityMessage m : messagesReturned) {
      if (m.getName().equals("My First Facility Message")) {
        foundMessage = m;
        break;
      }
    }

    assertThat(foundMessage).isNotNull();
    assertThat(foundMessage.getAssigningAuthority()).isEqualTo("dfn-688");
    assertThat(foundMessage.getCreateBy()).isNotNull();
    assertThat(foundMessage.getCreateDate()).isNotNull();
    assertThat(foundMessage.getMessage()).isEqualTo("My First Facility Message Description");
    assertThat(foundMessage.getName()).isEqualTo("My First Facility Message");
    assertThat(foundMessage.isEnabled()).isEqualTo(true);
    assertThat(foundMessage.getId()).isNotNull();
    assertThat(foundMessage.getProgress()).isEqualTo(BroadcastFacilityMessage.PROGRESS_PENDING);
    assertThat(foundMessage.getComments()).isNull();

    service.remove(foundMessage);
  }

  @Test
  public void editAndGetFacilityMessage() throws Exception {
    BroadcastFacilityMessage message = createFacilityMessage();
    service.updateBroadcastFacilityMessageTemplate(null, message);

    message.setMessage("This is modified message");
    message.setProgress("bad progress");
    service.updateBroadcastFacilityMessageTemplate(null, message);
    List<BroadcastFacilityMessage> messagesReturned = service.getMessages();

    // Verify the new protocol is there
    BroadcastFacilityMessage foundMessage = null;
    for (BroadcastFacilityMessage m : messagesReturned) {
      if (m.getName().equals("My First Facility Message")) {
        foundMessage = m;
        break;
      }
    }

    String today = DateParser.getTodaysDate(null, null);
    assertThat(foundMessage).isNotNull();
    assertThat(foundMessage.getAssigningAuthority()).isEqualTo("dfn-688");
    assertThat(foundMessage.getCreateBy()).isEqualTo("One Vehu");
    assertThat(foundMessage.getCreateDate()).isEqualTo(today);
    assertThat(foundMessage.getMessage()).isEqualTo("This is modified message");
    assertThat(foundMessage.getName()).isEqualTo("My First Facility Message");
    assertThat(foundMessage.isEnabled()).isEqualTo(true);
    assertThat(foundMessage.getComments().getUser()).isEqualTo("One Vehu");
    assertThat(foundMessage.getComments().getDate()).isEqualTo(today);
    assertThat(foundMessage.getComments().getTime()).isNotNull();
    assertThat(foundMessage.getProgress()).isEqualTo(BroadcastFacilityMessage.PROGRESS_PENDING);
    assertThat(foundMessage.getId()).isNotNull();

    service.remove(foundMessage);
  }

  @Test
  public void createScheduleAndExpireTest() throws Exception {
    BroadcastFacilityMessage message = createFacilityMessage();
    String today = DateParser.getTodaysDate(null, null);
    message.setStartDate(today);
    message.setEndDate(today);

    // Create BCM message with custom schedule 1 minute from now
    Schedule schedule = new Schedule();
    DateTime dateTime = new DateTime();
    int hour = dateTime.getHourOfDay() % 12;

    ScheduleTime scheduleTime = new ScheduleTime();
    
    scheduleTime.setHour(dateTime.getHourOfDay());
    scheduleTime.setMin(dateTime.getMinuteOfHour() + 1);
    scheduleTime.setMeridian(hour == dateTime.getHourOfDay() ? Meridian.AM : Meridian.PM);

    schedule.setScheduleTime(scheduleTime);

    TypeEverySoMany scheduleType = new TypeEverySoMany();
    scheduleType.setName("Custom");
    scheduleType.setInterval(new TimeInterval(TimeIntervalUnit.MINUTE, 10));
    schedule.setScheduleType(scheduleType);

    message.setSchedule(schedule);

    // save the BCM message, the send message and expire job should be scheduled
    scheduler.pauseAll();
    BroadcastFacilityMessage savedMessage = serviceToTestScheduler.updateBroadcastFacilityMessageTemplate(null, message);
    assertNotNull(savedMessage);
    assertThat(savedMessage.getProgress()).isEqualTo(BroadcastFacilityMessage.PROGRESS_PENDING);

    // check init and expire job scheduled
    JobKey expireJobkey = JobKey.jobKey(savedMessage.getId(), BroadcastMessageScheduler.JOB_EXPIRE_GROUP);
    JobKey initJobkey = JobKey.jobKey(savedMessage.getId(), BroadcastMessageScheduler.JOB_FIRST_GROUP);
    boolean JobScheduled = scheduler.checkExists(initJobkey);
    Assert.assertTrue(JobScheduled);
    JobScheduled = scheduler.checkExists(expireJobkey);
    Assert.assertTrue(JobScheduled);
    scheduler.resumeAll();
    //scheduler.triggerJob(initJobkey);
    
    // wait for a minute to let the send message job run
    Thread.sleep(61000);

    // check message status changed
    savedMessage = serviceToTestScheduler.getById(savedMessage.getId());
    assertNotNull(savedMessage); 
    assertThat(savedMessage.getProgress()).isEqualTo(BroadcastFacilityMessage.PROGRESS_IN_PROGRESS);
    
    // trigger the expire job
    scheduler.triggerJob(expireJobkey);
    Thread.sleep(0000);

    // check jobs got unscheduled and message status changed
    JobScheduled = scheduler.checkExists(initJobkey);
    Assert.assertFalse(JobScheduled);
    savedMessage = serviceToTestScheduler.getById(savedMessage.getId());
    assertNotNull(savedMessage); 
    assertThat(savedMessage.getProgress()).isEqualTo(BroadcastFacilityMessage.PROGRESS_DELIVERED);
    service.remove(savedMessage);
  }

  @Test
  public void createScheduleOnceAndDisableTest() throws Exception {
    BroadcastFacilityMessage message = createFacilityMessage();
    String today = DateParser.getTodaysDate(null, null);
    message.setStartDate(today);
    message.setEndDate(today);

    // Create BCM message with once schedule 1 minute from now
    Schedule schedule = new Schedule();
    DateTime dateTime = new DateTime();
    int hour = dateTime.getHourOfDay() % 12;

    ScheduleTime scheduleTime = new ScheduleTime();
    
    scheduleTime.setHour(dateTime.getHourOfDay());
    scheduleTime.setMin(dateTime.getMinuteOfHour() + 1);
    scheduleTime.setMeridian(hour == dateTime.getHourOfDay() ? Meridian.AM : Meridian.PM);

    schedule.setScheduleTime(scheduleTime);

    TypeOnce scheduleType = new TypeOnce();
    scheduleType.setName("Once");
    schedule.setScheduleType(scheduleType);

    message.setSchedule(schedule);

    scheduler.pauseAll();
    // save the BCM message, the send message and expire job should be scheduled
    BroadcastFacilityMessage savedMessage = serviceToTestScheduler.updateBroadcastFacilityMessageTemplate(null, message);
    assertNotNull(savedMessage);
    assertThat(savedMessage.getProgress()).isEqualTo(BroadcastFacilityMessage.PROGRESS_PENDING);

    // check init and expire job scheduled
    JobKey expireJobkey = JobKey.jobKey(savedMessage.getId(), BroadcastMessageScheduler.JOB_EXPIRE_GROUP);
    JobKey initJobkey = JobKey.jobKey(savedMessage.getId(), BroadcastMessageScheduler.JOB_FIRST_GROUP);
    boolean JobScheduled = scheduler.checkExists(initJobkey);
    Assert.assertTrue(JobScheduled);
    JobScheduled = scheduler.checkExists(expireJobkey);
    Assert.assertTrue(JobScheduled);
    scheduler.resumeAll();
    //scheduler.triggerJob(initJobkey);
    
    // let the send message job run
    int i = 0;
    while (scheduler.checkExists(initJobkey) && i++ < 60) {
      Thread.sleep(1000);
    }

    // check message status changed
    savedMessage = serviceToTestScheduler.getById(savedMessage.getId());
    assertNotNull(savedMessage); 
    assertThat(savedMessage.getProgress()).isEqualTo(BroadcastFacilityMessage.PROGRESS_DELIVERED);
    
    JobScheduled = scheduler.checkExists(initJobkey);
    Assert.assertFalse(JobScheduled);

    // disable the BCM message
    savedMessage.setEnabled(false);
    savedMessage = serviceToTestScheduler.updateBroadcastFacilityMessageTemplate(null, savedMessage);
    
    // check related job being unscheduled
    JobScheduled = scheduler.checkExists(expireJobkey);
    Assert.assertFalse(JobScheduled);
    service.remove(savedMessage);
}

  public BroadcastFacilityMessage createFacilityMessage() {
    BroadcastFacilityMessage message = new BroadcastFacilityMessage();
    message.setName("My First Facility Message");
    message.setMessage("My First Facility Message Description");
    message.setEnabled(true);
    message.setAssigningAuthority(defaultAssigningAuthority);
    return message;
  }
  
  public static Facility createFacility() {
      Facility facility = new Facility();
      facility.setFacilityId(defaultAssigningAuthority);
      facility.setTimezone("America/New_York");
      return facility;
    }

  public static MhpUser createMhpUser(String assigningAuthority) {
    PatientIdentifier userIdentifier = new PatientIdentifier(assigningAuthority, "");
    MhpUser user = new MhpUser();
    user.setFirstName("One");
    user.setLastName("Vehu");
    user.setUserIdentifier(userIdentifier);
    return user;
  }


}
