package gov.va.mobilehealth.courier.service;

import static org.fest.assertions.api.Assertions.assertThat;

import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.mongodb.MongoClient;

import gov.va.mobilehealth.courier.bcm.appointments.EdipiIdentifier;
import gov.va.mobilehealth.courier.bcm.appointments.db.AssessmentProfileFutureAppointmentsDAO;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.PatientPreference;
import gov.va.mobilehealth.courier.core.UserIdentifier;
import gov.va.mobilehealth.courier.core.protocol.Meridian;
import gov.va.mobilehealth.courier.core.protocol.PatientProtocol;
import gov.va.mobilehealth.courier.core.protocol.PatientProtocolStatus;
import gov.va.mobilehealth.courier.core.protocol.Protocol;
import gov.va.mobilehealth.courier.core.protocol.ProtocolTemplateBuilder;
import gov.va.mobilehealth.courier.core.protocol.Schedule;
import gov.va.mobilehealth.courier.core.protocol.ScheduleTime;
import gov.va.mobilehealth.courier.core.protocol.Template;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeOnce;
import gov.va.mobilehealth.courier.db.PatientDAO;
import gov.va.mobilehealth.courier.db.PatientDaoTestHelper;
import gov.va.mobilehealth.courier.db.PatientProtocolDAO;
import gov.va.mobilehealth.courier.db.ProtocolDAO;
import gov.va.mobilehealth.courier.helper.QuartzScheduleFactoryTestHelper;
import gov.va.mobilehealth.courier.scheduler.PatientProtocolScheduler;
import gov.va.mobilehealth.courier.utils.DateParser;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;

import org.joda.time.DateTime;
import org.joda.time.Period;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;
import org.quartz.JobKey;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.ServletContext;
import javax.ws.rs.WebApplicationException;

public class PatientsServiceTest {

  private static MongoClient mongoClient;

  private PatientsService service;
  private static Scheduler scheduler; 
  private String patientId;
  static String protocolId = "52a8f798da063c40a95fc9b8";
  private static PatientProtocolDAO patientProtocolDAO;
  private static ProtocolDAO protocolDAO;
  private static PatientDAO patientDAO;
  private final MhpUser mhpUser = BroadCastFacilityMessageServiceTest.createMhpUser("dfn-688");

  @Rule
  public ExpectedException thrown = ExpectedException.none();

  @Before
  public void start() throws SchedulerException, IOException{
    Patient patient = patientDAO.update(getPatient());
    patientId = patient.getId();
    
    service = new PatientsService() {
        protected PatientProtocolScheduler getPatientProtocolScheduler(ServletContext context) throws SchedulerException {
            return new PatientProtocolScheduler(scheduler);
        };
    };
}

  @BeforeClass
  public static void setup() throws IOException, SchedulerException {
    MongoDBConnection.init("mongodb://localhost", "courier");
    mongoClient = new MongoClient();
    mongoClient.getDB("courier").getCollection(AssessmentProfileFutureAppointmentsDAO.AppointmentsCollectionName).drop();
    scheduler = QuartzScheduleFactoryTestHelper.initTestScheduler();
    
    mongoClient.getDB("courier").getCollection("patientProtocol").drop();
    patientProtocolDAO = new PatientProtocolDAO();
    mongoClient.getDB("courier").getCollection("protocols").drop();
    protocolDAO = new ProtocolDAO();
    mongoClient.getDB("courier").getCollection("patients").drop();
    patientDAO = new PatientDAO();
  }

  @AfterClass
  public static void tearDown() {
    mongoClient.getDB("courier").getCollection("patientProtocol").drop();
    mongoClient.getDB("courier").getCollection("protocols").drop();
    mongoClient.getDB("courier").getCollection("patients").drop();
    MongoDBConnection.close();
  }


  @Test
  public void canUpdatePatient(){
    Patient patient = new Patient();
    UserIdentifier userIdentifier = new UserIdentifier();
    userIdentifier.setAssigningAuthority(EdipiIdentifier.EDIPI_ASSIGNING_AUTHORITY);
    userIdentifier.setUniqueId("100840");
    patient.setPatientIdentifier(userIdentifier);
    patient.setPreference(new PatientPreference());
    patient.getPreference().setReceiveApptReminder(false);

    PatientsService patientsService = new PatientsService();
    patientsService.updatePatient(patient);

  }
  
  @Test
  public void canUpdatePatientProtocol() throws Exception {
      PatientProtocol protocol = getPatientProtocol();
      protocolDAO.update(protocol.getProtocol());
      
      scheduler.pauseAll();
      service.updatePatientProtocol(patientId, protocol, mhpUser, null);
      
      List<PatientProtocol> protocolReturned = patientProtocolDAO.findByPatientId(patientId);

      // Verify the new protocol is there
      PatientProtocol foundProtocol = null;
      for (PatientProtocol p : protocolReturned) {
        if (p.getProtocol().getName().equals("Blood Pressure Test")) {
            foundProtocol = p;
          break;
        }
      }
      
      String today = DateParser.getTodaysDate(null, null);
      assertThat(foundProtocol).isNotNull();
      assertThat(foundProtocol.getId()).isNotNull();
      assertThat(foundProtocol.getAssignedDate()).isEqualTo(today);
      assertThat(foundProtocol.getStatus()).isEqualTo(PatientProtocolStatus.ACTIVE);

      // check init and expire job scheduled
      JobKey expireJobkey = JobKey.jobKey(PatientProtocolScheduler.getJobId(foundProtocol.getId(), 0), PatientProtocolScheduler.JOB_EXPIRE_GROUP);
      JobKey initJobkey = JobKey.jobKey(PatientProtocolScheduler.getJobId(foundProtocol.getId(), 0), PatientProtocolScheduler.JOB_FIRST_GROUP);
      boolean JobScheduled = scheduler.checkExists(initJobkey);
      Assert.assertTrue(JobScheduled);
      JobScheduled = scheduler.checkExists(expireJobkey);
      Assert.assertTrue(JobScheduled);
      
      // set protoocol status to pause
      foundProtocol.setStatus(PatientProtocolStatus.PAUSED);
      service.updatePatientProtocol(patientId, foundProtocol, mhpUser, null);
      
      // set protoocol status to resume
      foundProtocol.setStatus(PatientProtocolStatus.RESUME);
      service.updatePatientProtocol(patientId, foundProtocol, mhpUser, null);
      
      // check init removed and expire job still exists
      JobScheduled = scheduler.checkExists(initJobkey);
      Assert.assertTrue(JobScheduled);
      JobScheduled = scheduler.checkExists(expireJobkey);
      Assert.assertTrue(JobScheduled);

      // disable the protocol and check scheduled jobs are cleared
      foundProtocol.setStatus(PatientProtocolStatus.REMOVED);
      service.updatePatientProtocol(patientId, foundProtocol, mhpUser, null);
      
      // check init removed and expire job still exists
      JobScheduled = scheduler.checkExists(initJobkey);
      Assert.assertFalse(JobScheduled);
      JobScheduled = scheduler.checkExists(expireJobkey);
      Assert.assertFalse(JobScheduled);

      patientProtocolDAO.removeFromDB(foundProtocol.getId());
  }

  @Test
  public void validatePatientConsent() throws IOException{
    Patient patient = getPatient();
    // patient has consented through staff registration
    patient.setConsent(Patient.STAFF_ASSISTED_CONSENT);
    patient = patientDAO.update(patient);
    String pId = patient.getId();
    
    service.checkPatientConsent(pId);

    // patient only goes through self registration
    patient.setConsent(Patient.PATIENT_CONSENT);
    patient = patientDAO.update(patient);

    //set up exception expectations
    thrown.expect(WebApplicationException.class);

    try {
        //now execute what should throw
        service.checkPatientConsent(pId);
    } finally {
        patientDAO.removeFromDB(pId);
    }
  }
  
  private PatientProtocol getPatientProtocol() {
      PatientProtocol p = new PatientProtocol();
      p.setPatientId(patientId);
      DateTime today = new DateTime();
      DateTimeFormatter fmt = DateTimeFormat.forPattern("yyyy-MM-dd");
      Period pd = Period.days(1);

      Template template = getBloodPressureProtocol();
      template.setStartDate(today.toString(fmt));
      template.setEndDate((today.plus(pd)).toString(fmt));

      List<Template> templates = new ArrayList<Template>();
      templates.add(template);
      Protocol protocol = new Protocol();
      protocol.setId(protocolId);
      protocol.setTemplates(templates);
      protocol.setName("Blood Pressure Test");
      p.setProtocol(protocol);
      p.setStatus(PatientProtocolStatus.ACTIVE);

      return p;
    }

  private Template getBloodPressureProtocol() {

      final Template protocol = ProtocolTemplateBuilder.buildProtocolTemplateWithScheduleTypeOnce();

      Schedule schedule = new Schedule();
      DateTime dateTime = new DateTime();
      int hour = dateTime.getHourOfDay() % 12;

      ScheduleTime scheduleTime = new ScheduleTime();
      
      scheduleTime.setHour(hour);
      scheduleTime.setMin(dateTime.getMinuteOfHour() + 1);
      scheduleTime.setMeridian(hour == dateTime.getHourOfDay()? Meridian.AM : Meridian.PM);

      schedule.setScheduleTime(scheduleTime);

      TypeOnce scheduleType = new TypeOnce();
      scheduleType.setName("Once");
      schedule.setScheduleType(scheduleType);
      protocol.setSchedule(schedule);

      return protocol;
    }

  private static Patient getPatient(){
      Patient p = PatientDaoTestHelper.getPatient();
      p.getPreference().setTimeZone("America/New_York");
      return p;
  }
}
