package gov.va.mobilehealth.courier.utils.auth;

import gov.va.mobilehealth.courier.core.StaffRole;
import gov.va.mobilehealth.courier.db.StaffRoleDAO;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.springframework.security.access.AccessDecisionVoter;
import org.springframework.security.access.ConfigAttribute;
import org.springframework.security.access.SecurityConfig;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.web.FilterInvocation;

import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.security.AppUser;
import com.agilex.healthcare.mobilehealthplatform.security.Roles;
import com.mongodb.MongoClient;

import static org.fest.assertions.api.Assertions.assertThat;

public class ClinicalAccessRoleVoterTest {
  private static MongoClient mongoClient;

  private String ClinicalAccessAttribute = "USER_ADMIN";
  private String userName = "test";
  private String assigningAuthority = "EDIPI";
  private String uniqueId_good = "good_id";
  private String uniqueId_good_2nd = "good_id_second";
  private String uniqueId_bad = "bad_id";

  @BeforeClass
  public static void setup() throws IOException {
    MongoDBConnection.init("mongodb://localhost", "courier");
    mongoClient = new MongoClient();
  }

  @Before
  public void setUp() throws Exception {
    mongoClient.getDB("courier").getCollection("staffRole").drop();
    StaffRoleDAO dao = new StaffRoleDAO();

    StaffRole p1 = getStaffRole();
    dao.update(p1);

    StaffRole p2 = getStaffRole();
    p2.getUserIdentifier().setUniqueId(uniqueId_good_2nd);
    String[] roles = {"bcm_admin"};
    p2.setRoles(roles);
    dao.update(p2);
  }

  @AfterClass
  public static void tearDown() {
    mongoClient.getDB("courier").getCollection("staffRole").drop();
    MongoDBConnection.close();
  }

  private StaffRole getStaffRole() {
    gov.va.mobilehealth.courier.core.UserIdentifier userIdentifier = new gov.va.mobilehealth.courier.core.UserIdentifier();
    userIdentifier.setUniqueId(uniqueId_good);
    userIdentifier.setAssigningAuthority(assigningAuthority);

    StaffRole staffRole = new StaffRole();
    String[] roles = {"admin"};
    staffRole.setRoles(roles);

    staffRole.setUserIdentifier(userIdentifier);
    ;

    return staffRole;
  }

  protected boolean supportsConfigAttribute(String config, AccessDecisionVoter<FilterInvocation> decisionVoter) {
    AccessDecisionVoter<FilterInvocation> voter = decisionVoter;
    ConfigAttribute configAttribute = new SecurityConfig(config);
    return voter.supports(configAttribute);
  }

  @Test
  public void supportsConfigAttributeTest() {
    ClinicalAccessRoleVoter voter = new ClinicalAccessRoleVoter();

    ConfigAttribute configAttribute = new SecurityConfig("USER_ADMIN");
    assertThat(voter.supports(configAttribute)).isEqualTo(true);

    configAttribute = new SecurityConfig("USER_BCM_ADMIN");
    assertThat(voter.supports(configAttribute)).isEqualTo(true);

    configAttribute = new SecurityConfig("IS_AUTHENTICATED_ANONYMOUSLY");
    assertThat(voter.supports(configAttribute)).isEqualTo(false);
  }

  /**
   * Test revealed scenario where a null attribute was passed to the voter during deployment.  Ensure that scenario
   * does not throw an exception, forcing deploy to abort
   */
  @Test
  public void supportsConfigWithNullAttribute() {
    ClinicalAccessRoleVoter voter = new ClinicalAccessRoleVoter();
    ConfigAttribute configAttribute;
    configAttribute = createAttribute(null);
    assertThat(voter.supports(configAttribute)).isEqualTo(false);
  }

  @Test
  public void testAccessAdmin() throws Exception {
    ClinicalAccessRoleVoter voter = new ClinicalAccessRoleVoter();
    Authentication userAuthentication = createUserAuthentication(uniqueId_good);

    Collection<ConfigAttribute> attributes = createAttributes(ClinicalAccessAttribute);

    int result = voter.vote(userAuthentication, null, attributes);

    assertThat(result).isEqualTo(AccessDecisionVoter.ACCESS_GRANTED);

    attributes = createAttributes("USER_BCM_ADMIN");

    result = voter.vote(userAuthentication, null, attributes);

    assertThat(result).isEqualTo(AccessDecisionVoter.ACCESS_DENIED);
  }

  @Test
  public void testAccessBCMAdmin() throws Exception {
    ClinicalAccessRoleVoter voter = new ClinicalAccessRoleVoter();
    Authentication userAuthentication = createUserAuthentication(uniqueId_good_2nd);

    Collection<ConfigAttribute> attributes = createAttributes(ClinicalAccessAttribute);

    int result = voter.vote(userAuthentication, null, attributes);

    assertThat(result).isEqualTo(AccessDecisionVoter.ACCESS_DENIED);

    attributes = createAttributes("USER_BCM_ADMIN");

    result = voter.vote(userAuthentication, null, attributes);

    assertThat(result).isEqualTo(AccessDecisionVoter.ACCESS_GRANTED);
  }

  @Test
  public void testAccessDenied() throws Exception {
    ClinicalAccessRoleVoter voter = new ClinicalAccessRoleVoter();
    Authentication userAuthentication = createUserAuthentication(uniqueId_bad);

    Collection<ConfigAttribute> attributes = createAttributes(ClinicalAccessAttribute);

    int result = voter.vote(userAuthentication, null, attributes);

    assertThat(result).isEqualTo(AccessDecisionVoter.ACCESS_DENIED);

    attributes = createAttributes("USER_BCM_ADMIN");

    result = voter.vote(userAuthentication, null, attributes);

    assertThat(result).isEqualTo(AccessDecisionVoter.ACCESS_DENIED);
  }

  @Test
  public void testAccessAbstainedWhenUnsupportedAttributesAreSpecified() throws Exception {
    ClinicalAccessRoleVoter voter = new ClinicalAccessRoleVoter();
    Authentication userAuthentication = createUserAuthentication(uniqueId_bad);

    Collection<ConfigAttribute> attributes = createAttributes("ROLE_STAFF");

    int result = voter.vote(userAuthentication, null, attributes);

    assertThat(result).isEqualTo(AccessDecisionVoter.ACCESS_ABSTAIN);
  }

  private Authentication createUserAuthentication(String uniqueId) {
    List<GrantedAuthority> authorities = createPatientAuthorities();

    AppUser principal = createPrincipal(userName, new PatientIdentifier(assigningAuthority, uniqueId), authorities);
    setPermittedPatient(principal, assigningAuthority, uniqueId);

    Object credentials = null;

    Authentication authentication = createAuthentication(authorities, principal, credentials);
    return authentication;
  }

  private Collection<ConfigAttribute> createAttributes(final String role) {
    Collection<ConfigAttribute> attributes = new ArrayList<ConfigAttribute>();
    ConfigAttribute attribute = createAttribute(role);
    attributes.add(attribute);
    return attributes;
  }

  private ConfigAttribute createAttribute(final String role) {
    return new ConfigAttribute() {
      private static final long serialVersionUID = 9208629368448580596L;

      @Override
      public String getAttribute() {
        return role;
      }
    };
  }

  private void setPermittedPatient(AppUser principal, String defaultAssigningAuthority1, String targetPatientId) {
    Patient advocatePatientTarget = new Patient();
    PatientIdentifier patientIdentifier = new PatientIdentifier();
    patientIdentifier.setAssigningAuthority(defaultAssigningAuthority1);
    patientIdentifier.setUniqueId(targetPatientId);
    advocatePatientTarget.setPatientIdentifier(patientIdentifier);
    principal.getMhpUser().setPatient(advocatePatientTarget);
  }

  protected List<GrantedAuthority> createPatientAuthorities() {
    return createAuthorities(Roles.ROLE_STAFF);
  }

  protected List<GrantedAuthority> createAuthorities(String role) {
    List<GrantedAuthority> authorities = new ArrayList<GrantedAuthority>();
    authorities.add(new SimpleGrantedAuthority(role));
    return authorities;
  }

  protected AppUser createPrincipal(String userName, PatientIdentifier patientIdentifier, List<GrantedAuthority> authorities) {
    String password = "";
    MhpUser authenticatedUser = new MhpUser();
    authenticatedUser.setUserIdentifier(patientIdentifier);
    authenticatedUser.setRightOfAccessAccepted(true);

    AppUser principal = new AppUser(userName, password, authorities, authenticatedUser);
    return principal;
  }

  protected UsernamePasswordAuthenticationToken createAuthentication(List<GrantedAuthority> authorities, AppUser principal, Object credentials) {
    return new UsernamePasswordAuthenticationToken(principal, credentials, authorities);
  }
}
