package gov.va.mobilehealth.courier.bcm.appointments;

import com.agilex.healthcare.mobilehealthplatform.domain.Appointments;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import gov.va.mobilehealth.courier.bcm.appointments.datefilter.DateFilterFactory;
import gov.va.mobilehealth.courier.bcm.appointments.db.AssessmentProfileFutureAppointmentsDAO;
import gov.va.mobilehealth.courier.bcm.appointments.scheduler.SendAppointmentReminderJobCreator;
import gov.va.mobilehealth.courier.db.DAOFactory;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;


/**
 * Coordinator of various activies:
 * Retrieving future appointments from CDS
 * Mapping to Internal Structures
 * Saving to db
 * creating appointment reminder messages
 * Created by clarkee on 10/27/14.
 */
public class AppointmentReminderService {

  /**
   * Coordinating several components to perform the single purpose of this class, to generate
   * appointment reminders based upon the current schedule
   * @param edipiIdentifier
   * @return
   */
  public List<AppointmentReminder> queryAppointmentsAndCreateReminders(EdipiIdentifier edipiIdentifier) throws IOException {
    Appointments cdsAppointments = getFutureAppointments(edipiIdentifier);
    List<AssessmentProfileFutureAppointment> futureAppointments = mapAppointments(edipiIdentifier, cdsAppointments);
    refreshPatientFutureAppointments(edipiIdentifier, futureAppointments);
    List<AppointmentReminder> reminders = createReminders(futureAppointments);

    //Perform scheduling of the reminders
    SendAppointmentReminderJobCreator creator = new SendAppointmentReminderJobCreator();
    creator.createReminderJobs(reminders, edipiIdentifier);

    return reminders;
  }


  public void removeExistingReminders(EdipiIdentifier edipiIdentifier){
    SendAppointmentReminderJobCreator creator = new SendAppointmentReminderJobCreator();
    creator.cleanJobsForPatient(edipiIdentifier);
  }

  /**
   * Retrieve the future appointments for a patient
   * @param edipi
   * @return
   */
  private Appointments getFutureAppointments(EdipiIdentifier edipi) {

    final AppointmentsService appointmentsService = new AppointmentsService();

    //TODO: Need to make the days configurable
    final DateFilter dateFilter = DateFilterFactory.createDaysFromNowFilter(6);

    return appointmentsService.getFutureAppointments(edipi, dateFilter);
  }


  /**
   * Convert CDSLib Appointments to Assessment Profile Appointments
   * @param edipiIdentifier
   * @param appointments
   * @return
   */
  private List<AssessmentProfileFutureAppointment> mapAppointments(EdipiIdentifier edipiIdentifier, Appointments appointments) {
    FutureAppointmentMapper mapper = new FutureAppointmentMapper();
    return mapper.mapAppointments(edipiIdentifier, appointments);
  }

  /**
   * Reset the assessment profile with the latest Future Appointments for the given patient
   * @param edipiIdentifier
   * @param futureAppointments
   * @throws IOException
   */
  private  void refreshPatientFutureAppointments(EdipiIdentifier edipiIdentifier, List<AssessmentProfileFutureAppointment> futureAppointments) throws IOException {
    AssessmentProfileFutureAppointmentsDAO dao = DAOFactory.createAssessmentProfileFutureAppointmentsDAO();
    dao.removeFromDB(edipiIdentifier);
    for (AssessmentProfileFutureAppointment futureAppointment : futureAppointments) {
      dao.saveOrUpdateAppointment(futureAppointment);
    }
  }

  /**
   * create appointment reminders for the given appointments
   * @param appointments
   * @return
   */
  private List<AppointmentReminder> createReminders(List<AssessmentProfileFutureAppointment> appointments) {
    AppointmentReminderGenerator generator = new AppointmentReminderGenerator();
    return generator.createAppointmentReminders(appointments);
  }
}
