package gov.va.mobilehealth.courier.bcm.appointments.db;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Optional;
import com.google.common.base.Preconditions;
import com.mongodb.*;
import com.mongodb.util.JSON;
import gov.va.mobilehealth.courier.bcm.appointments.AssessmentProfileFutureAppointment;
import gov.va.mobilehealth.courier.bcm.appointments.EdipiIdentifier;
import gov.va.mobilehealth.courier.bcm.appointments.IcnIdentifier;
import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import org.bson.types.ObjectId;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class AssessmentProfileFutureAppointmentsDAO {
  private static final ObjectMapper MAPPER = MongoDocumentBase
    .newCustomObjectMapper();

  protected final DBCollection collection;
  public final static String AppointmentsCollectionName = "assessmentProfile.futureAppointments";

  public AssessmentProfileFutureAppointmentsDAO() {

    this.collection = MongoDBConnection.getDB().getCollection(
      AppointmentsCollectionName);
  }

  public Optional<AssessmentProfileFutureAppointment> findById(String id) throws IOException {
    DBObject dbObject = collection.findOne(new BasicDBObject("_id",
      new ObjectId(id)));

    Preconditions
      .checkNotNull(dbObject, "Message ID does not exist in DB");

    return Optional.fromNullable(MAPPER.readValue(dbObject.toString(),
      AssessmentProfileFutureAppointment.class));
  }

  public Optional<AssessmentProfileFutureAppointment> update(AssessmentProfileFutureAppointment appointment) throws IOException {
    BasicDBObject dbObject = MAPPER.readValue(
      MAPPER.writeValueAsString(appointment), BasicDBObject.class);
    if (appointment.getId() != null) {
      dbObject.put("_id", new ObjectId(appointment.getId()));
    }
    collection.save(dbObject);

    return findById(appointment.getId());
  }

  public List<AssessmentProfileFutureAppointment> findAll() throws IOException {
    List<AssessmentProfileFutureAppointment> appointments = new ArrayList<AssessmentProfileFutureAppointment>();
    DBCursor cursor = collection.find();
    try {
      while (cursor.hasNext()) {
        appointments.add(MAPPER.readValue(cursor.next().toString(),
          AssessmentProfileFutureAppointment.class));
      }
    } finally {
      cursor.close();
    }

    return appointments;
  }

  public AssessmentProfileFutureAppointment saveOrUpdateAppointment(AssessmentProfileFutureAppointment appointment) throws IOException {
    ObjectId oId;

    DBObject dbObject = (DBObject) JSON.parse(MAPPER
      .writeValueAsString(appointment));
    if (appointment.getId() != null) {
      oId = new ObjectId(appointment.getId());
      dbObject.put("_id", oId);
      BasicDBObject searchQuery = new BasicDBObject().append("_id", oId);
      collection.update(searchQuery, dbObject);
    } else {
      collection.insert(dbObject);
      oId = (ObjectId) dbObject.get("_id");
      appointment.setId(oId.toString());

    }
    return appointment;

  }

  public int removeAssessmentProfileFutureAppointment(AssessmentProfileFutureAppointment appointment) throws IOException {

    if (null == appointment || null == appointment.getId()) {
      return -1;
    }
    ObjectId oId = new ObjectId(appointment.getId());
    BasicDBObject searchQuery = new BasicDBObject().append("_id", oId);

    return removeByQuery(searchQuery);
  }

  public int removeFromDB(String id) throws IOException {
    BasicDBObject searchQuery = new BasicDBObject("_id", new ObjectId(id));
    return removeByQuery(searchQuery);
  }

  public int removeFromDB(EdipiIdentifier edipiIdentifier) {
    BasicDBObject query = createSearchByEdipiQuery(edipiIdentifier);
    return removeByQuery(query);
  }

  private int removeByQuery(BasicDBObject searchQuery) {
    WriteResult wr = collection.remove(searchQuery);
    return wr.getN();
  }

  public List<AssessmentProfileFutureAppointment> findByPatient(IcnIdentifier icnIdentifier) throws IOException {
    BasicDBObject query = new BasicDBObject();
    query.append(AssessmentProfileFutureAppointment.PatientIcnIdentifierFieldName + ".icnId", icnIdentifier.getIcnId());


    return findByQuery(query);
  }

  public List<AssessmentProfileFutureAppointment> findByPatient(EdipiIdentifier edipiIdentifier) throws IOException {
    BasicDBObject query = createSearchByEdipiQuery(edipiIdentifier);
    return findByQuery(query);
  }

  private BasicDBObject createSearchByEdipiQuery(EdipiIdentifier edipiIdentifier) {
    BasicDBObject query = new BasicDBObject();
    query.append(AssessmentProfileFutureAppointment.PatientEdipiIdentifierFieldName + ".edipiId", edipiIdentifier.getEdipiId());
    return query;
  }

  public List<AssessmentProfileFutureAppointment> findByQuery(BasicDBObject query) throws IOException {

    List<AssessmentProfileFutureAppointment> appointments = new ArrayList<AssessmentProfileFutureAppointment>();
    DBCursor cursor = collection.find(query);
    cursor.sort(new BasicDBObject("createdDateUTC", -1));
    try {
      while (cursor.hasNext()) {
        appointments.add(MAPPER.readValue(cursor.next().toString(), AssessmentProfileFutureAppointment.class));
      }
    } finally {
      cursor.close();
    }

    return appointments;
  }
}
