package gov.va.mobilehealth.courier.db;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Optional;
import com.google.common.base.Preconditions;
import com.mongodb.*;
import com.mongodb.util.JSON;

import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import gov.va.mobilehealth.courier.core.protocol.BroadcastFacilityMessage;
import gov.va.mobilehealth.courier.core.protocol.PatientProtocol;
import gov.va.mobilehealth.courier.core.protocol.Protocol;
import gov.va.mobilehealth.courier.scheduler.BroadcastMessageScheduler;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;

import org.bson.types.ObjectId;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class BroadcastFacilityMessageDAO {
  private static final ObjectMapper MAPPER = MongoDocumentBase
    .newCustomObjectMapper();

  protected final DBCollection collection;
  public final static String FacilityMessagesCollectionName = "facilityMessages";
  private static final Logger LOGGER = LoggerFactory.getLogger(BroadcastFacilityMessageDAO.class);

  public BroadcastFacilityMessageDAO() {

    this.collection = MongoDBConnection.getDB().getCollection(
      FacilityMessagesCollectionName);
  }

  public Optional<BroadcastFacilityMessage> findById(String id) throws IOException {
    DBObject dbObject = collection.findOne(new BasicDBObject("_id",
      new ObjectId(id)));

    Preconditions
      .checkNotNull(dbObject, "Message ID does not exist in DB");

    return Optional.fromNullable(MAPPER.readValue(dbObject.toString(),
      BroadcastFacilityMessage.class));
  }

  public Optional<BroadcastFacilityMessage> update(BroadcastFacilityMessage message) throws IOException {
    BasicDBObject dbObject = MAPPER.readValue(
      MAPPER.writeValueAsString(message), BasicDBObject.class);
    if (message.getId() != null) {
      dbObject.put("_id", new ObjectId(message.getId()));
    }
    collection.save(dbObject);

    return findById(message.getId());
  }

  public List<BroadcastFacilityMessage> findAll() throws IOException {
    List<BroadcastFacilityMessage> messages = new ArrayList<BroadcastFacilityMessage>();
    DBCursor cursor = collection.find();
    try {
      while (cursor.hasNext()) {
        messages.add(MAPPER.readValue(cursor.next().toString(),
          BroadcastFacilityMessage.class));
      }
    } finally {
      cursor.close();
    }

    return messages;
  }

  public int removeBroadcastFacilityMessage(BroadcastFacilityMessage facilityMessage) throws IOException {

    if (null == facilityMessage || null == facilityMessage.getId()) {
      return -1;
    }
    ObjectId oId = new ObjectId(facilityMessage.getId());
    BasicDBObject searchQuery = new BasicDBObject().append("_id", oId);

    WriteResult wr = collection.remove(searchQuery);
    return wr.getN();
  }

  public BroadcastFacilityMessage saveOrUpdateFacilityMessage(BroadcastFacilityMessage message) throws IOException {
    ObjectId oId;

    DBObject dbObject = (DBObject) JSON.parse(MAPPER
      .writeValueAsString(message));
    if (message.getId() != null) {
      oId = new ObjectId(message.getId());
      dbObject.put("_id", oId);
      BasicDBObject searchQuery = new BasicDBObject().append("_id", oId);
      collection.update(searchQuery, dbObject);
    } else {
      collection.insert(dbObject);
      oId = (ObjectId) dbObject.get("_id");
      message.setId(oId.toString());

    }
    return message;

  }

  public void removeFromDB(String id) throws IOException {
    BasicDBObject searchQuery = new BasicDBObject("_id", new ObjectId(id));
    collection.remove(searchQuery);
  }

  public List<BroadcastFacilityMessage> findByAssigningAuthority(String assigningAuthority) throws IOException {
    List<BroadcastFacilityMessage> messages = new ArrayList<BroadcastFacilityMessage>();
    DBCursor cursor = collection.find(new BasicDBObject(BroadcastFacilityMessage.assigningAuthorityField, assigningAuthority));
    cursor.sort(new BasicDBObject("createdDateUTC", -1));
    try {
      while (cursor.hasNext()) {
        messages.add(MAPPER.readValue(cursor.next().toString(), BroadcastFacilityMessage.class));
      }
    } finally {
      cursor.close();
    }

    return messages;
  }

  public void updateMessageProgress(String id, String progress) throws IOException {
      Preconditions.checkNotNull(id, "message ID cannot be null");

      BasicDBObject updateQuery = new BasicDBObject();
      updateQuery.append("$set",
          new BasicDBObject().append("progress", progress));

      BasicDBObject searchQuery = new BasicDBObject();
      searchQuery.append("_id", new ObjectId(id));

      collection.update(searchQuery, updateQuery, false, false);
  }

}
