package gov.va.mobilehealth.courier.db;

import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Optional;
import com.mongodb.BasicDBObject;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.util.JSON;

import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.UserIdentifier;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;

import org.bson.types.ObjectId;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

public class PatientDAO {
  public static final int ALL = 0;
  public static final int NAME = 1;
  public static final int SSN = 2;
  public static final int MOBILE = 3;
  public static final int L4SSN = 4;
  public static final int NUM_OF_NAME_PARAMS = 3;

  private static final ObjectMapper MAPPER = MongoDocumentBase.newCustomObjectMapper();
  private static final Logger LOGGER = LoggerFactory.getLogger(PatientDAO.class);
  protected final DBCollection collection;

  public PatientDAO() {

    this.collection = MongoDBConnection.getDB().getCollection("patients");
  }

  public Optional<Patient> findById(String id) throws IOException {
    BasicDBObject whereQuery = new BasicDBObject("_id", new ObjectId(id));
    Patient p = findOnebyQuery(whereQuery);

    return Optional.fromNullable(p);
  }

  public List<Patient> findAll() throws IOException {
    return findListbyQuery(null);
  }

  public Optional<Patient> findByMDN(String mdn) throws IOException {
    BasicDBObject whereQuery = new BasicDBObject("preference.phoneNumber", mdn);
    Patient p = findOnebyQuery(whereQuery);

    return Optional.fromNullable(p);
  }


  public Optional<Patient> findByPatientIdentifier(UserIdentifier patientIdentifier) throws IOException {
    return findByPatientIdentifier(patientIdentifier.getAssigningAuthority(), patientIdentifier.getUniqueId());
  }

  public Optional<Patient> findByPatientIdentifier(PatientIdentifier patientIdentifier) throws IOException {
    return findByPatientIdentifier(patientIdentifier.getAssigningAuthority(), patientIdentifier.getUniqueId());
  }

  private Optional<Patient> findByPatientIdentifier(String assigningAuthority, String uniqueId) throws IOException {
    BasicDBObject whereQuery = new BasicDBObject();
    whereQuery.append("patientIdentifier.uniqueId", uniqueId);
    whereQuery.append("patientIdentifier.assigningAuthority", assigningAuthority);
    //whereQuery.put("patientIdentifier", patientIdentifier);
    Patient p = findOnebyQuery(whereQuery);

    return Optional.fromNullable(p);
  }

  public Patient update(Patient patient) throws IOException {
    ObjectId oId;
    // only calculate if it is a newly registered user
    // since we don't allow change in last name and ssn
    if (patient.getId() == null) {
        patient.setLast4SSNFromSSN(patient.getSsn());
        patient.setLastNameInitialFromLastName(patient.getLastName());
    }

    DBObject dbObject = (DBObject) JSON.parse(MAPPER.writeValueAsString(patient));
    if (patient.getId() != null) {
      oId = new ObjectId(patient.getId());
      dbObject.put("_id", oId);
      BasicDBObject searchQuery = new BasicDBObject().append("_id", oId);

      collection.update(searchQuery, dbObject);
    } else {
      collection.insert(dbObject);
      oId = (ObjectId) dbObject.get("_id");
      patient.setId(oId.toString());
    }
    return patient;
  }

  public List<Patient> findByName(String lastname, String firstname, String middlename) throws IOException {
    BasicDBObject whereQuery = new BasicDBObject();

    if (lastname != null && !lastname.isEmpty()) {
      whereQuery.put("lastName", new BasicDBObject("$regex", lastname).append("$options", "i"));
    }
    if (firstname != null && !firstname.isEmpty()) {
      whereQuery.put("firstName", new BasicDBObject("$regex", firstname).append("$options", "i"));
    }
    if (middlename != null && !middlename.isEmpty()) {
      whereQuery.put("middleName", new BasicDBObject("$regex", middlename).append("$options", "i"));
    }
    return findListbyQuery(whereQuery);

  }

  public List<Patient> findBroadcastPatientByFacility(String facility) throws IOException {
      BasicDBObject whereQuery = new BasicDBObject();
      whereQuery.put("facilityId", facility);
      whereQuery.append("preference.receiveFacilityMessage", true);
      return findListbyQuery(whereQuery);
  }

  public List<Patient> findAppointmentReminderPatients() throws IOException {
    BasicDBObject whereQuery = new BasicDBObject();
    whereQuery.append("preference.receiveApptReminder", true);
    return findListbyQuery(whereQuery);

  }

  public List<Patient> findBySsn(String ssn) throws IOException {
    BasicDBObject whereQuery = new BasicDBObject();
    whereQuery.put("ssn", ssn);
    return findListbyQuery(whereQuery);
  }

  public List<Patient> findByMobile(String mobile) throws IOException {
    BasicDBObject whereQuery = new BasicDBObject();
    whereQuery.put("preference.phoneNumber", mobile);
    return findListbyQuery(whereQuery);

  }

  public List<Patient> findByLastnameInit4Ssn(String last4SSN, String lastNameInitial) throws IOException {
    BasicDBObject whereQuery = new BasicDBObject();
    BasicDBObject keysForIndex = new BasicDBObject();

    if (last4SSN != null && !last4SSN.isEmpty()) {
      whereQuery.put("last4SSN", last4SSN);
    }
    if (lastNameInitial != null && !lastNameInitial.isEmpty()) {
      whereQuery.put("lastNameInitial", lastNameInitial);
    }

    keysForIndex.put("last4SSN", 1);
    keysForIndex.put("lastNameInitial", 1);
    collection.ensureIndex(keysForIndex);

    return findListbyQuery(whereQuery);
  }

  public List<Patient> findListbyQuery(BasicDBObject query) throws IOException {
    List<Patient> patients = new ArrayList<Patient>();
    DBCursor cursor = collection.find(query);
    Patient p;
    try {
      while (cursor.hasNext()) {
        p = MAPPER.readValue(cursor.next().toString(), Patient.class);
        p.getAgeFromDoB();
        patients.add(p);
      }
    } finally {
      cursor.close();
    }

    return patients;
  }

  public Patient findOnebyQuery(BasicDBObject query) throws IOException {
    Patient patient = null;

    DBObject dbObject = collection.findOne(query);

    if (dbObject != null) {
      patient = MAPPER.readValue(dbObject.toString(), Patient.class);
    }

    return patient;
  }

  public void removeFromDB(String id) throws IOException {
    BasicDBObject searchQuery = new BasicDBObject("_id", new ObjectId(id));
    collection.remove(searchQuery);
  }

}
