package gov.va.mobilehealth.courier.db;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Optional;
import com.google.common.base.Preconditions;
import com.mongodb.*;
import com.mongodb.util.JSON;
import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import gov.va.mobilehealth.courier.core.protocol.*;
import gov.va.mobilehealth.courier.utils.DateParser;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;
import org.bson.types.ObjectId;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

public class PatientProtocolDAO {
  private static final ObjectMapper MAPPER = MongoDocumentBase.newCustomObjectMapper();

  protected final DBCollection collection;

  public PatientProtocolDAO() {
    this.collection = MongoDBConnection.getDB().getCollection("patientProtocol");
  }

  public Optional<PatientProtocol> findById(String id) throws IOException {
    DBObject dbObject = collection.findOne(new BasicDBObject("_id", new ObjectId(id)));

    Preconditions.checkNotNull(dbObject, "patient protocol ID does not exist in DB");

    return Optional.fromNullable(MAPPER.readValue(dbObject.toString(), PatientProtocol.class));
  }

  public List<PatientProtocol> findByPatientId(String patientId) throws IOException {
    List<PatientProtocol> patientProtocol = new ArrayList<PatientProtocol>();
    DBCursor cursor = collection.find(new BasicDBObject("patientId", patientId));
    try {
      while (cursor.hasNext()) {
        patientProtocol.add(MAPPER.readValue(cursor.next().toString(), PatientProtocol.class));
      }
    } finally {
      cursor.close();
    }

    return patientProtocol;
  }

  public List<PatientProtocol> findByPatientIdAndStatus(String patientId, PatientProtocolStatus patientProtocolStatus) throws IOException {
    List<PatientProtocol> patientProtocol = new ArrayList<PatientProtocol>();
    BasicDBObject query = new BasicDBObject("patientId", patientId).
      append("status", patientProtocolStatus.toString());
    DBCursor cursor = collection.find(query);

    try {
      while (cursor.hasNext()) {
        patientProtocol.add(MAPPER.readValue(cursor.next().toString(), PatientProtocol.class));
      }
    } finally {
      cursor.close();
    }

    return patientProtocol;
  }

  /**
   * Query patientProtocol collection to get all the patient protocols with a given statuses
   * Example of the generated query:
   * db.patientProtocol.find({"patientId" : "52b9f1519d16ccbfab2fcff5", "status": {"$in":["Active","Paused","Expired","Removed"]}})
   *
   * @param patientId
   * @param patientProtocolStatuses
   * @return
   * @throws IOException
   */
  public List<PatientProtocol> findByPatientIdAndStatus(String patientId, List<PatientProtocolStatus> patientProtocolStatuses) throws IOException {
    List<PatientProtocol> patientProtocols = new ArrayList<PatientProtocol>();

    BasicDBList docStatuses = new BasicDBList();

    for(PatientProtocolStatus patientProtocolStatus : patientProtocolStatuses){
      docStatuses.add(patientProtocolStatus.toString());
    }

    DBObject inClause = new BasicDBObject("$in", docStatuses);

    BasicDBObject query = new BasicDBObject("status", inClause);
    query.append("patientId", patientId);

    DBCursor cursor = collection.find(query);

    try {
      while (cursor.hasNext()) {
        patientProtocols.add(MAPPER.readValue(cursor.next().toString(), PatientProtocol.class));
      }
    } finally {
      cursor.close();
    }

    return patientProtocols;
  }


  public List<PatientProtocol> findByPatientIdAndStatus(String patientId, PatientProtocolStatus patientProtocolStatus1,PatientProtocolStatus patientProtocolStatus2) throws IOException {
    List<PatientProtocol> patientProtocol = new ArrayList<PatientProtocol>();

    BasicDBObject query1 = new BasicDBObject("patientId", patientId).
            append("status", patientProtocolStatus1.toString());

    BasicDBObject query2 = new BasicDBObject("patientId", patientId).
            append("status", patientProtocolStatus2.toString());

    BasicDBList conditionalOperator = new BasicDBList();
    conditionalOperator.add(query1);
    conditionalOperator.add(query2);
    DBObject query = new BasicDBObject("$or", conditionalOperator);

    DBCursor cursor = collection.find(query);

    try {
      while (cursor.hasNext()) {
        patientProtocol.add(MAPPER.readValue(cursor.next().toString(), PatientProtocol.class));
      }
    } finally {
      cursor.close();
    }

    return patientProtocol;
  }

  public String insert(PatientProtocol pProtocol) throws IOException {
    DBObject dbObject = (DBObject) JSON.parse(MAPPER.writeValueAsString(pProtocol));

    collection.insert(dbObject);
    ObjectId oId = (ObjectId) dbObject.get("_id");
    pProtocol.setId(oId.toString());
    return pProtocol.getId();
  }

  public String save(PatientProtocol pProtocol) throws IOException {
    DBObject dbObject = (DBObject) JSON.parse(MAPPER.writeValueAsString(pProtocol));
    if (pProtocol.getId() != null) {
      dbObject.put("_id", new ObjectId(pProtocol.getId()));
    }
    collection.save(dbObject);
    if (pProtocol.getId() == null) {
      ObjectId oId = (ObjectId) dbObject.get("_id");
      pProtocol.setId(oId.toString());
    }
    return pProtocol.getId();
  }

  // TODO: differentiate update from save
  public void update(PatientProtocol pProtocol) throws IOException {
    ObjectId oId;

    DBObject dbObject = (DBObject) JSON.parse(MAPPER.writeValueAsString(pProtocol));
    if (pProtocol.getId() != null) {
      oId = new ObjectId(pProtocol.getId());
      dbObject.put("_id", oId);
      BasicDBObject searchQuery = new BasicDBObject().append("_id", oId);

      collection.update(searchQuery, dbObject);
    } else {
      collection.insert(dbObject);
      oId = (ObjectId) dbObject.get("_id");
      pProtocol.setId(oId.toString());
    }
  }

  public void delete(String protocolId, Comment comment) throws IOException {
    Preconditions.checkNotNull(protocolId, "protocol ID cannot be null");
    Preconditions.checkNotNull(comment, "comment cannot be null");

    Optional<PatientProtocol> p = findById(protocolId);
    if (!p.isPresent()) {
      throw new NullPointerException();
    }

    PatientProtocol patientProtocol = p.get();
    patientProtocol.setStatus(PatientProtocolStatus.REMOVED);
    patientProtocol.setAllTemplateStatus(PatientProtocolStatus.REMOVED);
    patientProtocol.setLastEditedDate(DateParser.getTodaysDate(null, null));
    patientProtocol.addComments(comment);

    save(patientProtocol);
  }

  public void removeFromDB(String protocolId) throws IOException {

    BasicDBObject searchQuery = new BasicDBObject("_id", new ObjectId(protocolId));

    collection.remove(searchQuery);
  }

  public void updateStatus(String protocolId, PatientProtocolStatus status) throws IOException {
    Preconditions.checkNotNull(protocolId, "protocol ID cannot be null");
    Preconditions.checkNotNull(status, "status cannot be null");

    BasicDBObject searchQuery = new BasicDBObject("_id", new ObjectId(protocolId));

    BasicDBObject newDocument = new BasicDBObject("$set", new BasicDBObject()
      .append("status", status.toString())
      .append("lastEditedDate", DateParser.getTodaysDate(null, null)));

    collection.update(searchQuery, newDocument);
  }

  public List<PatientProtocol> findAll() throws IOException {
    List<PatientProtocol> patientProtocol = new ArrayList<PatientProtocol>();
    DBCursor cursor = collection.find();
    try {
      while (cursor.hasNext()) {
        patientProtocol.add(MAPPER.readValue(cursor.next().toString(), PatientProtocol.class));
      }
    } finally {
      cursor.close();
    }

    return patientProtocol;
  }

  public List<PatientProtocol> findByPatientIdAndProtocolName(String patientId, String protocolName) throws IOException {
    List<PatientProtocol> patientProtocol = new ArrayList<PatientProtocol>();

    String pattern = "^" + protocolName + "$";
    DBCursor cursor = collection.find(new BasicDBObject("patientId", patientId)
      .append("protocol.name", Pattern.compile(pattern, Pattern.CASE_INSENSITIVE)));

    try {
      while (cursor.hasNext()) {
        patientProtocol.add(MAPPER.readValue(cursor.next().toString(), PatientProtocol.class));
      }
    } finally {
      cursor.close();
    }

    return patientProtocol;
  }

  /**
   * Sets the protocol status to expired and add audit trail
   */
  public void expireProtocol(final String protocolId, final int templateIndex) throws IOException {
    Preconditions.checkNotNull(protocolId, "protocol ID cannot be null");

    Optional<PatientProtocol> p = findById(protocolId);
    if (!p.isPresent()) {
      throw new NullPointerException();
    }

    PatientProtocol patientProtocol = p.get();
    String date = DateParser.getTodaysDate(null, null);
    String time = DateParser.getCurrentTime(null, null);

    Template template = patientProtocol.getProtocol().getTemplates().get(templateIndex);

    List<ChangeEntry> foundSoFar = new ArrayList<ChangeEntry>();
    foundSoFar.add(new ChangeEntry("status",
      template.getStatus() != null ? template.getStatus().toString() : "",
      PatientProtocolStatus.EXPIRED.toString(), template.getName()));
    Comment c = new Comment();
    c.setChanges(foundSoFar);
    c.setDate(date);
    c.setTime(time);
    c.setUser("System");
    patientProtocol.addComments(c);

    template.setStatus(PatientProtocolStatus.EXPIRED);
    boolean allExpired = true;
    for (Template t : patientProtocol.getProtocol().getTemplates()) {
      if (t.getStatus().equals(PatientProtocolStatus.ACTIVE)) {
        allExpired = false;
        break;
      }
    }
    if (allExpired) {
      foundSoFar.add(new ChangeEntry("status",
        patientProtocol.getStatus() != null ? patientProtocol.getStatus().toString() : "",
        PatientProtocolStatus.EXPIRED.toString(), ""));
      patientProtocol.setStatus(PatientProtocolStatus.EXPIRED);
    }
    patientProtocol.setLastEditedDate(date);

    save(patientProtocol);

  }

}
