package gov.va.mobilehealth.courier.db;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.mongodb.*;
import com.mongodb.util.JSON;
import gov.va.mobilehealth.cms.jackson.Jackson;
import gov.va.mobilehealth.courier.core.MongoDocumentBase;
import gov.va.mobilehealth.courier.core.PatientResponseServiceData;
import gov.va.mobilehealth.courier.core.PatientServiceReading;
import gov.va.mobilehealth.courier.utils.MongoDBConnection;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class PatientResponseServiceDataDAO {
  private static final ObjectMapper MAPPER = MongoDocumentBase.newCustomObjectMapper();
  protected final DBCollection collection;

  public PatientResponseServiceDataDAO() {
    this.collection = MongoDBConnection.getDB().getCollection("patientResponseServiceData");
  }

  public List<PatientResponseServiceData> findByPatientId(String id) throws IOException {

    BasicDBObject whereQuery = new BasicDBObject("patientId", id);
    return findByQuery(whereQuery);
  }

  public List<PatientResponseServiceData> findByPatientIdSortByDate(String id) throws IOException {

    BasicDBObject whereQuery = new BasicDBObject("patientId", id);
    return findByQueryAndSort(whereQuery);
  }

  public List<PatientResponseServiceData> findByPatientIdAndServiceId(String patientId, String serviceId) throws IOException {

    BasicDBObject query = new BasicDBObject("patientId", patientId)
      .append("patientServiceReading.serviceId", serviceId);
    return findByQuery(query);
  }

  public void save(PatientResponseServiceData patientResponseServiceData) throws IOException {
    //use regular mapper here, since we do not want to use the MAPPER between
    // MongoDB objectID and string _id
    ObjectMapper regularMAPPER = Jackson.newObjectMapper();
    DBObject dbObject = (DBObject) JSON.parse(regularMAPPER.writeValueAsString(patientResponseServiceData));
    collection.insert(dbObject);

  }

  public List<PatientServiceReading> findServiceNameByPatientId(String id) throws IOException {
    // create our pipeline operations, first with the $match
    DBObject match = new BasicDBObject("$match", new BasicDBObject("patientId", id));

    // build the $projection operation
    DBObject fields = new BasicDBObject("patientServiceReading.serviceName", 1);
    fields.put("patientServiceReading.serviceId", 1);
    fields.put("_id", 0);
    DBObject project = new BasicDBObject("$project", fields);

    // Now the $group operation
    DBObject groupFields = new BasicDBObject("_id", "$patientServiceReading.serviceName");
    groupFields.put("serviceName", new BasicDBObject("$first", "$patientServiceReading.serviceName"));
    groupFields.put("serviceId", new BasicDBObject("$first", "$patientServiceReading.serviceId"));
    DBObject group = new BasicDBObject("$group", groupFields);

    // build the $projection operation
    DBObject fields2 = new BasicDBObject("serviceName", 1);
    fields2.put("serviceId", 1);
    fields2.put("_id", 0);
    DBObject project2 = new BasicDBObject("$project", fields2);

    // run aggregation
    AggregationOutput output = collection.aggregate(match, group, project2);

    List<PatientServiceReading> patientServices = new ArrayList<PatientServiceReading>();

    Iterator it = output.results().iterator();
    while (it.hasNext()) {
      PatientServiceReading patientService = MAPPER.readValue(it.next().toString(), PatientServiceReading.class);
      patientServices.add(patientService);
    }

    return patientServices;
  }

  protected List<PatientResponseServiceData> findByQuery(BasicDBObject query) throws IOException {
    List<PatientResponseServiceData> patientResponseServiceDataList = new ArrayList<PatientResponseServiceData>();

    PatientResponseServiceData p;
    DBObject dbo;
    DBCursor cursor = collection.find(query);
    try {
      while (cursor.hasNext()) {
        dbo = cursor.next();
        p = MAPPER.readValue(dbo.toString(), PatientResponseServiceData.class);
        patientResponseServiceDataList.add(p);
      }
    } finally {
      cursor.close();
    }

    return patientResponseServiceDataList;
  }

  protected List<PatientResponseServiceData> findByQueryAndSort(BasicDBObject query) throws IOException {
    List<PatientResponseServiceData> patientResponseServiceDataList = new ArrayList<PatientResponseServiceData>();

    PatientResponseServiceData p;
    DBObject dbo;
    DBCursor cursor = collection.find(query);
    cursor.sort(new BasicDBObject("patientResponseReceivedTimeUTC", -1));
    try {
      while (cursor.hasNext()) {
        dbo = cursor.next();
        p = MAPPER.readValue(dbo.toString(), PatientResponseServiceData.class);
        patientResponseServiceDataList.add(p);
      }
    } finally {
      cursor.close();
    }

    return patientResponseServiceDataList;
  }

}
