package gov.va.mobilehealth.courier.protocol;

import com.google.common.base.Optional;
import gov.va.mobilehealth.courier.core.PatientReading;
import gov.va.mobilehealth.courier.core.PatientResponseServiceData;
import gov.va.mobilehealth.courier.core.PatientServiceReading;
import gov.va.mobilehealth.courier.core.protocol.*;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.PatientMessageDAO;
import gov.va.mobilehealth.courier.db.PatientProtocolDAO;
import gov.va.mobilehealth.courier.db.PatientResponseServiceDataDAO;
import gov.va.mobilehealth.courier.scheduler.PatientProtocolScheduler;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.utils.CamelProducerFactory;
import org.apache.camel.ProducerTemplate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class MessageProcessorService {

  public static final String NUMBER_PATTERN = "[0-9]*\\.?[0-9]+";
  public static final String NO_MATCH_MESSAGE = "Thank you. You are not subscribed to any protocols at this time.";
  public static final String SUCCESS_MESSAGE = "Thank you. Your text message was received.";

  private static final Logger LOGGER = LoggerFactory.getLogger(MessageProcessorService.class);
  private ProducerTemplate outgoingMsgProducer;

  public MessageProcessorService(ProducerTemplate outgoingMsgProducer) {
    this.outgoingMsgProducer = outgoingMsgProducer;
  }

  public void messageProcess(PatientProtocolScheduler patientProtocolScheduler, final CMSMessage msg) throws IOException {
    doMessageProcess(patientProtocolScheduler, msg);
  }

  protected void doMessageProcess(final PatientProtocolScheduler patientProtocolScheduler, final CMSMessage msg) throws IOException {

    List<PatientProtocol> protocolList = getPatientActiveProtocols(msg.getPatientId());

    if (protocolList.size() == 0) {
      msg.setText(NO_MATCH_MESSAGE);
    } else {

      Optional<PatientProtocolSearchResult> searchResultOptional = findMatchProtocol(protocolList, msg.getText());

      // send SMS message
      if (!searchResultOptional.isPresent()) {
        msg.setText(FormatValidator.INCORRECT_FORMAT_MESSAGE);
      } else {
        PatientProtocol patientProtocol = searchResultOptional.get().getPatientProtocol();
        int templateIndex = searchResultOptional.get().getTemplateIndex();
        Vital template = (Vital) patientProtocol.getProtocol().getTemplates().get(templateIndex);

        LOGGER.debug("Incoming message: {}.", msg);

        PatientResponseServiceData patientResponseServiceData = buildPatientResponseServiceData(patientProtocol, template, msg);

        LOGGER.debug("patientResponseServiceData: {}.", patientResponseServiceData);

        // IMPORTANT: status field in patientResponseServiceData might be changed during validation
        String sendMessage = validatePatientReading(patientProtocol, template, patientResponseServiceData);
        if (!(FormatValidator.INCORRECT_FORMAT_MESSAGE.equals(sendMessage) || ValidRangeValidator.INVALID_MESSAGE.equals(sendMessage))) {

          // store the patientResponseServiceData
          getPatientResponseServiceDataDAO().save(patientResponseServiceData);

          // if the protocol schedule type is once, expire the protocol and delete all the scheduled jobs
          if ("Once".equalsIgnoreCase(template.getSchedule().getScheduleType().getName())) {

              getPatientProtocolDAO().expireProtocol(patientProtocol.getId(), msg.getTemplateIndex());
              patientProtocolScheduler.delete(patientProtocol);
          }
        }

        msg.setServiceName(patientProtocol.getProtocol().getName());
        msg.setAlertLevel(patientResponseServiceData.getAlertLevel());
        msg.setText(sendMessage);
        msg.setPatientProtocolId(patientProtocol.getId());
        msg.setTemplateIndex(templateIndex);

        // update the status of the patient reading response
        getPatientMessageDAO().updateStatus(msg);

      }
    }

    outgoingMsgProducer.sendBody(msg);
  }

  protected List<PatientProtocol> getPatientActiveProtocols(String patientId) throws IOException {
    return getPatientProtocolDAO().findByPatientIdAndStatus(patientId,
            PatientProtocol.ACTIVE_PATIENT_PROTOCOL_STATUSES);
  }

  protected Optional<PatientProtocolSearchResult> findMatchProtocol(List<PatientProtocol> patientProtocols, String msg) {
    // loop through protocols and do match
    for (PatientProtocol patientProtocol : patientProtocols) {
      List<Template> templates = patientProtocol.getProtocol().getTemplates();
      for (int i = 0; i < templates.size(); i++) {
        // only vital templates count
        Template t = templates.get(i);
        if (t instanceof Vital) {
          Vital template = (Vital) t;
          if (msg.matches(template.getService().getPattern())) {
            return Optional.of(new PatientProtocolSearchResult(patientProtocol, i));
          }
        }
      }
    }
    return Optional.absent();
  }

  /**
   * find the correct message according to the readings send in my patient
   *
   * @param patientProtocol the patient patientProtocol we compare against
   * @param template matched template in patientProtocol
   * @param patientResponseServiceData sms message patient send in
   * @return return message (normal, above/below desirable, above/below critical)
   */
  // IMPORTANT: status field in patientResponseServiceData might be changed during validation
  protected String validatePatientReading(final PatientProtocol patientProtocol, final Vital template, final PatientResponseServiceData patientResponseServiceData) {
    // level is to make sure we send out the message that is more critical if there are
    // more than one readings need to be evaluated.
    // e.g., if we already found out the systolic value is invalid, we will not overwrite
    // the message if the diastolic value is above desirable
    ServiceReadingValidator serviceReadingValidator = buildServiceReadingValidatorChain();

    return serviceReadingValidator.validate(patientResponseServiceData, template);
  }

  protected ServiceReadingValidator buildServiceReadingValidatorChain() {

    ServiceReadingValidator alertValidator = new AlertValidator();

    ServiceReadingValidator criticalValidator = new CriticalValidator();
    criticalValidator.setNext(alertValidator);

    ServiceReadingValidator validRangeValidator = new ValidRangeValidator();
    validRangeValidator.setNext(criticalValidator);

    ServiceReadingValidator formatValidator = new FormatValidator();
    formatValidator.setNext(validRangeValidator);

    return formatValidator;
  }

  protected PatientResponseServiceData buildPatientResponseServiceData(final PatientProtocol protocol, final Vital template, final CMSMessage message) {

    CMSService service = template.getService();
    // find numbers/float numbers in input string
    List<String> patientReadingValues = getReadings(message.getText());

    List<PatientReading> patientReadings = new ArrayList<PatientReading>(patientReadingValues.size());
    int index = 0;
    for (String patientReadingValue : patientReadingValues) {
      Reading reading = service.getReadings().get(index);

      PatientReading patientReading = new PatientReading();
      patientReading.setName(reading.getName());
      patientReading.setUnit(reading.getUnit());
      patientReading.setValue(Float.valueOf(patientReadingValue));

      patientReadings.add(patientReading);
      
      // only increase index if there is required reading in service
      // it is possible that the vet sends in more than required numbers
      // we still need to record that, since in FormatValidator, 
      // it compares the size of this structure (PatientReading) to 
      // the size of template service readings. Need to refactor the logic
      // if there is time
      if (index < service.getReadings().size() - 1) {
          index++;
      }
    }

    PatientServiceReading patientServiceReading = new PatientServiceReading();
    patientServiceReading.setServiceName(service.getName());
    patientServiceReading.setServiceId(service.getServiceId());
    patientServiceReading.setReadings(patientReadings);

    PatientResponseServiceData patientResponseServiceData = new PatientResponseServiceData();
    patientResponseServiceData.setPatientId(protocol.getPatientId());
    patientResponseServiceData.setPatientResponseReceivedTimeUTC(message.getNetworkSubmitTime());
    patientResponseServiceData.setPatientResponseReceivedTime(message.getAdjustedSubmitTime());
    patientResponseServiceData.setTimeZone(message.getTimeZone());
    patientResponseServiceData.setPatientServiceReading(patientServiceReading);
    patientResponseServiceData.setAlertLevel(message.getAlertLevel());

    return patientResponseServiceData;
  }

  protected List<String> getReadings(String text) {
    Pattern pattern = Pattern.compile(NUMBER_PATTERN);
    Matcher matcher = pattern.matcher(text);
    List<String> readings = new ArrayList<String>();
    while (matcher.find()) {
      readings.add(matcher.group());
    }
    return readings;
  }

  protected PatientProtocolDAO getPatientProtocolDAO() {
    return DAOFactory.createPatientProtocolDAO();
  }

  protected PatientMessageDAO getPatientMessageDAO() {
    return DAOFactory.createPatientMessageDAO();
  }

  protected PatientResponseServiceDataDAO getPatientResponseServiceDataDAO() {
    return DAOFactory.createPatientResponseServiceDataDAO();
  }

  static class PatientProtocolSearchResult {
    private final PatientProtocol patientProtocol;
    private final int templateIndex;

    public PatientProtocolSearchResult(PatientProtocol patientProtocol, int templateIndex) {
      this.patientProtocol = patientProtocol;
      this.templateIndex = templateIndex;
    }

    public PatientProtocol getPatientProtocol() {
      return patientProtocol;
    }

    public int getTemplateIndex() {
      return templateIndex;
    }
  }
}