package gov.va.mobilehealth.courier.resources;

import gov.va.mobilehealth.courier.core.protocol.BroadcastFacilityMessage;
import gov.va.mobilehealth.courier.service.BroadCastFacilityMessageService;

import org.codehaus.enunciate.Facet;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.servlet.ServletContext;
import javax.validation.Valid;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.UriInfo;

import java.net.UnknownHostException;
import java.util.List;

@Service
@Path("facilityMessages")
@Produces(MediaType.APPLICATION_JSON)
@Facet(name = "private")
public class BroadcastFacilityMessagesResource {

  private static final Logger LOGGER = LoggerFactory.getLogger(BroadcastFacilityMessagesResource.class);

  //Resource class should be responsible for handling serialization and web/https specifics
  //Delegate everything else to the service
  private final BroadCastFacilityMessageService broadCastFacilityMessageService;


  public BroadcastFacilityMessagesResource() throws UnknownHostException {
    broadCastFacilityMessageService = new BroadCastFacilityMessageService();
  }

  //TODO: Get by id
    /**
     * Perform a search for a broadcast message by its ID.
     *
     * <pre>
     *
     * Example of broadcast message object:
     * {
     *  "enabled":true,
     *  "name":"VA Information",
     *  "assigningAuthority":"dfn-688",
     *  "createDate":"2014-10-12",
     *  "createBy":"One Vehu",
     *  "_id":"5464d6c7b52c1820def94f4f",
     *  "message":"The Veterans Health Administration is America's largest integrated health care system with over 1,700 sites of care, serving 8.76 million Veterans each year.",
     *  "startDate":"2014-10-01",
     *  "endDate":"2015-10-25",
     *  "schedule":{
     *      "acceptUnscheduledReading":false,
     *      "scheduleType":{
     *          "@class":"gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany",
     *          "name":"Custom",
     *          "description":"Every 10 minute",
     *          "interval":{
     *              "value":10,"unit":"MINUTE"
     *          }
     *      },
     *      "scheduleTime":{
     *          "hour":8,
     *          "min":30,
     *          "meridian":"AM",
     *          "description":"(At 8:30 AM)"
     *      },
     *      "isStartTimeRelative":false,
     *      "isEndTimeRelative":false
     *  },
     *  "progress":"Pending Delivery",
     *  "comments":{
     *      "content":"Disable",
     *      "user":"ONE VEHU",
     *      "time":"12:05 PM",
     *      "date":"2014-10-12",
     *      "changes":[]
     *  },
     *  "createdDateUTC":"2014-10-12 16:05:57"
     * }
     * </pre>
     *
     * @param id  Type ID of the message
     * @return The message.
     * @see gov.va.mobilehealth.courier.core.protocol.BroadcastFacilityMessage
     */
  @Path("/{id}")
  @GET
  public BroadcastFacilityMessage getBroadcastFacilityMessage(@Context UriInfo info, @Context HttpHeaders headers, @PathParam("id") String id) throws Exception {
    return broadCastFacilityMessageService.getById(id);
  }

    /**
     * Retrieves a list of broadcast messages based on the facility.
     *
     * <pre>
     * Example of a list of broadcast messages:
     * [
     *    {
     *      "enabled":true,
     *      "name":"VA Information",
     *      "assigningAuthority":"dfn-688",
     *      "createDate":"2014-10-12",
     *      "createBy":"One Vehu",
     *      "_id":"5464d6c7b52c1820def94f4f",
     *      "message":"The Veterans Health Administration is America's largest integrated health care system with over 1,700 sites of care, serving 8.76 million Veterans each year.",
     *      "startDate":"2014-10-01",
     *      "endDate":"2015-10-25",
     *      "schedule":{
     *          "acceptUnscheduledReading":false,
     *          "scheduleType":{
     *              "@class":"gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany",
     *              "name":"Custom",
     *              "description":"Every 10 minute",
     *              "interval":{
     *                  "value":10,
     *                  "unit":"MINUTE"
     *              }
     *          },
     *          "scheduleTime":{
     *              "hour":8,
     *              "min":30,
     *              "meridian":"AM",
     *              "description":"(At 8:30 AM)"
     *          },
     *          "isStartTimeRelative":false,
     *          "isEndTimeRelative":false
     *      },
     *      "progress":"Pending Delivery",
     *      "comments":{
     *          "content":"Disable",
     *          "user":"ONE VEHU",
     *          "time":"12:05 PM",
     *          "date":"2014-10-12",
     *          "changes":[
     *          ]
     *      },
     *      "createdDateUTC":"2014-10-12 16:05:57"
     *    },
     *    {
     *      "enabled":true,
     *      "name":"Breast Wellness",
     *      "assigningAuthority":"dfn-688",
     *      "createDate":"2014-10-09",
     *      "createBy":"Two Vehu",
     *      "_id":"5464d6c7b52c1820def94f50",
     *      "message":"Mobile Mammograms - Parkshore Center - October 27",
     *      "startDate":"2014-10-01",
     *      "endDate":"2015-10-27",
     *      "schedule":{
     *          "acceptUnscheduledReading":false,
     *          "scheduleType":{
     *              "@class":"gov.va.mobilehealth.courier.core.protocol.schedule.TypeDaysOfWeek",
     *              "name":"Specified Days",
     *              "description":"Mon., Thu., Sat.",
     *              "monday":true,
     *              "tuesday":false,
     *              "wednesday":false,
     *              "thursday":true,
     *              "friday":false,
     *              "saturday":true,
     *              "sunday":false
     *          },
     *          "scheduleTime":{
     *              "hour":8,
     *              "min":30,
     *              "meridian":"PM",
     *              "description":"(At 8:30 PM)"
     *          },
     *          "isStartTimeRelative":false,
     *          "isEndTimeRelative":false
     *      },
     *      "progress":"Delivery In Progress",
     *      "comments":{
     *          "content":null,
     *          "user":null,
     *          "time":null,
     *          "date":null,
     *          "changes":[
     *          ]
     *      },
     *      "createdDateUTC":"2014-10-09 18:05:57"
     *    }
     *  ]
     * </pre>
     *
     * @return A list of broadcast messages.
     */
  @GET
  public List<BroadcastFacilityMessage> getBroadcastFacilityMessageByAssigningAuthority(@Context UriInfo info, @Context HttpHeaders headers) throws Exception {
    return broadCastFacilityMessageService.getMessages();
  }

    /**
     * Saves/updates a broadcast message.
     *
     * <pre>
     * Example of broadcast message to save:
     *
     * {
     *  "enabled":true,
     *  "name":"Breast Wellness",
     *  "assigningAuthority":"dfn-688",
     *  "createDate":"10/09/2014",
     *  "createBy":"Two Vehu",
     *  "_id":"5464d6c7b52c1820def94f50",
     *  "message":"Mobile Mammograms - Parkshore Center - October 27",
     *  "startDate":"2014-11-13T05:00:00.000Z",
     *  "endDate":"2015-10-27",
     *  "schedule":{
     *      "acceptUnscheduledReading":false,
     *      "scheduleType":{
     *          "@class":"gov.va.mobilehealth.courier.core.protocol.schedule.TypeDaysOfWeek",
     *          "name":"Specified Days",
     *          "description":"Mon., Thu., Sat.",
     *          "monday":true,
     *          "tuesday":false,
     *          "wednesday":false,
     *          "thursday":true,
     *          "friday":false,
     *          "saturday":true,
     *          "sunday":false
     *      },
     *      "scheduleTime":{
     *          "hour":8,
     *          "min":"30",
     *          "meridian":"PM",
     *          "description":"(At 8:30 PM)"
     *      },
     *      "isStartTimeRelative":false,
     *      "isEndTimeRelative":false
     *  },
     *  "progress":"Delivery In Progress",
     *  "comments":{
     *      "content":null,
     *      "user":null,
     *      "time":null,
     *      "date":null,
     *      "changes":[
     *      ]
     *  },
     *  "createdDateUTC":"2014-10-09 18:05:57"
     * }
     * </pre>
     *
     * @param context
     * @param message
     * @throws Exception
     */
  @Produces(MediaType.APPLICATION_JSON)
  @Consumes(MediaType.APPLICATION_JSON)
  @PUT
  public void updateBroadcastFacilityMessageTemplate(@Context ServletContext context, @Valid BroadcastFacilityMessage message, @Context UriInfo info, @Context HttpHeaders headers) throws Exception {
    broadCastFacilityMessageService.updateBroadcastFacilityMessageTemplate(context, message);
  }

}
