package gov.va.mobilehealth.courier.resources;

import com.fasterxml.jackson.databind.ObjectMapper;

import gov.va.mobilehealth.cms.jackson.Jackson;
import gov.va.mobilehealth.courier.core.protocol.ScheduleType;
import gov.va.mobilehealth.courier.core.protocol.TimeInterval;
import gov.va.mobilehealth.courier.core.protocol.TimeIntervalUnit;
import gov.va.mobilehealth.courier.core.protocol.dictionary.Dictionary;
import gov.va.mobilehealth.courier.core.protocol.dictionary.Schedule;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeDaysOfWeek;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeOnce;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.DictionaryDAO;
import gov.va.mobilehealth.courier.db.TimeZoneDAO;

import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.UriInfo;

import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;

import org.codehaus.enunciate.Facet;
import org.springframework.stereotype.Service;

/**
 * Endpoint related to static lists of data generally useful for front end application
 */
@Service
@Path("dictionary")
@Produces(MediaType.APPLICATION_JSON)
@Consumes(MediaType.APPLICATION_JSON)
@Facet( name = "private")
public class DictionaryResource {
  private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
  private final DictionaryDAO dictionaryDAO;
  private final TimeZoneDAO timeZoneDAO;

  public DictionaryResource() throws UnknownHostException {
    this.dictionaryDAO = DAOFactory.createDictionaryDAO();
    this.timeZoneDAO = DAOFactory.createTimeZoneDAO();
  }

  /**
   * Return lists of data for services, schedules and timezones
   *<pre>
   * {
   *"services": [
   *    {
   *    "@class": "gov.va.mobilehealth.courier.core.protocol.dictionary.ServiceDictionary",
   *    "name": "Blood Pressure",
   *    "readings": [
   *        {
   *            "name": "Systolic",
   *            "valid": {
   *                "low": 70,
   *                "high": 270
   *            },
   *            "alert": {
   *                "low": 80,
   *                "high": 140
   *            },
   *            "critical": {
   *                "low": 75,
   *                "high": 200
   *            },
   *            "unit": "mmHg"
   *        },
   *        {
   *            "name": "Diastolic",
   *            "valid": {
   *                "low": 40,
   *                "high": 140
   *            },
   *            "alert": {
   *                "low": 60,
   *                "high": 90
   *            },
   *            "critical": {
   *                "low": 55,
   *                "high": 100
   *            },
   *            "unit": "mmHg"
   *        }
   *    ],
   *    "keywords": [
   *        "BP",
   *        "BLOOD"
   *    ],
   *    "_id": "52e829f9bf890cf9ee685d85"
   *    },
   *    {
   *        "@class": "gov.va.mobilehealth.courier.core.protocol.dictionary.ServiceDictionary",
   *        "name": "Pulse Oximetry (SpO2)",
   *        "readings": [
   *            {
   *                "name": "SpO2",
   *                "valid": {
   *                    "low": 60,
   *                    "high": 100
   *                },
   *                "alert": {
   *                    "low": 70,
   *                    "high": 100
   *                },
   *                "critical": {
   *                    "low": 65,
   *                    "high": 100
   *                },
   *                "unit": "% Sat"
   *            }
   *        ],
   *        "keywords": [
   *            "OX",
   *            "OXYGEN"
   *        ],
   *        "_id": "52e829f9bf890cf9ee685d86"
   *    },
   *    {
   *        "@class": "gov.va.mobilehealth.courier.core.protocol.dictionary.ServiceDictionary",
   *        "name": "Blood Glucose",
   *        "readings": [
   *        {
   *            "name": "Glucose",
   *            "valid": {
   *                "low": 30,
   *                "high": 300
   *            },
   *            "alert": {
   *                "low": 40,
   *                "high": 200
   *            },
   *            "unit": "mg/dl"
   *        }
   *        ],
   *        "keywords": [
   *            "BG",
   *            "GLUCOSE",
   *            "SUGAR"
   *        ],
   *        "_id": "52e829f9bf890cf9ee685d87"
   *    },
   *    {
   *        "@class": "gov.va.mobilehealth.courier.core.protocol.dictionary.ServiceDictionary",
   *        "name": "Glucose Before Eating",
   *        "readings": [
   *        {
   *            "name": "Glucose",
   *            "valid": {
   *                "low": 30,
   *                "high": 300
   *            },
   *            "alert": {
   *                "low": 40,
   *                "high": 200
   *            },
   *            "unit": "mg/dl"
   *        }
   *        ],
   *        "keywords": [
   *            "BGB",
   *            "BEFORE"
   *        ],
   *        "_id": "52e829f9bf890cf9ee685d88"
   *    },
   *    {
   *        "@class": "gov.va.mobilehealth.courier.core.protocol.dictionary.ServiceDictionary",
   *        "name": "Glucose After Eating",
   *        "readings": [
   *        {
   *            "name": "Glucose",
   *            "valid": {
   *                "low": 30,
   *                "high": 300
   *            },
   *            "alert": {
   *                "low": 40,
   *                "high": 200
   *            },
   *            "unit": "mg/dl"
   *        }
   *        ],
   *        "keywords": [
   *            "BGA",
   *            "AFTER"
   *        ],
   *        "_id": "52e829f9bf890cf9ee685d89"
   *    },
   *    {
   *        "@class": "gov.va.mobilehealth.courier.core.protocol.dictionary.ServiceDictionary",
   *        "name": "Weight Loss",
   *        "readings": [
   *        {
   *            "name": "Weight",
   *            "valid": {
   *                "low": 50,
   *                "high": 250
   *            },
   *            "alert": {
   *                "low": 70,
   *                "high": 200
   *            },
   *            "unit": "lbs"
   *        }
   *        ],
   *        "keywords": [
   *            "WE",
   *            "WEIGHT"
   *        ],
   *        "_id": "52e829f9bf890cf9ee685d8a"
   *    },
   *],
   *"schedule": {
   *    "scheduleType": [
   *    {
   *        "@class": "gov.va.mobilehealth.courier.core.protocol.schedule.TypeOnce",
   *        "name": "Once",
   *        "description": "Once"
   *    },
   *    {
   *        "@class": "gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany",
   *        "name": "Custom",
   *        "description": "Every 0 minute",
   *        "interval": {
   *            "value": 0,
   *            "unit": "MINUTE"
   *        }
   *    },
   *    {
   *        "@class": "gov.va.mobilehealth.courier.core.protocol.schedule.TypeDaysOfWeek",
   *        "name": "Specified Days",
   *        "description": "",
   *        "monday": false,
   *        "tuesday": false,
   *        "wednesday": false,
   *        "thursday": false,
   *        "friday": false,
   *        "saturday": false,
   *        "sunday": false
   *    }
   *    ]
   *},
   *"timeZones": [
   *    {
   *        "@class": "gov.va.mobilehealth.courier.core.protocol.dictionary.TimeZone",
   *        "value": "America/Anchorage",
   *        "description": "Alaska (GMT-09:00)",
   *        "_id": "528a739eda06d04179851fa9"
   *    },
   *    {
   *        "@class": "gov.va.mobilehealth.courier.core.protocol.dictionary.TimeZone",
   *        "value": "Atlantic/Bermuda",
   *        "description": "Atlantic (GMT-04:00)",
   *        "_id": "528a739eda06d04179851fb9"
   *    },
   *    {
   *        "@class": "gov.va.mobilehealth.courier.core.protocol.dictionary.TimeZone",
   *        "value": "America/Chicago",
   *        "description": "Central (GMT-06:00)",
   *        "_id": "528a739eda06d04179851fc9"
   *    },
   *    {
   *        "@class": "gov.va.mobilehealth.courier.core.protocol.dictionary.TimeZone",
   *        "value": "Pacific/Guam",
   *        "description": "Chamorro (GMT+10:00)",
   *        "_id": "528a739eda06d04179851fd9"
   *    },
   *    {
   *        "@class": "gov.va.mobilehealth.courier.core.protocol.dictionary.TimeZone",
   *        "value": "America/New_York",
   *        "description": "Eastern (GMT-05:00)",
   *        "_id": "528a739eda06d04179851fa8"
   *    },
   *    {
   *        "@class": "gov.va.mobilehealth.courier.core.protocol.dictionary.TimeZone",
   *        "value": "Pacific/Honolulu",
   *        "description": "Hawaii-Aleutian (GMT-10:00)",
   *        "_id": "528a739eda06d04179851fb8"
   *    },
   *    {
   *        "@class": "gov.va.mobilehealth.courier.core.protocol.dictionary.TimeZone",
   *        "value": "America/Denver",
   *        "description": "Mountain (GMT-07:00)",
   *        "_id": "528a739eda06d04179851fc8"
   *    },
   *    {
   *        "@class": "gov.va.mobilehealth.courier.core.protocol.dictionary.TimeZone",
   *        "value": "America/Los_Angeles",
   *        "description": "Pacific (GMT-08:00)",
   *        "_id": "528a739eda06d04179851fd8"
   *    },
   *    {
   *        "@class": "gov.va.mobilehealth.courier.core.protocol.dictionary.TimeZone",
   *        "value": "Pacific/Pago_Pago",
   *        "description": "Samoa (GMT-11:00)",
   *        "_id": "528a739eda06d04179851fe8"
   *    }
   *]
   *}
   * </pre>
   **
   * @return Dictionary
   */
  @GET
  public Dictionary listDictionary(@Context UriInfo info, @Context HttpHeaders headers) throws Exception {
    //List<DictionaryItem> list = (List<DictionaryItem>) dictionaryDAO.findAll();

    Dictionary dictionary = dictionaryDAO.findAll();

      /*List<TimeZone> timeZones = timeZoneDAO.findAll();
      Dictionary dictionary = new Dictionary();
    	dictionary.setServices(list);     */
    // dictionary.setTimeZones(timeZones);

    // schedule
    Schedule schedule = new Schedule();
    List<ScheduleType> scheduleTypeList = new ArrayList<ScheduleType>();

    TypeOnce scheduleType1 = new TypeOnce();
    TypeEverySoMany scheduleType2 = new TypeEverySoMany();
    TypeDaysOfWeek scheduleType3 = new TypeDaysOfWeek();

    scheduleType1.setName("Once");
    scheduleType2.setName("Custom");
    scheduleType3.setName("Specified Days");
    scheduleType2.setInterval(new TimeInterval(TimeIntervalUnit.MINUTE, 0));

    scheduleTypeList.add(scheduleType1);
    scheduleTypeList.add(scheduleType2);
    scheduleTypeList.add(scheduleType3);

    schedule.setScheduleType(scheduleTypeList);
    dictionary.setSchedule(schedule);
    return dictionary;
  }


}

