package gov.va.mobilehealth.courier.resources;

import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.PatientIncomingMessage;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.PatientDAO;
import gov.va.mobilehealth.courier.db.PatientMessageDAO;
import gov.va.mobilehealth.courier.resources.helper.PatientMessageHandler;
import gov.va.mobilehealth.courier.scheduler.PatientProtocolScheduler;
import gov.va.mobilehealth.courier.utils.CamelProducerFactory;
import gov.va.mobilehealth.courier.utils.DateParser;

import org.apache.camel.ProducerTemplate;
import org.quartz.SchedulerException;
import org.quartz.ee.servlet.QuartzInitializerListener;
import org.quartz.impl.StdSchedulerFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.servlet.ServletContext;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriInfo;

import java.io.IOException;
import java.util.Date;

import org.codehaus.enunciate.Facet;

@Service
@Path("/messagereceiving")
@Facet( name = "private")
public class MessageReceivingResource {
  private static final Logger LOGGER = LoggerFactory.getLogger(MessageReceivingResource.class);
  private final PatientMessageDAO patientResponseDAO;
  private final PatientDAO patientDAO;
  private final ProducerTemplate incomingMsgProducer;

  public MessageReceivingResource() {
    this.patientResponseDAO = DAOFactory.createPatientResponseDAO();
    this.patientDAO = DAOFactory.createPatientDAO();

    this.incomingMsgProducer = CamelProducerFactory.getIncomingMsgProducer();
  }

  /**
   * Rest end point to receive message from AT&T
   *
   * <pre>
   * request:
   * cms/service/messagereceiving?destination=61429925830&
   * clientMessageId=10332&source=61494225344&text=KEYWORD+SUBKEYWORD+VALUE&
   * from=61494225344&to=61429925830
   * &networkSubmitTime=2011-11-16+10%3A26%3A13&clientSubmitTime=2011-11-16+10%3A26%3A13
   *
   * response:
   *   204 no content
   * </pre>
   *
   */
  @GET
  public void receiveMessageWithQueryParams(@QueryParam("destination") String destination, @QueryParam("text") String text, @QueryParam("source") String source,
                                            @QueryParam("networkSubmitTime") String networkSubmitTimeString, @QueryParam("clientSubmitTime") String clientSubmitTimeString,
                                            @QueryParam("clientMessageId") String clientMessageId, @Context UriInfo info, @Context HttpHeaders headers) throws IOException {

    processReceivedMessage(destination, text, source, networkSubmitTimeString, clientSubmitTimeString, clientMessageId);
  }

  /**
   * Rest end point to receive message from AT&T
   *
   * <pre>
   * request:
   * cms/service/messagereceiving
   *
   * request payload:
   *
   * destination=61429925830&
   * clientMessageId=10332&source=61494225344&text=KEYWORD+SUBKEYWORD+VALUE&
   * from=61494225344&to=61429925830
   * &networkSubmitTime=2011-11-16+10%3A26%3A13&clientSubmitTime=2011-11-16+10%3A26%3A13
   *
   * response:
   *   204 no content
   * </pre>
   *
   */
  @POST
  public void receiveMessageWithFormParams(@FormParam("destination") String destination, @FormParam("text") String text, @FormParam("source") String source,
                                           @FormParam("networkSubmitTime") String networkSubmitTimeString, @FormParam("clientSubmitTime") String clientSubmitTimeString,
                                           @FormParam("clientMessageId") String clientMessageId, @Context UriInfo u, @Context HttpHeaders headers) throws IOException {

//    if (!((ContainerRequest) headers).  .contentType.toString().equals("application/x-www-form-urlencoded")) {
//      LOGGER.error("Unexpected contentType received on");
//    }

    if (headers.getRequestHeader("content-type") != null) {
      LOGGER.info(headers.getRequestHeader("content-type").toString());
    }
    processReceivedMessage(destination, text, source, networkSubmitTimeString, clientSubmitTimeString, clientMessageId);
  }

  private void processReceivedMessage(String destination, String text, String source, String networkSubmitTimeString, String clientSubmitTimeString, String clientMessageId) {
    LOGGER.info("Received SMS Message from network provider");
    Date networkSubmitTime = DateParser.parseAttDate(networkSubmitTimeString);
    LOGGER.info("networkSubmitTime = {}", networkSubmitTime);
    Date clientSubmitTime = DateParser.parseAttDate(clientSubmitTimeString);
    String[] msgDeliveryType = {PatientMessageHandler.APP_NAME, PatientMessageHandler.SMS_MESSAGE_DELIVERY_TYPE};

    PatientIncomingMessage patientResponse = new PatientIncomingMessage(source, destination);
    patientResponse.setClientMessageId(clientMessageId);
    patientResponse.setClientSubmitTime(clientSubmitTimeString);
    patientResponse.setNetworkSubmitTime(networkSubmitTimeString);
    patientResponse.setText(text);
    patientResponse.setType(MessageType.IN);
    patientResponse.setMsgDeliveryType(msgDeliveryType);

    LOGGER.debug("Sending message to Inbound queue");
    incomingMsgProducer.sendBody(patientResponse);
    LOGGER.info("Done Sending message to Inbound queue");

    //ATT Gateway expects OK/200 on success.  Without this, we will return a No Content/204
    //return Response.status(Response.Status.OK).build();
  }
}
