package gov.va.mobilehealth.courier.resources;

import com.fasterxml.jackson.databind.ObjectMapper;

import gov.va.mobilehealth.cms.jackson.Jackson;
import gov.va.mobilehealth.courier.core.ErrorResponse;
import gov.va.mobilehealth.courier.core.PatientResponseServiceData;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.PatientResponseServiceDataDAO;

import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import java.util.List;

import org.codehaus.enunciate.Facet;
import org.springframework.stereotype.Service;

@Service
@Path("patient")
@Produces(MediaType.APPLICATION_JSON)
@Facet( name = "private")
public class PatientResponseResource {
  private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
  private final PatientResponseServiceDataDAO patientResponseServiceDataDAO;

  public PatientResponseResource() {
    this.patientResponseServiceDataDAO = DAOFactory.createPatientResponseServiceDataDAO();
  }

  /**
   * Get patient's vital readings history
   *
   * <pre>
   * request:
   *   cms/service/patient/52b9f1519d16ccbfab2fcff4/readings
   *
   * response:
   *   patient's vital readings array
   *   
   * [
   *   {
   *     "_id": "3291354e086a53bdd9c9d39e",
   *     "patientResponseReceivedTime": "2014-03-22 11:00:00",
   *     "patientId": "52b9f1519d16ccbfab2fcff4",
   *     "alertLevel": null,
   *     "patientServiceReading": {
   *       "serviceName": "Blood Pressure",
   *       "serviceId": "52e829f9bf890cf9ee685d85",
   *       "readings": [
   *         {
   *           "name": "Systolic",
   *           "value": 120,
   *           "unit": "mmHg"
   *         },
   *         {
   *           "name": "Diastolic",
   *           "value": 77,
   *           "unit": "mmHg"
   *         }
   *       ]
   *     },
   *     "patientResponseReceivedTimeUTC": "2014-03-22 11:00:00",
   *     "timeZone": null
   *   },
   *   {
   *     "_id": "3291354e086a53bd49c9d39e",
   *     "patientResponseReceivedTime": "2014-03-22 07:00:00",
   *     "patientId": "52b9f1519d16ccbfab2fcff4",
   *     "alertLevel": null,
   *     "patientServiceReading": {
   *       "serviceName": "Blood Pressure",
   *       "serviceId": "52e829f9bf890cf9ee685d85",
   *       "readings": [
   *         {
   *           "name": "Systolic",
   *           "value": 139,
   *           "unit": "mmHg"
   *         },
   *         {
   *           "name": "Diastolic",
   *           "value": 80,
   *           "unit": "mmHg"
   *         }
   *       ]
   *     },
   *     "patientResponseReceivedTimeUTC": "2014-03-22 07:00:00",
   *     "timeZone": null
   *   }
   * ]   
   * </pre>
   *
   */
  @Path("/{id}/readings")
  @GET
  public List<PatientResponseServiceData> findPatientResponseServiceData(@PathParam("id") String patientId, @Context UriInfo info, @Context HttpHeaders headers) throws Exception {

    List<PatientResponseServiceData> patientRsponseServiceDataList = patientResponseServiceDataDAO.findByPatientId(patientId);
    if (patientRsponseServiceDataList == null) {
      throw new WebApplicationException(Response.status(Response.Status.NOT_FOUND)
        .entity(new ErrorResponse("no such patient response ")).type(MediaType.APPLICATION_JSON).build());
    }
    return patientRsponseServiceDataList;

  }

  /**
   * Get patient's vital readings history by service Id
   *
   * <pre>
   * request:
   *   cms/service/patient/52b9f1519d16ccbfab2fcff4/readings/52e829f9bf890cf9ee685d85
   *
   * response:
   *   patient's vital readings array of the specified service
   *   
   * [
   *   {
   *     "_id": "3291354e086a53bdd9c9d39e",
   *     "patientResponseReceivedTime": "2014-03-22 11:00:00",
   *     "patientId": "52b9f1519d16ccbfab2fcff4",
   *     "alertLevel": null,
   *     "patientServiceReading": {
   *       "serviceName": "Blood Pressure",
   *       "serviceId": "52e829f9bf890cf9ee685d85",
   *       "readings": [
   *         {
   *           "name": "Systolic",
   *           "value": 120,
   *           "unit": "mmHg"
   *         },
   *         {
   *           "name": "Diastolic",
   *           "value": 77,
   *           "unit": "mmHg"
   *         }
   *       ]
   *     },
   *     "patientResponseReceivedTimeUTC": "2014-03-22 11:00:00",
   *     "timeZone": null
   *   },
   *   {
   *     "_id": "3291354e086a53bd49c9d39e",
   *     "patientResponseReceivedTime": "2014-03-22 07:00:00",
   *     "patientId": "52b9f1519d16ccbfab2fcff4",
   *     "alertLevel": null,
   *     "patientServiceReading": {
   *       "serviceName": "Blood Pressure",
   *       "serviceId": "52e829f9bf890cf9ee685d85",
   *       "readings": [
   *         {
   *           "name": "Systolic",
   *           "value": 139,
   *           "unit": "mmHg"
   *         },
   *         {
   *           "name": "Diastolic",
   *           "value": 80,
   *           "unit": "mmHg"
   *         }
   *       ]
   *     },
   *     "patientResponseReceivedTimeUTC": "2014-03-22 07:00:00",
   *     "timeZone": null
   *   }
   * ]   
   * </pre>
   *
   */
  @Path("/{id}/readings/{serviceId}")
  @GET
  public List<PatientResponseServiceData> findPatientReading(@PathParam("id") String patientId, @PathParam("serviceId") String serviceId, @Context UriInfo info, @Context HttpHeaders headers) throws Exception {

    try {
      List<PatientResponseServiceData> patientResponseServiceDataList = patientResponseServiceDataDAO.findByPatientIdAndServiceId(patientId, serviceId);
      if (patientResponseServiceDataList == null) {
        throw new WebApplicationException(Response.status(Response.Status.NOT_FOUND)
          .entity(new ErrorResponse("no such patient response reading")).type(MediaType.APPLICATION_JSON).build());
      }


      return patientResponseServiceDataList;
    } catch (NullPointerException e) {
      throw new WebApplicationException(Response.status(Response.Status.NOT_FOUND)
        .entity(new ErrorResponse(e.getMessage())).type(MediaType.APPLICATION_JSON).build());


    }
  }


}
