package gov.va.mobilehealth.courier.resources;

import gov.va.mobilehealth.cms.jackson.Jackson;
import gov.va.mobilehealth.courier.core.ErrorResponse;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.PatientMessageHistory;
import gov.va.mobilehealth.courier.core.PatientPreference;
import gov.va.mobilehealth.courier.core.PatientPreferenceAuditTrail;
import gov.va.mobilehealth.courier.core.protocol.Motivational;
import gov.va.mobilehealth.courier.core.protocol.PatientProtocol;
import gov.va.mobilehealth.courier.core.protocol.PatientProtocolStatus;
import gov.va.mobilehealth.courier.core.protocol.Protocol;
import gov.va.mobilehealth.courier.core.protocol.Template;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.PatientDAO;
import gov.va.mobilehealth.courier.db.PatientMessageHistoryDAO;
import gov.va.mobilehealth.courier.db.PatientPreferenceAuditTrailDAO;
import gov.va.mobilehealth.courier.db.PatientProtocolDAO;
import gov.va.mobilehealth.courier.db.ProtocolDAO;
import gov.va.mobilehealth.courier.resources.helper.ErrorCode;
import gov.va.mobilehealth.courier.resources.helper.ErrorMessageConfigure;
import gov.va.mobilehealth.courier.resources.helper.ProcessOptInMessage;
import gov.va.mobilehealth.courier.resources.helper.ValidateUniquePhoneNumber;
import gov.va.mobilehealth.courier.service.PatientsService;
import gov.va.mobilehealth.courier.utils.DateParser;
import gov.va.mobilehealth.courier.utils.SSNFormatter;

import java.net.UnknownHostException;
import java.util.List;

import javax.servlet.ServletContext;
import javax.validation.Valid;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import org.aeonbits.owner.ConfigFactory;
import org.codehaus.enunciate.Facet;
import org.quartz.SchedulerException;
import org.springframework.stereotype.Service;

import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserFactory;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Optional;

/**
 * This class provides both REST endpoints and helper methods for Patients
 */

@Service
@Path("patients")
@Produces(MediaType.APPLICATION_JSON)
@Consumes(MediaType.APPLICATION_JSON)
public class PatientsResource {
  private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
  private final PatientDAO patientDAO;
  private final PatientProtocolDAO patientProtocolDAO;
  private final ProtocolDAO protocolDAO;
  private final PatientMessageHistoryDAO patientMessageHistoryDAO;
  private final PatientPreferenceAuditTrailDAO patientPreferenceAuditTrailDAO;
  private final static String DOB_FORMAT = "MM/dd/yyyy";
  static ErrorMessageConfigure errorMessageConfig = getErrorMessageConfig();
  private final PatientsService patientsService;

  static ErrorMessageConfigure getErrorMessageConfig() {
    return ConfigFactory.create(ErrorMessageConfigure.class, System.getProperties(), System.getenv());
  }

  public PatientsResource(){
    this.patientDAO = DAOFactory.createPatientDAO();
    this.patientProtocolDAO = DAOFactory.createPatientProtocolDAO();
    this.protocolDAO = DAOFactory.createProtocolDAO();
    this.patientMessageHistoryDAO = DAOFactory.createPatientMessageHistoryDAO();
    this.patientPreferenceAuditTrailDAO = DAOFactory.createPatientPreferenceAuditTrailDAO();

    patientsService = new PatientsService();

  }

    /**
     * Returns a list of all patients
     *
     * <pre>
     * Example Patient object:
     *[
     *{
     *"_id": "5464d6c6b52c1820def94f18",
     *"firstName": "zztest",
     *"middleName": null,
     *"lastName": "patient23",
     *"dob": "1900-11-06",
     *"ssn": "999-99-9923",
     *"gender": "male",
     *"patientExistInCms": false,
     *"preference": {
     *"messageStarted": true,
     *"messageStopped": false,
     *"messagePaused": false,
     *"phoneNumber": "15715000023",
     *"homePhoneNumber": null,
     *"messagingPreference": null,
     *"email": null,
     *"timeZone": "America/New_York",
     *"phoneType": null,
     *"basicPhoneNonSecure": false,
     *"receiveFacilityMessage": false,
     *"receiveApptReminder": false
     *},
     *"patientIdentifier": {
     *"uniqueId": "PATID23",
     *"assigningAuthority": "EDIPI"
     *},
     *"lastNameInitial": "P",
     *"last4SSN": "9923",
     *"consent": false,
     *"consentDate": null,
     *"facilityId": null,
     *"age": 114
     *}
     *]
     * </pre>
     * @return List of Patient objects (see example above)
     * @throws Exception
     * @see gov.va.mobilehealth.courier.core.Patient
     */
  @GET
  @Facet( name = "public")
  public List<Patient> listPatient(@Context UriInfo info, @Context HttpHeaders headers) throws Exception {
    return patientDAO.findAll();
  }

    /**
     * Perform a search for a patient by the chosen criteria and return a list of matches.
     *
     * <pre>
     * Example Patient object:
     *[
     *{
     *"_id": "5464d6c6b52c1820def94f18",
     *"firstName": "zztest",
     *"middleName": null,
     *"lastName": "patient23",
     *"dob": "1900-11-06",
     *"ssn": "999-99-9923",
     *"gender": "male",
     *"patientExistInCms": false,
     *"preference": {
     *"messageStarted": true,
     *"messageStopped": false,
     *"messagePaused": false,
     *"phoneNumber": "15715000023",
     *"homePhoneNumber": null,
     *"messagingPreference": null,
     *"email": null,
     *"timeZone": "America/New_York",
     *"phoneType": null,
     *"basicPhoneNonSecure": false,
     *"receiveFacilityMessage": false,
     *"receiveApptReminder": false
     *},
     *"patientIdentifier": {
     *"uniqueId": "PATID23",
     *"assigningAuthority": "EDIPI"
     *},
     *"lastNameInitial": "P",
     *"last4SSN": "9923",
     *"consent": false,
     *"consentDate": null,
     *"facilityId": null,
     *"age": 114
     *}
     *]
     * </pre>
     *
     * @param type  Type of search to perform (SSN, Mobile #, Name, Last 4 of SSN)
     * @param value Search criteria (based on type passed in)
     * @return List of Patient objects (see example above)
     * @throws Exception
     * @see gov.va.mobilehealth.courier.core.Patient
     */
  @GET
  @Path("/search")
  @Facet( name = "public")
  public List<Patient> searchPatient(@QueryParam("type") int type,
                                     @QueryParam("value") String value, @Context UriInfo info, @Context HttpHeaders headers) throws Exception {
    // TODO: may need to move the logic to patient DAO for easy testing
    List<Patient> pList = null;
    if (value == null) {
      throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
        .entity(new ErrorResponse("requested field missing")).type(MediaType.APPLICATION_JSON).build());
    }
    try {
      if (type == PatientDAO.SSN) {
        value = SSNFormatter.ssnFormatter(value);
        if (value == null) {
          throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
            .entity(new ErrorResponse("requested field wrong format")).type(MediaType.APPLICATION_JSON).build());
        }
        pList = patientDAO.findBySsn(value);
      } else if (type == PatientDAO.MOBILE) {
        pList = patientDAO.findByMobile(value);
        if (pList.size() > 1) {
          throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
                  .entity(new ErrorResponse(ErrorCode.MSG_DUPLICATE_MOBILE_PHONE_NUMBER_FOUND, errorMessageConfig.duplicateMobilePhoneNumber())).type(MediaType.APPLICATION_JSON).build());

        }
      } else if (type == PatientDAO.NAME) {
        String[] inputs = value.split(",", PatientDAO.NUM_OF_NAME_PARAMS);
        String[] names = new String[]{"", "", ""};
        boolean isEmpty = true;
        for (int i = 0; i < inputs.length; i++) {
          names[i] = inputs[i];
          if (!names[i].isEmpty()) {
            isEmpty = false;
          }
        }
        if (isEmpty) {
          throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
            .entity(new ErrorResponse("requested field wrong format")).type(MediaType.APPLICATION_JSON).build());
        }
        if (!names[0].isEmpty() && names[1].isEmpty() && names[2].isEmpty()) {
          pList = patientDAO.findByName(names[0], null, null);
          List<Patient> pList2 = patientDAO.findByName(null, names[0], null);
          pList.addAll(pList2);
        } else {
          pList = patientDAO.findByName(names[0], names[1], names[2]);
        }
      } else if (type == PatientDAO.L4SSN) {
        if (!value.isEmpty() && value.trim().length() == 5) {
          String lastNameInitial = value.substring(0, 1).toUpperCase();
          String l4SSN = value.substring(1, 5);
          if (!lastNameInitial.isEmpty() && !l4SSN.isEmpty()) {
            pList = patientDAO.findByLastnameInit4Ssn(l4SSN, lastNameInitial);
          }

        }


      } else {
        throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
          .entity(new ErrorResponse("incorrect search type")).type(MediaType.APPLICATION_JSON).build());
      }
    } catch (WebApplicationException e) {
      throw e;
    } catch (NullPointerException e) {
      throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
        .entity(new ErrorResponse(e.getMessage())).type(MediaType.APPLICATION_JSON).build());
    }
    return pList;
  }

    /**
     * Returns a single patient record given the patient id
     *
     * <pre>
     * Example Patient object:
     *[
     *{
     *"_id": "5464d6c6b52c1820def94f18",
     *"firstName": "zztest",
     *"middleName": null,
     *"lastName": "patient23",
     *"dob": "1900-11-06",
     *"ssn": "999-99-9923",
     *"gender": "male",
     *"patientExistInCms": false,
     *"preference": {
     *"messageStarted": true,
     *"messageStopped": false,
     *"messagePaused": false,
     *"phoneNumber": "15715000023",
     *"homePhoneNumber": null,
     *"messagingPreference": null,
     *"email": null,
     *"timeZone": "America/New_York",
     *"phoneType": null,
     *"basicPhoneNonSecure": false,
     *"receiveFacilityMessage": false,
     *"receiveApptReminder": false
     *},
     *"patientIdentifier": {
     *"uniqueId": "PATID23",
     *"assigningAuthority": "EDIPI"
     *},
     *"lastNameInitial": "P",
     *"last4SSN": "9923",
     *"consent": false,
     *"consentDate": null,
     *"facilityId": null,
     *"age": 114
     *}
     *]
     * </pre>
     *
     * @param id    ID of the patient record to return
     * @return  A single Patient object
     * @throws Exception
     * @see gov.va.mobilehealth.courier.core.Patient
     */
  @Path("/{id}")
  @GET
  @Facet( name = "private")
  public Patient getPatient(@PathParam("id") String id, @Context UriInfo info, @Context HttpHeaders headers) throws Exception {
	  Optional<Patient> existingPatient = patientDAO.findById(id);
	  Patient patient = new Patient();
	  if (existingPatient.isPresent()) {
		  patient = existingPatient.get();
		  patient.getAgeFromDoB();
		  patient.setPatientExistInCms(true);
	    }
	return patient;
  }

    /**
     * Returns the list of protocols associated with the given patient ID
     *
     * <pre>
     * [
     *{
     *"_id": "546518990cf22fc55ec2dbe4",
     *"patientId": "5464d6c6b52c1820def94f10",
     *"assignedDate": "2014-11-13",
     *"lastEditedDate": "2014-11-13",
     *"protocol": {
     *"status": true,
     *"name": "Blood Pressure Protocol3",
     *"description": "This is blood pressure protocol with every so many schedule type",
     *"area": "Blood",
     *"createDate": "2014-03-17",
     *"createBy": "One Vehu",
     *"comments": [],
     *"templates": [
     *{
     *"@class": "gov.va.mobilehealth.courier.core.protocol.Vital",
     *"status": "Active",
     *"type": "Vitals",
     *"name": "Blood Pressure Protocol3",
     *"description": "Blood Pressure",
     *"serviceMessage": "Hello, just a quick reminder that your readings are due. Thanks.",
     *"reminderMessage": "reminder message",
     *"confirmationMessage": "Your reading is fine",
     *"service": {
     *"_id": "52e829f9bf890cf9ee685d85",
     *"name": "Blood Pressure",
     *"readings": [
     *{
     *"name": "systolic",
     *"valid": {
     *"low": 70,
     *"high": 262
     *},
     *"alert": {
     *"low": 80,
     *"high": 180
     *},
     *"critical": {
     *"low": 75,
     *"high": 230
     *},
     *"unit": "mmHg"
     *},
     *{
     *"name": "diastolic",
     *"valid": {
     *"low": 50,
     *"high": 120
     *},
     *"alert": {
     *"low": 60,
     *"high": 100
     *},
     *"critical": {
     *"low": 55,
     *"high": 110
     *},
     *"unit": "mmHg"
     *}
     *],
     *"firstLevelMessage": {
     *"below": "below level message",
     *"above": "above level message"
     *},
     *"secondLevelMessage": {
     *"below": "below level message 2",
     *"above": "above level message 2"
     *},
     *"criticalMessage": {
     *"below": "this is low breach message",
     *"above": "this is high breach message"
     *},
     *"keywords": [
     *"BP",
     *"BLOOD"
     *],
     *"pattern": "(?i).*\\s*(bp|blood)(\\s|\\d).*"
     *},
     *"schedule": {
     *"acceptUnscheduledReading": false,
     *"scheduleType": {
     *"@class": "gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany",
     *"name": "Custom",
     *"description": "Every 10 minute",
     *"interval": {
     *"value": 10,
     *"unit": "MINUTE"
     *}
     *},
     *"scheduleTime": {
     *"hour": 8,
     *"min": 30,
     *"meridian": "AM",
     *"description": "(At 8:30 AM)"
     *},
     *"isStartTimeRelative": false,
     *"relativeStartTime": {
     *"value": 0,
     *"unit": "MINUTE"
     *},
     *"isEndTimeRelative": false,
     *"relativeEndTime": {
     *"value": 10,
     *"unit": "MONTH"
     *},
     *"reminderNumber": 0,
     *"reminderInterval": {
     *"value": 240,
     *"unit": "MINUTE"
     *},
     *"breach": {
     *"repeatBreachValues": false,
     *"breachesPerRepeatingAlert": 0,
     *"breachesDays": 0,
     *"criticalPercentValues": false,
     *"percentTriggersCriticalBreach": 0
     *}
     *},
     *"_id": "52d970ac9386d1ea9b2fcc84",
     *"startDate": "2014-11-13",
     *"endDate": "2015-09-13"
     *}
     *],
     *"_id": "5421c0b7da99b0ba6eaddfc6"
     *},
     *"status": "Active",
     *"comments": [
     *{
     *"content": null,
     *"user": "EIGHT VEHU",
     *"time": "3:46 PM",
     *"date": "2014-11-13",
     *"changes": [
     *{
     *"fieldName": "status",
     *"oldValue": "",
     *"newValue": "Active",
     *"source": ""
     *},
     *{
     *"fieldName": "startDate",
     *"oldValue": null,
     *"newValue": "2014-11-13",
     *"source": "Vitals: Blood Pressure"
     *},
     *{
     *"fieldName": "endDate",
     *"oldValue": null,
     *"newValue": "2015-09-13",
     *"source": "Vitals: Blood Pressure"
     *}
     *]
     *}
     *]
     *}
     *]
     *
     *
     * </pre>
     * @param id    ID of the patient record to return
     * @return  List of PatientProtocol objects
     * @throws Exception
     * @see gov.va.mobilehealth.courier.core.protocol.PatientProtocol
     */
  @Path("/{id}/protocols")
  @GET
  @Facet( name = "private")
  public List<PatientProtocol> listPatientProtocol(@PathParam("id") String id, @Context UriInfo info, @Context HttpHeaders headers) throws Exception {

    return patientProtocolDAO.findByPatientId(id);
  }

    /**
     * Returns a list of messages given a patient ID
     *
     * <pre>
     * [
     *{
     *"networkSubmitTime": "2014-11-13 20:46:44",
     *"_id": "546518b40cf22fc55ec2dbe5",
     *"type": "Out",
     *"date": "2014-11-13",
     *"time": "15:46:44",
     *"text": "Hello, just a quick reminder that your readings are due. Thanks.",
     *"destination": "Annie",
     *"alertLevel": null,
     *"serviceName": "Blood Pressure Protocol3",
     *"timeZone": "America/New_York",
     *"adjustedSubmitTime": "2014-11-13 15:46:44"
     *},
     *{
     *"networkSubmitTime": "2014-11-13 20:50:00",
     *"_id": "546519780cf22fc55ec2dbe6",
     *"type": "Out",
     *"date": "2014-11-13",
     *"time": "15:50:00",
     *"text": "Hello, just a quick reminder that your readings are due. Thanks.",
     *"destination": "Annie",
     *"alertLevel": null,
     *"serviceName": "Blood Pressure Protocol3",
     *"timeZone": "America/New_York",
     *"adjustedSubmitTime": "2014-11-13 15:50:00"
     *}
     *]
     *
     * </pre>
     * @param id    ID of the patient record to return
     * @return  List of PatientMessageHistory objects
     * @throws Exception
     * @see gov.va.mobilehealth.courier.core.PatientMessageHistory
     */
  @Path("/{id}/messages")
  @GET
  @Facet( name = "private")
  public List<PatientMessageHistory> listPatientMessageHistory(@PathParam("id") String id, @Context UriInfo info, @Context HttpHeaders headers) throws Exception {

    return patientMessageHistoryDAO.findByPatientId(id, false);
  }

    /**
     *
     *<pre>
     *
     *An example of the patient's protocol object:
     *
     *[
     *{
     *"_id": "546518990cf22fc55ec2dbe4",
     *"patientId": "5464d6c6b52c1820def94f10",
     *"assignedDate": "2014-11-13",
     *"lastEditedDate": "2014-11-13",
     *"protocol": {
     *"status": true,
     *"name": "Blood Pressure Protocol3",
     *"description": "This is blood pressure protocol with every so many schedule type",
     *"area": "Blood",
     *"createDate": "2014-03-17",
     *"createBy": "One Vehu",
     *"comments": [],
     *"templates": [
     *{
     *"@class": "gov.va.mobilehealth.courier.core.protocol.Vital",
     *"status": "Active",
     *"type": "Vitals",
     *"name": "Blood Pressure Protocol3",
     *"description": "Blood Pressure",
     *"serviceMessage": "Hello, just a quick reminder that your readings are due. Thanks.",
     *"reminderMessage": "reminder message",
     *"confirmationMessage": "Your reading is fine",
     *"service": {
     *"_id": "52e829f9bf890cf9ee685d85",
     *"name": "Blood Pressure",
     *"readings": [
     *{
     *"name": "systolic",
     *"valid": {
     *"low": 70,
     *"high": 262
     *},
     *"alert": {
     *"low": 80,
     *"high": 180
     *},
     *"critical": {
     *"low": 75,
     *"high": 230
     *},
     *"unit": "mmHg"
     *},
     *{
     *"name": "diastolic",
     *"valid": {
     *"low": 50,
     *"high": 120
     *},
     *"alert": {
     *"low": 60,
     *"high": 100
     *},
     *"critical": {
     *"low": 55,
     *"high": 110
     *},
     *"unit": "mmHg"
     *}
     *],
     *"firstLevelMessage": {
     *"below": "below level message",
     *"above": "above level message"
     *},
     *"secondLevelMessage": {
     *"below": "below level message 2",
     *"above": "above level message 2"
     *},
     *"criticalMessage": {
     *"below": "this is low breach message",
     *"above": "this is high breach message"
     *},
     *"keywords": [
     *"BP",
     *"BLOOD"
     *],
     *"pattern": "(?i).*\\s*(bp|blood)(\\s|\\d).*"
     *},
     *"schedule": {
     *"acceptUnscheduledReading": false,
     *"scheduleType": {
     *"@class": "gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany",
     *"name": "Custom",
     *"description": "Every 10 minute",
     *"interval": {
     *"value": 10,
     *"unit": "MINUTE"
     *}
     *},
     *"scheduleTime": {
     *"hour": 8,
     *"min": 30,
     *"meridian": "AM",
     *"description": "(At 8:30 AM)"
     *},
     *"isStartTimeRelative": false,
     *"relativeStartTime": {
     *"value": 0,
     *"unit": "MINUTE"
     *},
     *"isEndTimeRelative": false,
     *"relativeEndTime": {
     *"value": 10,
     *"unit": "MONTH"
     *},
     *"reminderNumber": 0,
     *"reminderInterval": {
     *"value": 240,
     *"unit": "MINUTE"
     *},
     *"breach": {
     *"repeatBreachValues": false,
     *"breachesPerRepeatingAlert": 0,
     *"breachesDays": 0,
     *"criticalPercentValues": false,
     *"percentTriggersCriticalBreach": 0
     *}
     *},
     *"_id": "52d970ac9386d1ea9b2fcc84",
     *"startDate": "2014-11-13",
     *"endDate": "2015-09-13"
     *}
     *],
     *"_id": "5421c0b7da99b0ba6eaddfc6"
     *},
     *"status": "Active",
     *"comments": [
     *{
     *"content": null,
     *"user": "EIGHT VEHU",
     *"time": "3:46 PM",
     *"date": "2014-11-13",
     *"changes": [
     *{
     *"fieldName": "status",
     *"oldValue": "",
     *"newValue": "Active",
     *"source": ""
     *},
     *{
     *"fieldName": "startDate",
     *"oldValue": null,
     *"newValue": "2014-11-13",
     *"source": "Vitals: Blood Pressure"
     *},
     *{
     *"fieldName": "endDate",
     *"oldValue": null,
     *"newValue": "2015-09-13",
     *"source": "Vitals: Blood Pressure"
     *}
     *]
     *}
     *]
     *}
     *]
     *
     *
     * </pre>
     * @param patientId
     * @param protocolId
     * @return
     * @throws Exception
     */
    // TODO:  WHY IS THIS CATCHING NPE WHEN IT IS USING OPTIONAL?
  @Path("/{id}/protocols/{pid}")
  @GET
  @Facet( name = "private")
  public PatientProtocol getPatientProtocol(@PathParam("id") String patientId
    , @PathParam("pid") String protocolId, @Context UriInfo info, @Context HttpHeaders headers) throws Exception {

    try {
      final Optional<PatientProtocol> protocol = patientProtocolDAO.findById(protocolId);
      if (!protocol.isPresent()) {
        throw new WebApplicationException(Response.status(Response.Status.NOT_FOUND)
          .entity(new ErrorResponse("no such patient protocol")).type(MediaType.APPLICATION_JSON).build());
      }
      return protocol.get();
    } catch (NullPointerException e) {
      throw new WebApplicationException(Response.status(Response.Status.NOT_FOUND)
        .entity(new ErrorResponse(e.getMessage())).type(MediaType.APPLICATION_JSON).build());
    }
  }

    /**
     *
     * Add a new protocol to a patient
     *
     * <pre>
     * Protocol Template object:
     *
     * {
     *"_id": null,
     *"patientId": "5464d6c6b52c1820def94f0a",
     *"assignedDate": null,
     *"lastEditedDate": null,
     *"protocol": {
     *"status": true,
     *"name": "Blood Pressure Protocol3",
     *"description": "This is blood pressure protocol with every so many schedule type",
     *"area": "Blood",
     *"createDate": "2014-03-17",
     *"createBy": "One Vehu",
     *"comments": [],
     *"templates": [
     *{
     *"@class": "gov.va.mobilehealth.courier.core.protocol.Vital",
     *"status": "Active",
     *"type": "Vitals",
     *"name": "Blood Pressure Protocol3",
     *"description": "Blood Pressure",
     *"serviceMessage": "Hello, just a quick reminder that your readings are due. Thanks.",
     *"reminderMessage": "reminder message",
     *"confirmationMessage": "Your reading is fine",
     *"service": {
     *"_id": "52e829f9bf890cf9ee685d85",
     *"name": "Blood Pressure",
     *"readings": [
     *{
     *"name": "systolic",
     *"valid": {
     *"low": 70,
     *"high": 262
     *},
     *"alert": {
     *"low": 80,
     *"high": 180
     *},
     *"critical": {
     *"low": 75,
     *"high": 230
     *},
     *"unit": "mmHg"
     *},
     *{
     *"name": "diastolic",
     *"valid": {
     *"low": 50,
     *"high": 120
     *},
     *"alert": {
     *"low": 60,
     *"high": 100
     *},
     *"critical": {
     *"low": 55,
     *"high": 110
     *},
     *"unit": "mmHg"
     *}
     *],
     *"firstLevelMessage": {
     *"below": "below level message",
     *"above": "above level message"
     *},
     *"secondLevelMessage": {
     *"below": "below level message 2",
     *"above": "above level message 2"
     *},
     *"criticalMessage": {
     *"below": "this is low breach message",
     *"above": "this is high breach message"
     *},
     *"keywords": [
     *"BP",
     *"BLOOD"
     *],
     *"pattern": "(?i).*\\s*(bp|blood)(\\s|\\d).*"
     *},
     *"schedule": {
     *"acceptUnscheduledReading": false,
     *"scheduleType": {
     *"@class": "gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany",
     *"name": "Custom",
     *"description": "Every 10 minute",
     *"interval": {
     *"value": 10,
     *"unit": "MINUTE"
     *}
     *},
     *"scheduleTime": {
     *"hour": 8,
     *"min": 30,
     *"meridian": "AM",
     *"description": "(At 8:30 AM)"
     *},
     *"isStartTimeRelative": false,
     *"relativeStartTime": {
     *"value": 0,
     *"unit": "MINUTE"
     *},
     *"isEndTimeRelative": false,
     *"relativeEndTime": {
     *"value": 10,
     *"unit": "MONTH"
     *},
     *"reminderNumber": 0,
     *"reminderInterval": {
     *"value": 240,
     *"unit": "MINUTE"
     *},
     *"breach": {
     *"repeatBreachValues": false,
     *"breachesPerRepeatingAlert": 0,
     *"breachesDays": 0,
     *"criticalPercentValues": false,
     *"percentTriggersCriticalBreach": 0
     *}
     *},
     *"_id": "52d970ac9386d1ea9b2fcc84",
     *"startDate": "2014-11-14",
     *"endDate": "2015-09-14"
     *}
     *],
     *"_id": "5421c0b7da99b0ba6eaddfc6"
     *},
     *"status": null,
     *"comments": []
     *}
     * </pre>
     *
     * @param patientId ID of the patient the protocol is being added to
     * @param protocolId    ID of the protocol to add
     * @return  Protocol that was added
     * @throws Exception
     * @see gov.va.mobilehealth.courier.core.protocol.PatientProtocol
     */
  @Path("/{id}/protocol-template/{pid}")
  @POST
  @Facet( name = "private")
  public PatientProtocol addPatientProtocol(@PathParam("id") String patientId
    , @PathParam("pid") String protocolId, @Context UriInfo info, @Context HttpHeaders headers) throws Exception {
      
    // first need to check if the patient has agreed on staff assisted consent
    patientsService.checkPatientConsent(patientId);

    Optional<Patient> p = patientDAO.findById(patientId);
    if (!p.isPresent()) {
        throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
                .entity(new ErrorResponse(ErrorCode.MSG_PATIENT_NO_FOUND, errorMessageConfig.patientNotFound())).type(MediaType.APPLICATION_JSON).build());
    }

    Protocol protocol = protocolDAO.findById(protocolId).get();
    // clear comments history of the protocol when it is assigned to patient
    // since it is not needed and the data could be big
    protocol.getComments().clear();
    PatientProtocol pProtocol = new PatientProtocol(p.get().getId(), protocol);
    for (Template t : pProtocol.getProtocol().getTemplates()) {
      t.getDefaultStartDate();
      t.getDefaultEndDate();
      t.setStatus(PatientProtocolStatus.ACTIVE);
      if (t instanceof Motivational) {
        ((Motivational) t).resetLastMessageSent();
      }
    }

    return pProtocol;
  }

    /**
     * Update data concerning an existing protocol on a patient
     *
     * <pre>
     * The protocol object:
     *
     *
     * {
     *"_id": null,
     *"patientId": "5464d6c6b52c1820def94f0a",
     *"assignedDate": null,
     *"lastEditedDate": null,
     *"protocol": {
     *"status": true,
     *"name": "Blood Pressure Protocol3",
     *"description": "This is blood pressure protocol with every so many schedule type",
     *"area": "Blood",
     *"createDate": "2014-03-17",
     *"createBy": "One Vehu",
     *"comments": [],
     *"templates": [
     *{
     *"@class": "gov.va.mobilehealth.courier.core.protocol.Vital",
     *"status": "Active",
     *"type": "Vitals",
     *"name": "Blood Pressure Protocol3",
     *"description": "Blood Pressure",
     *"serviceMessage": "Hello, just a quick reminder that your readings are due. Thanks.",
     *"reminderMessage": "reminder message",
     *"confirmationMessage": "Your reading is fine",
     *"service": {
     *"_id": "52e829f9bf890cf9ee685d85",
     *"name": "Blood Pressure",
     *"readings": [
     *{
     *"name": "systolic",
     *"valid": {
     *"low": 70,
     *"high": 262
     *},
     *"alert": {
     *"low": 80,
     *"high": 180
     *},
     *"critical": {
     *"low": 75,
     *"high": 230
     *},
     *"unit": "mmHg"
     *},
     *{
     *"name": "diastolic",
     *"valid": {
     *"low": 50,
     *"high": 120
     *},
     *"alert": {
     *"low": 60,
     *"high": 100
     *},
     *"critical": {
     *"low": 55,
     *"high": 110
     *},
     *"unit": "mmHg"
     *}
     *],
     *"firstLevelMessage": {
     *"below": "below level message",
     *"above": "above level message"
     *},
     *"secondLevelMessage": {
     *"below": "below level message 2",
     *"above": "above level message 2"
     *},
     *"criticalMessage": {
     *"below": "this is low breach message",
     *"above": "this is high breach message"
     *},
     *"keywords": [
     *"BP",
     *"BLOOD"
     *],
     *"pattern": "(?i).*\\s*(bp|blood)(\\s|\\d).*"
     *},
     *"schedule": {
     *"acceptUnscheduledReading": false,
     *"scheduleType": {
     *"@class": "gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany",
     *"name": "Custom",
     *"description": "Every 10 minute",
     *"interval": {
     *"value": 10,
     *"unit": "MINUTE"
     *}
     *},
     *"scheduleTime": {
     *"hour": 8,
     *"min": 30,
     *"meridian": "AM",
     *"description": "(At 8:30 AM)"
     *},
     *"isStartTimeRelative": false,
     *"relativeStartTime": {
     *"value": 0,
     *"unit": "MINUTE"
     *},
     *"isEndTimeRelative": false,
     *"relativeEndTime": {
     *"value": 10,
     *"unit": "MONTH"
     *},
     *"reminderNumber": 0,
     *"reminderInterval": {
     *"value": 240,
     *"unit": "MINUTE"
     *},
     *"breach": {
     *"repeatBreachValues": false,
     *"breachesPerRepeatingAlert": 0,
     *"breachesDays": 0,
     *"criticalPercentValues": false,
     *"percentTriggersCriticalBreach": 0
     *}
     *},
     *"_id": "52d970ac9386d1ea9b2fcc84",
     *"startDate": "2014-11-14",
     *"endDate": "2015-09-14"
     *}
     *],
     *"_id": "5421c0b7da99b0ba6eaddfc6"
     *},
     *"status": null,
     *"comments": []
     *}
     *
     * </pre>
     *
     * @param patientId ID of the patient the protocol is being changed on
     * @param patientProtocol   PatientProtocol object
     * @throws Exception
     */
  @PUT
  @Path("/{id}/protocols")
  @Facet( name = "private")
  public void updatePatientProtocol(@PathParam("id") String patientId, @Valid PatientProtocol patientProtocol
          , @Context ServletContext context, @Context UriInfo info, @Context HttpHeaders headers) throws Exception {
      // first need to check if the patient has agreed on staff assisted consent
      patientsService.checkPatientConsent(patientId);

      patientsService.updatePatientProtocol(patientId, patientProtocol, MhpUserFactory.createFromSecurityContext(), context);
  }

    /**
     * Update information for a patient
     *
     * <pre>
     * Example Patient object:
     *{
     *"_id": "5464d6c6b52c1820def94f0a",
     *"firstName": "zztest",
     *"middleName": null,
     *"lastName": "patient04",
     *"dob": "1900-12-12",
     *"ssn": "999-99-9904",
     *"gender": "male",
     *"patientExistInCms": true,
     *"preference": {
     *"messageStarted": true,
     *"messageStopped": false,
     *"messagePaused": false,
     *"phoneNumber": "15715000004",
     *"homePhoneNumber": null,
     *"messagingPreference": null,
     *"email": null,
     *"timeZone": "America/New_York",
     *"phoneType": "basic",
     *"basicPhoneNonSecure": true,
     *"receiveFacilityMessage": false,
     *"receiveApptReminder": false
     *},
     *"patientIdentifier": {
     *"uniqueId": "D123404",
     *"assigningAuthority": "EDIPI"
     *},
     *"lastNameInitial": "P",
     *"last4SSN": "9904",
     *"consent": true,
     *"consentDate": "2014-04-30",
     *"facilityId": null,
     *"age": 113
     *}
     * </pre>
     *
     * @param patient   Updated Patient object
     * @return  The updated Patient object (see example above)
     * @throws Exception
     * @see gov.va.mobilehealth.courier.core.Patient
     */
  @PUT
  @Facet( name = "public")
  public Patient updatePatient(@Valid Patient patient, @Context ServletContext context, @Context UriInfo info, @Context HttpHeaders headers) throws Exception {
	  try {
	      long x = Long.parseLong(patient.getPreference().getPhoneNumber());

	    } catch (NumberFormatException nFE) {
	      throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
	        .entity(new ErrorResponse("Mobile Phone number not in correct format")).type(MediaType.APPLICATION_JSON).build());

	    }
	    if (patient.getPreference().getHomePhoneNumber() != null && !patient.getPreference().getHomePhoneNumber().isEmpty()) {
	      try {
	        long x = Long.parseLong(patient.getPreference().getHomePhoneNumber());

	      } catch (NumberFormatException nFE) {
	        throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
	          .entity(new ErrorResponse("Home Phone number not in correct format")).type(MediaType.APPLICATION_JSON).build());

	      }
	    }


	   if (patient.getDob() != null) {
	        patient.setDob(DateParser.convertDateFormat(patient.getDob(),DOB_FORMAT));
	   }
	   if ( patient.getConsentDate() == null || patient.getConsentDate().isEmpty()){
		   patient.setConsentDate(DateParser.getTodaysDate(null,null));
	   }
    //PatientPreference newPatientPreference = new PatientPreference();
    PatientPreference newPatientPreference = patient.getPreference();

    MhpUser user = MhpUserFactory.createFromSecurityContext();
    String today = DateParser.getTodaysDate(null, null);
    String time = DateParser.getCurrentTime(null, null);
    String cmsUser = user.getFirstName() + " " + user.getLastName();

    PatientPreferenceAuditTrail patientPreferenceAuditTrail = new PatientPreferenceAuditTrail();
    ValidateUniquePhoneNumber validateUniquePhoneNumber = new ValidateUniquePhoneNumber();


    if (patient.getId() == null) {

      validateUniquePhoneNumber.checkUniquePhoneNumber(patient.getPreference().getPhoneNumber());

      patient = patientDAO.update(patient);
      ProcessOptInMessage processOptInMessage = new ProcessOptInMessage();
      processOptInMessage.sendOptInMessage(patient );
      patientPreferenceAuditTrail.addAuditTrail(null, patient, today, time, cmsUser, patient.getId());

    }
    else {
    Optional<Patient> p = patientDAO.findById(patient.getId());
    if (p.isPresent()) {
      Patient oldPatient = p.get();
      PatientPreference oldPatientPreference = oldPatient.getPreference();
      if (!oldPatientPreference.getPhoneNumber().equals(newPatientPreference.getPhoneNumber())) {
        validateUniquePhoneNumber.checkUniquePhoneNumber(newPatientPreference.getPhoneNumber());
      }
      if (!oldPatient.getPreference().getTimeZone().equals(patient.getPreference().getTimeZone())) {
        patientsService.reschedulePatientProtocols(patient, context);
      }
      patientDAO.update(patient);
      patientPreferenceAuditTrail.addAuditTrail(oldPatient, patient, today, time, cmsUser, patient.getId());
    }


    } //comment for testing


    //Perform other update ops
    //TODO: Move the bulk of code is this PUT to the Patient Service and consolidate with the verteran faciing registration page that performs (mostly) the same ops
    patientsService.updatePatient(patient);

    patientPreferenceAuditTrailDAO.save(patientPreferenceAuditTrail);
	   return patient;
  }

    /**
     * Delete a protocol from a patient
     *
     * <pre>
     * Protocol object:
     *
     * {
     *"_id": "546627780cf22fc55ec2dbf6",
     *"patientId": "5464d6c6b52c1820def94f0a",
     *"assignedDate": "2014-11-14",
     *"lastEditedDate": "2014-11-14",
     *"protocol": {
     *"status": true,
     *"name": "Blood Pressure Protocol3",
     *"description": "This is blood pressure protocol with every so many schedule type",
     *"area": "Blood",
     *"createDate": "2014-03-17",
     *"createBy": "One Vehu",
     *"comments": [],
     *"templates": [
     *{
     *"@class": "gov.va.mobilehealth.courier.core.protocol.Vital",
     *"status": "Removed",
     *"type": "Vitals",
     *"name": "Blood Pressure Protocol3",
     *"description": "Blood Pressure",
     *"serviceMessage": "Hello, just a quick reminder that your readings are due. Thanks.",
     *"reminderMessage": "reminder message",
     *"confirmationMessage": "Your reading is fine",
     *"service": {
     *"_id": "52e829f9bf890cf9ee685d85",
     *"name": "Blood Pressure",
     *"readings": [
     *{
     *"name": "systolic",
     *"valid": {
     *"low": 70,
     *"high": 262
     *},
     *"alert": {
     *"low": 80,
     *"high": 180
     *},
     *"critical": {
     *"low": 75,
     *"high": 230
     *},
     *"unit": "mmHg"
     *},
     *{
     *"name": "diastolic",
     *"valid": {
     *"low": 50,
     *"high": 120
     *},
     *"alert": {
     *"low": 60,
     *"high": 100
     *},
     *"critical": {
     *"low": 55,
     *"high": 110
     *},
     *"unit": "mmHg"
     *}
     *],
     *"firstLevelMessage": {
     *"below": "below level message",
     *"above": "above level message"
     *},
     *"secondLevelMessage": {
     *"below": "below level message 2",
     *"above": "above level message 2"
     *},
     *"criticalMessage": {
     *"below": "this is low breach message",
     *"above": "this is high breach message"
     *},
     *"keywords": [
     *"BP",
     *"BLOOD"
     *],
     *"pattern": "(?i).*\\s*(bp|blood)(\\s|\\d).*"
     *},
     *"schedule": {
     *"acceptUnscheduledReading": false,
     *"scheduleType": {
     *"@class": "gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany",
     *"name": "Custom",
     *"description": "Every 10 minute",
     *"interval": {
     *"value": 10,
     *"unit": "MINUTE"
     *}
     *},
     *"scheduleTime": {
     *"hour": 8,
     *"min": 30,
     *"meridian": "AM",
     *"description": "(At 8:30 AM)"
     *},
     *"isStartTimeRelative": false,
     *"relativeStartTime": {
     *"value": 0,
     *"unit": "MINUTE"
     *},
     *"isEndTimeRelative": false,
     *"relativeEndTime": {
     *"value": 10,
     *"unit": "MONTH"
     *},
     *"reminderNumber": 0,
     *"reminderInterval": {
     *"value": 240,
     *"unit": "MINUTE"
     *},
     *"breach": {
     *"repeatBreachValues": false,
     *"breachesPerRepeatingAlert": 0,
     *"breachesDays": 0,
     *"criticalPercentValues": false,
     *"percentTriggersCriticalBreach": 0
     *}
     *},
     *"_id": "52d970ac9386d1ea9b2fcc84",
     *"startDate": "2014-11-14",
     *"endDate": "2015-09-14"
     *}
     *],
     *"_id": "5421c0b7da99b0ba6eaddfc6"
     *},
     *"status": "Removed",
     *"comments": [
     *{
     *"content": null,
     *"user": "EIGHT VEHU",
     *"time": "11:02 AM",
     *"date": "2014-11-14",
     *"changes": [
     *{
     *"fieldName": "status",
     *"oldValue": "",
     *"newValue": "Active",
     *"source": ""
     *},
     *{
     *"fieldName": "startDate",
     *"oldValue": null,
     *"newValue": "2014-11-14",
     *"source": "Vitals: Blood Pressure"
     *},
     *{
     *"fieldName": "endDate",
     *"oldValue": null,
     *"newValue": "2015-09-14",
     *"source": "Vitals: Blood Pressure"
     *}
     *]
     *},
     *{
     *"content": "sdfsdf",
     *"user": "EIGHT VEHU",
     *"time": "11:22 AM",
     *"date": "2014-11-14",
     *"changes": [
     *{
     *"fieldName": "status",
     *"oldValue": "Active",
     *"newValue": "Removed",
     *"source": ""
     *},
     *{
     *"fieldName": "protocol.templates.status",
     *"oldValue": "Active",
     *"newValue": "Removed",
     *"source": "Vitals: Blood Pressure"
     *}
     *]
     *}
     *]
     *}
     * </pre>
     *
     * @param patientId ID of the patient the protocol is being removed from
     * @param protocolId    ID of the protocol to delete
     * @throws Exception
     */
  @DELETE
  @Path("/{id}/protocols/{pid}")
  @Facet( name = "private")
  public void deletePatientProtocol(@PathParam("id") String patientId
    , @Context ServletContext context
    , @PathParam("pid") String protocolId, @Context UriInfo info, @Context HttpHeaders headers) throws Exception {

      patientsService.deletePatientProtocol(patientId, protocolId, context);
  }
  
}

