package gov.va.mobilehealth.courier.resources;

import com.google.common.base.Optional;

import gov.va.mobilehealth.courier.core.ErrorResponse;
import gov.va.mobilehealth.courier.core.protocol.Template;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.ProtocolTemplateDAO;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import java.net.UnknownHostException;

import org.codehaus.enunciate.Facet;

@Service
@Path("templates/{id: [a-zA-Z0-9]+}")
@Produces(MediaType.APPLICATION_JSON)
@Consumes(MediaType.APPLICATION_JSON)
@Facet( name = "private")
public class ProtocolTemplateResource {

  private static final Logger LOGGER = LoggerFactory.getLogger(ProtocolTemplateResource.class);

  private final ProtocolTemplateDAO protocolTemplateDAO;

  public ProtocolTemplateResource() throws UnknownHostException {
    this.protocolTemplateDAO = DAOFactory.createProtocolTemplateDAO();
  }

  /**
   * Get protocol templates by its Id. This is for future enhancement. Not currently used by Annie. 
   *
   * <pre>
   * request:
   *   cms/service/templates/53da7b430cf27f4540f08295
   *
   * response:
   *  protocol template
   *
   * {
   *   "@class": "gov.va.mobilehealth.courier.core.protocol.Vital",
   *   "status": "Active",
   *   "type": "Vitals",
   *   "description": "Blood Pressure",
   *   "serviceMessage": "Please take your BP twice, take the average and send me the readings.  Please text BP, followed by the two numbers e.g..: BP 133 82 Thanks, Annie",
   *   "reminderMessage": "It is 8 o'clock - did you take your BP meds? If you forgot, iit is better to take them now than to skip them completely.",
   *   "confirmationMessage": "Your BP is staying too far outside the safe range.  Please contact the nurse advice line today and tell the nurse how your are doing.",
   *   "service": {
   *     "_id": "52e829f9bf890cf9ee685d85",
   *     "name": "Blood Pressure",
   *     "readings": [
   *       {
   *         "name": "Systolic",
   *         "valid": {
   *           "low": 89,
   *           "high": 202
   *         },
   *         "alert": {
   *           "low": 92,
   *           "high": 135
   *         },
   *         "critical": {
   *           "low": 90,
   *           "high": 200
   *         },
   *         "unit": "mmHg"
   *       },
   *       {
   *         "name": "Diastolic",
   *         "valid": {
   *           "low": 40,
   *           "high": 110
   *         },
   *         "alert": {
   *           "low": 50,
   *           "high": 85
   *         },
   *         "critical": {
   *           "low": 45,
   *           "high": 100
   *         },
   *         "unit": "mmHg"
   *       }
   *     ],
   *     "firstLevelMessage": {
   *       "below": "Your BP is a little LOW today - I like the low number above 90 and the bottom above 41.  Please send another reading tomorrow.  Take care, Annie.",
   *       "above": "Your BP is a little HIGH today - I like the high number below 135 and the bottom below 85.  Please send another reading tomorrow.  Take care, Annie."
   *     },
   *     "secondLevelMessage": {},
   *     "criticalMessage": {
   *       "below": "Your BP is very low today.  It worries me if the top number is below 90 or the bottom is below 40.  Please rest an hour then send me another reading.  Annie.",
   *       "above": "Your BP is quite high today.  It worries me if the top is over 200 or the bottom is over 110.  Rest an hour, then send me another reading.  Take care, Annie."
   *     },
   *     "keywords": [
   *       "BP",
   *       "BLOOD"
   *     ],
   *     "pattern": "(?i).*\\s*(BP|BLOOD)(\\s|\\d).*"
   *   },
   *   "schedule": {
   *     "acceptUnscheduledReading": false,
   *     "scheduleType": {
   *       "@class": "gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany",
   *       "name": "Custom",
   *       "description": "Every 12 hour",
   *       "interval": {
   *         "value": 12,
   *         "unit": "HOUR"
   *       }
   *     },
   *     "scheduleTime": {
   *       "hour": 8,
   *       "min": 0,
   *       "meridian": "AM",
   *       "description": "(At 8:00 AM)"
   *     },
   *     "isStartTimeRelative": false,
   *     "relativeStartTime": {
   *       "value": 0,
   *       "unit": "MINUTE"
   *     },
   *     "isEndTimeRelative": false,
   *     "relativeEndTime": {
   *       "value": 0,
   *       "unit": "MINUTE"
   *     },
   *     "reminderNumber": 1,
   *     "reminderInterval": {
   *       "value": 2,
   *       "unit": "WEEK"
   *     },
   *     "breach": {
   *       "repeatBreachValues": false,
   *       "breachesPerRepeatingAlert": 0,
   *       "breachesDays": 0,
   *       "criticalPercentValues": false,
   *       "percentTriggersCriticalBreach": 0
   *     }
   *   },
   *   "_id": "53da7b430cf27f4540f08295"
   * }
   * </pre>
   *
   */
  @GET
  public Template getProtocolTemplate(@PathParam("id") String id, @Context UriInfo info, @Context HttpHeaders headers) throws Exception {

    try {
      final Optional<Template> protocol = protocolTemplateDAO.findById(id);
      if (!protocol.isPresent()) {
        throw new WebApplicationException(Response.status(Response.Status.NOT_FOUND)
          .entity(new ErrorResponse("no such protocol")).type(MediaType.APPLICATION_JSON).build());
      }
      return protocol.get();
    } catch (NullPointerException e) {
      throw new WebApplicationException(Response.status(Response.Status.NOT_FOUND)
        .entity(new ErrorResponse(e.getMessage())).type(MediaType.APPLICATION_JSON).build());
    }
  }

}
