package gov.va.mobilehealth.courier.resources;

import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserFactory;
import com.google.common.base.Optional;

import gov.va.mobilehealth.courier.core.ErrorResponse;
import gov.va.mobilehealth.courier.core.protocol.*;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.ProtocolDAO;
import gov.va.mobilehealth.courier.db.ProtocolTemplateDAO;
import gov.va.mobilehealth.courier.utils.DateParser;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.validation.Valid;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import java.net.UnknownHostException;
import java.util.List;

import org.codehaus.enunciate.Facet;

@Service
@Path("protocols")
@Produces(MediaType.APPLICATION_JSON)
@Facet( name = "private")
public class ProtocolsResource {

  private static final Logger LOGGER = LoggerFactory.getLogger(ProtocolsResource.class);
  private final ProtocolDAO protocolDAO;
  private final ProtocolTemplateDAO templateDAO;

  public ProtocolsResource() throws UnknownHostException {
    this.protocolDAO = DAOFactory.createProtocolDAO();
    this.templateDAO = DAOFactory.createProtocolTemplateDAO();
  }

    /**
     * <pre>
     * Protocol object:
     *
     * {
     *"_id": "546627780cf22fc55ec2dbf6",
     *"patientId": "5464d6c6b52c1820def94f0a",
     *"assignedDate": "2014-11-14",
     *"lastEditedDate": "2014-11-14",
     *"protocol": {
     *"status": true,
     *"name": "Blood Pressure Protocol3",
     *"description": "This is blood pressure protocol with every so many schedule type",
     *"area": "Blood",
     *"createDate": "2014-03-17",
     *"createBy": "One Vehu",
     *"comments": [],
     *"templates": [
     *{
     *"@class": "gov.va.mobilehealth.courier.core.protocol.Vital",
     *"status": "Removed",
     *"type": "Vitals",
     *"name": "Blood Pressure Protocol3",
     *"description": "Blood Pressure",
     *"serviceMessage": "Hello, just a quick reminder that your readings are due. Thanks.",
     *"reminderMessage": "reminder message",
     *"confirmationMessage": "Your reading is fine",
     *"service": {
     *"_id": "52e829f9bf890cf9ee685d85",
     *"name": "Blood Pressure",
     *"readings": [
     *{
     *"name": "systolic",
     *"valid": {
     *"low": 70,
     *"high": 262
     *},
     *"alert": {
     *"low": 80,
     *"high": 180
     *},
     *"critical": {
     *"low": 75,
     *"high": 230
     *},
     *"unit": "mmHg"
     *},
     *{
     *"name": "diastolic",
     *"valid": {
     *"low": 50,
     *"high": 120
     *},
     *"alert": {
     *"low": 60,
     *"high": 100
     *},
     *"critical": {
     *"low": 55,
     *"high": 110
     *},
     *"unit": "mmHg"
     *}
     *],
     *"firstLevelMessage": {
     *"below": "below level message",
     *"above": "above level message"
     *},
     *"secondLevelMessage": {
     *"below": "below level message 2",
     *"above": "above level message 2"
     *},
     *"criticalMessage": {
     *"below": "this is low breach message",
     *"above": "this is high breach message"
     *},
     *"keywords": [
     *"BP",
     *"BLOOD"
     *],
     *"pattern": "(?i).*\\s*(bp|blood)(\\s|\\d).*"
     *},
     *"schedule": {
     *"acceptUnscheduledReading": false,
     *"scheduleType": {
     *"@class": "gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany",
     *"name": "Custom",
     *"description": "Every 10 minute",
     *"interval": {
     *"value": 10,
     *"unit": "MINUTE"
     *}
     *},
     *"scheduleTime": {
     *"hour": 8,
     *"min": 30,
     *"meridian": "AM",
     *"description": "(At 8:30 AM)"
     *},
     *"isStartTimeRelative": false,
     *"relativeStartTime": {
     *"value": 0,
     *"unit": "MINUTE"
     *},
     *"isEndTimeRelative": false,
     *"relativeEndTime": {
     *"value": 10,
     *"unit": "MONTH"
     *},
     *"reminderNumber": 0,
     *"reminderInterval": {
     *"value": 240,
     *"unit": "MINUTE"
     *},
     *"breach": {
     *"repeatBreachValues": false,
     *"breachesPerRepeatingAlert": 0,
     *"breachesDays": 0,
     *"criticalPercentValues": false,
     *"percentTriggersCriticalBreach": 0
     *}
     *},
     *"_id": "52d970ac9386d1ea9b2fcc84",
     *"startDate": "2014-11-14",
     *"endDate": "2015-09-14"
     *}
     *],
     *"_id": "5421c0b7da99b0ba6eaddfc6"
     *},
     *"status": "Removed",
     *"comments": [
     *{
     *"content": null,
     *"user": "EIGHT VEHU",
     *"time": "11:02 AM",
     *"date": "2014-11-14",
     *"changes": [
     *{
     *"fieldName": "status",
     *"oldValue": "",
     *"newValue": "Active",
     *"source": ""
     *},
     *{
     *"fieldName": "startDate",
     *"oldValue": null,
     *"newValue": "2014-11-14",
     *"source": "Vitals: Blood Pressure"
     *},
     *{
     *"fieldName": "endDate",
     *"oldValue": null,
     *"newValue": "2015-09-14",
     *"source": "Vitals: Blood Pressure"
     *}
     *]
     *},
     *{
     *"content": "sdfsdf",
     *"user": "EIGHT VEHU",
     *"time": "11:22 AM",
     *"date": "2014-11-14",
     *"changes": [
     *{
     *"fieldName": "status",
     *"oldValue": "Active",
     *"newValue": "Removed",
     *"source": ""
     *},
     *{
     *"fieldName": "protocol.templates.status",
     *"oldValue": "Active",
     *"newValue": "Removed",
     *"source": "Vitals: Blood Pressure"
     *}
     *]
     *}
     *]
     *}
     * </pre>
     * @return A list of protocols.
     * @throws Exception
     */
  @GET
  public List<Protocol> listProtocols(@Context UriInfo info, @Context HttpHeaders headers) throws Exception {

    return protocolDAO.findAll();
  }

    /**
     * <pre>
     * Protocol object:
     *
     * {
     *"_id": "546627780cf22fc55ec2dbf6",
     *"patientId": "5464d6c6b52c1820def94f0a",
     *"assignedDate": "2014-11-14",
     *"lastEditedDate": "2014-11-14",
     *"protocol": {
     *"status": true,
     *"name": "Blood Pressure Protocol3",
     *"description": "This is blood pressure protocol with every so many schedule type",
     *"area": "Blood",
     *"createDate": "2014-03-17",
     *"createBy": "One Vehu",
     *"comments": [],
     *"templates": [
     *{
     *"@class": "gov.va.mobilehealth.courier.core.protocol.Vital",
     *"status": "Removed",
     *"type": "Vitals",
     *"name": "Blood Pressure Protocol3",
     *"description": "Blood Pressure",
     *"serviceMessage": "Hello, just a quick reminder that your readings are due. Thanks.",
     *"reminderMessage": "reminder message",
     *"confirmationMessage": "Your reading is fine",
     *"service": {
     *"_id": "52e829f9bf890cf9ee685d85",
     *"name": "Blood Pressure",
     *"readings": [
     *{
     *"name": "systolic",
     *"valid": {
     *"low": 70,
     *"high": 262
     *},
     *"alert": {
     *"low": 80,
     *"high": 180
     *},
     *"critical": {
     *"low": 75,
     *"high": 230
     *},
     *"unit": "mmHg"
     *},
     *{
     *"name": "diastolic",
     *"valid": {
     *"low": 50,
     *"high": 120
     *},
     *"alert": {
     *"low": 60,
     *"high": 100
     *},
     *"critical": {
     *"low": 55,
     *"high": 110
     *},
     *"unit": "mmHg"
     *}
     *],
     *"firstLevelMessage": {
     *"below": "below level message",
     *"above": "above level message"
     *},
     *"secondLevelMessage": {
     *"below": "below level message 2",
     *"above": "above level message 2"
     *},
     *"criticalMessage": {
     *"below": "this is low breach message",
     *"above": "this is high breach message"
     *},
     *"keywords": [
     *"BP",
     *"BLOOD"
     *],
     *"pattern": "(?i).*\\s*(bp|blood)(\\s|\\d).*"
     *},
     *"schedule": {
     *"acceptUnscheduledReading": false,
     *"scheduleType": {
     *"@class": "gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany",
     *"name": "Custom",
     *"description": "Every 10 minute",
     *"interval": {
     *"value": 10,
     *"unit": "MINUTE"
     *}
     *},
     *"scheduleTime": {
     *"hour": 8,
     *"min": 30,
     *"meridian": "AM",
     *"description": "(At 8:30 AM)"
     *},
     *"isStartTimeRelative": false,
     *"relativeStartTime": {
     *"value": 0,
     *"unit": "MINUTE"
     *},
     *"isEndTimeRelative": false,
     *"relativeEndTime": {
     *"value": 10,
     *"unit": "MONTH"
     *},
     *"reminderNumber": 0,
     *"reminderInterval": {
     *"value": 240,
     *"unit": "MINUTE"
     *},
     *"breach": {
     *"repeatBreachValues": false,
     *"breachesPerRepeatingAlert": 0,
     *"breachesDays": 0,
     *"criticalPercentValues": false,
     *"percentTriggersCriticalBreach": 0
     *}
     *},
     *"_id": "52d970ac9386d1ea9b2fcc84",
     *"startDate": "2014-11-14",
     *"endDate": "2015-09-14"
     *}
     *],
     *"_id": "5421c0b7da99b0ba6eaddfc6"
     *},
     *"status": "Removed",
     *"comments": [
     *{
     *"content": null,
     *"user": "EIGHT VEHU",
     *"time": "11:02 AM",
     *"date": "2014-11-14",
     *"changes": [
     *{
     *"fieldName": "status",
     *"oldValue": "",
     *"newValue": "Active",
     *"source": ""
     *},
     *{
     *"fieldName": "startDate",
     *"oldValue": null,
     *"newValue": "2014-11-14",
     *"source": "Vitals: Blood Pressure"
     *},
     *{
     *"fieldName": "endDate",
     *"oldValue": null,
     *"newValue": "2015-09-14",
     *"source": "Vitals: Blood Pressure"
     *}
     *]
     *},
     *{
     *"content": "sdfsdf",
     *"user": "EIGHT VEHU",
     *"time": "11:22 AM",
     *"date": "2014-11-14",
     *"changes": [
     *{
     *"fieldName": "status",
     *"oldValue": "Active",
     *"newValue": "Removed",
     *"source": ""
     *},
     *{
     *"fieldName": "protocol.templates.status",
     *"oldValue": "Active",
     *"newValue": "Removed",
     *"source": "Vitals: Blood Pressure"
     *}
     *]
     *}
     *]
     *}
     * </pre>
     * @return A list of active protocols.
     * @throws Exception
     */
  @GET
  @Path("/list-active")
  public List<Protocol> listActiveProtocols(@Context UriInfo info, @Context HttpHeaders headers) throws Exception {

    return protocolDAO.findAllActive();
  }

    /**
     * * <pre>
     * Protocol template object:
     *
     * {
     *"_id": "546627780cf22fc55ec2dbf6",
     *"patientId": "5464d6c6b52c1820def94f0a",
     *"assignedDate": "2014-11-14",
     *"lastEditedDate": "2014-11-14",
     *"protocol": {
     *"status": true,
     *"name": "Blood Pressure Protocol3",
     *"description": "This is blood pressure protocol with every so many schedule type",
     *"area": "Blood",
     *"createDate": "2014-03-17",
     *"createBy": "One Vehu",
     *"comments": [],
     *"templates": [
     *{
     *"@class": "gov.va.mobilehealth.courier.core.protocol.Vital",
     *"status": "Removed",
     *"type": "Vitals",
     *"name": "Blood Pressure Protocol3",
     *"description": "Blood Pressure",
     *"serviceMessage": "Hello, just a quick reminder that your readings are due. Thanks.",
     *"reminderMessage": "reminder message",
     *"confirmationMessage": "Your reading is fine",
     *"service": {
     *"_id": "52e829f9bf890cf9ee685d85",
     *"name": "Blood Pressure",
     *"readings": [
     *{
     *"name": "systolic",
     *"valid": {
     *"low": 70,
     *"high": 262
     *},
     *"alert": {
     *"low": 80,
     *"high": 180
     *},
     *"critical": {
     *"low": 75,
     *"high": 230
     *},
     *"unit": "mmHg"
     *},
     *{
     *"name": "diastolic",
     *"valid": {
     *"low": 50,
     *"high": 120
     *},
     *"alert": {
     *"low": 60,
     *"high": 100
     *},
     *"critical": {
     *"low": 55,
     *"high": 110
     *},
     *"unit": "mmHg"
     *}
     *],
     *"firstLevelMessage": {
     *"below": "below level message",
     *"above": "above level message"
     *},
     *"secondLevelMessage": {
     *"below": "below level message 2",
     *"above": "above level message 2"
     *},
     *"criticalMessage": {
     *"below": "this is low breach message",
     *"above": "this is high breach message"
     *},
     *"keywords": [
     *"BP",
     *"BLOOD"
     *],
     *"pattern": "(?i).*\\s*(bp|blood)(\\s|\\d).*"
     *},
     *"schedule": {
     *"acceptUnscheduledReading": false,
     *"scheduleType": {
     *"@class": "gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany",
     *"name": "Custom",
     *"description": "Every 10 minute",
     *"interval": {
     *"value": 10,
     *"unit": "MINUTE"
     *}
     *},
     *"scheduleTime": {
     *"hour": 8,
     *"min": 30,
     *"meridian": "AM",
     *"description": "(At 8:30 AM)"
     *},
     *"isStartTimeRelative": false,
     *"relativeStartTime": {
     *"value": 0,
     *"unit": "MINUTE"
     *},
     *"isEndTimeRelative": false,
     *"relativeEndTime": {
     *"value": 10,
     *"unit": "MONTH"
     *},
     *"reminderNumber": 0,
     *"reminderInterval": {
     *"value": 240,
     *"unit": "MINUTE"
     *},
     *"breach": {
     *"repeatBreachValues": false,
     *"breachesPerRepeatingAlert": 0,
     *"breachesDays": 0,
     *"criticalPercentValues": false,
     *"percentTriggersCriticalBreach": 0
     *}
     *},
     *"_id": "52d970ac9386d1ea9b2fcc84",
     *"startDate": "2014-11-14",
     *"endDate": "2015-09-14"
     *}
     *],
     *"_id": "5421c0b7da99b0ba6eaddfc6"
     *},
     *"status": "Removed",
     *"comments": [
     *{
     *"content": null,
     *"user": "EIGHT VEHU",
     *"time": "11:02 AM",
     *"date": "2014-11-14",
     *"changes": [
     *{
     *"fieldName": "status",
     *"oldValue": "",
     *"newValue": "Active",
     *"source": ""
     *},
     *{
     *"fieldName": "startDate",
     *"oldValue": null,
     *"newValue": "2014-11-14",
     *"source": "Vitals: Blood Pressure"
     *},
     *{
     *"fieldName": "endDate",
     *"oldValue": null,
     *"newValue": "2015-09-14",
     *"source": "Vitals: Blood Pressure"
     *}
     *]
     *},
     *{
     *"content": "sdfsdf",
     *"user": "EIGHT VEHU",
     *"time": "11:22 AM",
     *"date": "2014-11-14",
     *"changes": [
     *{
     *"fieldName": "status",
     *"oldValue": "Active",
     *"newValue": "Removed",
     *"source": ""
     *},
     *{
     *"fieldName": "protocol.templates.status",
     *"oldValue": "Active",
     *"newValue": "Removed",
     *"source": "Vitals: Blood Pressure"
     *}
     *]
     *}
     *]
     *}
     * </pre>
      * @param id
     * @return
     * @throws Exception
     */
  @Path("/{id: [a-zA-Z0-9]+}")
  @GET
  public Protocol getProtocolTemplate(@PathParam("id") String id, @Context UriInfo info, @Context HttpHeaders headers) throws Exception {

    try {
      final Optional<Protocol> protocol = protocolDAO.findById(id);
      if (!protocol.isPresent()) {
        throw new WebApplicationException(Response.status(Response.Status.NOT_FOUND)
          .entity(new ErrorResponse("no such protocol")).type(MediaType.APPLICATION_JSON).build());
      }
      return protocol.get();
    } catch (NullPointerException e) {
      throw new WebApplicationException(Response.status(Response.Status.NOT_FOUND)
        .entity(new ErrorResponse(e.getMessage())).type(MediaType.APPLICATION_JSON).build());
    }
  }

    /**
     * * <pre>
     * Protocol template object:
     *
     * {
     *"_id": "546627780cf22fc55ec2dbf6",
     *"patientId": "5464d6c6b52c1820def94f0a",
     *"assignedDate": "2014-11-14",
     *"lastEditedDate": "2014-11-14",
     *"protocol": {
     *"status": true,
     *"name": "Blood Pressure Protocol3",
     *"description": "This is blood pressure protocol with every so many schedule type",
     *"area": "Blood",
     *"createDate": "2014-03-17",
     *"createBy": "One Vehu",
     *"comments": [],
     *"templates": [
     *{
     *"@class": "gov.va.mobilehealth.courier.core.protocol.Vital",
     *"status": "Removed",
     *"type": "Vitals",
     *"name": "Blood Pressure Protocol3",
     *"description": "Blood Pressure",
     *"serviceMessage": "Hello, just a quick reminder that your readings are due. Thanks.",
     *"reminderMessage": "reminder message",
     *"confirmationMessage": "Your reading is fine",
     *"service": {
     *"_id": "52e829f9bf890cf9ee685d85",
     *"name": "Blood Pressure",
     *"readings": [
     *{
     *"name": "systolic",
     *"valid": {
     *"low": 70,
     *"high": 262
     *},
     *"alert": {
     *"low": 80,
     *"high": 180
     *},
     *"critical": {
     *"low": 75,
     *"high": 230
     *},
     *"unit": "mmHg"
     *},
     *{
     *"name": "diastolic",
     *"valid": {
     *"low": 50,
     *"high": 120
     *},
     *"alert": {
     *"low": 60,
     *"high": 100
     *},
     *"critical": {
     *"low": 55,
     *"high": 110
     *},
     *"unit": "mmHg"
     *}
     *],
     *"firstLevelMessage": {
     *"below": "below level message",
     *"above": "above level message"
     *},
     *"secondLevelMessage": {
     *"below": "below level message 2",
     *"above": "above level message 2"
     *},
     *"criticalMessage": {
     *"below": "this is low breach message",
     *"above": "this is high breach message"
     *},
     *"keywords": [
     *"BP",
     *"BLOOD"
     *],
     *"pattern": "(?i).*\\s*(bp|blood)(\\s|\\d).*"
     *},
     *"schedule": {
     *"acceptUnscheduledReading": false,
     *"scheduleType": {
     *"@class": "gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany",
     *"name": "Custom",
     *"description": "Every 10 minute",
     *"interval": {
     *"value": 10,
     *"unit": "MINUTE"
     *}
     *},
     *"scheduleTime": {
     *"hour": 8,
     *"min": 30,
     *"meridian": "AM",
     *"description": "(At 8:30 AM)"
     *},
     *"isStartTimeRelative": false,
     *"relativeStartTime": {
     *"value": 0,
     *"unit": "MINUTE"
     *},
     *"isEndTimeRelative": false,
     *"relativeEndTime": {
     *"value": 10,
     *"unit": "MONTH"
     *},
     *"reminderNumber": 0,
     *"reminderInterval": {
     *"value": 240,
     *"unit": "MINUTE"
     *},
     *"breach": {
     *"repeatBreachValues": false,
     *"breachesPerRepeatingAlert": 0,
     *"breachesDays": 0,
     *"criticalPercentValues": false,
     *"percentTriggersCriticalBreach": 0
     *}
     *},
     *"_id": "52d970ac9386d1ea9b2fcc84",
     *"startDate": "2014-11-14",
     *"endDate": "2015-09-14"
     *}
     *],
     *"_id": "5421c0b7da99b0ba6eaddfc6"
     *},
     *"status": "Removed",
     *"comments": [
     *{
     *"content": null,
     *"user": "EIGHT VEHU",
     *"time": "11:02 AM",
     *"date": "2014-11-14",
     *"changes": [
     *{
     *"fieldName": "status",
     *"oldValue": "",
     *"newValue": "Active",
     *"source": ""
     *},
     *{
     *"fieldName": "startDate",
     *"oldValue": null,
     *"newValue": "2014-11-14",
     *"source": "Vitals: Blood Pressure"
     *},
     *{
     *"fieldName": "endDate",
     *"oldValue": null,
     *"newValue": "2015-09-14",
     *"source": "Vitals: Blood Pressure"
     *}
     *]
     *},
     *{
     *"content": "sdfsdf",
     *"user": "EIGHT VEHU",
     *"time": "11:22 AM",
     *"date": "2014-11-14",
     *"changes": [
     *{
     *"fieldName": "status",
     *"oldValue": "Active",
     *"newValue": "Removed",
     *"source": ""
     *},
     *{
     *"fieldName": "protocol.templates.status",
     *"oldValue": "Active",
     *"newValue": "Removed",
     *"source": "Vitals: Blood Pressure"
     *}
     *]
     *}
     *]
     *}
     * </pre>
     * @param protocol The protocol object to update.
     * @return
     * @throws Exception
     */
  @Path("/{id: [a-zA-Z0-9]+}")
  @Consumes(MediaType.APPLICATION_JSON)
  @PUT
  public void updateProtocolTemplate(@Valid Protocol protocol, @Context UriInfo info, @Context HttpHeaders headers) throws Exception {
    for (Template template : protocol.getTemplates()) {
      // validate name, description, area
      // TODO need to move to protocol class def
      if (protocol.getName() == null || protocol.getDescription() == null || protocol.getArea() == null) {
        throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
          .entity(new ErrorResponse("Missing required field")).type(MediaType.APPLICATION_JSON).build());
      }

      if (template instanceof Vital) {
        Vital vital = (Vital) template;
        CMSService s = vital.getService();
        // validate keywords
        if (!CMSService.validateKeywords(s.getKeywords())) {
          throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
            .entity(new ErrorResponse("Invalide Keywords")).type(MediaType.APPLICATION_JSON).build());
        }
        // we need to assign the pattern of service to protocol
        // we may need to revisit later when we decide how many services a
        // protocol template can have
        s.setPatternFromKeywords(s.getKeywords());
      }
    }
    try {
      // if new protocol, set created by and created on field
      if (protocol.getId() == null) {
        MhpUser user = MhpUserFactory.createFromSecurityContext();
        String cmsUser = user.getFirstName() + " " + user.getLastName();

        protocol.setCreateBy(cmsUser);
        if (protocol.getCreateDate() == null) {
          protocol.setCreateDate(DateParser.getTodaysDate(null, null));
        }
      }

      // if new template, save template
      for (Template template : protocol.getTemplates()) {
        if (template.getId() == null) {
          template.setStatus(PatientProtocolStatus.ACTIVE);
        }
        if (template instanceof Vital) {
          template.setDescription(((Vital) template).getService().getName());
        }
        templateDAO.saveOrUpdateProtocol(template);
      }
      
      // add audit trail
      String today = DateParser.getTodaysDate(null, null);
      String time = DateParser.getCurrentTime(null, null);

      MhpUser user = MhpUserFactory.createFromSecurityContext();
      String cmsUser = user.getFirstName() + " " + user.getLastName();
      if (protocol.getId() == null) {
        protocol.addAuditTrail(null, today, time, cmsUser);
      } else {
        Optional<Protocol> existingOne = protocolDAO.findById(protocol.getId());
        if (existingOne.isPresent()) {
          Protocol existingProtocol = existingOne.get();
          protocol.addAuditTrail(existingProtocol, today, time, cmsUser);
        }
      }

      protocolDAO.saveOrUpdateProtocol(protocol);
    } catch (NullPointerException e) {
      throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
        .entity(new ErrorResponse(e.getMessage())).type(MediaType.APPLICATION_JSON).build());
    }
  }

}
