package gov.va.mobilehealth.courier.resources;

import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserFactory;
import com.google.common.base.Optional;

import gov.va.mobilehealth.courier.core.*;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.PatientDAO;
import gov.va.mobilehealth.courier.db.PatientMessageDAO;
import gov.va.mobilehealth.courier.db.PatientPreferenceAuditTrailDAO;
import gov.va.mobilehealth.courier.resources.helper.ErrorCode;
import gov.va.mobilehealth.courier.resources.helper.ErrorMessageConfigure;
import gov.va.mobilehealth.courier.resources.helper.ProcessOptInMessage;
import gov.va.mobilehealth.courier.resources.helper.ValidateUniquePhoneNumber;
import gov.va.mobilehealth.courier.service.PatientsService;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.utils.CamelProducerFactory;
import gov.va.mobilehealth.courier.utils.DateParser;
import gov.va.mobilehealth.courier.utils.authConfig.AuthConfig;

import org.aeonbits.owner.ConfigFactory;
import org.apache.camel.ProducerTemplate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.validation.Valid;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import java.io.IOException;

import org.codehaus.enunciate.Facet;

@Service
@Path("registration")
@Produces(MediaType.APPLICATION_JSON)
@Consumes(MediaType.APPLICATION_JSON)
public class RegistrationResource {
  private static final Logger LOGGER = LoggerFactory.getLogger(RegistrationResource.class);
  private final PatientDAO patientDAO;
  private final PatientPreferenceAuditTrailDAO patientPreferenceAuditTrailDAO;
  final static String DATE_OF_BIRTH_FORMAT = "MMM dd, yyyy";

  private final PatientMessageDAO patientMessageDAO;


  static AuthConfig getAuthConfig() {
    return ConfigFactory.create(AuthConfig.class, System.getProperties(), System.getenv());
  }

  public RegistrationResource() {
    this.patientDAO = DAOFactory.createPatientDAO();
    this.patientPreferenceAuditTrailDAO = DAOFactory.createPatientPreferenceAuditTrailDAO();
    this.patientMessageDAO = DAOFactory.createPatientMessageDAO();
  }

  /**
   * Create an empty veteran's user profile
   *
   * <pre>
   * request:
   *   vet/service/registration
   *
   * response:
   * {
   *   "_id": null,
   *   "firstName": null,
   *   "middleName": null,
   *   "lastName": null,
   *   "dob": null,
   *   "ssn": null,
   *   "gender": null,
   *   "patientExistInCms": false,
   *   "preference": {
   *     "messageStarted": false,
   *     "messageStopped": false,
   *     "messagePaused": false,
   *     "phoneNumber": null,
   *     "homePhoneNumber": null,
   *     "messagingPreference": null,
   *     "email": null,
   *     "timeZone": null,
   *     "phoneType": null,
   *     "basicPhoneNonSecure": false,
   *     "receiveFacilityMessage": false,
   *     "receiveApptReminder": false
   *   },
   *   "patientIdentifier": null,
   *   "lastNameInitial": null,
   *   "last4SSN": null,
   *   "consent": false,
   *   "consentDate": null,
   *   "facilityId": null,
   *   "age": 0
   * }
   *
   * </pre>
   *
   */
  @POST
  @Facet( name = "private")
  public Patient registerPatient(@Context UriInfo info, @Context HttpHeaders headers) throws Exception {
    Patient p = new Patient();
    PatientPreference pref = new PatientPreference();
    p.setPreference(pref);
    return p;
  }

  /**
   * Get veteran's user profile
   *
   * <pre>
   * request:
   *   vet/service/registration/userProfile
   *
   * response:
   *   veteran's user profile
   *   
   * {
   *   "_id": "52b9f1519d16ccbfab2fcff4",
   *   "firstName": "zztest",
   *   "middleName": null,
   *   "lastName": "patient01",
   *   "dob": "1900-05-05",
   *   "ssn": "999-99-9901",
   *   "gender": "male",
   *   "patientExistInCms": true,
   *   "preference": {
   *     "messageStarted": true,
   *     "messageStopped": false,
   *     "messagePaused": false,
   *     "phoneNumber": "1571P0O0R0T",
   *     "homePhoneNumber": null,
   *     "messagingPreference": null,
   *     "email": null,
   *     "timeZone": "America/Chicago",
   *     "phoneType": null,
   *     "basicPhoneNonSecure": false,
   *     "receiveFacilityMessage": false,
   *     "receiveApptReminder": false
   *   },
   *   "patientIdentifier": {
   *     "uniqueId": "D123401",
   *     "assigningAuthority": "EDIPI"
   *   },
   *   "lastNameInitial": "P",
   *   "last4SSN": "9901",
   *   "consent": true,
   *   "consentDate": "2014-04-30",
   *   "facilityId": null,
   *   "age": 114
   * }
   * </pre>
   *
   */
  @Path("/userProfile")
  @GET
  @Facet( name = "private")
  public Patient findPatient(@Context UriInfo info, @Context HttpHeaders headers) throws Exception {

    MhpUser user = MhpUserFactory.createFromSecurityContext();
    Patient patient = new Patient();

    UserIdentifier patientIdentifier = new UserIdentifier();
    patientIdentifier.setAssigningAuthority(user.getPatient().getPatientIdentifier().getAssigningAuthority());
    patientIdentifier.setUniqueId(user.getPatient().getPatientIdentifier().getUniqueId());
    LOGGER.debug("uniqid " + user.getPatient().getPatientIdentifier().getUniqueId());
    Optional<Patient> p = patientDAO.findByPatientIdentifier(patientIdentifier);

    if (p.isPresent()) {
      patient = p.get();
      patient.getAgeFromDoB();
      //set the flag
      patient.setPatientExistInCms(true);
    } else {
      patient = mapNonEditablePatientData(user.getPatient(), patient);
      patient = mapEditablePatientData(user.getPatient(), patient);
      if (patient.getFirstName() == null) {     // this is more for messaging app
        patient.setFirstName(user.getFirstName());
      }
      if (patient.getLastName() == null) {
        patient.setLastName(user.getLastName());
      }
      patient.setPatientExistInCms(false);
      patient.setConsentDate(DateParser.getTodaysDate(null, null));
    }
    return patient;
  }

  private Patient mapNonEditablePatientData(com.agilex.healthcare.mobilehealthplatform.domain.Patient mhpPatient, Patient patient) {

    if (mhpPatient.getPatientIdentifier().getFirstName() != null) {
      patient.setFirstName(mhpPatient.getPatientIdentifier().getFirstName());
    } else {
      patient.setFirstName(mhpPatient.getFirstName());
    }

    if (mhpPatient.getPatientIdentifier().getLastName() != null) {
      patient.setLastName(mhpPatient.getPatientIdentifier().getLastName());
    } else {
      patient.setLastName(mhpPatient.getLastName());
    }
    StringBuilder tempSsn = new StringBuilder();
    if (mhpPatient.getPatientIdentifier().getSsn() != null) {
      tempSsn.append(mhpPatient.getPatientIdentifier().getSsn().substring(0, 3));
      tempSsn.append("-");
      tempSsn.append((mhpPatient.getPatientIdentifier().getSsn().substring(3, 5)));
      tempSsn.append("-");
      tempSsn.append((mhpPatient.getPatientIdentifier().getSsn().substring(5, 9)));
      patient.setSsn(tempSsn.toString());
    } else {
      if (mhpPatient.getSsn() != null) {
        tempSsn.append(mhpPatient.getSsn().substring(0, 3));
        tempSsn.append("-");
        tempSsn.append((mhpPatient.getSsn().substring(3, 5)));
        tempSsn.append("-");
        tempSsn.append((mhpPatient.getSsn().substring(5, 9)));
      }

      patient.setSsn(tempSsn.toString());
    }

    if (mhpPatient.getGender() != null) {
      patient.setGender(mhpPatient.getGender());
    }
    if (mhpPatient.getDateOfBirth() != null) {
      patient.setDob(DateParser.convertDateFormat(mhpPatient.getDateOfBirth().toString(), DATE_OF_BIRTH_FORMAT));
    }

    return patient;

  }

  private Patient mapEditablePatientData(com.agilex.healthcare.mobilehealthplatform.domain.Patient mhpPatient, Patient patient) {

    PatientPreference patientPreference = new PatientPreference();
    patientPreference.setPhoneNumber(mhpPatient.getTextMessagingPhoneNumber());
    patient.setPreference(patientPreference);

    return patient;

  }

    /**
     * Updates patient information given a valid patient object
     *
     * <pre>
     * request:
     *   vet/service/registration
     *
     * request payload:
     * {
     *   "_id": "52b9f1519d16ccbfab2fcff4",
     *   "firstName": "zztest",
     *   "middleName": null,
     *   "lastName": "patient01",
     *   "dob": "05/05/1900",
     *   "ssn": "999-99-9901",
     *   "gender": "male",
     *   "patientExistInCms": true,
     *   "preference": {
     *     "messageStarted": true,
     *     "messageStopped": false,
     *     "messagePaused": false,
     *     "phoneNumber": "1571P0O0R0T",
     *     "homePhoneNumber": null,
     *     "messagingPreference": "non-secure",
     *     "email": null,
     *     "timeZone": "America/Chicago",
     *     "phoneType": "smart",
     *     "basicPhoneNonSecure": false,
     *     "receiveFacilityMessage": true,
     *     "receiveApptReminder": false
     *   },
     *   "patientIdentifier": {
     *     "uniqueId": "D123401",
     *     "assigningAuthority": "EDIPI"
     *   },
     *   "lastNameInitial": "P",
     *   "last4SSN": "9901",
     *   "consent": true,
     *   "consentDate": "2014-04-30",
     *   "facilityId": "dfn-688",
     *   "age": 114
     * }
     *
     * response:
     *   204 no content
     *
     * </pre>
     * @param patient
     * @throws Exception
     * @see gov.va.mobilehealth.courier.core.Patient
     */
  @PUT
  public void updatePatientProtocol(@Valid Patient patient, @Context UriInfo info, @Context HttpHeaders headers) throws Exception {

    try {
      long x = Long.parseLong(patient.getPreference().getPhoneNumber());

    } catch (NumberFormatException nFE) {
      throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
              .entity(new ErrorResponse("Mobile Phone number not in correct format")).type(MediaType.APPLICATION_JSON).build());

    }
    if (patient.getPreference().getHomePhoneNumber() != null && !patient.getPreference().getHomePhoneNumber().isEmpty()) {
      try {
        long x = Long.parseLong(patient.getPreference().getHomePhoneNumber());

      } catch (NumberFormatException nFE) {
        throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
                .entity(new ErrorResponse("Home Phone number not in correct format")).type(MediaType.APPLICATION_JSON).build());

      }
    }
    MhpUser user = MhpUserFactory.createFromSecurityContext();
    patient = mapNonEditablePatientData(user.getPatient(), patient);
    String today = DateParser.getTodaysDate(null, null);
    String time = DateParser.getCurrentTime(null, null);
    String cmsUser = user.getFirstName() + " " + user.getLastName();

    UserIdentifier patientIdentifier = new UserIdentifier();
    patientIdentifier.setAssigningAuthority(user.getPatient().getPatientIdentifier().getAssigningAuthority());
    patientIdentifier.setUniqueId(user.getPatient().getPatientIdentifier().getUniqueId());
    patient.setPatientIdentifier(patientIdentifier);

    PatientPreference newPatientPreference = new PatientPreference();
    PatientPreference oldPatientPreference = new PatientPreference();
    PatientPreferenceAuditTrail patientPreferenceAuditTrail = new PatientPreferenceAuditTrail();
    ValidateUniquePhoneNumber validateUniquePhoneNumber = new ValidateUniquePhoneNumber();
    if (patient.getPreference() != null) {
      newPatientPreference = patient.getPreference();
    }

    Optional<Patient> p = patientDAO.findByPatientIdentifier(patientIdentifier);

    if (p.isPresent()) {
      Patient oldPatient = p.get();
      oldPatientPreference = oldPatient.getPreference();
      if (!oldPatientPreference.getPhoneNumber().equals(newPatientPreference.getPhoneNumber())) {
        validateUniquePhoneNumber.checkUniquePhoneNumber(newPatientPreference.getPhoneNumber());
      }
      patientDAO.update(patient);
      patientPreferenceAuditTrail.addAuditTrail(oldPatient, patient, today, time, cmsUser, oldPatient.getId());
    } else {
      PatientPreference patientPreference = new PatientPreference();
      validateUniquePhoneNumber.checkUniquePhoneNumber(patient.getPreference().getPhoneNumber());
      patient.setConsentDate(DateParser.getTodaysDate(null, null));

      patient = patientDAO.update(patient);
      patientPreferenceAuditTrail.addAuditTrail(null, patient, today, time, cmsUser, patient.getId());
      ProcessOptInMessage processOptInMessage = new ProcessOptInMessage();
      processOptInMessage.sendOptInMessage(patient );


    } //comment for testing

    //Perform other update ops
    //TODO: Move the bulk of code is this PUT to the Patient Service and consolidate with the verteran faciing registration page that performs (mostly) the same ops
    PatientsService patientsService = new PatientsService();
    patientsService.updatePatient(patient);

    patientPreferenceAuditTrailDAO.save(patientPreferenceAuditTrail);
  }



}

