package gov.va.mobilehealth.courier.resources;

import java.io.IOException;

import gov.va.mobilehealth.courier.core.ErrorResponse;
import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.SMS;
import gov.va.mobilehealth.courier.core.ThroughMessage;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.PatientDAO;
import gov.va.mobilehealth.courier.resources.helper.ErrorCode;
import gov.va.mobilehealth.courier.resources.helper.ErrorMessageConfigure;
import gov.va.mobilehealth.courier.resources.helper.OutgoingMessageProcessor;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.smsgateway.communication.MessageProcessor;
import gov.va.mobilehealth.courier.utils.CamelProducerFactory;
import gov.va.mobilehealth.courier.utils.authConfig.AuthConfig;

import org.aeonbits.owner.ConfigFactory;
import org.apache.camel.ProducerTemplate;
import org.codehaus.enunciate.Facet;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import com.google.common.base.Optional;

import javax.servlet.ServletContext;
import javax.validation.Valid;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.HeaderParam;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

/**
 * This class provides REST endpoint to send SMS message to patients
 */

@Service
@Path("/messages")
public class SendMessageResource {
    private static final Logger LOGGER = LoggerFactory.getLogger(SendMessageResource.class);
    static ErrorMessageConfigure errorMessageConfig = getErrorMessageConfig();

    static ErrorMessageConfigure getErrorMessageConfig() {
      return ConfigFactory.create(ErrorMessageConfigure.class, System.getProperties(), System.getenv());
    }

    public SendMessageResource() {
    }

    /**
     * Post message sent to patient
     *
     * <pre>
     * request body:
     *   {
     *     "id": "52b9f1519d16ccbfab2fcfc4",
     *     "text": "test message"
     *   }
     *
     *   //id: patient Id in the system
     *   //text: message body, cannot exceed 160 charactors
     * </pre>
     *
     * @return 204 if success. 400 with error
     * {
     *      message: "error message"
     * }
     * @throws Exception
     */
    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @Facet( name = "public")
    public void sendMessage(@Valid ThroughMessage message, @Context UriInfo info, @Context HttpHeaders headers) throws Exception {
        processThroughMessage(message.getId(), message.getText());
    }

    private void processThroughMessage(String patientId, String text) throws WebApplicationException, IOException{
        if ((text == null) || (patientId == null)) {
            throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
                    .entity(new ErrorResponse(ErrorCode.MSG_MISSING_REQUIRED_FIELD, errorMessageConfig.missingRequiredField())).type(MediaType.APPLICATION_JSON).build());
        }

        String messageText = text.trim();
        int length = messageText.length();

        if (length > 160 || length <= 0 ) {
            throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
                    .entity(new ErrorResponse(ErrorCode.MSG_MESSAGE_LENGTH_ERROR, errorMessageConfig.messageLengthError())).type(MediaType.APPLICATION_JSON).build());
        }

        PatientDAO patientDAO = DAOFactory.createPatientDAO();
        Optional<Patient> result = patientDAO.findById(patientId);

        if (!result.isPresent()) {
            throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
                    .entity(new ErrorResponse(ErrorCode.MSG_PATIENT_NO_FOUND, errorMessageConfig.patientNotFound())).type(MediaType.APPLICATION_JSON).build());
        }

        CMSMessage cmsMsg = createThroughMessage(result.get(), messageText);

        ProducerTemplate outgoingMsgProducer = CamelProducerFactory.getOutgoingMsgProducer();
        outgoingMsgProducer.sendBody(cmsMsg);
    }


    /**
     * simple helper to fill in message
     * @param patient
     * @param text
     * @return CMSMessage created
     */
    private CMSMessage createThroughMessage(Patient patient, String text) {
        CMSMessage cmsMsg = new CMSMessage(OutgoingMessageProcessor.API, patient, text, MessageType.OUT);
        return cmsMsg;
    }
}
