package gov.va.mobilehealth.courier.resources.helper;

import com.google.common.base.Optional;

import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.PatientIncomingMessage;
import gov.va.mobilehealth.courier.core.protocol.Comment;
import gov.va.mobilehealth.courier.core.protocol.PatientProtocol;
import gov.va.mobilehealth.courier.core.protocol.PatientProtocolStatus;
import gov.va.mobilehealth.courier.core.protocol.PatientResponseStatus;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.PatientDAO;
import gov.va.mobilehealth.courier.db.PatientMessageDAO;
import gov.va.mobilehealth.courier.db.PatientProtocolDAO;
import gov.va.mobilehealth.courier.protocol.MessageProcessorService;
import gov.va.mobilehealth.courier.scheduler.PatientProtocolScheduler;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.utils.CamelProducerFactory;
import gov.va.mobilehealth.courier.utils.DateParser;

import org.apache.camel.Exchange;
import org.apache.camel.Processor;
import org.apache.camel.ProducerTemplate;
import org.quartz.SchedulerException;
import org.quartz.ee.servlet.QuartzInitializerListener;
import org.quartz.impl.StdSchedulerFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.ServletContext;

import java.io.IOException;
import java.util.List;

//1. encapsulate message sending, new class that is injected?  reduce code duplication, allow test inheriting class to override implementation there
//ProducerTemplate outgoingMsgProducer = CamelProducerFactory.getOutgoingMsgProducer();
//
//  outgoingMsgProducer.sendBody(cmsMsg);
//2.  other dependencies ?  dao?


public class IncomingMessageProcessor implements Processor {

  private static final Logger LOGGER = LoggerFactory.getLogger(IncomingMessageProcessor.class);
  private final PatientDAO patientDAO = DAOFactory.createPatientDAO();
  private final PatientProtocolDAO patientProtocolDAO = DAOFactory.createPatientProtocolDAO();
  private final PatientMessageDAO patientResponseDAO = DAOFactory.createPatientResponseDAO();

  private final ServletContext servletContext;

  public static final String INVALID_MESSAGE = "Thank you. Your readings are not in valid range, please try one more time.";
  public static final String INCORRECT_FORMAT_MESSAGE = "Sorry, I did not understand your text. Please try sending again with the required information and correct format.";
  public static final String APP_NAME = "Annie";
  private final ProducerTemplate outboundMessageProducer;

  public IncomingMessageProcessor(ServletContext servletContext) {
    this.servletContext = servletContext;
    this.outboundMessageProducer = getOutgoingMsgProducer();
  }

  protected ProducerTemplate getOutgoingMsgProducer() {
    return CamelProducerFactory.getOutgoingMsgProducer();
  }

  protected PatientProtocolScheduler getPatientProtocolScheduler() throws SchedulerException {
    StdSchedulerFactory schedulerFactory = (StdSchedulerFactory) this.servletContext
      .getAttribute(QuartzInitializerListener.QUARTZ_FACTORY_KEY);

    return new PatientProtocolScheduler(schedulerFactory.getScheduler());
  }

  @Override
  public void process(Exchange exchange) throws Exception {

    PatientIncomingMessage patientResponse = exchange.getIn().getBody(PatientIncomingMessage.class);

    LOGGER.debug("IncomingMessageProcessor.process() - patientIncomingMessage: {}", patientResponse);

    Optional<Patient> patientOptional = patientDAO.findByMDN(patientResponse.getSource());

    if (patientOptional.isPresent()) {
      Patient patient = patientOptional.get();
      patientResponse.setPatientId(patient.getId());
      patientResponse.setTimeZone(patient.getPreference().getTimeZone());
      patientResponse.setAdjustedSubmitTime(DateParser.convertTimeZone(
        patientResponse.getNetworkSubmitTime(), "UTC", patient
          .getPreference().getTimeZone(), null
      ));

      if (MessageType.START.toString().equalsIgnoreCase(patientResponse.getText())) {
        patient.getPreference().setMessageStarted(true);
        patient.getPreference().setMessageStopped(false);

        patientDAO.update(patient);
        processStart(patientResponse);
      } else if ((MessageType.HELP.toString().equalsIgnoreCase(patientResponse.getText()))) {

        processHelp(patientResponse, patient);

      } else if ((MessageType.STOP.toString().equalsIgnoreCase(patientResponse.getText()))) {
        LOGGER.debug("In the incoming Message listner - getText() in STOP", patientResponse.getText());
        stopProtocol(patientResponse, patient, patientResponse.getText());
      } else if ((MessageType.PAUSE.toString().equalsIgnoreCase(patientResponse.getText()))) {

        pauseProtocol(patientResponse, patient, patientResponse.getText());

      } else if ((MessageType.RESUME.toString().equalsIgnoreCase(patientResponse.getText()))) {

        resumeProtocol(patientResponse, patient, patientResponse.getText());

      } else {

        patientResponse.setAlertLevel(PatientResponseStatus.UNDECIDED.toString());
        patientResponseDAO.insertIncomingMessage(patientResponse);
// if patient sent reading and sent start before
        if ((patient.getPreference() != null) && patient.getPreference().isMessageStarted()) {
          processReading(patientResponse, patientOptional.get().getId());

        }
        // patient sent reading before sending start
        else {

          processReadingBeforeStart(patientResponse, patientOptional.get().getId());
        }

      }
    }

  }

  public void processReadingBeforeStart(PatientIncomingMessage patientResponse, String patientId) {
    CMSMessage cmsMsg = new ResponseMessageGenerator().generateReadingBeforeStartResponse(patientResponse, patientId);
    sendMessage(cmsMsg);
  }

  public void processReading(PatientIncomingMessage patientResponse, String patientId) throws IOException, SchedulerException {

    CMSMessage cmsMsg = new ResponseMessageGenerator().generateReadingResponse(patientResponse, patientId);

    MessageProcessorService msgService = new MessageProcessorService(this.outboundMessageProducer);
    msgService.messageProcess(getPatientProtocolScheduler(), cmsMsg);
    deletePatientReminderJobs(cmsMsg);
  }

  public void deletePatientReminderJobs(CMSMessage cmsg) throws SchedulerException {

    // try delete only on successful msg received.
    if (cmsg.getText().compareTo(INVALID_MESSAGE) != 0 &&
      cmsg.getText().compareTo(INCORRECT_FORMAT_MESSAGE) != 0) {
      getPatientProtocolScheduler().deleteReminderJobs(
        PatientProtocolScheduler.getJobId(cmsg.getPatientProtocolId(), cmsg.getTemplateIndex()));
    }
  }

  public void processStart(PatientIncomingMessage patientResponse) throws Exception {

    LOGGER.debug("In the processing messageCommand start response ", patientResponse.getText());
    patientResponseDAO.insertIncomingMessage(patientResponse);

    ResponseMessageGenerator generator = new ResponseMessageGenerator();
    CMSMessage cmsMsg = generator.generatorStartResponse(patientResponse);

    sendMessage(cmsMsg);
  }

  private void sendMessage(CMSMessage cmsMsg) {
    this.outboundMessageProducer.sendBody(cmsMsg);
  }


  public void pauseProtocol(PatientIncomingMessage patientResponse, Patient patient, String messageCommand) throws Exception {

    patientResponseDAO.insertIncomingMessage(patientResponse);
    List<PatientProtocol> patientProtocolList = patientProtocolDAO.findByPatientId(patient.getId());
    String today = DateParser.getTodaysDate(null, null);
    String time = DateParser.getCurrentTime(null, null);

    CMSMessage cmsMsg;


    if (patient.getPreference() != null && !patient.getPreference().isMessageStarted()) {
      cmsMsg = new ResponseMessageGenerator().generatePauseStartErrorResponse(patientResponse, patient, messageCommand);
    } else {

      cmsMsg = new ResponseMessageGenerator().generatePauseResponse(patientResponse, patient, messageCommand);
      updatePatientProtocolsScheduleForPause(patientProtocolList, today, time);
      updatePatientPreferencesForPause(patient);
    }

    applyStartError(cmsMsg, patient);
    sendMessage(cmsMsg);
  }


  private void updatePatientPreferencesForPause(Patient patient) throws IOException {
    patient.getPreference().setMessagePaused(true);

    patientDAO.update(patient);
  }


  //Delete existing scheduled protocols for the patient.  Then update protocol status and comment and save.
  private void updatePatientProtocolsScheduleForPause(List<PatientProtocol> patientProtocolList, String today, String time) throws IOException, SchedulerException {

    for (PatientProtocol patientProtocol : patientProtocolList) {
      Optional<PatientProtocol> existingOne = patientProtocolDAO.findById(patientProtocol.getId());

      // Only change the state of the protocol IF it is active.
      if (existingOne.isPresent() && (patientProtocol.getStatus() == PatientProtocolStatus.ACTIVE || patientProtocol.getStatus() == PatientProtocolStatus.RESUME)) {
        PatientProtocol existingPatientProtocol = existingOne.get();
        patientProtocol.setAllTemplateStatus(PatientProtocolStatus.PAUSED);
        getPatientProtocolScheduler().delete(patientProtocol);
        patientProtocol.setLastEditedDate(today);
        patientProtocol.setStatus(PatientProtocolStatus.PAUSED);
        //get audit trail
        Comment comment = new Comment();
        comment.setContent("Veteran paused protocol");
        comment.setUser("System");
        comment.setDate(today);
        comment.setTime(time);
        patientProtocol.addComments(comment);
        patientProtocol.addAuditTrail(existingPatientProtocol, today, time, "System");

        patientProtocolDAO.save(patientProtocol);
      }

    }
  }

  private void updatePatientProtocolsScheduleForResume(List<PatientProtocol> patientProtocolList, String timeZone, String today, String time) throws IOException, SchedulerException {

    for (PatientProtocol patientProtocol : patientProtocolList) {
      Optional<PatientProtocol> existingOne = patientProtocolDAO.findById(patientProtocol.getId());

      // Only resume a protocol if it was in a paused state. Do not enable the inactive protocols.
      if (existingOne.isPresent() && (patientProtocol.getStatus() == PatientProtocolStatus.PAUSED)) {
        PatientProtocol existingPatientProtocol = existingOne.get();
        patientProtocol.setAllTemplateStatus(PatientProtocolStatus.RESUME);
        getPatientProtocolScheduler().update(patientProtocol, timeZone);

        patientProtocol.setLastEditedDate(today);
        patientProtocol.setStatus(PatientProtocolStatus.RESUME);
        //get audit trail
        Comment c = new Comment();
        c.setContent("Veteran resumed protocol");
        c.setDate(today);
        c.setTime(time);
        patientProtocol.addComments(c);
        patientProtocol.addAuditTrail(existingPatientProtocol, today, time, "System");
        patientProtocolDAO.save(patientProtocol);
      }
    }


  }

  private void updatePatientPreferencesForResume(Patient patient) throws IOException {
    patient.getPreference().setMessagePaused(false);
    patient.getPreference().setMessageStarted(true);
    patientDAO.update(patient);

  }

  private void updatePatientProtocolsScheduleForStop(List<PatientProtocol> patientProtocolList, String today, String time) throws IOException, SchedulerException {

    for (PatientProtocol patientProtocol : patientProtocolList) {
      Optional<PatientProtocol> existingOne = patientProtocolDAO.findById(patientProtocol.getId());

      if (existingOne.isPresent()) {
        PatientProtocol existingPatientProtocol = existingOne.get();
        patientProtocol.setAllTemplateStatus(PatientProtocolStatus.REMOVED);
        getPatientProtocolScheduler().delete(patientProtocol);
        patientProtocol.setLastEditedDate(today);
        patientProtocol.setStatus(PatientProtocolStatus.REMOVED);
        //get audit trail
        Comment comment = new Comment();
        comment.setContent("Veteran stopped protocol");
        comment.setUser("System");
        comment.setDate(today);
        comment.setTime(time);
        patientProtocol.addComments(comment);
        patientProtocol.addAuditTrail(existingPatientProtocol, today, time, "System");

        patientProtocolDAO.save(patientProtocol);
      }

    }


  }

  public void resumeProtocol(PatientIncomingMessage patientResponse, Patient patient, String messageCommand) throws Exception {

    patientResponseDAO.insertIncomingMessage(patientResponse);
    List<PatientProtocol> patientProtocolList = patientProtocolDAO.findByPatientIdAndStatus(patient.getId(), PatientProtocolStatus.PAUSED);
    String today = DateParser.getTodaysDate(null, null);
    String time = DateParser.getCurrentTime(null, null);

    CMSMessage cmsMsg = null;


    if (patient.getPreference() != null && !patient.getPreference().isMessageStarted()) {
      cmsMsg = new ResponseMessageGenerator().generateResumeStartError(patientResponse, patient, messageCommand);

    } else {
      if (!patient.getPreference().isMessagePaused()) {
        cmsMsg = new ResponseMessageGenerator().generateResumePauseError(patientResponse, patient, messageCommand);

      } else {
        cmsMsg = new ResponseMessageGenerator().generateResumeResponse(patientResponse, patient, messageCommand);
        updatePatientProtocolsScheduleForResume(patientProtocolList, patient.getPreference().getTimeZone(), today, time);
        updatePatientPreferencesForResume(patient);

      }

    }
    applyStartError(cmsMsg, patient);
    sendMessage(cmsMsg);

  }

  public void processHelp(PatientIncomingMessage patientResponse, Patient patient) throws Exception {
// veteran sent Help
    patientResponseDAO.insertIncomingMessage(patientResponse);

    ResponseMessageGenerator generator = new ResponseMessageGenerator();
    CMSMessage cmsMsg = generator.generateHelpResponse(patientResponse, patient);

    sendMessage(cmsMsg);
  }

  public void stopProtocol(PatientIncomingMessage patientResponse, Patient patient, String messageCommand) throws Exception {

    patientResponseDAO.insertIncomingMessage(patientResponse);
    List<PatientProtocol> patientProtocolList = patientProtocolDAO.findByPatientIdAndStatus(patient.getId(), PatientProtocol.ACTIVE_AND_PAUSED_PATIENT_PROTOCOL);
    //PatientProtocol patientProtocol = new PatientProtocol();
    String today = DateParser.getTodaysDate(null, null);
    String time = DateParser.getCurrentTime(null, null);
    CMSMessage cmsMsg;

    if (patient.getPreference() != null && !patient.getPreference().isMessageStarted()) {
      cmsMsg = new ResponseMessageGenerator().genreateStopStartError(patientResponse, patient, messageCommand);
    } else {
      cmsMsg = new ResponseMessageGenerator().generateStopResponse(patientResponse, patient, messageCommand);
      updatePatientProtocolsScheduleForStop(patientProtocolList, today, time);
      updatePatientPreferencesForStop(patient);
    }

    sendMessage(cmsMsg);
  }

  private void updatePatientPreferencesForStop(Patient patient) throws IOException {

    patient.getPreference().setMessageStopped(true);
    patient.getPreference().setMessageStarted(false);
    patient.getPreference().setMessagePaused(false);

    patientDAO.update(patient);

  }

  private CMSMessage applyStartError(CMSMessage message, Patient p1) {
      if ((!p1.getPreference().isMessageStarted() && (!p1.getPreference().isMessageStopped() || !ResponseMessageGenerator.STOP_RESPONSE.equals(message.getText())))) {
          message.setText(ResponseMessageGenerator.START_ERROR);
      }
      return message;
  }

}
