package gov.va.mobilehealth.courier.resources.helper;

import com.google.common.base.Optional;

import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.PatientPreference;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.PatientDAO;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.smsgateway.communication.MessageProcessor;
import gov.va.mobilehealth.courier.smsgateway.communication.MessageProcessorBase;
import gov.va.mobilehealth.courier.utils.authConfig.AuthConfig;

import org.aeonbits.owner.ConfigFactory;
import org.apache.camel.Exchange;
import org.apache.camel.Processor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class OutgoingMessageProcessor implements Processor {

  private static final Logger LOGGER = LoggerFactory.getLogger(OutgoingMessageProcessor.class);
  private final PatientDAO patientDAO;
  public static final String NON_SECURE_MESSAGE = "non-secure";
  private static final String ALERT_NEW_MESSAGE_IN_APP = "You have a new message! Please login to view your message. Talk to you soon. Thanks, Annie. ";
  public static final String API = "API";

  static AuthConfig authConfig = getAuthConfig();
  private CMSMessage lastMessage;
  private CMSMessage lastNoticeMessage;
  private MessageProcessorBase messageProcessor;

  static AuthConfig getAuthConfig() {
    return ConfigFactory.create(AuthConfig.class, System.getProperties(), System.getenv());
  }
  public OutgoingMessageProcessor(){
    this.patientDAO = DAOFactory.createPatientDAO();
    this.messageProcessor = new MessageProcessor();
  }

  //Allow message processor to be specified (for test)

  public OutgoingMessageProcessor(MessageProcessorBase messageProcessor){
    this.patientDAO = DAOFactory.createPatientDAO();
    this.messageProcessor = messageProcessor;
  }


  // if message is only sent if in process of veteran registration or ,
  //  if non-secured method  and start message already sent (isMessageStart is true) or in response to Stop or Help

  @Override
  public void process(Exchange exchange) throws Exception {

    PatientMessageHandler pHandler = new PatientMessageHandler();
    LOGGER.debug("In the consumer-processor- getting the message from exchange");
    CMSMessage message = (CMSMessage) exchange.getIn().getBody(CMSMessage.class);
    LOGGER.debug("In the consumer-processor, the text is" + message.getText());
    CMSMessage notice = null;

    // Only Opt-in message is set to true at this point, all other messages are by patient preference
    if (!message.isForceSendViaSMS()) {

      String patientId = message.getPatientId();

      Optional<Patient> patient = patientDAO.findById(patientId);

      if (patient.isPresent()) {

        Patient p1 = patient.get();

        if (isPrefereceNonSecuredMessaging(p1)) { // veteran sent by basic cell phone or smart phone,sending response to phone
          message.setForceSendViaSMS(true);
        }

        // Add logic to send a notice message if the patient don't agree to receive non secure message

        notice = generateSmsNotice(message, p1);

      }

      // the destination of the original message needs to be set to Annie
      // if the message is not sent through SMS
      // this is need to sort the message in right order if an SMS notification
      // also needs to be sent out
      message = setMessageDestination(message);
    }

    LOGGER.debug("In the consumer-processor, the text is  flag for sendsms" + message.isForceSendViaSMS());
    LOGGER.debug("In the consumer-processor, the text is   text" + message.getText());


    getProcessor().sendSingleMessage(message, pHandler);
    if (notice != null) {
      getProcessor().sendSingleMessage(notice, pHandler);
    }


    //set properties to retrieve messages for testing purposes
    this.setLastMessage(message);
    this.setLastNoticeMessage(notice);
  }


  public MessageProcessorBase getProcessor() {
    return messageProcessor;
  }

  public CMSMessage setMessageDestination(CMSMessage message) {
    if (!message.isForceSendViaSMS()) {
      message.setDestination(IncomingMessageProcessor.APP_NAME);
    }
     return message;
  }
  /**
   * generate SMS notice message is necessary
   * condition is
   * 1) message should not be sent through SMS
   * 2) patient uses smart phone
   *
   * @param message, original message
   * @param p1, patient info
   * @return if meet requirement, the SMS notice message;
   * otherwise, null
   */
  protected CMSMessage generateSmsNotice(CMSMessage message, Patient patient) {
    CMSMessage notice = null;
    // generate sms notice message
    // only if the message should not be sent through SMS
    // and the patient has smart phone
    if (!message.isForceSendViaSMS() && patient.getPreference().getNotificationDeliveryPreference().isSendSms()) {
      notice = new CMSMessage(message);
      notice.setAlertLevel(null);
      notice.setForceSendViaSMS(true);
      notice.setText(ALERT_NEW_MESSAGE_IN_APP + authConfig.annieUrl());
      LOGGER.debug("Alert veteran new message in secure app");
      if (!patient.getPreference().getNotificationDeliveryPreference().isSendAnnie()) {
          notice.setForceSendViaAnnie(false);
      }
    } 
    return notice;
  }


  protected PatientDAO getPatientDAO() {
    return DAOFactory.createPatientDAO();
  }

  private boolean isPrefereceNonSecuredMessaging(Patient patient) {
    return patient.getPreference().getMessageDeliveryPreference().isSendSms();
  }

  public void setLastMessage(CMSMessage lastMessage) {
    this.lastMessage = lastMessage;
  }

  public CMSMessage getLastMessage() {
    return lastMessage;
  }

  public void setLastNoticeMessage(CMSMessage lastNoticeMessage) {
    this.lastNoticeMessage = lastNoticeMessage;
  }

  public CMSMessage getLastNoticeMessage() {
    return lastNoticeMessage;
  }
}
