package gov.va.mobilehealth.courier.scheduler;

import gov.va.mobilehealth.courier.core.protocol.BroadcastFacilityMessage;
import gov.va.mobilehealth.courier.core.protocol.TimeInterval;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeDaysOfWeek;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeOnce;
import gov.va.mobilehealth.courier.utils.DateParser;
import org.quartz.Job;
import org.quartz.JobDetail;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Date;

import static org.quartz.JobBuilder.newJob;

public class BroadcastMessageScheduler extends SchedulerBase {
  public static final String JOB_FIRST_GROUP = "initFacilityMessage";
  public static final String TRIGGER_FIRST_GROUP = "initFacilityMessage";
  public static final String JOB_EXPIRE_GROUP = "expireFacilityMessage";
  public static final String TRIGGER_EXPIRE_GROUP = "expireFacilityMessage";
  private static final Logger LOGGER = LoggerFactory.getLogger(BroadcastMessageScheduler.class);
  protected Class<? extends Job> patientProtocolInitJobClass;
  protected Class<? extends Job> patientProtocolExpireJobClass;

  public BroadcastMessageScheduler(Scheduler scheduler) {
    this.scheduler = scheduler;
    this.patientProtocolInitJobClass = FacilityMessageInitJob.class;
    this.patientProtocolExpireJobClass = FacilityMessageExpireJob.class;
  }

  protected void SetPatientProtocolInitJobClass(Class<? extends Job> patientProtocolInitJobClass) {
    this.patientProtocolInitJobClass = patientProtocolInitJobClass;
  }

  protected Class<? extends Job> GetPatientProtocolInitJobClass() {
    return this.patientProtocolInitJobClass;
  }

  protected void SetPatientProtocolExpireJobClass(Class<? extends Job> patientProtocolExpireJobClass) {
    this.patientProtocolExpireJobClass = patientProtocolExpireJobClass;
  }

  protected Class<? extends Job> GetPatientProtocolExpireJobClass() {
    return this.patientProtocolExpireJobClass;
  }

  public void update(BroadcastFacilityMessage facilityMessage, String timezone) {

    String jobName = facilityMessage.getId();
    String jobGroup = JOB_FIRST_GROUP;

    String cronScheduleStr = "";

    try {
      scheduleProtocolExpireJob(facilityMessage, timezone);

      cleanJob(jobName, JOB_FIRST_GROUP, TRIGGER_FIRST_GROUP);

      JobDetail job = newJob(GetPatientProtocolInitJobClass())
        .withIdentity(jobName, jobGroup)
        .build();

      String scheduleTimeStr = ProtocolChronGenerator.getStartDateTime(
        trimDate(facilityMessage.getStartDate()), facilityMessage
          .getSchedule().getScheduleTime());
      Date scheduleTime = DateParser.parseDateWithTimeZone(scheduleTimeStr, timezone);

      String scheduledEndTimeStr = ProtocolChronGenerator.getStartDateTime(
        trimDate(facilityMessage.getEndDate()), facilityMessage
          .getSchedule().getScheduleTime());
      Date scheduledEndTime = DateParser.parseDateWithTimeZone(scheduledEndTimeStr, timezone);

      Date newScheduledEndTime = new Date(scheduledEndTime.getTime() + (1000 * 60 * 60 * 24));

      if (facilityMessage.getSchedule().getScheduleType() instanceof TypeOnce) {
        scheduleSimpleTriggerJob(job, jobName, jobGroup, scheduleTime, newScheduledEndTime);
      } else if (facilityMessage.getSchedule().getScheduleType() instanceof TypeEverySoMany) {
        TimeInterval timeInterval = ((TypeEverySoMany) facilityMessage.getSchedule().getScheduleType()).getInterval();
        scheduleCalendarIntervalTriggerJob(job, jobName, jobGroup, scheduleTime, newScheduledEndTime, timeInterval, timezone);

      } else if (facilityMessage.getSchedule().getScheduleType() instanceof TypeDaysOfWeek) {
        cronScheduleStr = ProtocolChronGenerator.getCronSchedule(facilityMessage.getSchedule());
        scheduleCronTriggerJob(job, jobName, jobGroup, cronScheduleStr, newScheduledEndTime, timezone);
      }

    } catch (SchedulerException e) {
      LOGGER.error("Scheduler Exception", e);
    }

  }

  // only for facility message. Really should enforce front end to send in correct date format
  private static String trimDate(String date) {
    return (date.length() <= "yyyy-MM-dd".length()
      ? date
      : date.substring(0, "yyyy-MM-dd".length()));
  }

  public void scheduleProtocolExpireJob(BroadcastFacilityMessage facilityMessage, String timezone) {
    String jobId = facilityMessage.getId();
    if (timezone == null) {
      timezone = "America/New_York";
    }
    try {
      cleanJob(jobId, JOB_EXPIRE_GROUP, TRIGGER_EXPIRE_GROUP);

      JobDetail expireJob = newJob(GetPatientProtocolExpireJobClass())
        .withIdentity(jobId, JOB_EXPIRE_GROUP)
        .build();

      Date endDateLastMinute = getExpireTime(facilityMessage, timezone);
      java.util.Calendar cal = java.util.Calendar.getInstance();
      cal.setTime(endDateLastMinute);

      Date newScheduledEndTime = new Date(cal.getTime().getTime() + (1000 * 60 * 60 * 24));

      this.scheduleSimpleTriggerJob(expireJob, jobId, JOB_EXPIRE_GROUP, cal.getTime(), newScheduledEndTime);

    } catch (SchedulerException e) {
      LOGGER.error("Scheduler Exception", e);
    }

  }

  public static Date getExpireTime(BroadcastFacilityMessage facilityMessage, String timezone) {
    String endDateStr = trimDate(facilityMessage.getEndDate());
    Date endDate = DateParser.parse(endDateStr, "yyyy-MM-dd");
    Date endDateLastMinute = DateParser.getEndofDate(endDate, timezone);
    return endDateLastMinute;
  }

  /**
   * delete  jobs associated with a particular template in patientProtocol
   *
   * @param jobId the id of the patient protocol
   */
  public void delete(final String jobId) {
    try {
      cleanJob(jobId, JOB_FIRST_GROUP, TRIGGER_FIRST_GROUP);

      cleanJob(jobId, JOB_EXPIRE_GROUP, TRIGGER_EXPIRE_GROUP);
    } catch (SchedulerException e) {
      LOGGER.error("Scheduler Exception", e);
    }
  }


}
