package gov.va.mobilehealth.courier.scheduler;

import com.google.common.base.Optional;

import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.protocol.BroadcastFacilityMessage;
import gov.va.mobilehealth.courier.db.BroadcastFacilityMessageDAO;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import gov.va.mobilehealth.courier.utils.CamelProducerFactory;

import org.apache.camel.ProducerTemplate;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Date;
import java.util.List;

public class FacilityMessageInitJob implements Job {

  private static final Logger LOGGER = LoggerFactory.getLogger(FacilityMessageInitJob.class);
  protected JobExecutionException jobException;

  @Override
  public void execute(JobExecutionContext context)
          throws JobExecutionException {
    String jobName = context.getJobDetail().getKey().getName();
    
    try {
        // get patient protocol from protocol ID
        BroadcastFacilityMessageDAO broadcastFacilityMessageDAO = DAOFactory.createBroadcastFacilityMessageDAO();
        final Optional<BroadcastFacilityMessage> foundFacilityMessage = broadcastFacilityMessageDAO.findById(jobName);
        if (!foundFacilityMessage.isPresent()) {
          LOGGER.debug("Cannot find facility message: " + jobName);
          jobException = new JobExecutionException("PatientProtocolInitJob Cannot find patient protocol");
          //unschedule all triggers associated with this job
          // so that it does not run again
          jobException.setUnscheduleAllTriggers(true);
          throw jobException;
        }
        
        BroadcastFacilityMessage facilityMessage = foundFacilityMessage.get();
        
        Date nextTime = context.getNextFireTime();
        Date expireTime = BroadcastMessageScheduler.getExpireTime(facilityMessage, facilityMessage.getTimezone());
        String progress;
        if (nextTime == null || nextTime.after(expireTime)) {
            progress = BroadcastFacilityMessage.PROGRESS_DELIVERED;
        } else {
            progress = BroadcastFacilityMessage.PROGRESS_IN_PROGRESS;
        }
        broadcastFacilityMessageDAO.updateMessageProgress(jobName, progress);
        
        // get patient from patient Id
        final List<Patient> patientList = DAOFactory.createPatientDAO().findBroadcastPatientByFacility(facilityMessage.getAssigningAuthority());
        //send message to outbound queue
        ProducerTemplate outgoingMsgProducer = CamelProducerFactory.getOutgoingMsgProducer();
        
        if (outgoingMsgProducer != null) {
            for (Patient p : patientList) {
                CMSMessage cmsMsg = new CMSMessage(null, p, "", MessageType.OUT);
                String text = facilityMessage.getMessage();
                cmsMsg.setText(text);
    
                outgoingMsgProducer.sendBody(cmsMsg);
            }
        }
    } catch (Exception e) {
        LOGGER.error("Job execution exception", e);

        jobException = new JobExecutionException(e);
        // Set refire immediately to false for now. Need to 
        // revisit strategy in dealing with exceptions.
        jobException.setRefireImmediately(false);
        throw jobException;
    }
  }

}

