package gov.va.mobilehealth.courier.scheduler;

import gov.va.mobilehealth.courier.core.protocol.PatientProtocol;
import gov.va.mobilehealth.courier.core.protocol.Template;
import gov.va.mobilehealth.courier.core.protocol.TimeInterval;
import gov.va.mobilehealth.courier.core.protocol.Vital;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeDaysOfWeek;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeEverySoMany;
import gov.va.mobilehealth.courier.core.protocol.schedule.TypeOnce;
import gov.va.mobilehealth.courier.utils.DateParser;
import org.joda.time.DateTime;
import org.joda.time.Duration;
import org.joda.time.Period;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.quartz.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Date;

import static org.quartz.JobBuilder.newJob;
import static org.quartz.SimpleScheduleBuilder.simpleSchedule;
import static org.quartz.TriggerBuilder.newTrigger;

public class PatientProtocolScheduler extends SchedulerBase {
  public static final String JOB_FIRST_GROUP = "initProtocol";
  public static final String TRIGGER_FIRST_GROUP = "initProtocol";
  public static final String JOB_LAST_GROUP = "lastProtocol";
  public static final String TRIGGER_LAST_GROUP = "lastProtocol";
  public static final String JOB_REMINDER_GROUP = "reminderProtocol";
  public static final String TRIGGER_REMINDER_GROUP = "reminderProtocol";
  public static final String JOB_EXPIRE_GROUP = "expireProtocol";
  public static final String TRIGGER_EXPIRE_GROUP = "expireProtocol";
  private static final Logger LOGGER = LoggerFactory.getLogger(PatientProtocolScheduler.class);
  protected Class<? extends Job> patientProtocolInitJobClass;
  protected Class<? extends Job> patientProtocolReminderJobClass;
  protected Class<? extends Job> patientProtocolLastJobClass;

  public PatientProtocolScheduler(Scheduler scheduler) {
    this.scheduler = scheduler;
    this.patientProtocolInitJobClass = PatientProtocolInitJob.class;
    this.patientProtocolReminderJobClass = PatientProtocolReminderJob.class;
    this.patientProtocolLastJobClass = PatientProtocolLastJob.class;
  }

  public static String getJobId(final String protocolId, final int templateIndex) {
    return protocolId + "_" + templateIndex;
  }

  public static String[] splitJobName(String jobId) {
    if (jobId.contains("_")) {
      String[] parts = jobId.split("_");
      return parts;
    } else {
      throw new IllegalArgumentException("jobId format error: " + jobId);
    }
  }

  protected void SetPatientProtocolInitJobClass(Class<? extends Job> patientProtocolInitJobClass) {
    this.patientProtocolInitJobClass = patientProtocolInitJobClass;
  }

  protected Class<? extends Job> GetPatientProtocolInitJobClass() {
    return this.patientProtocolInitJobClass;
  }

  protected void SetPatientProtocolReminderJobClass(Class<? extends Job> patientProtocolReminderJobClass) {
    this.patientProtocolReminderJobClass = patientProtocolReminderJobClass;
  }

  protected Class<? extends Job> GetPatientProtocolReminderJobClass() {
    return this.patientProtocolReminderJobClass;
  }

  protected void SetPatientProtocolLastJobClass(Class<? extends Job> patientProtocolLastJobClass) {
    this.patientProtocolLastJobClass = patientProtocolLastJobClass;
  }

  protected Class<? extends Job> GetPatientProtocolLastJobClass() {
    return this.patientProtocolLastJobClass;
  }

  public void createReminders(PatientProtocol patientProtocol, int templateIndex) {
    String jobName = getJobId(patientProtocol.getId(), templateIndex);
    String jobGroup = JOB_REMINDER_GROUP;
    String triggerGroup = TRIGGER_REMINDER_GROUP;

    Template protocol = patientProtocol.getProtocol().getTemplates().get(templateIndex);
    Period period = protocol.getSchedule().getReminderInterval().createPeriod();
    int reminderRepeatCount = protocol.getSchedule().getReminderNumber() - 1;
    DateTimeFormatter fmt = DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss");

    try {
      cleanJob(jobName, jobGroup, triggerGroup);
      if (reminderRepeatCount < 0) {
        return;
      }
      DateTime now = new DateTime();
      DateTime later = now.plus(period);
      Duration duration = new Duration(now, later);
      String timeStr = later.toString(fmt);
      scheduleReminderJob(jobName, timeStr, reminderRepeatCount, duration);
    } catch (SchedulerException e) {
      LOGGER.error("SchedulerException", e);
    }

  }

  /* We need to loop through all templates in one protocol to schedule jobs
   *
   */
  public void update(PatientProtocol patientProtocol, String timezone) {
    for (int i = 0; i < patientProtocol.getProtocol().getTemplates().size(); i++) {
      update(patientProtocol, i, timezone);
    }
  }

  public void update(PatientProtocol patientProtocol, int templateIndex, String timezone) {

    String jobName = getJobId(patientProtocol.getId(), templateIndex);
    String jobGroup = JOB_FIRST_GROUP;

    String triggerName = getJobId(patientProtocol.getId(), templateIndex);
    String triggerGroup = TRIGGER_FIRST_GROUP;
    String cronScheduleStr = "";

    try {

      scheduleProtocolExpireJob(patientProtocol, templateIndex, timezone);

      Template template = patientProtocol.getProtocol().getTemplates().get(templateIndex);
      cleanJob(jobName, JOB_FIRST_GROUP, TRIGGER_FIRST_GROUP);
      cleanJob(jobName, JOB_REMINDER_GROUP, TRIGGER_REMINDER_GROUP);

      JobDetail job = newJob(GetPatientProtocolInitJobClass())
        .withIdentity(jobName, jobGroup)
        .build();

      String scheduleTimeStr = ProtocolChronGenerator.getStartDateTime(template);
      Date scheduleTime = DateParser.parseDateWithTimeZone(scheduleTimeStr, timezone);

      String scheduledEndTimeStr = ProtocolChronGenerator.getLastDateTime(template);

      Date scheduledEndTime = DateParser.parseDateWithTimeZone(scheduledEndTimeStr, timezone);
      Date newScheduledEndTime = new Date(scheduledEndTime.getTime() + (1000 * 60 * 60 * 24));


      if (template.getSchedule().getScheduleType() instanceof TypeOnce) {
        scheduleSimpleTriggerJob(job, jobName, jobGroup, scheduleTime, newScheduledEndTime);
        return;
      } else if (template.getSchedule().getScheduleType() instanceof TypeEverySoMany) {

        TimeInterval timeInterval = ((TypeEverySoMany) template.getSchedule().getScheduleType()).getInterval();

        scheduleCalendarIntervalTriggerJob(job, jobName, jobGroup, scheduleTime, newScheduledEndTime, timeInterval, timezone);

      } else if (template.getSchedule().getScheduleType() instanceof TypeDaysOfWeek) {
        cronScheduleStr = ProtocolChronGenerator.getCronSchedule(template.getSchedule());
        scheduleCronTriggerJob(job, jobName, jobGroup, cronScheduleStr, newScheduledEndTime, timezone);
      }

      if (template instanceof Vital) {
        cleanJob(jobName, JOB_LAST_GROUP, TRIGGER_LAST_GROUP);

        JobDetail lastJob = newJob(GetPatientProtocolLastJobClass())
          .withIdentity(jobName, JOB_LAST_GROUP)
          .build();

        String runTimeStr = ProtocolChronGenerator.getLastDateTime(template);
        Date lastScheduleTime = DateParser.parseDateWithTimeZone(runTimeStr, timezone);
        scheduleSimpleTriggerJob(lastJob, jobName, JOB_LAST_GROUP, lastScheduleTime, newScheduledEndTime);
      }
    } catch (SchedulerException e) {
      LOGGER.error("SchedulerException", e);
    }
  }

  public void scheduleReminderJob(String jobName, String schedStr, int repeatCount, Duration interval) {
    try {
      repeatCount = repeatCount < 0 ? 0 : repeatCount;

      JobDetail jobReminder = newJob(GetPatientProtocolReminderJobClass())
        .withIdentity(jobName, JOB_REMINDER_GROUP)
        .build();

      Date scheduledTime = DateParser.parseAttDate(schedStr);

      SimpleScheduleBuilder sb = simpleSchedule().withRepeatCount(repeatCount);
      if (interval.getStandardMinutes() < 1440) {
        sb.withIntervalInMinutes((int) interval.getStandardMinutes());
      } else {
        sb.withIntervalInHours((int) interval.getStandardHours());
      }
      Trigger trigger = newTrigger().withIdentity(jobName, TRIGGER_REMINDER_GROUP).startAt(scheduledTime).withSchedule(sb).build();

      scheduler.scheduleJob(jobReminder, trigger);

    } catch (SchedulerException e) {
      LOGGER.error("SchedulerException", e);
    }
  }

  public void scheduleProtocolExpireJob(PatientProtocol protocol, int templateIndex, String timezone) {
    String jobId = getJobId(protocol.getId(), templateIndex);
    Template template = protocol.getProtocol().getTemplates().get(templateIndex);
    try {
      cleanJob(jobId, JOB_EXPIRE_GROUP, TRIGGER_EXPIRE_GROUP);

      JobDetail expireJob = newJob(PatientProtocolExpireJob.class)
        .withIdentity(jobId, JOB_EXPIRE_GROUP)
        .build();

      String endDateStr = template.getEndDate();
      Date endDate = DateParser.parse(endDateStr, "yyyy-MM-dd");
      Date endDateLastMinute = DateParser.getEndofDate(endDate, timezone);
      java.util.Calendar cal = java.util.Calendar.getInstance();
      cal.setTime(endDateLastMinute);

      Date newScheduledEndTime = new Date(cal.getTime().getTime() + (1000 * 60 * 60 * 24));

      this.scheduleSimpleTriggerJob(expireJob, jobId, JOB_EXPIRE_GROUP, cal.getTime(), newScheduledEndTime);

    } catch (SchedulerException e) {
      LOGGER.error("SchedulerException", e);
    }
  }

  /**
   * specify job id and delete reminder job
   *
   * @param jobId id of the job to be deleted
   */
  public void deleteReminderJobs(final String jobId) {
    try {
      cleanJob(jobId, JOB_REMINDER_GROUP, TRIGGER_REMINDER_GROUP);
    } catch (SchedulerException e) {
      LOGGER.error("SchedulerException", e);
    }
  }

  /**
   * delete all jobs associated with the patientProtocol
   *
   * @param patientProtocol patient protocol
   */
  public void delete(final PatientProtocol patientProtocol) {
    for (int i = 0; i < patientProtocol.getProtocol().getTemplates().size(); i++) {
      delete(patientProtocol.getId(), i);
    }
  }


  /**
   * delete  jobs associated with a particular template in patientProtocol
   *
   * @param patientProtocolId the id of the patient protocol
   * @param index             the index of the template in template list
   */
  public void delete(final String patientProtocolId, final int index) {
    String jobId = getJobId(patientProtocolId, index);
    try {
      cleanJob(jobId, JOB_FIRST_GROUP, TRIGGER_FIRST_GROUP);

      // delete interval reminders
      cleanJob(jobId, JOB_REMINDER_GROUP, TRIGGER_REMINDER_GROUP);

      // deletes last reminider message schedule
      cleanJob(jobId, JOB_LAST_GROUP, TRIGGER_LAST_GROUP);

      cleanJob(jobId, JOB_EXPIRE_GROUP, TRIGGER_EXPIRE_GROUP);
    } catch (SchedulerException e) {
      LOGGER.error("SchedulerException", e);
    }
  }


}
