package gov.va.mobilehealth.courier.service;

import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserFactory;
import com.google.common.base.Optional;

import gov.va.mobilehealth.courier.core.Facility;
import gov.va.mobilehealth.courier.core.protocol.BroadcastFacilityMessage;
import gov.va.mobilehealth.courier.core.protocol.Comment;
import gov.va.mobilehealth.courier.db.BroadcastFacilityMessageDAO;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.resources.helper.BadRequestException;
import gov.va.mobilehealth.courier.resources.helper.ErrorCode;
import gov.va.mobilehealth.courier.scheduler.BroadcastMessageScheduler;
import gov.va.mobilehealth.courier.utils.DateParser;

import java.io.IOException;
import java.util.List;

import javax.servlet.ServletContext;

import org.quartz.SchedulerException;
import org.quartz.ee.servlet.QuartzInitializerListener;
import org.quartz.impl.StdSchedulerFactory;

/**
 * The Service performs the business logic and makes calls to the DAO.  It should be unaware of web/http specifics, leaving
 * that to Resource class.  This also improves our ability to peform integration tests without having to deploy.
 * Created by clarkee on 10/4/14.
 */
public class BroadCastFacilityMessageService {
  private final BroadcastFacilityMessageDAO broadcastFacilityMessageDAO;
  public final static String MSG_INCORRECT_ASSIGNING_AUTHORITY = "Assigning Authority Does Not Match User's Assigning Authority";
  public final static String MSG_NOT_FOUND = "Facility Message Not Found";
  public final static String FACILITY_NOT_FOUND = "Facility Not Found";
  private MhpUser mhpUser;

  public BroadCastFacilityMessageService() {
    broadcastFacilityMessageDAO = DAOFactory.createBroadcastFacilityMessageDAO();
  }

  protected void setMhpUser(MhpUser user) {
    mhpUser = user;
  }
    
  protected MhpUser getMhpUser() {
    if (mhpUser == null) {
        mhpUser = MhpUserFactory.createFromSecurityContext();
    }
    return mhpUser;
  }
  
  public List<BroadcastFacilityMessage> getMessages() throws Exception {

    final List<BroadcastFacilityMessage> messages = broadcastFacilityMessageDAO.findByAssigningAuthority(getMhpUser().getUserIdentifier().getAssigningAuthority());
    return messages;
  }

  public BroadcastFacilityMessage updateBroadcastFacilityMessageTemplate(ServletContext context, BroadcastFacilityMessage message) throws Exception {
    //ensure assigningAuthority is null or matches security context
    if (message.getAssigningAuthority() == null || message.getAssigningAuthority().trim().equals("")) {
      message.setAssigningAuthority(getMhpUser().getUserIdentifier().getAssigningAuthority());
    } else {
      validateMessageAssigningAuthority(getMhpUser(), message);
    }

    String today = DateParser.getTodaysDate(null, null);
    String cmsUser = getMhpUser().getFirstName() + " " + getMhpUser().getLastName();

    Optional<Facility> result = DAOFactory.createFacilityDAO().findById(message.getAssigningAuthority());
    if (!result.isPresent()) {
        throw new BadRequestException(ErrorCode.FACILITY_NOT_FOUND, FACILITY_NOT_FOUND);
    }

    message.setTimezone(result.get().getTimezone());

    // if new message, set created by and created on field
    if (message.getId() == null) {
      message.setCreateBy(cmsUser);
      if (message.getCreateDate() == null) {
        message.setCreateDate(today);
      }
      message.setProgress(BroadcastFacilityMessage.PROGRESS_PENDING);
      message.setCreatedDateUTC(DateParser.getNow("UTC", null));
      message.setEnabled(true);
      // get the message ID
      message = broadcastFacilityMessageDAO.saveOrUpdateFacilityMessage(message);
    } else {
      // add last edit info
      String time = DateParser.getCurrentTime(null, null);
      Comment comment = new Comment(cmsUser, today, time);
      message.setComments(comment);
    }

    // schedule message job and set message progress
    message = scheduleMessageJobs(context, message);

    return broadcastFacilityMessageDAO.saveOrUpdateFacilityMessage(message);
  }

  protected BroadcastMessageScheduler getBroadcastMessageScheduler(ServletContext context) throws SchedulerException {
      StdSchedulerFactory factory = (StdSchedulerFactory) context
              .getAttribute(QuartzInitializerListener.QUARTZ_FACTORY_KEY);
      BroadcastMessageScheduler messageScheduler = new BroadcastMessageScheduler(factory.getScheduler());
      return messageScheduler;
  }

  public BroadcastFacilityMessage scheduleMessageJobs(ServletContext context, BroadcastFacilityMessage message) throws IOException, BadRequestException, SchedulerException {
      BroadcastMessageScheduler messageScheduler = getBroadcastMessageScheduler(context);

      if (message.isEnabled()) {
          messageScheduler.update(message, message.getTimezone());

          // Since the start date is always today or later,
          // delivery status is always pending
          message.setProgress(BroadcastFacilityMessage.PROGRESS_PENDING);
      } else {
          messageScheduler.delete(message.getId());
      }
      return message;
  }

  public void remove(BroadcastFacilityMessage message) throws IOException {
    broadcastFacilityMessageDAO.removeFromDB(message.getId());
  }

  public BroadcastFacilityMessage getById(String id) throws IOException, BadRequestException {
    try {
      Optional<BroadcastFacilityMessage> message = broadcastFacilityMessageDAO.findById(id);
      //ensure message assigning authority matches message
      validateMessageAssigningAuthority(getMhpUser(), message);
      return message.get();

    } catch (IllegalArgumentException ex) {
      throw new BadRequestException(ErrorCode.MSG_NOT_FOUND, MSG_NOT_FOUND);
    }
  }

  private void validateMessageAssigningAuthority(MhpUser user, Optional<BroadcastFacilityMessage> message) throws BadRequestException {
    if (message.isPresent()) validateMessageAssigningAuthority(user, message.get());
  }

  private void validateMessageAssigningAuthority(MhpUser user, BroadcastFacilityMessage message) throws BadRequestException {
    if (!message.getAssigningAuthority().equals(user.getUserIdentifier().getAssigningAuthority())) {
      throw new BadRequestException(ErrorCode.MSG_INCORRECT_ASSIGNING_AUTHORITY, MSG_INCORRECT_ASSIGNING_AUTHORITY);
    }
  }
}
