package gov.va.mobilehealth.courier.service;

import java.io.IOException;
import java.util.Date;
import java.util.List;

import javax.servlet.ServletContext;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import org.aeonbits.owner.ConfigFactory;
import org.quartz.SchedulerException;
import org.quartz.ee.servlet.QuartzInitializerListener;
import org.quartz.impl.StdSchedulerFactory;

import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.google.common.base.Optional;

import gov.va.mobilehealth.courier.bcm.appointments.AppointmentReminderService;
import gov.va.mobilehealth.courier.bcm.appointments.EdipiIdentifier;
import gov.va.mobilehealth.courier.core.ErrorResponse;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.protocol.Comment;
import gov.va.mobilehealth.courier.core.protocol.Motivational;
import gov.va.mobilehealth.courier.core.protocol.PatientProtocol;
import gov.va.mobilehealth.courier.core.protocol.PatientProtocolStatus;
import gov.va.mobilehealth.courier.core.protocol.Protocol;
import gov.va.mobilehealth.courier.core.protocol.Template;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.PatientDAO;
import gov.va.mobilehealth.courier.db.PatientProtocolDAO;
import gov.va.mobilehealth.courier.db.ProtocolDAO;
import gov.va.mobilehealth.courier.resources.helper.ErrorMessageConfigure;
import gov.va.mobilehealth.courier.scheduler.PatientProtocolScheduler;
import gov.va.mobilehealth.courier.utils.DateParser;

/**
 * Created by clarkee on 11/7/14.
 */
public class PatientsService {

    static ErrorMessageConfigure errorMessageConfig = getErrorMessageConfig();

    static ErrorMessageConfigure getErrorMessageConfig() {
      return ConfigFactory.create(ErrorMessageConfigure.class, System.getProperties(), System.getenv());
    }

    protected PatientProtocolScheduler getPatientProtocolScheduler(ServletContext context) throws SchedulerException {
        StdSchedulerFactory factory = (StdSchedulerFactory) context
                .getAttribute(QuartzInitializerListener.QUARTZ_FACTORY_KEY);
        PatientProtocolScheduler patientProtocolScheduler = new PatientProtocolScheduler(factory.getScheduler());
        return patientProtocolScheduler;
    }

  //For now only minimal updates to the patient service
  //TODO: Extract patient updates from the staff and veteran registration resources, and get the logic in here, too much duplication of code...
  public void updatePatient(Patient patient){
    removeApptRemindersOnUnsubscribe(patient);
  }

  private void removeApptRemindersOnUnsubscribe(Patient patient) {
    if (!patient.getPreference().isReceiveApptReminder()){
      AppointmentReminderService reminderService = new AppointmentReminderService();
      reminderService.removeExistingReminders(new EdipiIdentifier(patient.getPatientIdentifier().getUniqueId()));
    }
  }

  public void reschedulePatientProtocols(Patient patient, ServletContext context) throws IOException, SchedulerException {
      PatientProtocolDAO patientProtocolDAO = DAOFactory.createPatientProtocolDAO();
      List<PatientProtocol> protocolList = patientProtocolDAO.findByPatientId(patient.getId());
      for (PatientProtocol p : protocolList) {
          if (PatientProtocolStatus.ACTIVE.equals(p.getStatus())) {
              String timeZone = patient.getPreference().getTimeZone();
              getPatientProtocolScheduler(context).update(p, timeZone);
          }
      }
  }

  public void updatePatientProtocol(String patientId, PatientProtocol patientProtocol, MhpUser user, ServletContext context) throws Exception {
      PatientProtocolDAO patientProtocolDAO = DAOFactory.createPatientProtocolDAO();
      PatientDAO patientDAO = DAOFactory.createPatientDAO();
      ProtocolDAO protocolDAO = DAOFactory.createProtocolDAO();
      PatientProtocolScheduler patientProtocolScheduler = getPatientProtocolScheduler(context);
      try {
          List<PatientProtocol> patientProtocols = patientProtocolDAO.findByPatientIdAndProtocolName(patientId, patientProtocol.getProtocol().getName());
          if (patientProtocols.size() == 1) {
            if (!patientProtocols.get(0).getId().equals(patientProtocol.getId())) {
              throw new WebApplicationException(Response.status(Response.Status.CONFLICT)
                .entity(new ErrorResponse(1001, "A patient protocol with that name already exists for this patient. Please update the Protocol Name.")).type(MediaType.APPLICATION_JSON).build());

            }
          }
          String today = DateParser.getTodaysDate(null, null);
          String time = DateParser.getCurrentTime(null, null);

          String cmsUser = user.getFirstName() + " " + user.getLastName();

          validatePatientProtocol(patientProtocol);

          if (null == patientProtocol.getId()) {
            patientProtocol.setAssignedDate(today);
            Protocol protocol = protocolDAO.findById(patientProtocol.getProtocol().getId()).get();
            PatientProtocol pProtocol = new PatientProtocol(patientId, protocol);
            //get audit trail
            patientProtocol.addAuditTrail(pProtocol, today, time, cmsUser);
          } else {
            Optional<PatientProtocol> existingOne = patientProtocolDAO.findById(patientProtocol.getId());

            if (existingOne.isPresent()) {
              PatientProtocol existingPatientProtocol = existingOne.get();
              if (PatientProtocolStatus.ACTIVE.equals(patientProtocol.getStatus())) {
                if (PatientProtocolStatus.EXPIRED.equals(existingPatientProtocol.getStatus())
                  || PatientProtocolStatus.REMOVED.equals(existingPatientProtocol.getStatus())) {
                  // if patientProtocol status is changed from EXPIRED/REMOVED
                  // to ACTIVE, we need to reset assigned date
                  patientProtocol.setAssignedDate(today);
                  patientProtocol.setAllTemplateStatus(patientProtocol.getStatus());
                }
              } else if (PatientProtocolStatus.REMOVED.equals(patientProtocol.getStatus())) {
                if (PatientProtocolStatus.ACTIVE.equals(existingPatientProtocol.getStatus()) ||
                        PatientProtocolStatus.PAUSED.equals(existingPatientProtocol.getStatus()) ||
                        PatientProtocolStatus.RESUME.equals(existingPatientProtocol.getStatus())) {
                  // if patientProtocol is removed,
                  // we need to make sure all associated scheduled jobs are cleared
                  patientProtocolScheduler.delete(patientProtocol);
                  patientProtocol.setAllTemplateStatus(patientProtocol.getStatus());
                }
              }
              //get audit trail
              patientProtocol.addAuditTrail(existingPatientProtocol, today, time, cmsUser);
            }
          }
          //always
          patientProtocol.setLastEditedDate(today);
          patientProtocolDAO.save(patientProtocol);
          // only update scheduler if the protocol is active
          if (PatientProtocolStatus.ACTIVE.equals(patientProtocol.getStatus())) {
            final Optional<Patient> pat = patientDAO.findById(patientId);
            String timeZone = null;
            if (pat.isPresent()) {
              Patient patient = pat.get();
              timeZone = patient.getPreference().getTimeZone();
            }
            patientProtocolScheduler.update(patientProtocol, timeZone);
          }
        } catch (NullPointerException e) {
          throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
            .entity(new ErrorResponse(e.getMessage())).type(MediaType.APPLICATION_JSON).build());
        }
  }

  public void deletePatientProtocol(String patientId, String protocolId, ServletContext context) throws Exception {
      try {
        //        patientProtocolDAO.delete(protocolId, comment);
          // temporarily comment out post data.
          Comment c = new Comment();
          c.setContent("test");
          PatientProtocolDAO patientProtocolDAO = DAOFactory.createPatientProtocolDAO();
          patientProtocolDAO.delete(protocolId, c);
          Optional<PatientProtocol> patientProtocol = patientProtocolDAO.findById(protocolId);
          if (patientProtocol.isPresent()) {
              getPatientProtocolScheduler(context).delete(patientProtocol.get());
          }
      } catch (NullPointerException e) {
          throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
            .entity(new ErrorResponse(e.getMessage())).type(MediaType.APPLICATION_JSON).build());
      }
  }

  // do some data validation
  private void validatePatientProtocol(PatientProtocol pProtocol) throws WebApplicationException, IOException {
    Protocol protocol = pProtocol.getProtocol();

    final Optional<Patient> pat = DAOFactory.createPatientDAO().findById(pProtocol.getPatientId());
    String timeZone = null;
    if (pat.isPresent()) {
      Patient patient = pat.get();
      timeZone = patient.getPreference().getTimeZone();
    }

    // get today's date from the date parser library
    String todayStr = DateParser.getTodaysDate(timeZone, "yyyy-MM-dd");
    Date today = DateParser.parse(todayStr, "yyyy-MM-dd");

    for (Template t : protocol.getTemplates()) {
      Date templateEndDate = DateParser.parse(t.getEndDate(), "yyyy-MM-dd");

      // test if template's end date is in the past and throw a 406 exception if so
      if( today.compareTo(templateEndDate) > 0 ) {
        throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
          .entity(new ErrorResponse("End Date cannot be in the past.")).type(MediaType.APPLICATION_JSON).build());
      }

      if (t instanceof Motivational) {
        Motivational motivational = (Motivational) t;
        if ((motivational.getServiceMessage() == null) || (motivational.getServiceMessage().size() == 0)) {
          throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
            .entity(new ErrorResponse("Missing service message")).type(MediaType.APPLICATION_JSON).build());
        }
      }
    }
  }
  
  public void checkPatientConsent(String patientId) throws WebApplicationException, IOException
  {
      PatientDAO patientDAO = DAOFactory.createPatientDAO();
      Optional<Patient> p = patientDAO.findById(patientId);
      if (p.isPresent())
      {
          Patient patient = p.get();
          if (patient.getConsent() != Patient.STAFF_ASSISTED_CONSENT)
          {
              throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
                      .entity(new ErrorResponse(errorMessageConfig.patientNotConsentToProtocols())).type(MediaType.APPLICATION_JSON).build());
          }
      } else
      {
          throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST)
              .entity(new ErrorResponse(errorMessageConfig.patientNotFound())).type(MediaType.APPLICATION_JSON).build());
      }
  }

}
