package gov.va.mobilehealth.courier.utils;

import gov.va.mobilehealth.courier.bcm.appointments.scheduler.RefreshPatientAppointmentsConsumer;
import gov.va.mobilehealth.courier.resources.helper.IncomingMessageProcessor;
import gov.va.mobilehealth.courier.resources.helper.OutgoingMessageProcessor;
import org.aeonbits.owner.ConfigFactory;
import org.apache.camel.CamelContext;
import org.apache.camel.Consumer;
import org.apache.camel.Endpoint;
import org.apache.camel.ProducerTemplate;
import org.apache.camel.component.jms.JmsComponent;
import org.apache.camel.impl.DefaultCamelContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.jms.QueueConnectionFactory;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

public class CamelContextInitializerListener implements ServletContextListener {

  public static final String JMS_COMPONENT_NAME = "annie-jms";
  public static final String JMS_CONNECTION_FACTORY = "jms/AnnieJMSConnectionFactory";
  public static final String JMS_SERVER = "AnnieJMSServer";
  public static final String JMS_MODULE = "AnnieJMSModule";

  private static final Logger LOGGER = LoggerFactory.getLogger(CamelContextInitializerListener.class);
  CamelContext camelContext = new DefaultCamelContext();
  ProducerTemplate incomingQueueProducer;
  ProducerTemplate outgoingQueueProducer;
  Consumer incomingMessageConsumer;
  Consumer outgoingMessageConsumer;

  ProducerTemplate refreshPatientAppointmentsProducer;
  Consumer refreshPatientAppointmentsConsumer;

  @Override
  public void contextInitialized(ServletContextEvent servletContextEvent) {

    ServerConfig serverConfig = ConfigFactory.create(ServerConfig.class, System.getProperties(), System.getenv());

    if (serverConfig.jmsEnabled()) {
      try {
        initCamelContext();
        initAnnieCamelContext(servletContextEvent.getServletContext());
        initAppointmentsCamelContext(servletContextEvent.getServletContext());
      } catch (NamingException e) {
        LOGGER.error("Failed to initialize camel context", e);
        throw new RuntimeException("Failed to initialize camel context.", e);
      }
    }
  }

  private void initCamelContext() throws NamingException {
    // get weblogic context
    Context ctx = getInitialContext();

    //get queue connection factory and add it as JMS component to camel
    QueueConnectionFactory qconFactory = (QueueConnectionFactory) ctx.lookup(JMS_CONNECTION_FACTORY);
    camelContext.addComponent(JMS_COMPONENT_NAME, JmsComponent.jmsComponentAutoAcknowledge(qconFactory));
  }

  private void initAnnieCamelContext(ServletContext servletContext) {
    final String JMS_INBOUND_QUEUE = "AnnieJMSInboundQueue";
    final String INBOUND_QUEUE = JMS_SERVER + "/" + JMS_MODULE + "!" + JMS_INBOUND_QUEUE;

    final String JMS_OUTBOUND_QUEUE = "AnnieJMSOutboundQueue";
    final String OUTBOUND_QUEUE = JMS_SERVER + "/" + JMS_MODULE + "!" + JMS_OUTBOUND_QUEUE;

    try {
      LOGGER.debug("Initializing camel context...");

      //create a producer to send a message on inbound queue
      incomingQueueProducer = camelContext.createProducerTemplate();
      incomingQueueProducer.setDefaultEndpointUri(JMS_COMPONENT_NAME + ":" + INBOUND_QUEUE);

      //create a producer to send a message on outbound queue
      outgoingQueueProducer = camelContext.createProducerTemplate();
      outgoingQueueProducer.setDefaultEndpointUri(JMS_COMPONENT_NAME + ":" + OUTBOUND_QUEUE);

      CamelProducerFactory.init(incomingQueueProducer, outgoingQueueProducer);

      // create a event-driven consumer to listen to messages on the inbound queue
      Endpoint incomingEndpoint = camelContext.getEndpoint(JMS_COMPONENT_NAME + ":" + INBOUND_QUEUE);

      incomingMessageConsumer = incomingEndpoint.createConsumer(new IncomingMessageProcessor(servletContext));

      // create a event-driven consumer to listen to message on the outbound queue
      Endpoint outgoingEndpoint = camelContext.getEndpoint(JMS_COMPONENT_NAME + ":" + OUTBOUND_QUEUE);
      outgoingMessageConsumer = outgoingEndpoint.createConsumer(new OutgoingMessageProcessor());

      //start the event-driven consumer
      incomingMessageConsumer.start();

      //start the event-driven consumer
      outgoingMessageConsumer.start();

      //start the camel context
      camelContext.start();

      LOGGER.info("camel connection was initialized successfully.");

    } catch (Exception e) {
      LOGGER.error("Failed to initialize camel context", e);
      throw new RuntimeException("Failed to initialize camel context.", e);
    }
  }

  private void initAppointmentsCamelContext(ServletContext servletContext) {
    final String JMS_REFRESH_PATIENT_APPOINTMENTS_QUEUE = "AnnieJMSRefreshAppointmentsQueue";
    final String REFRESH_PATIENT_APPOINTMENTS_QUEUE = JMS_SERVER + "/" + JMS_MODULE + "!" + JMS_REFRESH_PATIENT_APPOINTMENTS_QUEUE;

    try {
      LOGGER.debug("Initializing Appointments camel context...");

      //create a producer to send a message on inbound queue
      refreshPatientAppointmentsProducer = camelContext.createProducerTemplate();
      refreshPatientAppointmentsProducer.setDefaultEndpointUri(JMS_COMPONENT_NAME + ":" + REFRESH_PATIENT_APPOINTMENTS_QUEUE);

      CamelProducerFactory.initAppointmentReminders(refreshPatientAppointmentsProducer);

      // create a event-driven consumer to listen to messages on the inbound queue
      Endpoint incomingEndpoint = camelContext.getEndpoint(JMS_COMPONENT_NAME + ":" + REFRESH_PATIENT_APPOINTMENTS_QUEUE);
      refreshPatientAppointmentsConsumer = incomingEndpoint.createConsumer(new RefreshPatientAppointmentsConsumer(servletContext));

      //start the event-driven consumer
      refreshPatientAppointmentsConsumer.start();

      //start the camel context
      camelContext.start();

      LOGGER.info("camel connection was initialized successfully.");

    } catch (Exception e) {
      LOGGER.error("Failed to initialize camel context", e);
      throw new RuntimeException("Failed to initialize camel context.", e);
    }
  }

  @Override
  public void contextDestroyed(ServletContextEvent servletContextEvent) {
    try {
      stopService(incomingQueueProducer);
      stopService(outgoingQueueProducer);
      stopService(refreshPatientAppointmentsProducer);

      stopService(incomingMessageConsumer);
      stopService(outgoingMessageConsumer);
      stopService(refreshPatientAppointmentsConsumer);

      camelContext.stop();
    } catch (Exception e) {
      LOGGER.error("Exception", e);
    }
  }

  private void stopService(org.apache.camel.Service service) throws Exception {
    if (service != null){
      service.stop();
    }
  }

  private Context getInitialContext() throws NamingException {
    return new InitialContext();
  }

}
