package gov.va.mobilehealth.courier.veteran.resources;

import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserFactory;
import com.google.common.base.Optional;

import gov.va.mobilehealth.courier.core.*;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.PatientDAO;
import gov.va.mobilehealth.courier.db.PatientMessageHistoryDAO;
import gov.va.mobilehealth.courier.resources.helper.IncomingMessageProcessor;
import gov.va.mobilehealth.courier.utils.CamelProducerFactory;
import gov.va.mobilehealth.courier.utils.DateParser;
import gov.va.mobilehealth.courier.veteran.resources.helper.VeteranResourceHelper;

import org.apache.camel.ProducerTemplate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.UriInfo;

import java.io.IOException;
import java.util.List;

import org.codehaus.enunciate.Facet;

@Service
@Path("my/messages")
@Produces(MediaType.APPLICATION_JSON)
@Facet( name = "private")
public class VeteranMessageResource {

  final static String DATETIME_FORMAT = "yyyy-MM-dd HH:mm:ss";
  private static final Logger LOGGER = LoggerFactory.getLogger(VeteranMessageResource.class);
  private final PatientDAO patientDAO;
  private final PatientMessageHistoryDAO patientMessageHistoryDAO;
  private final ProducerTemplate incomingMsgProducer;
  VeteranResourceHelper veteranResourceHelper = new VeteranResourceHelper();

  public VeteranMessageResource() {
    this.patientDAO = DAOFactory.createPatientDAO();
    this.patientMessageHistoryDAO = DAOFactory.createPatientMessageHistoryDAO();
    this.incomingMsgProducer = CamelProducerFactory.getIncomingMsgProducer();
  }

  /**
   * List veteran's message history
   *
   * <pre>
   * request:
   *   vet/service/my/messages
   *
   * response:
   *   message list array
   *   
   *  [
   *    {
   *      "networkSubmitTime": "2014-11-13 16:36:26",
   *      "_id": "5464de0a0cf2d5b9916902a7",
   *      "type": "Out",
   *      "date": "2014-11-13",
   *      "time": "11:36:26",
   *      "text": "Hi, this is Annie.  Please text the word \"START\" to begin receiving messages from me. I look forward to hearing from you. http://bit.ly/1l4mk2m",
   *      "destination": "17031112121",
   *      "alertLevel": null,
   *      "serviceName": null,
   *      "timeZone": "America/New_York",
   *      "adjustedSubmitTime": "2014-11-13 11:36:26"
   *    },
   *    {
   *      "networkSubmitTime": "2014-11-13 16:36:31",
   *      "_id": "5464de0f0cf2d5b9916902a8",
   *      "type": "In",
   *      "date": "2014-11-13",
   *      "time": "11:36:31",
   *      "text": "Help",
   *      "destination": "Annie",
   *      "alertLevel": null,
   *      "serviceName": null,
   *      "timeZone": "America/New_York",
   *      "adjustedSubmitTime": "2014-11-13 11:36:31"
   *    }
   *  ]
   * </pre>
   *
   */
  @GET
  public List<PatientMessageHistory> listVeteranMessageHistory(@Context UriInfo info, @Context HttpHeaders headers) throws IOException {
	String patientId = veteranResourceHelper.getPatientId();
    return patientMessageHistoryDAO.findByPatientId(patientId, true);
  }

  /**
   * List veteran's message history
   *
   * <pre>
   * request:
   *   vet/service/my/messages/new
   * request payload:
   *   {
   *       message: "start"
   *   }
   *   
   *   message: message body
   *
   * response:
   *   204 No Content
   * </pre>
   *
   */
  @POST
  @Path("/new")
  @Consumes(MediaType.APPLICATION_JSON)
  public void receiveMessageWithFormParams(SMS smsMesssage, @Context UriInfo info, @Context HttpHeaders headers) throws IOException {
    LOGGER.debug("****In the veteran message recieving resource, the message text is: " + smsMesssage.getMessage());
    if (smsMesssage != null && (smsMesssage.getMessage() != null && !smsMesssage.getMessage().isEmpty())) {
      processReceivedMessage(smsMesssage.getMessage());
    }
  }

  private void processReceivedMessage(String text) throws IOException {
    LOGGER.info("Received SMS Message from network provider");
    PatientPreference patientPreference = getPatient().getPreference();
    String source = patientPreference.getPhoneNumber();
    String timezone = patientPreference.getTimeZone();
    String[] msgDeliveryType = {IncomingMessageProcessor.APP_NAME};
    PatientIncomingMessage incomingMessage = new PatientIncomingMessage(source, IncomingMessageProcessor.APP_NAME);
    String nowTime = DateParser.getNow(null, null);
    incomingMessage.setNetworkSubmitTime(DateParser.convertTimeZone(nowTime, null, "UTC", null));
    incomingMessage.setClientSubmitTime(incomingMessage.getNetworkSubmitTime());
    incomingMessage.setTimeZone(timezone);
    incomingMessage.setAdjustedSubmitTime(DateParser.convertTimeZone(nowTime, null, timezone, null));
    incomingMessage.setMsgDeliveryType(msgDeliveryType);

    incomingMessage.setText(text);

    LOGGER.debug("****In the veteran message receiving resource, the message text is: {}", incomingMessage.getText());
    incomingMessage.setType(MessageType.IN);

    LOGGER.debug("Sending message to Inbound queue: {}", incomingMessage);
    incomingMsgProducer.sendBody(incomingMessage);
    LOGGER.info("Done Sending message to Inbound queue");

  }

  private Patient getPatient() throws IOException {
    Patient patient = null;
    MhpUser user = MhpUserFactory.createFromSecurityContext();

    UserIdentifier patientIdentifier = new UserIdentifier();
    patientIdentifier.setAssigningAuthority(user.getPatient().getPatientIdentifier().getAssigningAuthority());
    patientIdentifier.setUniqueId(user.getPatient().getPatientIdentifier().getUniqueId());
    Optional<Patient> p = patientDAO.findByPatientIdentifier(patientIdentifier);
    if (p.isPresent()) {
      patient = p.get();
    }
    return patient;
  }

}
