package gov.va.mobilehealth.courier.veteran.resources;

import gov.va.mobilehealth.courier.core.*;
import gov.va.mobilehealth.courier.db.DAOFactory;
import gov.va.mobilehealth.courier.db.PatientResponseServiceDataDAO;
import gov.va.mobilehealth.courier.veteran.resources.helper.VeteranResourceHelper;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.*;

import java.util.List;

import org.codehaus.enunciate.Facet;

@Service
@Path("my/readings")
@Produces(MediaType.APPLICATION_JSON)
@Facet( name = "private")
public class VeteranResponseResource {

  private static final Logger LOGGER = LoggerFactory.getLogger(VeteranResponseResource.class);
  private final PatientResponseServiceDataDAO patientResponseServiceDataDAO;
  VeteranResourceHelper veteranResourceHelper = new VeteranResourceHelper();

  public VeteranResponseResource() {
    this.patientResponseServiceDataDAO = DAOFactory.createPatientResponseServiceDataDAO();
  }

  /**
   * Get veteran's vital readings history
   *
   * <pre>
   * request:
   *   vet/service/my/readings
   *
   * response:
   *   vital readings array
   *   
   * [
   *   {
   *     "_id": "3291354e086a53bdd9c9d39e",
   *     "patientResponseReceivedTime": "2014-03-22 11:00:00",
   *     "patientId": "52b9f1519d16ccbfab2fcff4",
   *     "alertLevel": null,
   *     "patientServiceReading": {
   *       "serviceName": "Blood Pressure",
   *       "serviceId": "52e829f9bf890cf9ee685d85",
   *       "readings": [
   *         {
   *           "name": "Systolic",
   *           "value": 120,
   *           "unit": "mmHg"
   *         },
   *         {
   *           "name": "Diastolic",
   *           "value": 77,
   *           "unit": "mmHg"
   *         }
   *       ]
   *     },
   *     "patientResponseReceivedTimeUTC": "2014-03-22 11:00:00",
   *     "timeZone": null
   *   },
   *   {
   *     "_id": "3291354e086a53bd49c9d39e",
   *     "patientResponseReceivedTime": "2014-03-22 07:00:00",
   *     "patientId": "52b9f1519d16ccbfab2fcff4",
   *     "alertLevel": null,
   *     "patientServiceReading": {
   *       "serviceName": "Blood Pressure",
   *       "serviceId": "52e829f9bf890cf9ee685d85",
   *       "readings": [
   *         {
   *           "name": "Systolic",
   *           "value": 139,
   *           "unit": "mmHg"
   *         },
   *         {
   *           "name": "Diastolic",
   *           "value": 80,
   *           "unit": "mmHg"
   *         }
   *       ]
   *     },
   *     "patientResponseReceivedTimeUTC": "2014-03-22 07:00:00",
   *     "timeZone": null
   *   }
   * ]   
   * </pre>
   *
   */
  @GET
  public List<PatientResponseServiceData> findPatientResponseServiceData(@Context UriInfo info, @Context HttpHeaders headers) throws Exception {
	  
	LOGGER.debug("Retrieving Patient Service Data");
	String patientId = veteranResourceHelper.getPatientId();
	List<PatientResponseServiceData> patientRsponseServiceDataList = patientResponseServiceDataDAO.findByPatientIdSortByDate(patientId);
    if (patientRsponseServiceDataList == null) {
      throw new WebApplicationException(Response.status(Response.Status.NOT_FOUND)
        .entity(new ErrorResponse("no such patient response ")).type(MediaType.APPLICATION_JSON).build());
    }
    return patientRsponseServiceDataList;
  }

  /**
   * Get veteran's vital history
   *
   * <pre>
   * request:
   *   vet/service/my/readings/services
   *
   * response:
   *   vital array
   *   
   * [
   *   {
   *     "serviceName": "Pulse Oximetry (SpO2)",
   *     "serviceId": "52e829f9bf890cf9ee685d86",
   *     "readings": null
   *   },
   *   {
   *     "serviceName": "Blood Pressure",
   *     "serviceId": "52e829f9bf890cf9ee685d85",
   *     "readings": null
   *   }
   * ]
   * </pre>
   *
   */
  @Path("/services")
  @GET
  public List<PatientServiceReading> findPatientResponseServiceNames(@Context UriInfo info, @Context HttpHeaders headers) throws Exception {

	LOGGER.debug("Retrieving Patient Service Names");
	String patientId = veteranResourceHelper.getPatientId();
	List<PatientServiceReading> patientServices = patientResponseServiceDataDAO.findServiceNameByPatientId(patientId);
    if (patientServices == null) {
      throw new WebApplicationException(Response.status(Response.Status.NOT_FOUND)
        .entity(new ErrorResponse("no such patient response ")).type(MediaType.APPLICATION_JSON).build());
    }
    return patientServices;
  }

}
