package gov.va.mobilehealth.courier.resources;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.jayway.restassured.http.ContentType;
import com.jayway.restassured.response.Response;
import gov.va.mobilehealth.cms.jackson.Jackson;
import gov.va.mobilehealth.courier.core.protocol.BroadcastFacilityMessage;
import gov.va.mobilehealth.courier.utils.StaffSecurityProxy;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.IOException;
import java.util.List;

import static gov.va.mobilehealth.courier.testing.FixtureHelpers.fixture;
import static org.fest.assertions.api.Assertions.assertThat;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

/**
 * With the Resource class calling into the corresponding Service class,
 * Resource Integration Test should be focused on ensuring web specifics are being handled
 * endpoints are established and running, exceptions translate to correct http error codes, query param
 * path param, payloads are handled correctly.  Most variations of variables themselves should be handled
 * at the service layer
 */
public class BroadcastFacilityMessageResourceIntTest {

  protected static StaffSecurityProxy staffSecurityProxy;
  private static final ObjectMapper MAPPER = Jackson.newObjectMapper();

  @BeforeClass
  public static void initSecurity() {

    staffSecurityProxy = new StaffSecurityProxy();
    staffSecurityProxy.initAccessToken();
  }

  @AfterClass
  public static void closeSecurityConnection() {
    staffSecurityProxy.logoutAndCloseConnection();
  }

  @Test
  public void createAndGetFacilityMessage() throws IOException {
    String facilityMessage = fixture("fixtures/broadcastFacilityMessageResIntTest.json");

    // Post the new protocol
    staffSecurityProxy.addAccessToken().given()
      .contentType(ContentType.JSON).body(facilityMessage).when()
      .put("/service/facilityMessages").then().log().everything()
      .statusCode(204);

    // Get the list of protocol included the new protocol just created
    Response res = staffSecurityProxy.addAccessToken().given()
      .when()
      .get("/service/facilityMessages")
      .then().log().everything()
      .contentType(ContentType.JSON)
      .statusCode(200)
      .extract()
      .response();

    final List<BroadcastFacilityMessage> messagesReturned =
      MAPPER.readValue(res.asString(),
        new TypeReference<List<BroadcastFacilityMessage>>() {
        }
      );

    // Verify the new protocol is there
    BroadcastFacilityMessage foundMessage = null;
    for (BroadcastFacilityMessage m : messagesReturned) {
      if (m.getName().equals("My First Facility Message")) {
        foundMessage = m;
        break;
      }
    }

    assertThat(foundMessage).isNotNull();
    assertThat(foundMessage.getAssigningAuthority()).isEqualTo("dfn-688");
    assertThat(foundMessage.getCreateBy()).isNotNull();
    assertThat(foundMessage.getCreateDate()).isNotNull();
    assertThat(foundMessage.getMessage()).isEqualTo("My First Facility Message Description");
    assertThat(foundMessage.getName()).isEqualTo("My First Facility Message");
    assertThat(foundMessage.isEnabled()).isEqualTo(true);
    assertThat(foundMessage.getId()).isNotNull();
  }


  @Test
  public void createAndGetById() throws IOException {
    String facilityMessage = fixture("fixtures/broadcastFacilityMessageResIntTest.json");

    // Post the new protocol
    staffSecurityProxy.addAccessToken().given()
      .contentType(ContentType.JSON).body(facilityMessage).when()
      .put("/service/facilityMessages").then().log().everything()
      .statusCode(204);

    // Get the list of protocol included the new protocol just created
    Response res = staffSecurityProxy.addAccessToken().given()
      .when()
      .get("/service/facilityMessages")
      .then().log().everything()
      .contentType(ContentType.JSON)
      .statusCode(200)
      .extract()
      .response();

    final List<BroadcastFacilityMessage> messagesReturned =
      MAPPER.readValue(res.asString(),
        new TypeReference<List<BroadcastFacilityMessage>>() {
        }
      );

    // Verify the new protocol is there
    BroadcastFacilityMessage foundMessage = null;
    for (BroadcastFacilityMessage m : messagesReturned) {
      if (m.getName().equals("My First Facility Message")) {
        foundMessage = m;
        break;
      }
    }


    //Now get by Id
    // Get the list of protocol included the new protocol just created
    Response resById = staffSecurityProxy.addAccessToken().given()
      .when()
      .get("/service/facilityMessages/" + foundMessage.getId())
      .then().log().everything()
      .contentType(ContentType.JSON)
      .statusCode(200)
      .extract()
      .response();

    final BroadcastFacilityMessage messagesReturnedById =
      MAPPER.readValue(resById.asString(),
        new TypeReference<BroadcastFacilityMessage>() {
        }
      );

    assertNotNull(messagesReturnedById);
    assertEquals(messagesReturnedById.getId(), foundMessage.getId());

  }
}
