package gov.va.mobilehealth.courier.resources;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.jayway.restassured.RestAssured;
import com.jayway.restassured.response.Response;

import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import gov.va.mobilehealth.courier.resources.helper.ErrorCode;
import gov.va.mobilehealth.courier.resources.helper.ErrorMessageConfigure;
import gov.va.mobilehealth.courier.utils.StaffSecurityProxy;

import org.aeonbits.owner.ConfigFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static org.fest.assertions.api.Assertions.assertThat;

public class SendMessageResourceTest {
    private static final Logger LOGGER = LoggerFactory.getLogger(SendMessageResourceTest.class);
    protected static StaffSecurityProxy staffSecurityProxy;

    static ErrorMessageConfigure errorMessageConfig = getErrorMessageConfig();

    static ErrorMessageConfigure getErrorMessageConfig() {
      return ConfigFactory.create(ErrorMessageConfigure.class, System.getProperties(), System.getenv());
    }

    @BeforeClass
    public static void initSecurity() {
      staffSecurityProxy = new StaffSecurityProxy();
      staffSecurityProxy.initAccessToken();
    }

    @AfterClass
    public static void closeSecurityConnection() {
      staffSecurityProxy.logoutAndCloseConnection();
    }

    @Test
    public void sendMessageSuccess() throws JsonProcessingException {
        String patient = "{\"id\":\"52b9f1519d16ccbfab2fcff4\", \"text\":\"test\"}";
        staffSecurityProxy.addAccessToken()
            .header("Content-Type", "application/json" )
            .header("Accept", "application/json, text/plain, */*")
            .log()
            .everything()
            .and()
            .body(patient)
            .when()
            .post("/service/messages")
            .then().statusCode(204);
    }

    @Test
    public void sendMessageFailPatientNotFound() throws JsonProcessingException {
        String databody = "{\"id\":\"52b9f1519d16ccbfab2fcfa4\", \"text\":\"test\"}";
        String expected = "{\"code\":" + ErrorCode.MSG_PATIENT_NO_FOUND + ",\"message\":\"" + errorMessageConfig.patientNotFound() + "\"}";
        //String expected = "{\"message\":\"" + errorMessageConfig.patientNotFound() + "\"}";
        Response res = staffSecurityProxy.addAccessToken()
            .header("Content-Type", "application/json" )
            .header("Accept", "application/json, text/plain, */*")
            .log()
            .everything()
            .and()
            .body(databody)
            .when()
            .post("/service/messages")
            .then().statusCode(400)
            .extract()
            .response();
        String responseString = res.asString();
        assertThat(responseString).isEqualTo(expected);
    }

    @Test
    public void sendMessageFailMissingField() throws JsonProcessingException {
        String databody = "{\"id\":\"52b9f1519d16ccbfab2fcfa4\"}";
        String expected = "{\"code\":" + ErrorCode.MSG_MISSING_REQUIRED_FIELD + ",\"message\":\"" + errorMessageConfig.missingRequiredField() + "\"}";
        Response res = staffSecurityProxy.addAccessToken()
                .header("Content-Type", "application/json" )
                .header("Accept", "application/json, text/plain, */*")
                .log()
                .everything()
                .and()
                .body(databody)
                .when()
                .post("/service/messages")
                .then().statusCode(400)
                .extract()
                .response();
        String responseString = res.asString();
        assertThat(responseString).isEqualTo(expected);
    }

    @Test
    public void sendMessageFailMessageLength() throws JsonProcessingException {
        String databody = "{\"id\":\"52b9f1519d16ccbfab2fcff4\", \"text\":\"larger than 160 charactor string larger than 160 charactor string larger than 160 charactor string larger than 160 charactor string larger than 160 charactor string larger than 160 charactor string larger than 160 charactor string larger than 160 charactor string larger than 160 charactor string \"}";
        String expected = "{\"code\":" + ErrorCode.MSG_MESSAGE_LENGTH_ERROR + ",\"message\":\"" + errorMessageConfig.messageLengthError() + "\"}";
        Response res = staffSecurityProxy.addAccessToken()
                .header("Content-Type", "application/json" )
                .header("Accept", "application/json, text/plain, */*")
                .log()
                .everything()
                .and()
                .body(databody)
                .when()
                .post("/service/messages")
                .then().statusCode(400)
                .extract()
                .response();
        String responseString = res.asString();
        assertThat(responseString).isEqualTo(expected);
    }
}

