package gov.va.mobilehealth.courier.protocol;

import gov.va.mobilehealth.courier.core.MessageType;
import gov.va.mobilehealth.courier.core.Patient;
import gov.va.mobilehealth.courier.core.PatientPreference;
import gov.va.mobilehealth.courier.core.PatientResponseServiceData;
import gov.va.mobilehealth.courier.core.protocol.*;
import gov.va.mobilehealth.courier.helper.MockMessageProducer;
import gov.va.mobilehealth.courier.smsgateway.communication.CMSMessage;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import static org.fest.assertions.api.Assertions.assertThat;

@RunWith(Parameterized.class)
public class MessageProcessorValidatorTest {

  protected static final String HIGH_CRITICAL_MESSAGE = "Message for Critical High Level Alert";
  protected static final String LOW_CRITICAL_MESSAGE = "Message for Critical Low Level Alert";
  private static final String CONFIRMATION_MESSAGE = "Confirmation Message";
  private static final String HIGH_LEVEL_ALERT_MESSAGE = "Above Desirable Level Message";
  private static final String LOW_LEVEL_ALERT_MESSAGE = "Below Desirable Level Message";

  private static final float LOWEST_VALID_READING_1 = 70f;
  private static final float HIGHEST_VALID_READING_1 = 270f;
  private static final float SET_LOW_LEVEL_ALERT_READING_1 = 85f;
  private static final float SET_HIGH_LEVEL_ALERT_READING_1 = 140f;
  private static final float CRITICAL_LOW_LEVEL_ALERT_READING_1 = 80f;
  private static final float CRITICAL_HIGH_LEVEL_ALERT_READING_1 = 200f;


  private static PatientProtocol patientProtocol = buildPatientProtocol();

  private String messageText;
  private String expectedValidationMessage;


  public MessageProcessorValidatorTest(String messageText, String expectedValidationMessage) {
    this.messageText = messageText;
    this.expectedValidationMessage = expectedValidationMessage;
  }

  @Parameterized.Parameters(name = "messageText: {0}, expectedValidationMessage: {1} ")
  public static Collection<Object[]> data() throws Exception {
    return Arrays.asList(new Object[][]{
      // first reading
      {"BLood", FormatValidator.INCORRECT_FORMAT_MESSAGE},
      {"120", FormatValidator.INCORRECT_FORMAT_MESSAGE},
      {"BL " + HIGHEST_VALID_READING_1 + "/66", ValidRangeValidator.INVALID_MESSAGE},
      {"BL " + (HIGHEST_VALID_READING_1 - 1f) + "/66", HIGH_CRITICAL_MESSAGE},
      {"BL " + CRITICAL_HIGH_LEVEL_ALERT_READING_1 + "/66", HIGH_CRITICAL_MESSAGE},
      {"BL " + (CRITICAL_HIGH_LEVEL_ALERT_READING_1 - 1f) + "/66", HIGH_LEVEL_ALERT_MESSAGE},
      {"BL " + SET_HIGH_LEVEL_ALERT_READING_1 + "/66", HIGH_LEVEL_ALERT_MESSAGE},
      {"BL " + (SET_HIGH_LEVEL_ALERT_READING_1 - 1f) + "/66", CONFIRMATION_MESSAGE},
      {"BL " + (SET_LOW_LEVEL_ALERT_READING_1 + 1f) + "/66", CONFIRMATION_MESSAGE},
      {"BL " + SET_LOW_LEVEL_ALERT_READING_1 + "/66", LOW_LEVEL_ALERT_MESSAGE},
      {"BL " + (CRITICAL_LOW_LEVEL_ALERT_READING_1 + 1f) + "/66", LOW_LEVEL_ALERT_MESSAGE},
      {"BL " + CRITICAL_LOW_LEVEL_ALERT_READING_1 + "/66", LOW_CRITICAL_MESSAGE},
      {"BL " + (LOWEST_VALID_READING_1 + 1f) + "/66", LOW_CRITICAL_MESSAGE},
      {"BL " + LOWEST_VALID_READING_1 + "/66", ValidRangeValidator.INVALID_MESSAGE},
      // second reading
      {"BL 86/140", ValidRangeValidator.INVALID_MESSAGE},
      {"BL 86/139", HIGH_CRITICAL_MESSAGE},
      {"BL 86/100", HIGH_CRITICAL_MESSAGE},
      {"BL 86/99", HIGH_LEVEL_ALERT_MESSAGE},
      {"BL 86/90", HIGH_LEVEL_ALERT_MESSAGE},
      {"BL 86/89", CONFIRMATION_MESSAGE},
      {"BL 86/66", CONFIRMATION_MESSAGE},
      {"BL 86/65", LOW_LEVEL_ALERT_MESSAGE},
      {"BL 86/61", LOW_LEVEL_ALERT_MESSAGE},
      {"BL 86/60", LOW_CRITICAL_MESSAGE},
      {"BL 86/41", LOW_CRITICAL_MESSAGE},
      {"BL 86/40", ValidRangeValidator.INVALID_MESSAGE},
      // combination of readings - to prove priority of messages: invalid, critical, alert, confirmation
      {"BL 269/99", HIGH_CRITICAL_MESSAGE},
      {"BL 80/65", LOW_CRITICAL_MESSAGE},
      {"BL 85/139", HIGH_CRITICAL_MESSAGE},
      {"BL 85/60", LOW_CRITICAL_MESSAGE}
    });
  }

  protected static PatientProtocol buildPatientProtocol() {
    Vital template = new Vital();

    Reading reading1 = new Reading();
    reading1.setName("systolic");
    reading1.setValid(new ReadingThreshold(LOWEST_VALID_READING_1, HIGHEST_VALID_READING_1));
    reading1.setCritical(new ReadingThreshold(CRITICAL_LOW_LEVEL_ALERT_READING_1, CRITICAL_HIGH_LEVEL_ALERT_READING_1));
    reading1.setAlert(new ReadingThreshold(SET_LOW_LEVEL_ALERT_READING_1, SET_HIGH_LEVEL_ALERT_READING_1));
    reading1.setUnit("mmHg");

    Reading reading2 = new Reading();
    reading2.setName("diastolic");
    reading2.setValid(new ReadingThreshold(40, 140));
    reading2.setCritical(new ReadingThreshold(60, 100));
    reading2.setAlert(new ReadingThreshold(65, 90));
    reading2.setUnit("mmHg");

    CMSService cmsService = new CMSService();
    cmsService.setReadings(Arrays.asList(reading1, reading2));
    cmsService.setCriticalMessage(new MessageElement(LOW_CRITICAL_MESSAGE, HIGH_CRITICAL_MESSAGE));
    cmsService.setFirstLevelMessage(new MessageElement(LOW_LEVEL_ALERT_MESSAGE, HIGH_LEVEL_ALERT_MESSAGE));
    cmsService.setKeywords(Arrays.asList("Blood", "BP"));

    template.setService(cmsService);
    template.setConfirmationMessage(CONFIRMATION_MESSAGE);

    PatientProtocol patientProtocol = new PatientProtocol();

    List<Template> templates = new ArrayList<Template>();
    templates.add(template);
    Protocol protocol = new Protocol();
    protocol.setTemplates(templates);
    protocol.setName("Blood Pressure");
    patientProtocol.setProtocol(protocol);

    return patientProtocol;
  }

  @Test
  public void validatePatientReadingReturnsExpectedMessage() {
    MessageProcessorService messageProcessorService = new MessageProcessorService(new MockMessageProducer());

    Patient patient = new Patient();
    patient.setPreference(new PatientPreference());
//    CMSMessage cmsMessage = new CMSMessage();
    CMSMessage cmsMessage = new CMSMessage("Source", patient, messageText, MessageType.OUT);
    cmsMessage.setText(messageText);
    PatientResponseServiceData patientResponseServiceData = messageProcessorService.buildPatientResponseServiceData(patientProtocol, (Vital) patientProtocol.getProtocol().getTemplates().get(0), cmsMessage);

    assertThat(messageProcessorService.validatePatientReading(patientProtocol, (Vital) patientProtocol.getProtocol().getTemplates().get(0), patientResponseServiceData)).isEqualTo(expectedValidationMessage);
  }
}
